/*  Robodoc Markup Header
*@***h* AMSU-A_FCDR_STAR/amsu.fcdr.gridded.AmsuFcdrDailyProcessor.java
*
* NAME
*   amsu.fcdr.gridded.AmsuFcdrDailyProcessor.java
*
* PURPOSE
*   Process daily FCDR L1c files and generate daily gridded AMSU-A brightness temperature FCDR.
* 
* DESCRIPTION
*	This class is used to generate daily gridded brightness temperature FCDR for specified satellite.
*
* AUTHOR
*   Xianjun Hao, George Mason University 
*
* COPYRIGHT 
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*
* REVISION HISTORY
*   12/08/2015 Initial Version prepared for NCEI
*
*@*****
 */
package amsu.fcdr.gridded;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.lang.Exception;
import java.lang.Runnable;

import ucar.nc2.NetcdfFile;
import ucar.nc2.Variable;
import ucar.ma2.*;
import ucar.nc2.Attribute;
import ucar.nc2.Dimension;
import ucar.nc2.NetcdfFileWriter;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;


/**
 * Process daily FCDR L1c files and generate daily gridded AMSU-A brightness temperature FCDR.
 *
 * @author Xianjun Hao (xianjun.hao@noaa.gov)
 */
public class AmsuFcdrDailyProcessor implements AmsuFcdrL1cConstant, Runnable {

    /**
     * Information logger for FCDR processing.
     */
    final private static Logger  fcdrErrorLogger = LogManager.getLogger("ErrorLog");

    /**
     * Information logger for FCDR processing.
     */
    final private static Logger  fcdrInfoLogger = LogManager.getLogger("InfoLog");
    
    /**
     * year of the day for AMSU-A gridded brightness temperature FCDR generation.
     */
    private int year = 0;

    /**
     * Month of the day for AMSU-A gridded brightness temperature FCDR generation.
     */
    private int month = 0;

    /**
     * Calendar day of the day for AMSU-A gridded brightness temperature FCDR generation.
     */
    private int day = 0;

    /**
     * Julian day of the day for AMSU-A gridded brightness temperature FCDR generation.
     */
    private int jday = 0;

    /**
     * start time in seconds since 1978-1-1 00:00:00 UTC
     */
    private long secondsSince78_start = 0;

    /**
     * end time in second since 1978-1-1 00:00:00 UTC
     */
    private long secondsSince78_end = 0;

    /**
     * Short name of satellite, i.e. AQUA, M02, N15, N16, N17, N18 etc.
     */
    private String satName = null;

    /**
     * Root directory of FCDR L1C data. Data are stored in rootFCDRL1c/satName/year/.
     */
    private String rootFCDRL1c = null;

    /**
     * Root directory of merged FCDR L1c daily data. Data are stored in rootFCDL1d/sateName/year/.
     */
    private String rootFCDRL1d = null;

    /**
     * Root directory of gridded FCDR data. Data are stored in rootFCDRL1g/sateName/year/.
     */
    private String rootFCDRL3 = null;

    /**
     * Flag to indicate if keep intermediate data files (the quality controlled and temporally aligned daily level 1c data). Set it
     * to true to use intermediate data for testing or other purpose. Default value is false.
     */
    private boolean flagKeepIntermediateData = false;

    /**
     * List of FCDR L1c files for the specified date.
     */
    private File[] listFcdrL1c = null;

    private AmsuFcdrL1cIndex[] fcdrL1cIndex;

    /**
     * Number of FCDR L1c files for the specified day.
     */
    private int numFcdrL1c = 0;

    /**
     * Flag to indicate if there are available data to each scan period
     */
    private boolean[] vflag;

    /**
     * Number of data items for each scan period.
     */
    private int[] count;

    /**
     * nadir latitude, used to determine ascending/descending orbit
     */
    private float[] nadirLat;

    /**
     * BT variable list
     */
    private List<Variable> varBtList = null;

    /**
     * List of geolocation related variables
     */
    private List<Variable> varGeoList = null;

    /**
     * Dimension of 1D data.
     */
    private List<Dimension> varDim1 = null;

    /**
     * Dimension of 2D data.
     */
    private List<Dimension> varDim2 = null;

    /**
     * Name of quality controlled and temporally aligned daily L1c file (intermediate data file)
     */
    private String fileAligned = null;
    
    /**
     *  Flag to indicate if FCDR data available for the specified date.
     */
    private boolean isFcdrReady = false; 
            
    /**
     * Verbose flag for running status.
     */
    private boolean verbose = false;
    
    /**
     * Constructor. Create an instance for daily gridded FDCR generation for specified satellite and date.
     *
     * @param satName, short name of Satellite.
     * @param year, year.
     * @param month, month.
     * @param day, day.
     * @param rootFcdrL1c, root directory of FCDR L1c files.
     * @param rootFcdrL1d, root directory of daily merged FCDR L1c data (temporary).
     * @param rootFcdrL3, root directory of daily gridded FCDR data.
     * @param flagKeepIntermediateData, flag to indicate if keep intermediate data file.
     * @param verbose, flag to indicate if output more detailed processing information.
     */
    public AmsuFcdrDailyProcessor(String satName, int year, int month, int day, 
            String rootFcdrL1c, String rootFcdrL1d, String rootFcdrL3, boolean flagKeepIntermediateData, boolean verbose) {
        this.fcdrL1cIndex = null;
        this.satName = satName;
        this.year = year;
        this.month = month;
        this.day = day;
        this.jday = TimeUtility.getDayofYear(year, month, day);
        this.rootFCDRL1c = rootFcdrL1c;
        this.rootFCDRL1d = rootFcdrL1d;
        this.rootFCDRL3 = rootFcdrL3;
        this.flagKeepIntermediateData = flagKeepIntermediateData;
        this.verbose = verbose;
        this.fileAligned = this.getFullpathL1cDaily();
    }

    /**
     * Override method run to produce AMSU-A daily gridded FCDR.
     */
    public void run() {
        String msg = null;
        
        try {
            this.initL1c();
            if (this.isFcdrReady) {
                this.alignL1cData();
                // write aligned daily L1c data to file                    
                if (this.writeDailyL1c(this.fileAligned)){
                    // grid aligned daily L1c data and write to output file
                    this.produceGriddedData();    
                    msg = String.format("Produced daily gridded FCDR for %s, %4d-%02d-%02d", 
                        this.satName, this.year, this.month, this.day);
                    // delete temporary file
                    this.deleteAlignedL1c();
                } else {
                    msg = String.format("Error in writing daily aligned L1c: %s %4d-%02d-%02d ", 
                                this.satName, this.year, this.month, this.day); 
                }
            } else {
                msg = String.format("Error in processing %s %4d-%02d-%02d, FCDR L1c data files not available. ", 
                this.satName, this.year, this.month, this.day);
            }
                              
            if(this.verbose){                    
                fcdrInfoLogger.info(msg);
            }     
                
            
        } catch (Exception e) { // log exceptions
            msg = String.format("Errors, exception in processing %s %4d-%02d-%02d. ", 
                    this.satName, this.year, this.month, this.day);
            fcdrErrorLogger.error(msg);
            e.printStackTrace();
        }
    }

    /**
     * Produce gridded daily FCDR.
     */
    private void produceGriddedData(){
        AmsuFcdrGriddedProcessor gridProcessor =
                new AmsuFcdrGriddedProcessor(this.satName, this.year, this.month, this.day, this.rootFCDRL1c, 
                this.rootFCDRL3, this.rootFCDRL1d, this.flagKeepIntermediateData, this.fileAligned, this.verbose);
        gridProcessor.produceGriddedFcdr();
    }
    
    
    /**
     * Get the Julian day of current data for processing.
     *
     * @return the Julian day of current data for processing.
     */
    public int getJulDay() {
        return this.jday;
    }

    /**
     * Get current calendar day of data for processing.
     *
     * @return current calendar day of data for processing.
     */
    public int[] getCalDay() {
        int[] cday = new int[3];
        cday[0] = this.year;
        cday[1] = this.month;
        cday[2] = this.day;
        return cday;
    }

    /**
     * Initialize to setup essential parameters for daily data processing.
     */
    private void initL1c() {
        if (this.year == 0) {  // unset
            this.listFcdrL1c = null;
            this.numFcdrL1c = 0;
            this.vflag = null;
            this.count = null;
            this.nadirLat = null;
        } else {
            AmsuFcdrFileManager fcdrManager = new AmsuFcdrFileManager(this.rootFCDRL1c);
            this.listFcdrL1c = fcdrManager.getAllFCDRfilesForDate(this.satName, this.year, this.month, this.day);
            
            if (this.listFcdrL1c != null){                
                this.numFcdrL1c = this.listFcdrL1c.length;
                if (this.numFcdrL1c >0){
                    this.count = new int[NUMBER_SCAN_PER_DAY];
                    this.vflag = new boolean[NUMBER_SCAN_PER_DAY];
                    this.nadirLat = new float[NUMBER_SCAN_PER_DAY];
                    for (int k = 0; k < NUMBER_SCAN_PER_DAY; k++) {
                        this.vflag[k] = false;
                        this.nadirLat[k] = FILLED_BAD_GEOLOCATION; // initialize with filled invalid value first
                    }                    
                    if(this.createFolderL1cDaily()){
                        this.isFcdrReady = true;                        
                    }else {
                        this.isFcdrReady = false;
                    }
                } else {
                    this.isFcdrReady = false;
                }
            } else {
                this.isFcdrReady = false;
            }
        }
    }

    /**
     * Get list of AMSU FCDR L1c files for the specified day.
     *
     * @return, list of AMSU FCDR L1c files.
     */
    public File[] getFcdrL1cList() {
        return this.listFcdrL1c;
    }

    /**
     * Determine index of scans in each FCDR L1c file. 0-10799 for valid scans, and -1 for invalid scans.
     */
    public void alignL1cData() {
        int k;
        int j;
        int m;
        NetcdfFile fileNc = null;
        Variable dataVar = null;
        Variable flagVar = null;
        int[] shape = new int[2];
        int[] origin = new int[2];
        ArrayDouble.D1 dataArray1;
        ArrayFloat.D2 latArray;
        ArrayByte.D2 flagArray;
        int numScan = 0;
        double tmp = 0.0;
        byte dflag = 0;
        int numAll = 0;
        int numIn = 0;
        int numValid = 0;
        int numUnique = 0;
        int numBad = 0;
        float lat1 = -999.0f;
        float lat2 = -999.0f;
        String msg = "";

        // check number of orbital level-1c files for the day
        if (this.numFcdrL1c < 1) {
            // can't find AMSU-A level 1c files for the specified satellite and date, log it.
            msg = String.format("Couldn't find AMSU-A L1C files for satelite %1$s on %2$04d-%3$02d-%4$02d", this.satName,
                    this.year, this.month, this.day);
                fcdrErrorLogger.error(msg);
        } else {
            this.fcdrL1cIndex = new AmsuFcdrL1cIndex[this.numFcdrL1c];
            // check time, geolocation and data of each file to determine index
            this.secondsSince78_start = TimeUtility.getStartSecondSince78(year, month, day);
            this.secondsSince78_end = this.secondsSince78_start + 24 * 3600;

            // process file by file
            for (k = 0; k < this.numFcdrL1c; k++) {
                try {
                    fileNc = NetcdfFile.open(this.listFcdrL1c[k].getAbsolutePath());

                    // read scan time since 78
                    dataVar = fileNc.findVariable(AmsuFcdrL1cConstant.SCAN_TIME_SINCE78);
                    shape = dataVar.getShape();
                    dataArray1 = (ArrayDouble.D1) dataVar.read(origin, shape);
                    numScan = shape[0];
                    numAll = numAll + numScan;

                    // read quality flag, use channel 9 to check flag here
                    flagVar = fileNc.findVariable(AmsuFcdrL1cConstant.QUALITY_FLAGS_PIXEL_IMICA_CH9);
                    shape = flagVar.getShape();
                    flagArray = (ArrayByte.D2) flagVar.read(origin, shape);

                    // read latitude
                    dataVar = fileNc.findVariable(AmsuFcdrL1cConstant.LATITUDE);
                    shape = dataVar.getShape();
                    latArray = (ArrayFloat.D2) dataVar.read(origin, shape);

                    // check index 
                    this.fcdrL1cIndex[k] = new AmsuFcdrL1cIndex(numScan);
                    int[] ind = new int[numScan];

                    // check scan by scan
                    for (j = 0; j < numScan; j++) {
                        tmp = dataArray1.get(j);
                        if ((tmp >= this.secondsSince78_start) && (tmp < this.secondsSince78_end)) {
                            numIn = numIn + 1;
                            m = (int) Math.floor((tmp - this.secondsSince78_start) / AmsuFcdrL1cConstant.SECONDS_PER_SCAN);
                            count[m] = count[m] + 1;
                            // check if geolocation and data are valid.
                            // if valid, mark the timestamp to avoid repetition.
                            dflag = flagArray.get(j, 15);
                            if (((dflag & BIT_MASK_BAD_SCAN) == BIT_MASK_BAD_SCAN) || ((dflag & BIT_MASK_BAD_GEOLOCATION)
                                    == BIT_MASK_BAD_GEOLOCATION)) {
                                // skip this scan
                                this.fcdrL1cIndex[k].setFlag(j, false);
                                this.fcdrL1cIndex[k].setIndex(j, -1);
                                numBad = numBad + 1;
                            } else {
                                // keep this scan if hasn  't been used                               
                                numValid = numValid + 1;
                                if (!this.vflag[m]) { // hasn't been used, add it
                                    this.fcdrL1cIndex[k].setFlag(j, true);
                                    // set scan index
                                    this.fcdrL1cIndex[k].setIndex(j, m);
                                    this.vflag[m] = true;
                                    lat1 = latArray.get(j, INDEX_AMSU_A_NADIR_PIXEL_LEFT);
                                    lat2 = latArray.get(j, INDEX_AMSU_A_NADIR_PIXEL_RIGHT);
                                    this.nadirLat[m] = (lat1 + lat2) * 0.5f;
                                    numUnique = numUnique + 1;
                                } else {  // valid, but has been used, skip it.
                                    this.fcdrL1cIndex[k].setFlag(j, false);
                                    this.fcdrL1cIndex[k].setIndex(j, -1);
                                }
                            }
                        } else { // skip this scan
                            this.fcdrL1cIndex[k].setFlag(j, false);
                            this.fcdrL1cIndex[k].setIndex(j, -1);
                        }
                    }
                } catch (java.io.IOException e) {
                    e.printStackTrace();
                } catch (InvalidRangeException e) {
                    e.printStackTrace();
                } finally {
                    if (fileNc != null) {
                        try {
                            fileNc.close();
                        } catch (java.io.IOException ioe) {
                            ioe.printStackTrace();
                        }
                    }
                }
            } // end all files
        }
    }

    /**
     * Count number of valid scans for the specified day.
     *
     * @return number of valid scans.
     */
    public int getNumberOfValidScans() {
        int num = 0;
        int k, j, ns;
        boolean[] flag;

        for (k = 0; k < this.numFcdrL1c; k++) {
            flag = this.fcdrL1cIndex[k].getFlag();
            ns = this.fcdrL1cIndex[k].getNumScan();
            for (j = 0; j < ns; j++) {
                if (flag[j]) {
                    num = num + 1;
                }
            }
        }
        return num;
    }

    /**
     * Get nadir latitude
     *
     * @return array of nadir latitude.
     */
    public float[] getNadirLatitude() {
        return this.nadirLat;
    }

    /**
     * Write out brightness temperature to daily L1c file.
     *
     * @param fileOut, NetcdfFileWriter for output file.
     * @param varDim, Dimension of output.
     */
    private void writeDailyL1cBt(NetcdfFileWriter fileOut) {
        NetcdfFile fileIn = null;
        Variable varIn = null;

        int i = 0;
        int k = 0;
        int j = 0;
        int m = 0;
        int[] shape = new int[2];
        int[] origin = new int[2];
        int[] index;
        boolean[] flag;
        int len;
        int[] varSize = new int[]{NUMBER_SCAN_PER_DAY, NUMBER_PIXELS_PER_SCAN};
        ArrayShort.D2 arrIn = null;
        ArrayFloat.D2 farrIn = null;
        ArrayDouble.D1 darrIn = null;
        List<Attribute> attList = null;
        List<Variable> varList = null;
        String varName;
        short vtmp = 0;
        float stmp = 0;
        double dtmp = 0.0d;
        String msg = "";

        int row = 0;

        if (this.numFcdrL1c > 0) {

            // write scan time
            ArrayDouble.D1 arrTm = new ArrayDouble.D1(NUMBER_SCAN_PER_DAY);
            try {
                for (k = 0; k < this.numFcdrL1c; k++) {
                    // read data from input file
                    fileIn = NetcdfFile.open(this.listFcdrL1c[k].getAbsolutePath());
                    varIn = fileIn.findVariable(AmsuFcdrL1cConstant.SCAN_TIME_SINCE78);
                    darrIn = (ArrayDouble.D1) varIn.read();
                    shape = varIn.getShape();
                    index = this.fcdrL1cIndex[k].getIndex();
                    flag = this.fcdrL1cIndex[k].getFlag();
                    len = shape[0];
                    for (j = 0; j < len; j++) {
                        if (flag[j]) {
                            row = index[j];
                            dtmp = darrIn.get(j);
                            arrTm.setDouble(row, dtmp);
                        }
                    }
                    fileIn.close();
                }
                Variable varTm = fileOut.findVariable(AmsuFcdrL1cConstant.SCAN_TIME_SINCE78);
                fileOut.write(varTm, arrTm);

            } catch (Exception e) {
                e.printStackTrace();
            }

            // write geolocation, and angles  
            for (Variable varGeo : this.varGeoList) {
                varName = varGeo.getShortName();
                ArrayFloat.D2 arrGeo = new ArrayFloat.D2(AmsuFcdrL1cConstant.NUMBER_SCAN_PER_DAY,
                        AmsuFcdrL1cConstant.NUMBER_PIXELS_PER_SCAN);
                Index idxGeo = arrGeo.getIndex();
                try {
                    // compose files one by one
                    for (k = 0; k < this.numFcdrL1c; k++) {
                        // read data from input file
                        fileIn = NetcdfFile.open(this.listFcdrL1c[k].getAbsolutePath());
                        varIn = fileIn.findVariable(varName);
                        shape = varIn.getShape();
                        farrIn = (ArrayFloat.D2) varIn.read(origin, shape);
                        // compose to new file
                        index = this.fcdrL1cIndex[k].getIndex();
                        flag = this.fcdrL1cIndex[k].getFlag();

                        //len = index.length;
                        len = shape[0];

                        for (j = 0; j < len; j++) {
                            if (flag[j]) {
                                // valid data
                                row = index[j];
                                if (row < 0 || row >= NUMBER_SCAN_PER_DAY) {
                                    msg = String.format("Row=%1$d, error in processing AMSU-A l1c file %2$s.",
                                            row, this.listFcdrL1c[k].getAbsolutePath());
                                    fcdrErrorLogger.error(msg);
                                } else {
                                    for (m = 0; m < NUMBER_PIXELS_PER_SCAN; m++) {
                                        stmp = farrIn.get(j, m);
                                        arrGeo.setFloat(idxGeo.set(row, m), stmp);
                                    }
                                }
                            }
                        } // end for j

                        fileIn.close();

                    } // end for k

                    fileOut.write(varGeo, arrGeo);

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }  // end varBt   

            // write BT variable one by one
            for (Variable varBt : this.varBtList) {
                varName = varBt.getShortName();
                int vk = varName.indexOf(AmsuFcdrL1cConstant.PREFIX_FCDR_BRIGHTNESS_TEMPERATURE);
                if (vk < 0) {
                    continue;
                }
                ArrayShort.D2 arrData = new ArrayShort.D2(AmsuFcdrL1cConstant.NUMBER_SCAN_PER_DAY,
                        AmsuFcdrL1cConstant.NUMBER_PIXELS_PER_SCAN);
                Index idx = arrData.getIndex();
                try {
                    // compose files one by one
                    for (k = 0; k < this.numFcdrL1c; k++) {
                        // read data from input file
                        fileIn = NetcdfFile.open(this.listFcdrL1c[k].getAbsolutePath());
                        varIn = fileIn.findVariable(varName);
                        
                        shape = varIn.getShape();
                        arrIn = (ArrayShort.D2) varIn.read(origin, shape);

                        // compose to new file
                        index = this.fcdrL1cIndex[k].getIndex();
                        flag = this.fcdrL1cIndex[k].getFlag();

                        //len = index.length;
                        len = shape[0];

                        for (j = 0; j < len; j++) {
                            if (flag[j]) {
                                // valid data
                                row = index[j];
                                if (row < 0 || row >= AmsuFcdrL1cConstant.NUMBER_SCAN_PER_DAY) {
                                    msg = String.format("Row=%1$d, error in processing AMSU-A l1c file %2$s.",
                                            row, this.listFcdrL1c[k].getAbsolutePath());
                                    fcdrErrorLogger.error(msg);                                    
                                } else {
                                    for (m = 0; m < AmsuFcdrL1cConstant.NUMBER_PIXELS_PER_SCAN; m++) {
                                        vtmp = arrIn.get(j, m);
                                        arrData.setShort(idx.set(row, m), vtmp);
                                    }
                                }
                            }
                        } // end for j

                        fileIn.close();

                    } // end for k

                    fileOut.write(varBt, arrData);

                } catch (Exception e) {
                    e.printStackTrace();
                    fcdrErrorLogger.error("Error in writing " + fileOut.getNetcdfFile().getDetailInfo());
                }
            }  // end varBt   
        }  else  {
            
           msg = String.format("Unable to find AMSU FCDR L1c files for %s, %4d-%02d-%02d", this.satName, 
                    this.year, this.month, this.day);
           fcdrErrorLogger.warn(msg);
        }
    }

    /**
     * define daily L1c file.
     *
     * @param fileOut, NetcdfFileWriter for output file.
     * @param varDim, Dimension of output.
     */
    private void defineDailyNetcdf(NetcdfFileWriter fileOut) {
        int k = 0;
        List<Attribute> attList = null;
        int[] shape;
        float latNow, latPrevious, latNext;
        if (this.numFcdrL1c > 0) {

            // create Dimension, Along-track and Cross-Track
            Dimension dimAtrack = fileOut.addDimension(null, "Along-Track", AmsuFcdrL1cConstant.NUMBER_SCAN_PER_DAY);
            Dimension dimXtrack = fileOut.addDimension(null, "Cross-Track", AmsuFcdrL1cConstant.NUMBER_PIXELS_PER_SCAN);
            this.varDim1 = new ArrayList<Dimension>();
            this.varDim2 = new ArrayList<Dimension>();
            this.varDim1.add(dimAtrack);
            this.varDim2.add(dimAtrack);
            this.varDim2.add(dimXtrack);

            // add global attributes
            String version = String.format("%1$s.%2$s", AmsuFcdrGridConstant.VERSION_MAJOR_AMSU_A_FCDR_L3_DATA,
                    AmsuFcdrGridConstant.VERSION_REVISION_AMSU_A_FCDR_L3_DATA);
            fileOut.addGroupAttribute(null, new Attribute("Version", version));
            fileOut.addGroupAttribute(null, new Attribute("SatelliteId", this.satName));
            fileOut.addGroupAttribute(null, new Attribute("Year", (int) this.year));
            fileOut.addGroupAttribute(null, new Attribute("Month", (int) this.month));
            fileOut.addGroupAttribute(null, new Attribute("Day", (int) this.day));

            // create variables for later processing
            Variable vcross = fileOut.addVariable(null, "Cross-Track", DataType.SHORT, "Cross-Track");
            Variable vtime = fileOut.addVariable(null, "Time", DataType.FLOAT, "Along-Track");
            Variable vlat = fileOut.addVariable(null, "NadirLat", DataType.FLOAT, this.varDim1);
            vtime.addAttribute(new Attribute("units", "second"));
            vlat.addAttribute(new Attribute("units", "degree"));

            // create variable, setup attributes
            try {
                NetcdfFile fileIn = NetcdfFile.open(this.listFcdrL1c[0].getAbsolutePath());
                // get all variables
                List<Variable> varBtListIn = fileIn.getVariables();
                this.varBtList = new ArrayList<Variable>();

                for (Variable varIn : varBtListIn) {
                    // variable name
                    String varName = varIn.getShortName();
                    shape = varIn.getShape();
                    int vk = varName.indexOf(AmsuFcdrL1cConstant.PREFIX_FCDR_BRIGHTNESS_TEMPERATURE);
                    if (vk < 0) {
                        continue;
                    }
                    
                    // daily data
                    Variable varBt = fileOut.addVariable(null, varName, DataType.SHORT, this.varDim2);
                    this.varBtList.add(varBt);
                    // get attributes 
                    //varIn =fileIn.findVariable(varNameIn);              
                    attList = varIn.getAttributes();
                    // add attributes to variable for output
                    for (Attribute att : attList) {
                        String str = att.getShortName();
                        
                        int jk = str.indexOf("Chunk"); // remove ChunkSizes
                        if (jk < 0) {
                            varBt.addAttribute(att);
                        }
                    }
                }

                fileIn.close();

                // add geolocation related vairables
                this.varGeoList = new ArrayList<Variable>();
                // latitude
                Variable varLat = fileOut.addVariable(null, AmsuFcdrL1cConstant.LATITUDE, DataType.FLOAT, this.varDim2);
                this.varGeoList.add(varLat);
                // longitude
                Variable varLon = fileOut.addVariable(null, AmsuFcdrL1cConstant.LONGITUDE, DataType.FLOAT, this.varDim2);
                this.varGeoList.add(varLon);
                // sensor zenith angle
                Variable varVza = fileOut.addVariable(null, AmsuFcdrL1cConstant.SENSOR_ZENITH_ANGLE, DataType.FLOAT, this.varDim2);
                this.varGeoList.add(varVza);
                Variable varSza = fileOut.addVariable(null, AmsuFcdrL1cConstant.SOLAR_ZENITH_ANGLE, DataType.FLOAT, this.varDim2);
                this.varGeoList.add(varSza);

                // add orbit A/D
                Variable varOrbit = fileOut.addVariable(null, AmsuFcdrL1cConstant.FCDR_ORBIT_ASCENDING_DESCENDING_FLAG,
                        DataType.SHORT, this.varDim1);

                // add scan time
                Variable varScanTime = fileOut.addVariable(null, AmsuFcdrL1cConstant.SCAN_TIME_SINCE78, DataType.DOUBLE,
                        this.varDim1);
                varScanTime.addAttribute(new Attribute("units", "second"));

                fileOut.create();

                //          write out essential information for later processing
                Array dataTm = Array.factory(DataType.FLOAT, new int[]{NUMBER_SCAN_PER_DAY});
                Array dataXt = Array.factory(DataType.SHORT, new int[]{NUMBER_PIXELS_PER_SCAN});
                Array dataLat = Array.factory(DataType.FLOAT, new int[]{NUMBER_SCAN_PER_DAY});
                for (k = 0; k < NUMBER_SCAN_PER_DAY; k++) {
                    dataLat.setFloat(k, this.nadirLat[k]);
                    dataTm.setFloat(k, (k + 0.5f) * 8.0f);
                }
                for (k = 0; k < NUMBER_PIXELS_PER_SCAN; k++) {
                    dataXt.setShort(k, (short) k);
                }

                fileOut.write(vtime, dataTm);
                fileOut.write(vcross, dataXt);
                fileOut.write(vlat, dataLat);

                // write ascending/descending
                Array dataOrb = Array.factory(DataType.SHORT, new int[]{AmsuFcdrL1cConstant.NUMBER_SCAN_PER_DAY});
                // first scan
                if (this.nadirLat[0] >= AmsuFcdrL1cConstant.MINIMUM_LATITUDE) { // valid 
                    //dataOrb.setShort(0,AMSU_FCDR_L1C_Variable_Constant.ORBIT_INVALID);
                    if (this.nadirLat[1] >= AmsuFcdrL1cConstant.MINIMUM_LATITUDE) { // next scan
                        if (this.nadirLat[0] < this.nadirLat[1]) { // ascending
                            dataOrb.setShort(0, AmsuFcdrL1cConstant.ORBIT_ASCENDING);
                        } else { // descending
                            dataOrb.setShort(0, AmsuFcdrL1cConstant.ORBIT_DESCENDING);
                        }
                    } else {  // simply set as undefined now, further processing later
                        dataOrb.setShort(0, AmsuFcdrL1cConstant.ORBIT_UNDEFINED);
                    }
                } else { // invalid geolocation, mark as invalid                    
                    dataOrb.setShort(0, AmsuFcdrL1cConstant.ORBIT_UNDEFINED);
                }

                // reminding scans
                for (k = 1; k < (NUMBER_SCAN_PER_DAY - 1); k++) {
                    latNow = this.nadirLat[k];
                    latPrevious = this.nadirLat[k - 1];
                    latNext = this.nadirLat[k + 1];

                    if ((latPrevious < AmsuFcdrL1cConstant.MINIMUM_LATITUDE) && (latNext < AmsuFcdrL1cConstant.MINIMUM_LATITUDE)) {
                        //  geolocation of both previous and next scan  invalid, set this point undefined simply, 
                        //  will process in later steps
                        dataOrb.setShort(k, AmsuFcdrL1cConstant.ORBIT_UNDEFINED);
                    } else if (latNow >= AmsuFcdrL1cConstant.MINIMUM_LATITUDE) {
                        if (latNext < AmsuFcdrL1cConstant.MINIMUM_LATITUDE) {
                            // next scan, geolocation invalid, previous one will be valid
                            if (latNow > latPrevious) {
                                // ascending
                                dataOrb.setShort(k, AmsuFcdrL1cConstant.ORBIT_ASCENDING);
                            } else {
                                // descending
                                dataOrb.setShort(k, AmsuFcdrL1cConstant.ORBIT_DESCENDING);
                            }
                        } else // next scan, geolocation valid
                        {
                            if (latNow < latNext) {
                                // ascending
                                dataOrb.setShort(k, AmsuFcdrL1cConstant.ORBIT_ASCENDING);
                            } else {
                                // descending
                                dataOrb.setShort(k, AmsuFcdrL1cConstant.ORBIT_DESCENDING);
                            }
                        }

                    } else // current scan invalid, check if both previous anbd next scans are valid
                    {
                        if ((latPrevious >= AmsuFcdrL1cConstant.MINIMUM_LATITUDE) && (latNext >= AmsuFcdrL1cConstant.MINIMUM_LATITUDE)) {
                            if (latNext > latPrevious) {
                                dataOrb.setShort(k, AmsuFcdrL1cConstant.ORBIT_ASCENDING);
                            } else {
                                dataOrb.setShort(k, AmsuFcdrL1cConstant.ORBIT_DESCENDING);
                            }
                        } else {
                            // invalid
                            dataOrb.setShort(k, AmsuFcdrL1cConstant.ORBIT_UNDEFINED);
                        }
                    }
                } // end scans from 2 to (NUMBER_SCAN_PER_DAY-1)

                // last scan
                k = NUMBER_SCAN_PER_DAY - 1;
                latNow = this.nadirLat[k];
                latPrevious = this.nadirLat[k - 1];
                if (latNow < AmsuFcdrL1cConstant.MINIMUM_LATITUDE) { // invalid
                    dataOrb.setShort(k, AmsuFcdrL1cConstant.ORBIT_UNDEFINED);
                } else // check previous scan
                {
                    if (latPrevious < AmsuFcdrL1cConstant.MINIMUM_LATITUDE) {
                        dataOrb.setShort(k, AmsuFcdrL1cConstant.ORBIT_UNDEFINED);
                    } else if (latNow > latPrevious) {
                        dataOrb.setShort(k, AmsuFcdrL1cConstant.ORBIT_ASCENDING);
                    } else {
                        dataOrb.setShort(k, AmsuFcdrL1cConstant.ORBIT_DESCENDING);
                    }
                }

                // write A/D
                fileOut.write(varOrbit, dataOrb);

            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Write out composed daily L1c file
     *
     * @param filename, name of output daily L1c data file.
     * @return true if success, false if fail.
     */
    public boolean writeDailyL1c(String filename) {
        NetcdfFileWriter fileOut = null;
        int k = 0;
        boolean ret = false;
        
        // for debug only
        if(this.verbose) {
            String msgDebug = String.format("Find %d AMSU FCDR L1c files for %s, %4d-%02d-%02d", this.numFcdrL1c, this.satName, 
                    this.year, this.month, this.day);
            fcdrInfoLogger.debug(msgDebug);    
        }
        
        if(this.numFcdrL1c<1) {
            String msg = String.format("Unable to find AMSU FCDR L1c files for %s, %4d-%02d-%02d", this.satName, 
                    this.year, this.month, this.day);
            fcdrErrorLogger.error(msg);            
            return ret;
        }  
        
        try {
            // create netcdf file
            fileOut = NetcdfFileWriter.createNew(NetcdfFileWriter.Version.netcdf3, filename);
            if(fileOut == null){
                String msg = String.format("Failed to create new file for %s, %4d-%02d-%02d: %s", this.satName, 
                    this.year, this.month, this.day, filename);
                fcdrErrorLogger.error(msg);                 
                ret = false;
            } else {          
                // define variables
                this.defineDailyNetcdf(fileOut);
                // write BT data
                this.writeDailyL1cBt(fileOut);
                //close file
                fileOut.close();
                ret = true;
            }
        } catch (Exception e) {
            e.printStackTrace();
            ret = false;
            // try to make sure all open files are closed.
            try {
                fileOut.close();
            } catch (Exception ee) {

            }
        }
        return ret;
    }

    /**
     * Get name of Daily FCDR L1C file.
     *
     * @return full path of file for the specified date.
     */
    public String getFullpathL1cDaily() {
        String ret = null;
        String dirRoot = this.rootFCDRL1d;
        if (dirRoot.endsWith(File.separator)) {
            ret = String.format("%s%s%s%04d%sNESDIS-STAR_FCDR_AMSU-A_L1C_Daily_%s_%04d%02d%02d.nc", dirRoot, this.satName,
                    File.separator, this.year, File.separator, this.satName, this.year, this.month, this.day);
        } else {
            ret = String.format("%s%s%s%s%04d%sNESDIS-STAR_FCDR_AMSU-A_L1C_Daily_%s_%04d%02d%02d.nc", dirRoot, File.separator,
                    this.satName, File.separator, this.year, File.separator, this.satName, this.year, this.month, this.day);
        }
        return ret;
    }

    /**
     * Get directory name of daily FCDR files.
     *
     * @param dirRoot, root directory of FCDR daily data.
     * @return, name of folder for daily FCDR data.
     */
    public String getFolderL1cDaily(String dirRoot) {
        String ret = null;
        if (dirRoot.endsWith(File.separator)) {
            ret = String.format("%s%s%s%04d%s", dirRoot, this.satName, File.separator, this.year, File.separator);
        } else {
            ret = String.format("%s%s%s%s%04d%s", dirRoot, File.separator, this.satName, File.separator, this.year, File.separator);
        }
        return ret;
    }

    
    /**
     * Create directory of daily FCDR files. If keep intermediate data files, save in folder rootFCDRL1d/sateName/year
     * @return true if created folder correctly, false if failed to create sub-directories.
     */
    public boolean createFolderL1cDaily() {
        boolean ret = true;
        String dirRoot = this.rootFCDRL1d;
        String fullpath = null;
        if (dirRoot.endsWith(File.separator)) {
            fullpath = String.format("%s%s%s%04d%s", dirRoot, this.satName, File.separator, this.year, File.separator);
        } else {
            fullpath = String.format("%s%s%s%s%04d%s", dirRoot, File.separator, 
                        this.satName, File.separator, this.year, File.separator);
        }
            
        File folder = new File(fullpath);
        if (folder.exists()) {
            ret = true;
        } else {
            try {
                folder.mkdirs();
                if(this.verbose) {
                    String msg = String.format("Created folder in processing %s, %4d-%02d-%02d:  %s", 
                            this.satName, this.year, this.month, this.day, fullpath);
                    fcdrInfoLogger.debug(msg);   
                }
            } catch (Exception ioe) {
                ret = false;
                String msg = String.format("Error in processing %s, %4d-%02d-%02d, Failed to create folder + %s",
                            this.satName, this.year, this.month, this.day, fullpath);
                fcdrErrorLogger.error(msg);                    
            }
        }

        return ret;
    }

    /**
     * Get name of Daily FCDR L1C file.
     *
     * @param dirRoot, root directory of FCDR daily data.
     * @param satId, ID (short name) of satellite.
     * @param orbitId, ID for ascending/descending node.
     * @param year, year.
     * @param month, month.
     * @param day, day.
     * @return full path of file for the specified date.
     */
    public static String getFullpathL1cDaily(String dirRoot, String satId, char orbitId, int year, int month, int day) {
        String ret = null;
        if (dirRoot.endsWith(File.separator)) {
            ret = String.format("%s%s%s%04d%sNESDIS-STAR_FCDR_AMSU-A_L3_%s_%04d%02d%02d.nc", dirRoot, satId, File.separator, year,
                    File.separator, satId, year, month, day);
        } else {
            ret = String.format("%s%s%s%s%04d%sNESDIS-STAR_FCDR_AMSU-A_L3_%s_%04d%02d%02d.nc", dirRoot, File.separator, satId,
                    File.separator, year, File.separator, satId, year, month, day);
        }
        return ret;
    }


    
    /**
     * Delete aligned L1c file if don't keep it.
     */
    private void deleteAlignedL1c() {
        // delete it if don't want to keep it
        if(!this.flagKeepIntermediateData){
            try{
                File tmpFile = new File(this.fileAligned);
                if (tmpFile.exists()){
                    tmpFile.delete();  
                    if(this.verbose) {
                        String msg = String.format("Deleted temporary file %s", this.fileAligned);
                        fcdrInfoLogger.debug(msg);
                    }
                }
            } catch (Exception e){
                // unexpected condition
            }
        }                                        
    }
    
    
    
}


