/*  Robodoc Markup Header
*@***h*  AMSU-A_FCDR_STAR/amsu.fcdr.gridded.AmsuFcdrFileManager.java
*
* NAME
*   amsu.fcdr.gridded.AmsuFcdrFileManager.java
*
* PURPOSE
*   Manage file names in processing AMSU-A level-1c FCDR and generating daily gridded brightness temperature FCDR.
* 
* DESCRIPTION
*	This class defines file naming convrntion in processing AMSU-A level-1c FCDR and generating daily gridded 
*       brightness temperature FCDR.
*
* AUTHOR
*   Xianjun Hao, George Mason University 
*
* COPYRIGHT 
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*
* REVISION HISTORY
*   12/08/2015 Initial Version prepared for NCEI.
*
*@***** 
 */
package amsu.fcdr.gridded;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Arrays;
import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * This class defines file naming conversion in processing AMSU-A level-1c FCDR
 * and generating daily gridded brightness temperature FCDR.
 *
 * @author Xianjun Hao (xianjun.hao@noaa.gov)
 */
public class AmsuFcdrFileManager {

    /**
     * Root directory for AMSU-A level-1c FCDR data.
     */
    private String dir_FCDR_root = null;

    /**
     * CONSTRUCTOR.
     * @param dir_FCDR_root, root directory of AMSU-A level-1c FCDR data.
     */
    public AmsuFcdrFileManager(String dir_FCDR_root) {
        this.dir_FCDR_root = dir_FCDR_root;
    }

    /**
     * Get root directory of AMSU-A level-1c FCDR files.
     *
     * @return root directory of AMSU-A level-1c FCDR files.
     */
    public String getFCDRfileRoot() {
        return this.dir_FCDR_root;
    }

    /**
     * Set root directory of AMSU-A level-1c FCDR files.
     *
     * @param dir_FCDR_root, root directory of FCDR files.
     */
    public void setFCDRfileRoot(String dir_FCDR_root) {
        this.dir_FCDR_root = dir_FCDR_root;
    }

    /**
     * Get directory name of FCDR files for the specified satellite and year.
     *
     * @param satName, short name of satellite, i.e. (N15, N16, N17, N18, M02, AQUA)
     * @param year, year
     * @return directory name of FCDR files for the specified satellite and
     * year.
     */
    public String getFCDRfileFolder(String satName, int year) {
        String nameDirFCDR = this.dir_FCDR_root.concat(satName);
        nameDirFCDR = nameDirFCDR.concat("/");
        nameDirFCDR = nameDirFCDR.concat(Integer.toString(year));
        nameDirFCDR = nameDirFCDR.concat("/");
        return nameDirFCDR;
    }

    /**
     * Get list of potential FCDR files cover the current specified day but
     * named with the previous day.
     *
     * @param satName, short name of satellite, i.e. AQUA, M02, N15, N16, N17,
     * and N18.
     * @param year, year of the observation time.
     * @param month, month of the observation time.
     * @param day, day of the observation time.
     * @return list of potential FCDR files cover the current specified day but
     * named with the previous day..
     */
    public File[] getFirstFCDRfileOfDay(String satName, int year, int month, int day) {
        File[] fcdrFileList = null; // initialize file list
        FilenameFilter ncFilter = null; // initialize file filter

        /*determine directory of FCDR files for the specified day*/
        Calendar cal = new GregorianCalendar(year, month - 1, day);
        cal.add(Calendar.DAY_OF_YEAR, -1);
        int pyear = cal.get(Calendar.YEAR);
        int pmonth = cal.get(Calendar.MONTH) + 1;      // Calendar, month starts with 0, need to add 1 for normal use
        int pday = cal.get(Calendar.DAY_OF_MONTH);
        String nameDirFCDR = getFCDRfileFolder(satName, pyear);
        File dirFCDR = new File(nameDirFCDR);

        /* determine substring to filter files covering two consecutive days */
        String strPatt0 = satName.concat("_D");
        final String strPatt = strPatt0.concat(String.format("%8d", pyear * 10000 + pmonth * 100 + pday));
        final int lenSatName = satName.length();
        ncFilter = new FilenameFilter() {
            public boolean accept(File dir, String name) {
                int lenStrPatt = 10 + lenSatName;
                int indexPatt = name.indexOf(satName);
                if ((indexPatt < 0) || (name.length() < 50) || (!name.endsWith(".nc"))) {
                    return false;
                } else {
                    int k1 = indexPatt + lenSatName + 12;
                    int k2 = k1 + 6;
                    String str = name.substring(indexPatt, indexPatt + lenStrPatt);
                    // get start and end time (hour) of the orbital FCDR file
                    String st1 = name.substring(k1, k1 + 2);   // string for start hour
                    String st2 = name.substring(k2, k2 + 2);  // string for end hour    					
                    int ht1 = Integer.parseInt(st1);       // start hour
                    int ht2 = Integer.parseInt(st2);       // end hour   
                    // For different satellite, the temporal length of orbital data are different.  
                    // Use relaxed criteria to get potential FCDR files from previous day but may overlap with the current day.
                    // It is Ok to accept files not overlapping the current day in the list, will be neglected 
                    // in the temporal alignment step.
                    if (str.equals(strPatt)) {
                        if (ht1 < 20 || ht2 >= 10) {
                            return false;
                        } else {
                            return true;
                        }
                    } else {
                        return false;
                    }
                }
            }
        };

        fcdrFileList = dirFCDR.listFiles(ncFilter);
        if (fcdrFileList!=null) {
            if (fcdrFileList.length > 1) {
                Arrays.sort(fcdrFileList);
            }
        }
        return fcdrFileList;
    }

    /**
     * Get list of FCDR file for the specified date based on AMSU-A level-1c
     * FCDR file naming convention.
     *
     * @param satName, short name of satellite, i.e. AQUA, M02, N15, N16, N17,
     * and N18.
     * @param year, year of the observation time.
     * @param month, month of the observation time.
     * @param day, day of the observation time.
     * @return list of FCDR files for the specified observation day.
     */
    public File[] getFCDRfileListByDate(String satName, int year, int month, int day) {
        File[] fcdrFileList = null;  // initialize file list
        FilenameFilter ncFilter = null; // initialize file filter
        /*determine directory of FCDR files for the specified day*/
        String nameDirFCDR = getFCDRfileFolder(satName, year);
        File dirFCDR = new File(nameDirFCDR);
        /* determine substring to filter files */
        String strPatt0 = satName.concat("_D");
        final String strPatt = strPatt0.concat(String.format("%8d", year * 10000 + month * 100 + day));
        final int lenSatName = satName.length();
        ncFilter = new FilenameFilter() {
            public boolean accept(File dir, String name) {
                int lenStrPatt = 10 + lenSatName;
                int indexPatt = name.indexOf(satName);

                if ((indexPatt < 0) || (name.length() < 50) || (!name.endsWith(".nc"))) {
                    return false;
                } else {
                    String str = name.substring(indexPatt, indexPatt + lenStrPatt);
                    return str.equals(strPatt);
                }
            }
        };

        fcdrFileList = dirFCDR.listFiles(ncFilter);
        if(fcdrFileList!=null){
            if(fcdrFileList.length>=1){
                Arrays.sort(fcdrFileList);                
            }
        }
        return fcdrFileList;
    }

    /**
     * Get list of all FCDR file cover the specified date.
     *
     * @param satName, short name of satellite, i.e. AQUA, M02, N15, N16, N17,
     * and N18.
     * @param year, year of the observation time.
     * @param month, month of the observation time.
     * @param day, day of the observation time.
     * @return list of FCDR files cover the specified observation day.
     */
    public File[] getAllFCDRfilesForDate(String satName, int year, int month, int day) {
        int num1 = 0;
        int num2 = 0;
        File[] list1 = getFirstFCDRfileOfDay(satName, year, month, day);
        File[] list2 = getFCDRfileListByDate(satName, year, month, day);

        if (list1==null){
            num1 = 0;
        } else {
            num1 = list1.length;
        }
        if (list2==null){
            num2 = 0;
        } else {
            num2 = list2.length;
        }
                
        int num = num1 + num2;

        File[] listAll = new File[num];

        int k = 0;
        for (k = 0; k < num1; k++) {
            listAll[k] = list1[k];
        }
        
        for (k = 0; k < num2; k++) {
            listAll[k + num1] = list2[k];
        }

        return listAll;
    }

}
