/*  Robodoc Markup Header
*@***h* AMSU-A_FCDR_STAR/amsu.fcdr.gridded.AmsuFcdrGridConstant.java
*
* NAME
*   amsu.fcdr.gridded.AmsuFcdrGridConstant.java
*
* PURPOSE
*   Define constants in gridding daily AMSU-A brightness temperature FCDR.
* 
* DESCRIPTION
*	This interface defines constants and conventions in gridding daily AMSU-A brightness temperature FCDR.
*
* AUTHOR
*   Xianjun Hao, George Mason University 
*
* COPYRIGHT 
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*
* REVISION HISTORY
*   12/08/2015 Initial Version prepared for NCEI
*
*@***** 
 */
package amsu.fcdr.gridded;

/**
 * Define constants and conventions in gridding daily AMSU-A brightness
 * temperature FCDR.
 *
 * @author Xianjun Hao (xianjun.hao@noaa.gov)
 */
public interface AmsuFcdrGridConstant {

    /**
     * Starting latitude of the grid.
     */
    final public static double GRID_LATITUDE_START = 90.0d;

    /**
     * End latitude of the grid.
     */
    final public static double GRID_LATITUDE_END = -90.0d;

    /**
     * Starting longitude of the grid.
     */
    final public static float GRID_LONGITUDE_START = -180.0f;

    /**
     * End longitude of the grid.
     */
    final public static float GRID_LONGITUDE_END = 180.0f;

    /**
     * Step of latitude for gridding.
     */
    final public static float GRID_STEP_LATITUDE = -1.0f;

    /**
     * Step of longitude for gridding.
     */
    final public static float GRID_STEP_LONGITUDE = 1.0f;

    /**
     * Number of gridding samples in latitude.
     */
    final public static int GRID_NUM_LAT = 180;

    /**
     * Number of gridding samples in longitude.
     */
    final public static int GRID_NUM_LON = 360;

    /**
     * Maximum latitude.
     */
    final public static float GRID_MAXIMUM_LATITUDE = 90.0f;

    /**
     * Minimum latitude.
     */
    final public static float GRID_MINIMUM_LATITUDE = -90.0f;

    /**
     * Maximum longitude.
     */
    final public static float GRID_MAXIMUM_LONGITUDE = 180.0f;

    /**
     * Minimum longitude.
     */
    final public static float GRID_MINIMUM_LONGITUDE = -180.0f;

    /**
     * Filled value for invalid index.
     */
    final public static int GRID_INVALID_INDEX = -999;
    
    /**
     * Filled value for invalid TIME
     */
    final public static double FILLED_VALUE_INVALID_TIME_SINCE78 = -9999d;

    /**
     * Filled value for invalid VZA
     */
    final public static float FILLED_VALUE_INVALID_VZA = -9999.0f;
    
    /**
     * Filled value for invalid brightness temperature.
     */
    final public static float FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE = -9999.0f;
    
    /**
     *  Minimum value of valid brightness temperature.
     */
     final public static float VALID_MIN_BRIGHTNESS_TEMPERATURE = 150.0f;

    /**
     *  Maximum value of valid brightness temperature.
     */
     final public static float VALID_MAX_BRIGHTNESS_TEMPERATURE = 320.0f;
     
    /**
     * Major version number of the datasets.
     */
    final public static String VERSION_MAJOR_AMSU_A_FCDR_L3_DATA = "1";

    /**
     * Minor version number of the datasets.
     */
    final public static String VERSION_REVISION_AMSU_A_FCDR_L3_DATA = "0";

    /**
     * Name for ascending orbit.
     */
    final public static String ORBIT_ASCENDING_NAME = "ascending";

    /**
     * Name for descending orbit.
     */
    final public static String ORBIT_DESCENDING_NAME = "descending";
    
    /**
     * String for ascending orbit.
     */
    final public static String ORBIT_ASCENDING_STRING = "A";

    /**
     * String for descending orbit.
     */
    final public static String ORBIT_DESCENDING_STRING = "D";

    /**
     * Name for ascending orbit.
     */
    final public static String NAMETAG_ORBIT_ASCENDING = "ascending";

    /**
     * Name for descending orbit.
     */
    final public static String NAMETAG_ORBIT_DESCENDING = "descending";
    
    /**
     * Char for ascending orbit.
     */
    final public static char ORBIT_ASCENDING = 'A';

    /**
     * Char for descending orbit.
     */
    final public static char ORBIT_DESCENDING = 'D';

    /**
     * Flag for ascending orbit.
     */
    final public static short ORBIT_FLAG_ASCENDING = 0;

    /**
     * Flag for descending orbit.
     */
    final public static short ORBIT_FLAG_DESCENDING = 1;
    
    /**
     * ID of composition method 1, near nadir pixels only, in mapping level-1c
     * FCDR to gridded FCDR.
     */
    final public static short ID_METHOD_FCDR_OMPOSITION_NEAR_NADIR_ONLY = 1;

    /**
     * Name of composition method 1, near nadir pixels only, in mapping level-1c
     * FCDR to gridded FCDR.
     */
    final public static String NAME_METHOD_FCDR_OMPOSITION_NEAR_NADIR_ONLY = "Near-Nadir-Only";

    /**
     * ID of composition method 2, most nadir pixel, in mapping level-1c FCDR to
     * gridded FCDR.
     */
    final public static short ID_METHOD_FCDR_OMPOSITION_MOST_NADIR = 2;

    /**
     * Name of composition method 2, most nadir pixel, in mapping level-1c FCDR
     * to gridded FCDR.
     */
    final public static String Name_METHOD_FCDR_OMPOSITION_MOST_NADIR = "Most-Nadir";

    /**
     * ID of composition method 3, spatial mean, in mapping level-1c FCDR to
     * gridded FCDR.
     */
    final public static short ID_METHOD_FCDR_OMPOSITION_SPATIAL_MEAN = 3;

    /**
     * Name of composition method 3, spatial mean, in mapping level-1c FCDR to
     * gridded FCDR.
     */
    final public static String NAME_METHOD_FCDR_OMPOSITION_SPATIAL_MEAN = "Spatial-Mean";
    
   /**
   * Flag for AMSU_A FCDR L3 data processing. No Error.
   */
   final public static short AMSU_A_FCDR_L3_PROCESSING_STATUS_UNSTARTED = -1;

   /**
   * Flag for AMSU_A FCDR L3 data processing. No Error.
   */
   final public static short AMSU_A_FCDR_L3_PROCESSING_STATUS_SUCCESS = 0;

   /**
   * Flag for AMSU_A FCDR L3 data processing. Missing data files.
   */
   final public static short AMSU_A_FCDR_L3_PROCESSING_STATUS_MISSING_FILES = 1;
    
   /**
   * Flag for AMSU_A FCDR L3 data processing. Missing data.
   */
   final public static short AMSU_A_FCDR_L3_PROCESSING_STATUS_INVALID_DATA = 2;
    
    /**
     *  Start scan position for gridded FCDR generation (index start with 1).
     */
    final public static int INDEX_SCAN_POSITION_START = 8;

    /**
     *  End scan position for gridded FCDR generation (index start with 1).
     */
    final public static int INDEX_SCAN_POSITION_END = 23;

    /**
     *  Group name in netcdf file for near-nadir gridded FCDR.
     */
    final public static String NETCDF_GROUP_NAME_OPT1 = "fcdr-near-nadir-only";
    
    /**
     *  Group name in netcdf file for minimum-VZA gridded FCDR.
     */
    final public static String NETCDF_GROUP_NAME_OPT2 = "fcdr-minimum-vza";

    /**
     *  Group name in netcdf file for spatial-mean gridded FCDR.
     */
    final public static String NETCDF_GROUP_NAME_OPT3 = "fcdr-spatial-mean";
    
    /**
     * Dimension string for data.
     */
    final public static String NETCDF_GEO_DIMENSIION = "latitude longitude";
    
    /**
     *  Starting time when NOAA satellites started to use geodetic latitude.
     */    
    final public static long STARTING_TIME_NOAA_GEODETIC_LATITUDE = TimeUtility.getStartSecondSince78(2006, 7,6);

    /**
     * Naming convention for daily gridded FCDR.
     * NESDIS-STAR_FCDR-GRID_AMSU-A_V[Version]_[platform]_D[date]_[tag_orbit]_c[tag_file_creation_date].nc
     */
    final public static String NAMING_CONVENTION_FILE_GRIDDED_FCDR="NESDIS-STAR_FCDR-GRID_AMSU-A_V01R00_%s_D%4d%02d%02d_%s_C%s.nc";

    

    /**
     * Get brightness temperature variable name in product. 
     * @param selectedOrbitFlag orbit flag, 0 for ascending; 1 for descending.
     * @param channel AMSU-A channel ID
     * @param compositionOption composition method, 1 for near-nadir only; 2 for minimum VZA; 3 for block mean.
     * @return Variable name in product file (NetCDF format).
     */
    public static String getProductBtName(short selectedOrbitFlag, int channel, int compositionOption) {
        String varName = null;
        String selectedOrbitName = null;
        
        if (selectedOrbitFlag == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_ASCENDING_NAME;
        } else {
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_DESCENDING_NAME;
        }      
        
        switch (compositionOption) {
            case AmsuFcdrGridConstant.ID_METHOD_FCDR_OMPOSITION_NEAR_NADIR_ONLY:
                varName = String.format("BT_ch%d_IMICA_%s_nadir", channel, selectedOrbitName );
                break;
            case AmsuFcdrGridConstant.ID_METHOD_FCDR_OMPOSITION_MOST_NADIR:
                varName = String.format("BT_ch%d_IMICA_%s_minvza", channel, selectedOrbitName );                
                break;
            case AmsuFcdrGridConstant.ID_METHOD_FCDR_OMPOSITION_SPATIAL_MEAN:
                varName = String.format("BT_ch%d_IMICA_%s_mean", channel, selectedOrbitName );                
                break;
            default:
                break;   
        }                
        return varName;
    }

    /**
     * Get variable name for standard deviation of brightness temperature in product. 
     * @param selectedOrbitFlag orbit flag, 0 for ascending; 1 for descending.
     * @param channel AMSU-A channel ID
     * @return Variable name in product file (NetCDF format).
     */
    public static String getProductBtStdName(short selectedOrbitFlag, int channel) {
        String selectedOrbitName = null;
        
        if (selectedOrbitFlag == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_ASCENDING_NAME;
        } else {
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_DESCENDING_NAME;
        }      

        String varName = String.format("BT_ch%d_IMICA_%s_std", channel, selectedOrbitName );                
        
        return varName;
    }

    /**
     * Get variable name for time of the selected pixel in seconds since 1978.
     * @param selectedOrbitFlag, flag of selected orbit.
     * @return Variable name for time of selected pixel in seconds since 1978.
     */
    public static String getProductPixelTimeName(short selectedOrbitFlag) {
        String selectedOrbitName = null;
        if (selectedOrbitFlag == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_ASCENDING_NAME;
        } else {
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_DESCENDING_NAME;
        }      
//        String varName = String.format("time_IMICA_minvza_since_1978_%s", selectedOrbitName);
        String varName = String.format("scan_time_since_1978_%s_minvza", selectedOrbitName);
        return varName;
    }
    
    /**
     * Get variable name for view zenith angle of the selected pixel.
     * @param selectedOrbitFlag, flag of selected orbit.
     * @return variable name for view zenith angle of the selected pixel.
     */
    public static String getProductPixelVzaName(short selectedOrbitFlag) {
        String selectedOrbitName = null;
        if (selectedOrbitFlag == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_ASCENDING_NAME;
        } else {
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_DESCENDING_NAME;
        }      
        String varName = String.format("minimum_view_zenith_angle_%s", selectedOrbitName);
        return varName;
    }
    
}
