/*  Robodoc Markup Header
*@***h* AMSU-A_FCDR_STAR/amsu.fcdr.gridded.AmsuFcdrGriddedProcessor.java
*
* NAME
*   amsu.fcdr.gridded.AmsuFcdrGriddedProcessor.java
*
* PURPOSE
*   Produce AMSU-A daily gridded FCDR brightness temperature for the specified satellite and date.
* 
* DESCRIPTION
*	This class defines the core procedures to map and compose orbital AMSU-A level 1c data to daily data on 1*1 degree grid.
*
* AUTHOR
*   Xianjun Hao, George Mason University 
*
* COPYRIGHT 
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*
* REVISION HISTORY
*   12/08/2015 Initial Version prepared for NCEI
*
*@***** 
 */
package amsu.fcdr.gridded;

import java.lang.Math;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.lang.Runnable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import java.util.Date;
import org.apache.logging.log4j.Level;


import ucar.nc2.NetcdfFile;
import ucar.nc2.Variable;
import ucar.ma2.*;
import ucar.nc2.Attribute;
import ucar.nc2.Dimension;
import ucar.nc2.NetcdfFileWriter;
import ucar.nc2.Group;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * This class defines the core procedures to Produce AMSU-A daily gridded FCDR brightness temperature for the specified satellite
 * and date by mapping and composing orbital AMSU-A level 1c data to daily data on 1*1 degree grid.
 *
 * @author Xianjun Hao (xianjun.hao@noaa.gov)
 */
public class AmsuFcdrGriddedProcessor {
    /**
     * Information logger for FCDR processing.
     */
    final private static Logger  fcdrErrorLogger = LogManager.getLogger("ErrorLog");

    /**
     * Information logger for FCDR processing.
     */
    final private static Logger  fcdrInfoLogger = LogManager.getLogger("InfoLog");

    /**
     * Year.
     */
    private int year = 0;

    /**
     * Month of the year.
     */
    private int month = 0;

    /**
     * Day of the month.
     */
    private int day = 0;
    
    /**
     * seconds since 1970-01-01 00:00:00 0:00
     */
    private long secondFromEpoch = 0l;

    /**
     * Time bound (min) in seconds since 1970-01-01 00:00:00 0:00
     */
    private double timeBoundMin = 0.0d;

    /**
     * Time bound (max) in seconds since 1970-01-01 00:00:00 0:00
     */    
    private double timeBoundMax = 0.0d;
    
    /**
     * ID of satellite.
     */
    private String satId = null;

    /**
     * Root directory of FCDR L1C files (input).
     */
    private String rootIn = null;

    /**
     * Root directory of FCDR L3 files (output).
     */
    private String rootOut = null;

    /**
     * Root directory of intermediate data files (temporary).
     */
    private String rootTemp = null;

    /**
     * Flag to indicate if keep intermediate data files (the quality controlled and temporally aligned daily level 1c data). Set it
     * to true to use intermediate data for testing or other purpose. Default value is false.
     */
    private boolean flagKeepIntermediateData = false;

    /**
     * Full path of input file.
     */
    private String filenameIn = null;

    /**
     * Full path of output file for ascending orbit
     */
    private String filenameOutA = null;

    /**
     * Full path of output file for descending orbit
     */
    private String filenameOutD = null;

    /**
     * Orbit indicator for data processing.
     */
    private short currentOrbit = AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING;

    /**
     * NetCDF file for input file.
     */
    private NetcdfFile ncFileIn = null;

    /**
     * NetCDF writer for output file - ascending.
     */
    private NetcdfFileWriter ncFileOutA = null;

    /**
     * NetCDF writer for output file - descending.
     */
    private NetcdfFileWriter ncFileOutD = null;

    /**
     * BT variable list
     */
    private List<Variable> varBtList = null;

    /**
     * List of geolocation related variables
     */
    private List<Variable> varGeoList = null;

    /**
     * Flag of running status.
     */
    private int status;

    /**
     * Rows of pixels on grid.
     */
    private int[][] pixRow = null;

    /**
     * Columns of pixels in grid.
     */
    private int[][] pixCol = null;

    /**
     * orbit flag
     */
    private short[] orbitFlag = null;

    /**
     * number of scans.
     */
    private int numScan = 0;

    /**
     * Latitude of grid.
     */
    private float[] gridLat = null;

    /**
     * Longitude of grid
     */
    private float[] gridLon = null;
    
    /**
     * Dimension for 2d grid, ascending
     */
    private List<Dimension> dimGridA = null;

    /**
     * Dimension for 2d grid, descending
     */
    private List<Dimension> dimGridD = null;

    /**
     * Dimension for latitude, ascending grid
     */
    private List<Dimension> dimLatA = null;

    /**
     * Dimension for latitude, descending grid
     */
    private List<Dimension> dimLatD = null;

    /**
     * Dimension for longitude, ascending grid
     */
    private List<Dimension> dimLonA = null;

    /**
     * Dimension for longitude, descending grid
     */
    private List<Dimension> dimLonD = null;
    
    /**
     * Buffer for gridded FCDR with nadir pixels only.
     */
    private AmsuFcdrGriddedDailyDataNadir fcdrNadir = null;

    /**
     * Buffer for gridded FCDR with pixels with minimum VZA.
     */
    private AmsuFcdrGriddedDailyDataMinvza fcdrMinvza = null;

    /**
     * Buffer for gridded FCDR with block mean.
     */
    private AmsuFcdrGriddedDailyDataBlockmean fcdrBlockmean = null;
    
    /**
     *  verbose flag for running status
     */
    private boolean verbose = false;
    
    /**
     * Current time to set time tag for processing time of data files.
     */
    private Date processingDate = null;
    
    /**
     * Constructor. Create an empty instance.
     */
    public AmsuFcdrGriddedProcessor() {
        this.initGrid();
    }

    /**
     * Constructor.
     *
     * @param satId, short name of satellite.
     * @param year, year.
     * @param month, month of the year.
     * @param day, day of the month.
     * @param rootIn, root directory of the FCDR L1C data files (input).
     * @param rootOut, root directory of the FCDR L3 data files (output).
     * @param rootTemp, root directory of the intermediate data files (temporary).
     * @param flagKeepIntermediateData, flag to indicate if keep intermediate data files.
     * @param fileAligned, name of aligned daily L1c data file.
     * @param verbose, if show extra information during running.
     */
    public AmsuFcdrGriddedProcessor(String satId, int year, int month, int day, String rootIn, String rootOut, String rootTemp,
            boolean flagKeepIntermediateData, String fileAligned, boolean verbose) {
        this.initGrid();
        this.satId = satId;
        this.year = year;
        this.month = month;
        this.day = day;
        this.rootIn = rootIn;
        this.rootOut = rootOut;
        this.rootTemp = rootTemp;
        this.flagKeepIntermediateData = flagKeepIntermediateData;
        this.filenameIn = fileAligned;
        this.verbose = verbose;
        this.init();
    }

    /**
     * Setup parameters for AMSU-A FCDR L3 data processing.
     *
     * @param satId, short name of satellite.
     * @param year, year.
     * @param month, month of the year.
     * @param day, day of the month.
     * @param rootIn, root directory of the FCDR L1C data files (input).
     * @param rootOut, root directory of the FCDR L3 data files (output).
     */
    public void setup(String satId, int year, int month, int day, String rootIn, String rootOut) {
        this.satId = satId;
        this.year = year;
        this.month = month;
        this.day = day;
        this.rootIn = rootIn;
        this.rootOut = rootOut;
        this.init();
    }

 
    /**
     * Initialize latitude and longitude grid.
     */    
    private void initGrid() {
        int k = 0;
        this.gridLat = new float[AmsuFcdrGridConstant.GRID_NUM_LAT];
        this.gridLon = new float[AmsuFcdrGridConstant.GRID_NUM_LON];
        // get latitude for grid
        for (k = 0; k < AmsuFcdrGridConstant.GRID_NUM_LAT; k++) {
            this.gridLat[k] = (float) (AmsuFcdrGridConstant.GRID_LATITUDE_START + (k + 0.5) * ((float) AmsuFcdrGridConstant.GRID_STEP_LATITUDE));
        }
        // get longitude for grid
        for (k = 0; k < AmsuFcdrGridConstant.GRID_NUM_LON; k++) {
            this.gridLon[k] = (float) (AmsuFcdrGridConstant.GRID_LONGITUDE_START + (k + 0.5) * ((float) AmsuFcdrGridConstant.GRID_STEP_LONGITUDE));
        }
    }

    /**
     * Initialize instance for run.
     *
     * @return true, if initialize correctly; false, if initialization failed.
     */
    private boolean init() {        
        boolean ret = false;
        this.processingDate = new Date();        
        TimeZone utc = TimeZone.getTimeZone("UTC");
        Calendar calendar = new GregorianCalendar(this.year, this.month-1, this.day, 0, 0, 0);
        calendar.setTimeZone(utc);
        this.secondFromEpoch = calendar.getTimeInMillis()/1000;        
        this.timeBoundMin = this.secondFromEpoch;
        this.timeBoundMax = this.timeBoundMin + 24*3600;
        
        this.status = AmsuFcdrGridConstant.AMSU_A_FCDR_L3_PROCESSING_STATUS_UNSTARTED;
        this.currentOrbit = AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING;
        // get full path of input file
            this.createFolderL3Daily();
            this.filenameOutA = AmsuFcdrGriddedProcessor.getFullpathL3Daily(this.rootOut, this.satId, 
                    'A', this.year, this.month, this.day, this.processingDate);
            this.filenameOutD = AmsuFcdrGriddedProcessor.getFullpathL3Daily(this.rootOut, this.satId, 
                    'D', this.year, this.month, this.day, this.processingDate);
            
        // open the input netcdf file.
        try {
            this.ncFileIn = NetcdfFile.open(this.filenameIn);                      
            // get index for gridding - determine row and column of each pixel according to geolocation and grid information  
            if (AmsuFcdrL1cConstant.isLatitudeGeocentric(this.satId, this.year, this.month, this.day)) {
                // for geocentric geolocation, need to convert to geodetic latitude to keep consistent.
                this.getGridIndexWithGeodeticCorrection();
                if(this.verbose){
                    String msg = String.format("Converted geolocation to geodetic: %s, %4d-%02d-%02d", 
                        this.satId, this.year, this.month, this.day);
                    fcdrInfoLogger.debug(msg);
                }
            } else {
                // geodetic geolocation, gridding directly.
                this.getGridIndex();
            }
            
            ret = true;            
        } catch (Exception e) {
            // If in testing or debug mode, print out exception message.
            ret = false;
            this.status = AmsuFcdrGridConstant.AMSU_A_FCDR_L3_PROCESSING_STATUS_MISSING_FILES;
            String msg = String.format("Missing files for %s, %4d-%02d-%02d", this.satId, this.year, this.month, this.day);
            fcdrErrorLogger.error(msg);
        }
        return ret;
    }
    
    
   /**
    *  Grid L1c data to grid.
    */
    public void produceGriddedFcdr(){
        // ascending
        this.processAscending();
        //descending
        this.processDescending();        
        // close
        this.close();
    }

    /**
     * Grid L1c data for ascending orbit.
     */
    private void processAscending() {        
        // initialize ascending file
        this.initAscendingFile();  
        
        // OPT1: compose ascendding
        this.composeFcdrNadir(AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING);
        // OPT1: write ascendding data file
        this.writeFcdrOpt1(AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING);
        // OPT1: clean buffer
        this.fcdrNadir = null;
                

        // OPT2: compose ascending 
        this.composeFcdrMinvza(AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING);
        // OPT2: write ascenidng data file
        this.writeFcdrOpt2(AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING);
        this.fcdrMinvza = null;

        // OPT3: compose ascending
        this.composeFcdrBlockmean(AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING);
        //OPT3: write ascending ddata file
        this.writeFcdrOpt3(AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING);        
        //OPT3: clean buffer;
        this.fcdrBlockmean = null;
        
        // close  NetCDF file for ascending orbits
        if (this.ncFileOutA != null) {
            try {
                this.ncFileOutA.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }        
    }

    /**
     * Grid L1c data for descending orbit.
     */
    private void processDescending() {        
        // initialize ascending file
        this.initDescendingFile();  
        
        // OPT1: compose ascendding
        this.composeFcdrNadir(AmsuFcdrGridConstant.ORBIT_FLAG_DESCENDING);
        // OPT1: write ascendding data file
        this.writeFcdrOpt1(AmsuFcdrGridConstant.ORBIT_FLAG_DESCENDING);
        // OPT1: clean buffer
        this.fcdrNadir = null;
                

        // OPT2: compose ascending 
        this.composeFcdrMinvza(AmsuFcdrGridConstant.ORBIT_FLAG_DESCENDING);
        // OPT2: write ascenidng data file
        this.writeFcdrOpt2(AmsuFcdrGridConstant.ORBIT_FLAG_DESCENDING);
        this.fcdrMinvza = null;

        // OPT3: compose ascending
        this.composeFcdrBlockmean(AmsuFcdrGridConstant.ORBIT_FLAG_DESCENDING);
        //OPT3: write ascending ddata file
        this.writeFcdrOpt3(AmsuFcdrGridConstant.ORBIT_FLAG_DESCENDING);        
        //OPT3: clean buffer;
        this.fcdrBlockmean = null;
        
        // close  NetCDF file for ascending orbits
        if (this.ncFileOutD != null) {
            try {
                this.ncFileOutD.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }        
    }
    
    
    /**
     * Read geodetic geolocation and orbit flag, then, calculate rows and columns of pixels.
     */
    private void getGridIndex() {
        int i, j;
        float tmpLat, tmpLon;
        short tmpFlag;
        try {
            //  read latitude
            Variable dataVar = this.ncFileIn.findVariable(AmsuFcdrL1cConstant.LATITUDE);
            int[] shape = dataVar.getShape();
            int[] origin = new int[2];
            ArrayFloat.D2 latArr = (ArrayFloat.D2) dataVar.read(origin, shape);

            this.numScan = shape[0];
            int numVza = shape[1];
            // read longitude
            dataVar = this.ncFileIn.findVariable(AmsuFcdrL1cConstant.LONGITUDE);
            ArrayFloat.D2 lonArr = (ArrayFloat.D2) dataVar.read(origin, shape);

            // read orbit flag
            dataVar = this.ncFileIn.findVariable(AmsuFcdrL1cConstant.FCDR_ORBIT_ASCENDING_DESCENDING_FLAG);
            ArrayShort.D1 orbitArr = (ArrayShort.D1) dataVar.read();

            // set orbig flag
            this.orbitFlag = new short[numScan];
            // calculate row, col
            this.pixRow = new int[numScan][numVza];
            this.pixCol = new int[numScan][numVza];

            for (i = 0; i < numScan; i++) {
                tmpFlag = (short) orbitArr.get(i);
                this.orbitFlag[i] = tmpFlag;
                for (j = 0; j < numVza; j++) {                    
                    tmpLat = latArr.get(i, j);
                    tmpLon = lonArr.get(i, j);
                    if ((tmpLat < AmsuFcdrGridConstant.GRID_MINIMUM_LATITUDE) || (tmpLon < AmsuFcdrGridConstant.GRID_MINIMUM_LONGITUDE)) {
                        //  invalid geolocation data
                        pixRow[i][j] = AmsuFcdrGridConstant.GRID_INVALID_INDEX;
                        pixCol[i][j] = AmsuFcdrGridConstant.GRID_INVALID_INDEX;
                    } else {
                        pixRow[i][j] = (int) Math.floor((latArr.get(i, j)-AmsuFcdrGridConstant.GRID_LATITUDE_START) / AmsuFcdrGridConstant.GRID_STEP_LATITUDE);
                        // process special cases
                        if (pixRow[i][j] >= AmsuFcdrGridConstant.GRID_NUM_LAT) { // latitude= -90, usually impossible
                            pixRow[i][j] = AmsuFcdrGridConstant.GRID_NUM_LAT - 1;
                        }

                        pixCol[i][j] = (int) Math.floor((lonArr.get(i, j) - AmsuFcdrGridConstant.GRID_LONGITUDE_START) / AmsuFcdrGridConstant.GRID_STEP_LONGITUDE);
                        if (pixCol[i][j] >= AmsuFcdrGridConstant.GRID_NUM_LON) { // longitude = 180, set as -180 (index = 0) 
                            pixCol[i][j] = 0;
                        }
                    }
                } //for j
            }  // for i                    
        } catch (Exception nce) {
            nce.printStackTrace();
        }
    }


    /**
     * Read geocentric geolocation and orbit flag,  convert to geodetic latitude, then, calculate rows and columns of pixels.
     *  AMSU L1b geolocation data of NOAA satellites are geocentric until 07/06/2006, need to be converted to geodetic to keep 
     *  consistent. 
     */
    private void getGridIndexWithGeodeticCorrection() {
        int i, j;
        float tmpLat, tmpLon;
        short tmpFlag;
        double clat = 0.0d;
        float plat = 0.0f;
        double f = 1.0d/298.257223563d; 
        double f2 = (1.0d-f)*(1.0d-f); 
        double degree2pai = Math.PI/180.0d;
                
        try {
            //  read latitude
            Variable dataVar = this.ncFileIn.findVariable(AmsuFcdrL1cConstant.LATITUDE);
            int[] shape = dataVar.getShape();
            int[] origin = new int[2];
            ArrayFloat.D2 latArr = (ArrayFloat.D2) dataVar.read(origin, shape);

            this.numScan = shape[0];
            int numVza = shape[1];
            // read longitude
            dataVar = this.ncFileIn.findVariable(AmsuFcdrL1cConstant.LONGITUDE);
            ArrayFloat.D2 lonArr = (ArrayFloat.D2) dataVar.read(origin, shape);

            // read orbit flag
            dataVar = this.ncFileIn.findVariable(AmsuFcdrL1cConstant.FCDR_ORBIT_ASCENDING_DESCENDING_FLAG);
            ArrayShort.D1 orbitArr = (ArrayShort.D1) dataVar.read();

            // set orbig flag
            this.orbitFlag = new short[numScan];
            // calculate row, col
            this.pixRow = new int[numScan][numVza];
            this.pixCol = new int[numScan][numVza];

            for (i = 0; i < numScan; i++) {
                tmpFlag = (short) orbitArr.get(i);
                this.orbitFlag[i] = tmpFlag;
                for (j = 0; j < numVza; j++) {                    
                    tmpLat = latArr.get(i, j);
                    tmpLon = lonArr.get(i, j);
                    if ((tmpLat < AmsuFcdrGridConstant.GRID_MINIMUM_LATITUDE) || (tmpLon < AmsuFcdrGridConstant.GRID_MINIMUM_LONGITUDE)) {
                        //  invalid geolocation data
                        pixRow[i][j] = AmsuFcdrGridConstant.GRID_INVALID_INDEX;
                        pixCol[i][j] = AmsuFcdrGridConstant.GRID_INVALID_INDEX;
                    } else {
                        // convert geocentric latitude to geodetic latitude
                        clat = degree2pai*tmpLat;
                        plat = (float)(Math.atan(Math.tan(clat)/f2)/degree2pai);
                        
                        // get row 
                        pixRow[i][j] = (int) Math.floor((plat - AmsuFcdrGridConstant.GRID_LATITUDE_START) / AmsuFcdrGridConstant.GRID_STEP_LATITUDE);
                        // process special cases
                        if (pixRow[i][j] >= AmsuFcdrGridConstant.GRID_NUM_LAT) { // latitude= -90, usually impossible
                            pixRow[i][j] = AmsuFcdrGridConstant.GRID_NUM_LAT - 1;
                        }
                        
                        // get column
                        pixCol[i][j] = (int) Math.floor((tmpLon - AmsuFcdrGridConstant.GRID_LONGITUDE_START) / AmsuFcdrGridConstant.GRID_STEP_LONGITUDE);
                        if (pixCol[i][j] >= AmsuFcdrGridConstant.GRID_NUM_LON) { // longitude = 180, set as -180 (index = 0) 
                            pixCol[i][j] = 0;
                        }
                    }
                } //for j
            }  // for i                    
        } catch (Exception nce) {
            nce.printStackTrace();
        }
    }
    
    
    
    /**
     * Get status of current processing.
     *
     * @return, status.
     */
    public int getStatus() {
        return this.status;
    }

    /**
     * close current task.
     */
    private void close() {
        // clear row, col, and orbit flag.
        this.pixRow = null;
        this.pixCol = null;
        // If input NetCDF file open, close it.        
        if (this.ncFileIn != null) {
            try {
                this.ncFileIn.close();                
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
    

    /**
     *  Define global attributes for Netcdf files.
     */
    private void defineGlobalAttributes(NetcdfFileWriter ncWriter, short selectedOrbit){
        String[] items = null;

        if (selectedOrbit == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            ncWriter.addGroupAttribute(null, new Attribute("title", "NOAA Climate Data Record (CDR) of AMSU-A Level 1c Brightness "
                + "Temperature, Version 1.0 (Gridded Ascending)"));            
        } else {
            ncWriter.addGroupAttribute(null, new Attribute("title", "NOAA Climate Data Record (CDR) of AMSU-A Level 1c Brightness "
                + "Temperature, Version 1.0 (Gridded Descending)"));
        }
        ncWriter.addGroupAttribute(null, new Attribute("summary", "AMSU-A daily gridded brightness temperature FCDR products "
                + " of 11 channels (4-14) for both ascending and descending orbits. The AMSU-A gridded FCDR were generated from "
                + " the AMSU-A level-1c FCDR products of inter-calibrated brightness temperatures using "
                + "the Integrated Microwave Inter-Calibration Approach (IMICA)."));
        
        ncWriter.addGroupAttribute(null, new Attribute("Conventions", "CF-1.6"));
        ncWriter.addGroupAttribute(null, new Attribute("Metadata_Conventions", "CF-1.6,Unidata Dataset Discovery v1.0, "
                + " NOAA CDR v1.0, GDS v2.0"));
        ncWriter.addGroupAttribute(null, new Attribute("_nc3_strict", 1d));
        ncWriter.addGroupAttribute(null, new Attribute("acknowledgment", "This project was supported by a grant from the"
                + " NOAA Climate Data Record (CDR) Program"));
        ncWriter.addGroupAttribute(null, new Attribute("cdr_data_type", "Grid"));
        ncWriter.addGroupAttribute(null, new Attribute("program", "NOAA Climate Data Record Program for Satellites"));
        ncWriter.addGroupAttribute(null, new Attribute("comment", "The gridded products include three products based on "
                + "different gridding options:"
                + " 1)Brightness temperatures of near-nadir FOVs only; "
                + " 2)Brightness temperature of FOV with minimum view zenith angle;"
                + " 3)Average brightness temperatures of FOVs from multiple scan positions."));
        
        ncWriter.addGroupAttribute(null, new Attribute("contributor_name", "Cheng-Zhi Zou"));
        ncWriter.addGroupAttribute(null, new Attribute("contributor_role", "Principal Investigator and originator "
                + "of NOAA/STAR MSU/AMSU-A FCDR product"));
        ncWriter.addGroupAttribute(null, new Attribute("creator_email", "Cheng-Zhi.Zou@noaa.gov"));
        ncWriter.addGroupAttribute(null, new Attribute("creator_name", "Cheng-Zhi Zou"));

        // date and time
        String strDateFormat = "yyyy-MM-dd hh:mm:ss";
        DateFormat dateFormat = new SimpleDateFormat(strDateFormat);
        dateFormat.setTimeZone(TimeZone.getTimeZone( "UTC" ) );
        String formattedDate= String.format("%s Z", dateFormat.format(this.processingDate));
        ncWriter.addGroupAttribute(null, new Attribute("date_created", formattedDate));
        ncWriter.addGroupAttribute(null, new Attribute("cdm_data_type", "grid"));
        
        ncWriter.addGroupAttribute(null, new Attribute("geospatial_lat_max", 90.0f));
        ncWriter.addGroupAttribute(null, new Attribute("geospatial_lat_min", -90.0f));
        ncWriter.addGroupAttribute(null, new Attribute("geospatial_lat_units", "degrees_north"));
        ncWriter.addGroupAttribute(null, new Attribute("geospatial_lon_max", 180.0f));
        ncWriter.addGroupAttribute(null, new Attribute("geospatial_lon_min", -180.0f));       
        ncWriter.addGroupAttribute(null, new Attribute("geospatial_lon_units", "degrees_east"));

        // file Id
        if (selectedOrbit == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            items = this.filenameOutA.split(File.separator);
        } else {
            items = this.filenameOutD.split(File.separator);
        }
        int numItems = items.length;        
        ncWriter.addGroupAttribute(null, new Attribute("id", items[numItems-1]));            

        ncWriter.addGroupAttribute(null, new Attribute("institution", "NOAA/NESDIS/STAR>"
                + "Center for Satellite Applications and Research, NESDIS, NOAA, U.S. Department of Commerce"));
        ncWriter.addGroupAttribute(null, new Attribute("keywords", "Atmosphere > Atmospheric Temperature > "
                + "Air Temperature, Vertical Location > Troposphere"));
        ncWriter.addGroupAttribute(null, new Attribute("keywords_vocabulary", 
                "NASA Global Change Master Directory (GCMD) Earth Science Keywords, Version 8.0"));
        ncWriter.addGroupAttribute(null, new Attribute("license", "No constraints on data access or use."));
        ncWriter.addGroupAttribute(null, new Attribute("metadata_link", "gov.noaa.ncdc:C00757"));
        ncWriter.addGroupAttribute(null, new Attribute("naming_authority", "gov.noaa.ncdc"));
        ncWriter.addGroupAttribute(null, new Attribute("platform", 
                "NOAA-15, NOAA-16, NOAA-17, NOAA-18, Aqua, MetOp-A"));
        ncWriter.addGroupAttribute(null, new Attribute("platform_vocabulary", 
                "NASA/GCMD Platform Keywords. Version 8.1")); 
        ncWriter.addGroupAttribute(null, new Attribute("instrument_vocabulary", 
                "NASA/GCMD Instrument Keywords. Version 8.1"));         
        ncWriter.addGroupAttribute(null, new Attribute("processing_level", "NOAA Level 3"));
        ncWriter.addGroupAttribute(null, new Attribute("product_version", AmsuFcdrGridConstant.VERSION_MAJOR_AMSU_A_FCDR_L3_DATA));

        ncWriter.addGroupAttribute(null, new Attribute("publisher_name", "Cheng-Zhi Zou"));
        ncWriter.addGroupAttribute(null, new Attribute("publisher_email", "Cheng-Zhi.Zou@noaa.gov"));
        ncWriter.addGroupAttribute(null, new Attribute("publisher_url",
                "http://www.star.nesdis.noaa.gov/smcd/emb/mscat/index.php"));
        
        ncWriter.addGroupAttribute(null, new Attribute("references", "Wang, W., and C.-Z. Zou (2014). "
                + "AMSU-A-Only Atmospheric Temperature Data Records from the Lower Troposphere to the Top of the Stratosphere. "
                + "J. Atmos. Oceanic Technol., 31, 808-825, DOI: 10.1175/JTECH-D-13-00134.1"));
        ncWriter.addGroupAttribute(null, new Attribute("sensor", "AMSU-A>Advanced Microwave Sounding Unit-A"));
        ncWriter.addGroupAttribute(null, new Attribute("source", "AMSU Brightness Temperature FCDR - NOAA (doi:10.7289/V5X63JT2)"));
        ncWriter.addGroupAttribute(null, new Attribute("spatial_resolution", "1.0 latitude by 1.0 longitude"));
        ncWriter.addGroupAttribute(null, new Attribute("standard_name_vocabulary",
                "CF Standard Name Table (v26, 08 November 2013)"));        
        
          //:time_coverage_start = "2016-06-29T00:00:00Z";
          //  :time_coverage_end = "2016-06-29T23:59:58Z";
        String tm1 = String.format("%4d-%02d-%02dT00:00:00Z", this.year, this.month, this.day);
        String tm2 = String.format("%4d-%02d-%02dT23:59:59Z", this.year, this.month, this.day);
        ncWriter.addGroupAttribute(null, new Attribute("time_coverage_start", tm1));
        ncWriter.addGroupAttribute(null, new Attribute("time_coverage_end", tm2));
        ncWriter.addGroupAttribute(null, new Attribute("time_coverage_duration", "PT1D"));
          
        ncWriter.addGroupAttribute(null, new Attribute("Satellite Name", this.satId));
        ncWriter.addGroupAttribute(null, new Attribute("Year", (int) this.year));
        ncWriter.addGroupAttribute(null, new Attribute("Month", (int) this.month));
        ncWriter.addGroupAttribute(null, new Attribute("Day", (int) this.day));
        
        if (selectedOrbit == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            ncWriter.addGroupAttribute(null, new Attribute("Orbit", (String) "Ascending"));
        } else {
            ncWriter.addGroupAttribute(null, new Attribute("Orbit", (String) "Descending"));            
        }
        
        ncWriter.addGroupAttribute(null, new Attribute("doi", "doi:10.7289/V5X63JT2"));
    }
    
    
    /**
     * Initialize NetCDF file for ascending orbit
     */
    private void initAscendingFile() {
        int i = 0;
        int j = 0;
        int k = 0;
        float tmp = 0.0f;
        float tmp1 = 0.0f;
        float tmp2 = 0.0f;
         int NBound = 2;
        
        try {
            this.ncFileOutA = NetcdfFileWriter.createNew(NetcdfFileWriter.Version.netcdf3, this.filenameOutA);
            //this.groupOpt1A =  this.ncFileOutA.addGroup(null, AmsuFcdrGridConstant.NETCDF_GROUP_NAME_OPT1);
            
            // create Dimensions
            Dimension dimLon = this.ncFileOutA.addDimension(null, "longitude", AmsuFcdrGridConstant.GRID_NUM_LON);
            Dimension dimLat = this.ncFileOutA.addDimension(null, "latitude", AmsuFcdrGridConstant.GRID_NUM_LAT);
            Dimension dimTime = this.ncFileOutA.addDimension(null, "time", 1);
            Dimension dimBounds = this.ncFileOutA.addDimension(null, "bounds", NBound);

            this.dimLatA = new ArrayList<Dimension>();
            this.dimLatA.add(dimLat);

            this.dimLonA = new ArrayList<Dimension>();
            this.dimLonA.add(dimLon);

            this.dimGridA = new ArrayList<Dimension>();
            this.dimGridA.add(dimLat);
            this.dimGridA.add(dimLon);

            ArrayList<Dimension> dimBoundsLat = new ArrayList<Dimension>();
            dimBoundsLat.add(dimLat);
            dimBoundsLat.add(dimBounds);

            ArrayList<Dimension> dimBoundsLon = new ArrayList<Dimension>();
            dimBoundsLon.add(dimLon);
            dimBoundsLon.add(dimBounds);

            ArrayList<Dimension> dimListTimeBounds = new ArrayList<Dimension>();
            dimListTimeBounds.add(dimTime);
            dimListTimeBounds.add(dimBounds);

            
            // add global attributes
            this.defineGlobalAttributes(this.ncFileOutA, AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING);

            // define variables
            Variable vlon = this.ncFileOutA.addVariable(null, "longitude", DataType.FLOAT, this.dimLonA);
            vlon.addAttribute(new Attribute("units", "degree_east"));
            vlon.addAttribute(new Attribute("standard_name", "longitude"));
            vlon.addAttribute(new Attribute("long_name", "longitude of center of grid box"));
            vlon.addAttribute(new Attribute("valid_min", -180.0f));
            vlon.addAttribute(new Attribute("valid_max",  180.0f));
            vlon.addAttribute(new Attribute("bounds",  "longitude_bounds"));
            
            
            Variable vlat = this.ncFileOutA.addVariable(null, "latitude", DataType.FLOAT, this.dimLatA);
            vlat.addAttribute(new Attribute("units", "degree_north"));
            vlat.addAttribute(new Attribute("standard_name", "latitude"));
            vlat.addAttribute(new Attribute("long_name", "latitude of center of grid box"));
            vlat.addAttribute(new Attribute("valid_min", -90.0f));
            vlat.addAttribute(new Attribute("valid_max",  90.0f));
            vlat.addAttribute(new Attribute("bounds",  "latitude_bounds"));

            Variable vlatbounds = this.ncFileOutA.addVariable(null, "latitude_bounds", DataType.FLOAT, dimBoundsLat);
            vlatbounds.addAttribute(new Attribute("units", "degree_north"));
            vlatbounds.addAttribute(new Attribute("comment", "latitude values at the north and south bounds of each grid box."));

            Variable vlonbounds = this.ncFileOutA.addVariable(null, "longitude_bounds", DataType.FLOAT, dimBoundsLon);
            vlonbounds.addAttribute(new Attribute("units", "degree_east"));
            vlonbounds.addAttribute(new Attribute("comment", "longitude values at the west and east bounds of each grid box."));

            Variable vtime = this.ncFileOutA.addVariable(null, "time", DataType.DOUBLE, "time");
            vtime.addAttribute(new Attribute("units", "seconds since 1970-01-01 00:00:00 0:00"));
            vtime.addAttribute(new Attribute("standard_name", "time"));
            vtime.addAttribute(new Attribute("calendar", "Gregorian"));
            vtime.addAttribute(new Attribute("bounds", "time_bounds"));
            
            Variable vtimebounds = this.ncFileOutA.addVariable(null, "time_bounds", DataType.DOUBLE, dimListTimeBounds);
            vtimebounds.addAttribute(new Attribute("units", "seconds since 1970-01-01 00:00:00 0:00"));
            vtimebounds.addAttribute(new Attribute("comment", "time bounds for each time value"));
            
            
            // define variables for product 1
            this.defineNetcdfVarOpt1(AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING);
            // define variables for product 2
            this.defineNetcdfVarOpt2(AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING);
            // define variables for product 3
            this.defineNetcdfVarOpt3(AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING);
            
            // create file
            this.ncFileOutA.create();
            
            ArrayFloat.D1 dataLat = new ArrayFloat.D1(AmsuFcdrGridConstant.GRID_NUM_LAT);
            ArrayFloat.D1 dataLon = new ArrayFloat.D1(AmsuFcdrGridConstant.GRID_NUM_LON);
            ArrayFloat.D2 dataLatBounds = new ArrayFloat.D2(AmsuFcdrGridConstant.GRID_NUM_LAT, NBound);
            ArrayFloat.D2 dataLonBounds = new ArrayFloat.D2(AmsuFcdrGridConstant.GRID_NUM_LON, NBound);            

            // write longitude and latitude                        
            for (i=0; i<AmsuFcdrGridConstant.GRID_NUM_LAT; i++) {
                tmp = AmsuFcdrGridConstant.GRID_MAXIMUM_LATITUDE + (i+0.5f)*AmsuFcdrGridConstant.GRID_STEP_LATITUDE;
                dataLat.set(i, tmp );
                tmp1 = AmsuFcdrGridConstant.GRID_MAXIMUM_LATITUDE + i*AmsuFcdrGridConstant.GRID_STEP_LATITUDE;  // north bound
                tmp2 = tmp1 + AmsuFcdrGridConstant.GRID_STEP_LATITUDE;  // south bound
                dataLatBounds.set(i,0, tmp1);
                dataLatBounds.set(i,1, tmp2);                
            }
            
            for (i=0; i<AmsuFcdrGridConstant.GRID_NUM_LON; i++) {
                tmp = AmsuFcdrGridConstant.GRID_MINIMUM_LONGITUDE + (i+0.5f)*AmsuFcdrGridConstant.GRID_STEP_LONGITUDE;
                dataLon.set(i, tmp );
                tmp1 = AmsuFcdrGridConstant.GRID_MINIMUM_LONGITUDE + i*AmsuFcdrGridConstant.GRID_STEP_LONGITUDE;  // west
                tmp2 = tmp1 + AmsuFcdrGridConstant.GRID_STEP_LONGITUDE;
                dataLonBounds.set(i,0, tmp1);
                dataLonBounds.set(i,1, tmp2);
            }
            
            ArrayDouble.D1 dataTime = new ArrayDouble.D1(1);
            dataTime.set(0,(double)this.secondFromEpoch);

            ArrayDouble.D2 dataTimeBounds = new ArrayDouble.D2(1,2);
            dataTimeBounds.set(0,0, this.timeBoundMin);
            dataTimeBounds.set(0,1, this.timeBoundMax);
            
            this.ncFileOutA.write(vlat, dataLat);
            this.ncFileOutA.write(vlon, dataLon);
            this.ncFileOutA.write(vlatbounds, dataLatBounds);
            this.ncFileOutA.write(vlonbounds, dataLonBounds);
            this.ncFileOutA.write(vtime, dataTime);
            this.ncFileOutA.write(vtimebounds, dataTimeBounds);           
            
            // write BT data
        } catch (Exception nce) {
            nce.printStackTrace();
        }
    }

    /**
     * Initialize NetCDF file for descending orbit
     */
    private void initDescendingFile() {
        int i = 0;
        int j = 0;
        int k = 0;
        int NBound  = 2;
        float tmp = 0.0f;
        float tmp1 = 0.0f;
        float tmp2 = 0.0f;
        
        try {
            this.ncFileOutD = NetcdfFileWriter.createNew(NetcdfFileWriter.Version.netcdf3, this.filenameOutD);
            
            // create Dimensions
            Dimension dimLon = this.ncFileOutD.addDimension(null, "longitude", AmsuFcdrGridConstant.GRID_NUM_LON);
            Dimension dimLat = this.ncFileOutD.addDimension(null, "latitude", AmsuFcdrGridConstant.GRID_NUM_LAT);
            Dimension dimTime = this.ncFileOutD.addDimension(null, "time", 1);
            Dimension dimBounds = this.ncFileOutD.addDimension(null, "bounds", NBound);
            

            this.dimLatD = new ArrayList<Dimension>();
            this.dimLatD.add(dimLat);

            this.dimLonD = new ArrayList<Dimension>();
            this.dimLonD.add(dimLon);

            this.dimGridD = new ArrayList<Dimension>();
            this.dimGridD.add(dimLat);
            this.dimGridD.add(dimLon);

            ArrayList<Dimension> dimBoundsLat = new ArrayList<Dimension>();
            dimBoundsLat.add(dimLat);
            dimBoundsLat.add(dimBounds);

            ArrayList<Dimension> dimBoundsLon = new ArrayList<Dimension>();
            dimBoundsLon.add(dimLon);
            dimBoundsLon.add(dimBounds);

            ArrayList<Dimension> dimListTimeBounds = new ArrayList<Dimension>();
            dimListTimeBounds.add(dimTime);
            dimListTimeBounds.add(dimBounds);            
         
            // add global attributes
            this.defineGlobalAttributes(this.ncFileOutD, AmsuFcdrGridConstant.ORBIT_FLAG_DESCENDING);

            // define variables
            Variable vlon = this.ncFileOutD.addVariable(null, "longitude", DataType.FLOAT, this.dimLonD);
            vlon.addAttribute(new Attribute("units", "degree_east"));
            vlon.addAttribute(new Attribute("standard_name", "longitude"));
            vlon.addAttribute(new Attribute("long_name", "longitude of center of grid box"));
            vlon.addAttribute(new Attribute("valid_min", -180.0f));
            vlon.addAttribute(new Attribute("valid_max",  180.0f));
            vlon.addAttribute(new Attribute("bounds",  "longitude_bounds"));           
                       
            Variable vlat = this.ncFileOutD.addVariable(null, "latitude", DataType.FLOAT, this.dimLatD);
            vlat.addAttribute(new Attribute("units", "degree_north"));
            vlat.addAttribute(new Attribute("standard_name", "latitude"));
            vlat.addAttribute(new Attribute("long_name", "latitude of center of grid box"));
            vlat.addAttribute(new Attribute("valid_min", -90.0f));
            vlat.addAttribute(new Attribute("valid_max",  90.0f));
            vlat.addAttribute(new Attribute("bounds",  "latitude_bounds"));

            Variable vlatbounds = this.ncFileOutD.addVariable(null, "latitude_bounds", DataType.FLOAT, dimBoundsLat);
            vlatbounds.addAttribute(new Attribute("units", "degree_north"));
            vlatbounds.addAttribute(new Attribute("comment", "latitude values at the north and south bounds of each grid box."));

            Variable vlonbounds = this.ncFileOutD.addVariable(null, "longitude_bounds", DataType.FLOAT, dimBoundsLon);
            vlonbounds.addAttribute(new Attribute("units", "degree_east"));
            vlonbounds.addAttribute(new Attribute("comment", "longitude values at the west and east bounds of each grid box."));

            Variable vtime = this.ncFileOutD.addVariable(null, "time", DataType.DOUBLE, "time");
            vtime.addAttribute(new Attribute("units", "seconds since 1970-01-01 00:00:00 0:00"));
            vtime.addAttribute(new Attribute("standard_name", "time"));
            vtime.addAttribute(new Attribute("calendar", "Gregorian"));
            vtime.addAttribute(new Attribute("bounds", "time_bounds"));
            
            Variable vtimebounds = this.ncFileOutD.addVariable(null, "time_bounds", DataType.DOUBLE, dimListTimeBounds);
            vtimebounds.addAttribute(new Attribute("units", "seconds since 1970-01-01 00:00:00 0:00"));
            vtimebounds.addAttribute(new Attribute("comment", "time bounds for each time value"));
                        
            
            // define variables for product 1
            this.defineNetcdfVarOpt1(AmsuFcdrGridConstant.ORBIT_FLAG_DESCENDING);
            // define variables for product 2
            this.defineNetcdfVarOpt2(AmsuFcdrGridConstant.ORBIT_FLAG_DESCENDING);
            // define variables for product 3
            this.defineNetcdfVarOpt3(AmsuFcdrGridConstant.ORBIT_FLAG_DESCENDING);
            
            // create file
            this.ncFileOutD.create();
            ArrayFloat.D1 dataLat = new ArrayFloat.D1(AmsuFcdrGridConstant.GRID_NUM_LAT);
            ArrayFloat.D1 dataLon = new ArrayFloat.D1(AmsuFcdrGridConstant.GRID_NUM_LON);
            ArrayFloat.D2 dataLatBounds = new ArrayFloat.D2(AmsuFcdrGridConstant.GRID_NUM_LAT, NBound);
            ArrayFloat.D2 dataLonBounds = new ArrayFloat.D2(AmsuFcdrGridConstant.GRID_NUM_LON, NBound);   
            
            // write longitude and latitude                        
            for (i=0; i<AmsuFcdrGridConstant.GRID_NUM_LAT; i++) {
                tmp = AmsuFcdrGridConstant.GRID_MAXIMUM_LATITUDE + (i+0.5f)*AmsuFcdrGridConstant.GRID_STEP_LATITUDE;
                dataLat.set(i, tmp );
                tmp1 = AmsuFcdrGridConstant.GRID_MAXIMUM_LATITUDE + i*AmsuFcdrGridConstant.GRID_STEP_LATITUDE;  // north bound
                tmp2 = tmp1 + AmsuFcdrGridConstant.GRID_STEP_LATITUDE;  // south bound
                dataLatBounds.set(i,0, tmp1);
                dataLatBounds.set(i,1, tmp2);                                
            }
            
            for (i=0; i<AmsuFcdrGridConstant.GRID_NUM_LON; i++) {
                tmp = AmsuFcdrGridConstant.GRID_MINIMUM_LONGITUDE + (i+0.5f)*AmsuFcdrGridConstant.GRID_STEP_LONGITUDE;
                dataLon.set(i, tmp );
                tmp1 = AmsuFcdrGridConstant.GRID_MINIMUM_LONGITUDE + i*AmsuFcdrGridConstant.GRID_STEP_LONGITUDE;  // west
                tmp2 = tmp1 + AmsuFcdrGridConstant.GRID_STEP_LONGITUDE;
                dataLonBounds.set(i,0, tmp1);
                dataLonBounds.set(i,1, tmp2);                
            }

            ArrayDouble.D1 dataTime = new ArrayDouble.D1(1);
            dataTime.set(0,(float)this.secondFromEpoch);

            ArrayDouble.D2 dataTimeBounds = new ArrayDouble.D2(1,2);
            dataTimeBounds.set(0,0, this.timeBoundMin);
            dataTimeBounds.set(0,1, this.timeBoundMax);
            
            this.ncFileOutD.write(vlat, dataLat);
            this.ncFileOutD.write(vlon, dataLon);
            this.ncFileOutD.write(vlatbounds, dataLatBounds);
            this.ncFileOutD.write(vlonbounds, dataLonBounds);
            this.ncFileOutD.write(vtime, dataTime);
            this.ncFileOutD.write(vtimebounds, dataTimeBounds);                       
        } catch (Exception nce) {
            nce.printStackTrace();
        }
    }
    
    
    /**
     *  Define variable for data product 1 (near-nadir data).
     */
    private void defineNetcdfVarOpt1(short selectedOrbitFlag){
        NetcdfFileWriter ncWriter = null;
        String selectedOrbitName = null;
        int k = 0;
        int channel = 0; 
        String varName = null; 
        
        // checked selected orbit, ascending or descending
        if (selectedOrbitFlag == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            ncWriter = this.ncFileOutA;
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_ASCENDING_NAME;
        } else {
            ncWriter = this.ncFileOutD;
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_DESCENDING_NAME;
        }
        
        
        // define variables
       int[] channels = AmsuFcdrL1cConstant.SELECTED_CHANNELS;
       int numChannel = channels.length;
       for ( k=0; k<numChannel; k++) {
          channel = channels[k];
          varName = AmsuFcdrGridConstant.getProductBtName(selectedOrbitFlag, channel, 
                  AmsuFcdrGridConstant.ID_METHOD_FCDR_OMPOSITION_NEAR_NADIR_ONLY);
          // add variable          
          //Variable varBt = ncWriter.addVariable(null, varName, DataType.FLOAT, this.dimGridA);
          Variable varBt = ncWriter.addVariable(null, varName, DataType.FLOAT, AmsuFcdrGridConstant.NETCDF_GEO_DIMENSIION);
          
          // add attributes
          varBt.addAttribute(new Attribute("units","Kelvin" ));
          varBt.addAttribute(new Attribute("valid_max", AmsuFcdrGridConstant.VALID_MAX_BRIGHTNESS_TEMPERATURE));
          varBt.addAttribute(new Attribute("valid_min", AmsuFcdrGridConstant.VALID_MIN_BRIGHTNESS_TEMPERATURE));
          varBt.addAttribute(new Attribute("_FillValue", -9999.0f));
          varBt.addAttribute(new Attribute("coordinates","latitude longitude time" ));
          varBt.addAttribute(new Attribute("standard_name","brightness_temperature"));    
          varBt.addAttribute(new Attribute("long_name",varName));                
          String txt = String.format("Gridded brightness temperature of AMSU-A channel %d generated from Level-1C "
               + "FCDR data, using mean brightness temperature of near-nadir FOVs only (scan positions 15 and 16) "
                  + "in each grid box.", channel);
          varBt.addAttribute(new Attribute("comment",txt));         
       }
    }
    
    /**
     *  Define variable for data product 2 (Minimum VZA data).
     */
    private void defineNetcdfVarOpt2(short selectedOrbitFlag){
        NetcdfFileWriter ncWriter = null;
        String selectedOrbitName = null;
        int k = 0;
        int channel = 0; 
        String varName = null; 
        
        // checked selected orbit, ascending or descending
        if (selectedOrbitFlag == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            ncWriter = this.ncFileOutA;
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_ASCENDING_NAME;
        } else {
            ncWriter = this.ncFileOutD;
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_DESCENDING_NAME;
        }
        
        // define BT variables
       int[] channels = AmsuFcdrL1cConstant.SELECTED_CHANNELS;
       int numChannel = channels.length;
       for ( k=0; k<numChannel; k++) {
          channel = channels[k];
          varName = AmsuFcdrGridConstant.getProductBtName(selectedOrbitFlag, channel, 
                  AmsuFcdrGridConstant.ID_METHOD_FCDR_OMPOSITION_MOST_NADIR);
          // add variable          
//          Variable varBt = ncWriter.addVariable(null, varName, DataType.FLOAT, this.dimGridA);
          Variable varBt = ncWriter.addVariable(null, varName, DataType.FLOAT, AmsuFcdrGridConstant.NETCDF_GEO_DIMENSIION);
                            
          // add attributes
          varBt.addAttribute(new Attribute("units","Kelvin" ));
          varBt.addAttribute(new Attribute("valid_max", AmsuFcdrGridConstant.VALID_MAX_BRIGHTNESS_TEMPERATURE));
          varBt.addAttribute(new Attribute("valid_min", AmsuFcdrGridConstant.VALID_MIN_BRIGHTNESS_TEMPERATURE));
          varBt.addAttribute(new Attribute("_FillValue", -9999.0f));          
          varBt.addAttribute(new Attribute("coordinates","latitude longitude time" ));
          varBt.addAttribute(new Attribute("standard_name","brightness_temperature"));    
          varBt.addAttribute(new Attribute("long_name",varName));      
          String txt = String.format("Gridded brightness temperature of AMSU-A channel %d generated from Level-1C "
                  + "FCDR data, using FOVs with minimum viewing zenith angle.", channel);
          varBt.addAttribute(new Attribute("comment",txt));         
       }
       // define time variable: time_IMICA_minvza_since_1978_ascending
        varName = AmsuFcdrGridConstant.getProductPixelTimeName(selectedOrbitFlag);
//        Variable varTime = ncWriter.addVariable(null, varName, DataType.DOUBLE, this.dimGridA);
        Variable varTime = ncWriter.addVariable(null, varName, DataType.DOUBLE, AmsuFcdrGridConstant.NETCDF_GEO_DIMENSIION);
        varTime.addAttribute(new Attribute("units","seconds since 1978-01-01 00:00:00" ));
        varTime.addAttribute(new Attribute("_FillValue", -9999.0d));
        varTime.addAttribute(new Attribute("standard_name","time"));    
        varTime.addAttribute(new Attribute("long_name",varName));    
        varTime.addAttribute(new Attribute("comment"," scan start time (UTC)  since 1978-01-01 00:00:0 "
                + "of the selected FOV with minimum view zenith angle in each grid box"));    
        
       // define VZA variable
        varName = AmsuFcdrGridConstant.getProductPixelVzaName(selectedOrbitFlag);
//        Variable varVza = ncWriter.addVariable(null, varName, DataType.FLOAT, this.dimGridA);
        Variable varVza = ncWriter.addVariable(null, varName, DataType.FLOAT, AmsuFcdrGridConstant.NETCDF_GEO_DIMENSIION);
        varVza.addAttribute(new Attribute("units","degrees" ));
        varVza.addAttribute(new Attribute("_FillValue", -9999f));
        varVza.addAttribute(new Attribute("standard_name","sensor_zenith_angle"));                         
        varVza.addAttribute(new Attribute("long_name",varName));                         
        varVza.addAttribute(new Attribute("comment","minimum view zenith angle of FOVs in each grid box"));                         
    }
        
    /**
     *  Define variable for data product 3 (block mean).
     */
    private void defineNetcdfVarOpt3(short selectedOrbitFlag){
        NetcdfFileWriter ncWriter = null;
        String selectedOrbitName = null;
        int k = 0;
        int channel = 0; 
        String varName = null; 
        
        // checked selected orbit, ascending or descending
        if (selectedOrbitFlag == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            ncWriter = this.ncFileOutA;
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_ASCENDING_NAME;
        } else {
            ncWriter = this.ncFileOutD;
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_DESCENDING_NAME;
        }
        
        // define variables
       int[] channels = AmsuFcdrL1cConstant.SELECTED_CHANNELS;
       int numChannel = channels.length;
       for ( k=0; k<numChannel; k++) {
          channel = channels[k];
          varName = AmsuFcdrGridConstant.getProductBtName(selectedOrbitFlag, channel, 
                  AmsuFcdrGridConstant.ID_METHOD_FCDR_OMPOSITION_SPATIAL_MEAN);
          // add variable          
//          Variable varBt = ncWriter.addVariable(null, varName, DataType.FLOAT, this.dimGridA);
          Variable varBt = ncWriter.addVariable(null, varName, DataType.FLOAT, AmsuFcdrGridConstant.NETCDF_GEO_DIMENSIION);
          
          // add attributes
          varBt.addAttribute(new Attribute("units","Kelvin" ));
          varBt.addAttribute(new Attribute("valid_max", AmsuFcdrGridConstant.VALID_MAX_BRIGHTNESS_TEMPERATURE));
          varBt.addAttribute(new Attribute("valid_min", AmsuFcdrGridConstant.VALID_MIN_BRIGHTNESS_TEMPERATURE));
          varBt.addAttribute(new Attribute("_FillValue", -9999.0f));          
          varBt.addAttribute(new Attribute("coordinates","latitude longitude time" ));
          varBt.addAttribute(new Attribute("long_name",varName));      
          varBt.addAttribute(new Attribute("standard_name","brightness_temperature"));    
//          String cellMethod = String.format("%s:mean", varName);
          String cellMethod = String.format("area:mean");
          varBt.addAttribute(new Attribute("cell_methods",cellMethod));             
          String txt = String.format("Gridded brightness temperature of AMSU-A channel %d generated from Level-1C limb-corrected "
                  + "FCDR data, using the mean brightness temperature of pixels in each grid box "
                  + " from multiple scan positions (8-23).", channel);
          varBt.addAttribute(new Attribute("comment",txt));           
       }
       for ( k=0; k<numChannel; k++) {
          channel = channels[k];
          varName = AmsuFcdrGridConstant.getProductBtStdName(selectedOrbitFlag, channel);
          // add variable          
//          Variable varBt = ncWriter.addVariable(null, varName, DataType.FLOAT, this.dimGridA);
          Variable varBt = ncWriter.addVariable(null, varName, DataType.FLOAT, AmsuFcdrGridConstant.NETCDF_GEO_DIMENSIION);
          // add attributes
          varBt.addAttribute(new Attribute("units","Kelvin" ));
          varBt.addAttribute(new Attribute("coordinates","latitude longitude" ));
          varBt.addAttribute(new Attribute("standard_name","brightness_temperature"));   
//          String cellMethod = String.format("%s:standard_deviation", varName);
          String cellMethod = String.format("area:standard_deviation");
          varBt.addAttribute(new Attribute("cell_methods", cellMethod));             
          varBt.addAttribute(new Attribute("long_name",varName));          
          varBt.addAttribute(new Attribute("valid_min", 0.0f));
          varBt.addAttribute(new Attribute("_FillValue", -9999.0f));   
          String txt = String.format("Standard deviation of channel %d brightness temperature of pixels within grid box from "
                  + "multiple scan positions (8-23)", channel);
          varBt.addAttribute(new Attribute("comment",txt));                     
       }

    }
    
    /**
     * Write gridded near-nadir data to netcdf file for selected orbit (ascend/descending).
     * @param selectedOrbitFlag 
     */
    private void writeFcdrOpt1(short selectedOrbitFlag) {
        NetcdfFileWriter ncWriter = null;
        int k = 0;
        int i=0;
        int j = 0;
        int channel =0;
        String varName = null;
        String selectedOrbitName =null;
        int[] dataDim = new int[]{AmsuFcdrGridConstant.GRID_NUM_LAT, AmsuFcdrGridConstant.GRID_NUM_LON};
        Index2D idx = new Index2D(dataDim);
       
        
        // checked selected orbit, ascending or descending
        if (selectedOrbitFlag == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            ncWriter = this.ncFileOutA;
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_ASCENDING_NAME;
        } else {
            ncWriter = this.ncFileOutD;
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_DESCENDING_NAME;
        }
        
       int[] channels = AmsuFcdrL1cConstant.SELECTED_CHANNELS;
       int numChannel = channels.length;
       try {
       for ( k=0; k<numChannel; k++) {
          channel = channels[k];
          varName = String.format("BT_ch%d_IMICA_%s_nadir", channel, selectedOrbitName );
          // find variable          
          Variable varBt = ncWriter.findVariable(varName);
          // write data
          ArrayFloat.D2 buff = new ArrayFloat.D2(AmsuFcdrGridConstant.GRID_NUM_LAT, AmsuFcdrGridConstant.GRID_NUM_LON);
          for(i=0; i<AmsuFcdrGridConstant.GRID_NUM_LAT; i++) {
              for(j=0; j<AmsuFcdrGridConstant.GRID_NUM_LON; j++) {
                  idx.set(i,j);
                  buff.set(idx, this.fcdrNadir.fcdr_gridded_bt[i][j][k]); 
              }
          }
          ncWriter.write(varBt, buff);
       }
       } catch (Exception e){
           e.printStackTrace();
       }        
    }

    /**
     * Write gridded minvza data to netcdf file for selected orbit (ascend/descending).
     * @param selectedOrbitFlag 
     */
    private void writeFcdrOpt2(short selectedOrbitFlag) {
        NetcdfFileWriter ncWriter = null;
        int k = 0;
        int i=0;
        int j = 0;
        int channel =0;
        String varName = null;
        String selectedOrbitName =null;
        int[] dataDim = new int[]{AmsuFcdrGridConstant.GRID_NUM_LAT, AmsuFcdrGridConstant.GRID_NUM_LON};
        Index2D idx = new Index2D(dataDim);
       
        
        // checked selected orbit, ascending or descending
        if (selectedOrbitFlag == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            ncWriter = this.ncFileOutA;
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_ASCENDING_NAME;
        } else {
            ncWriter = this.ncFileOutD;
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_DESCENDING_NAME;
        }
        
       int[] channels = AmsuFcdrL1cConstant.SELECTED_CHANNELS;
       int numChannel = channels.length;
       try {
           // write BT data
            for ( k=0; k<numChannel; k++) {
                channel = channels[k];
                varName = AmsuFcdrGridConstant.getProductBtName(selectedOrbitFlag, channel, 
                            AmsuFcdrGridConstant.ID_METHOD_FCDR_OMPOSITION_MOST_NADIR);
                // find variable          
                Variable varBt = ncWriter.findVariable(varName);
                // write data
                ArrayFloat.D2 buff = new ArrayFloat.D2(AmsuFcdrGridConstant.GRID_NUM_LAT, AmsuFcdrGridConstant.GRID_NUM_LON);
                for(i=0; i<AmsuFcdrGridConstant.GRID_NUM_LAT; i++) {
                    for(j=0; j<AmsuFcdrGridConstant.GRID_NUM_LON; j++) {
                        idx.set(i,j);
                        buff.set(idx, this.fcdrMinvza.fcdr_gridded_bt[i][j][k]); 
                    }
                }
                ncWriter.write(varBt, buff);
            }
                        
            // write VZA data
            varName = AmsuFcdrGridConstant.getProductPixelVzaName(selectedOrbitFlag);
            Variable varVza = ncWriter.findVariable(varName);
            ArrayFloat.D2 buff1 = new ArrayFloat.D2(AmsuFcdrGridConstant.GRID_NUM_LAT, AmsuFcdrGridConstant.GRID_NUM_LON);
            for(i=0; i<AmsuFcdrGridConstant.GRID_NUM_LAT; i++) {
                for(j=0; j<AmsuFcdrGridConstant.GRID_NUM_LON; j++) {
                    idx.set(i,j);
                    buff1.set(idx, this.fcdrMinvza.minvza[i][j]); 
                }
            }
            ncWriter.write(varVza, buff1);    
            
            // write time data
            varName = AmsuFcdrGridConstant.getProductPixelTimeName(selectedOrbitFlag);
            Variable varTm = ncWriter.findVariable(varName);
            ArrayDouble.D2 buff2 = new ArrayDouble.D2(AmsuFcdrGridConstant.GRID_NUM_LAT, AmsuFcdrGridConstant.GRID_NUM_LON);
            for(i=0; i<AmsuFcdrGridConstant.GRID_NUM_LAT; i++) {
                for(j=0; j<AmsuFcdrGridConstant.GRID_NUM_LON; j++) {
                    idx.set(i,j);
                    buff2.set(idx, this.fcdrMinvza.time_minvza[i][j]); 
                }
            }
            ncWriter.write(varTm, buff2);                                
            
       } catch (Exception e){
           e.printStackTrace();
       }  
       
       
    }
    
    
    /**
     * Write gridded block-mean data to netcdf file for selected orbit (ascend/descending).
     * @param selectedOrbitFlag, flag for selected orbit.
     */
    private void writeFcdrOpt3(short selectedOrbitFlag) {
        NetcdfFileWriter ncWriter = null;
        int k = 0;
        int i=0;
        int j = 0;
        int channel =0;
        String varName = null;
        String selectedOrbitName =null;
        int[] dataDim = new int[]{AmsuFcdrGridConstant.GRID_NUM_LAT, AmsuFcdrGridConstant.GRID_NUM_LON};
        Index2D idx = new Index2D(dataDim);       
        
        // checked selected orbit, ascending or descending
        if (selectedOrbitFlag == AmsuFcdrGridConstant.ORBIT_FLAG_ASCENDING) {
            ncWriter = this.ncFileOutA;
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_ASCENDING_NAME;
        } else {
            ncWriter = this.ncFileOutD;
            selectedOrbitName = AmsuFcdrGridConstant.ORBIT_DESCENDING_NAME;
        }
        
       int[] channels = AmsuFcdrL1cConstant.SELECTED_CHANNELS;
       int numChannel = channels.length;
       
       // write mean BT
       try {
            for ( k=0; k<numChannel; k++) {
                channel = channels[k];
                varName = AmsuFcdrGridConstant.getProductBtName(selectedOrbitFlag, channel, 
                            AmsuFcdrGridConstant.ID_METHOD_FCDR_OMPOSITION_SPATIAL_MEAN);
                // find variable          
                Variable varBt = ncWriter.findVariable(varName);
                // write data
                ArrayFloat.D2 buff = new ArrayFloat.D2(AmsuFcdrGridConstant.GRID_NUM_LAT, AmsuFcdrGridConstant.GRID_NUM_LON);
                for(i=0; i<AmsuFcdrGridConstant.GRID_NUM_LAT; i++) {
                    for(j=0; j<AmsuFcdrGridConstant.GRID_NUM_LON; j++) {
                        idx.set(i,j);
                        buff.set(idx, this.fcdrBlockmean.fcdr_gridded_bt[i][j][k]); 
                    }
                }
                ncWriter.write(varBt, buff);
            }
       } catch (Exception e){
           e.printStackTrace();
       }      
       
       // write std
       try {
            for ( k=0; k<numChannel; k++) {
                channel = channels[k];
                varName = AmsuFcdrGridConstant.getProductBtStdName(selectedOrbitFlag, channel);
                // find variable          
                Variable varBt = ncWriter.findVariable(varName);
                // write data
                ArrayFloat.D2 buff = new ArrayFloat.D2(AmsuFcdrGridConstant.GRID_NUM_LAT, AmsuFcdrGridConstant.GRID_NUM_LON);
                for(i=0; i<AmsuFcdrGridConstant.GRID_NUM_LAT; i++) {
                    for(j=0; j<AmsuFcdrGridConstant.GRID_NUM_LON; j++) {
                        idx.set(i,j);
                        buff.set(idx, this.fcdrBlockmean.fcdr_gridded_bt_std[i][j][k]); 
                    }
                }
                ncWriter.write(varBt, buff);
            }
       } catch (Exception e){
           e.printStackTrace();
       }      
       
       
    }


    
    /**
     * Compose gridded FCDR with nadir pixels only.
     *
     * @param selectedOrbitFlag, flag of selected orbit, 0: ascending, 1: descending.
     */
    private void composeFcdrNadir(short selectedOrbitFlag) {
        int k = 0;
        int i = 0;
        int j = 0;
        int kscan = 0;
        int krow = 0;
        int kcol = 0;
        short vtmp = 0;

        int channel = 0;
        int ncol = AmsuFcdrGridConstant.GRID_NUM_LON;
        int nrow = AmsuFcdrGridConstant.GRID_NUM_LAT;

        int num_channel = AmsuFcdrL1cConstant.NUMBER_SELECTED_CHANNELS;
        int[] channels = AmsuFcdrL1cConstant.SELECTED_CHANNELS;
        this.fcdrNadir = new AmsuFcdrGriddedDailyDataNadir();
        float[][] pcount = new float[nrow][ncol];
        float[][] psum = new float[nrow][ncol];

        Attribute attrScale = null;
        Attribute attrRange = null;
        float scaleFactor = 0.0f;
        short[] validBtRange = new short[2];
        
        // compose FCDR channel by channel
        for (k = 0; k < num_channel; k++) {
            channel = channels[k];
            // initialize count and sum 
            for (i = 0; i < nrow; i++) {
                for (j = 0; j < ncol; j++) {
                    pcount[i][j] = 0.0f;
                    psum[i][j] = 0.0f;
                }
            }
            // read  channel data, IMICA brightness temperature 
            String varName = AmsuFcdrL1cConstant.getFcdrL1cVarName(channel, AmsuFcdrL1cConstant.ID_FCDR_L1C_PRODUCT_IMICA);
            Variable dataVar = this.ncFileIn.findVariable(varName);
            attrScale = dataVar.findAttribute(AmsuFcdrL1cConstant.ATTRIBUTE_BRIGHTNESS_TEMPERATURE_SCALE_FACTOR);
            scaleFactor = (float)attrScale.getValue(0);
            
            attrRange = dataVar.findAttribute(AmsuFcdrL1cConstant.ATTRIBUTE_BRIGHTNESS_TEMPERATURE_VALID_RANGE);
            validBtRange[0] = (short)attrRange.getValue(0);
            validBtRange[1] = (short)attrRange.getValue(1);
            
            int[] shape = dataVar.getShape();
            
            int[] origin = new int[2];
            try {
                ArrayShort.D2 btArr = (ArrayShort.D2) dataVar.read(origin, shape);
                // process scan by scan
                for (kscan = 0; kscan < shape[0]; kscan++) {
                    // check orbit, if this scan doesn't belong to the selected ascending/descending orbit, ignore it.
                    if (this.orbitFlag[kscan] != selectedOrbitFlag) {
                        continue;
                    }
                    // left nadir
                    krow = this.pixRow[kscan][AmsuFcdrL1cConstant.INDEX_AMSU_A_NADIR_PIXEL_LEFT - 1];
                    kcol = this.pixCol[kscan][AmsuFcdrL1cConstant.INDEX_AMSU_A_NADIR_PIXEL_LEFT - 1];
                    vtmp = btArr.get(kscan, AmsuFcdrL1cConstant.INDEX_AMSU_A_NADIR_PIXEL_LEFT - 1);
                    if ((vtmp >=validBtRange[0])&(vtmp <=validBtRange[1])) {
                        pcount[krow][kcol] = pcount[krow][kcol] + 1;
                        psum[krow][kcol] = psum[krow][kcol] + scaleFactor*((float)vtmp);
                    }
                    //right nadir
                    krow = this.pixRow[kscan][AmsuFcdrL1cConstant.INDEX_AMSU_A_NADIR_PIXEL_RIGHT - 1];
                    kcol = this.pixCol[kscan][AmsuFcdrL1cConstant.INDEX_AMSU_A_NADIR_PIXEL_RIGHT - 1];
                    vtmp = btArr.get(kscan, AmsuFcdrL1cConstant.INDEX_AMSU_A_NADIR_PIXEL_RIGHT - 1);
                    if ((vtmp >=validBtRange[0])&(vtmp <=validBtRange[1])) {
                        pcount[krow][kcol] = pcount[krow][kcol] +1;
                        psum[krow][kcol] = psum[krow][kcol] + scaleFactor*((float)vtmp);
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
                for (i = 0; i < nrow; i++) {
                    for (j = 0; j < ncol; j++) {
                        psum[i][j] = AmsuFcdrGridConstant.FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE;
                        pcount[i][j] = 0.0f;
                    }
                }
            }

            for (i = 0; i < nrow; i++) {
                for (j = 0; j < ncol; j++) {
                    if (pcount[i][j] > 0.5) {
                        this.fcdrNadir.fcdr_gridded_bt[i][j][k] = psum[i][j] / pcount[i][j];
                    } else {
                        this.fcdrNadir.fcdr_gridded_bt[i][j][k] = AmsuFcdrGridConstant.FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE;
                    }
                }
            }
        }
    }

    /**
     * Compose gridded FCDR with minimum VZA.
     *
     * @param selectedOrbitFlag, flag of selected orbit, 0: ascending, 1: descending.
     */
    private void composeFcdrMinvza(short selectedOrbitFlag) {
        int k = 0;
        int i = 0;
        int j = 0;
        int kscan = 0;
        int kpos = 0;
        int krow = 0;
        int kcol = 0;
        float vtmp = 0.0f;

        int channel = 0;
        int ncol = AmsuFcdrGridConstant.GRID_NUM_LON;
        int nrow = AmsuFcdrGridConstant.GRID_NUM_LAT;

        int scanPos1 = AmsuFcdrGridConstant.INDEX_SCAN_POSITION_START - 1;
        int scanPos2 = AmsuFcdrGridConstant.INDEX_SCAN_POSITION_END - 1;

        int num_channel = AmsuFcdrL1cConstant.NUMBER_SELECTED_CHANNELS;
        int[] channels = AmsuFcdrL1cConstant.SELECTED_CHANNELS;
        this.fcdrMinvza = new AmsuFcdrGriddedDailyDataMinvza();
        short[][] pcount = new short[nrow][ncol];
        float[][] pvza = new float[nrow][ncol];
        float[][] pbt = new float[nrow][ncol];
        double[][] ptime = new double[nrow][ncol];
        int[][] selectedScan = new int[nrow][ncol]; // row number for selected pixels with minimum VZA
        int[][] selectedPos = new int[nrow][ncol]; // col number for selected pixels with minimum VZA
        
        int[] origin = new int[2];
        int[] shape = null;
        
        Attribute attrScale = null;
        Attribute attrRange = null;
        float scaleFactor = 0.0f;
        short[] validBtRange = new short[2];

        ArrayDouble.D1 timeArr = null;
        ArrayFloat.D2 vzaArr = null;
        
        float tmpVza = 0.0f;
        double tmpTm = 0.0d;
        int mscan = 0;
        int mpos = 0;
                
        // read VZA data
        String vzaName = AmsuFcdrL1cConstant.SENSOR_ZENITH_ANGLE;
        Variable vzaVar = this.ncFileIn.findVariable(vzaName);
        shape = vzaVar.getShape();
        try {
            vzaArr = (ArrayFloat.D2) vzaVar.read(origin, shape);
        } catch (Exception e){
            e.printStackTrace();
        }

        // read time data
        String tmName = AmsuFcdrL1cConstant.SCAN_TIME_SINCE78;
        Variable tmVar = this.ncFileIn.findVariable(tmName);
        try {
            timeArr = (ArrayDouble.D1) tmVar.read();
        } catch (Exception e){
            e.printStackTrace();
        }
        
        // initialize arrays
        for(i=0; i< nrow; i++){
            for(j=0; j<ncol; j++) {
                pcount[i][j] = 0;
                ptime[i][j] = AmsuFcdrGridConstant.FILLED_VALUE_INVALID_TIME_SINCE78;
                //pvza[i][j] = AmsuFcdrGridConstant.FILLED_VALUE_INVALID_VZA;
                pvza[i][j] = Float.MAX_VALUE;
                selectedScan[i][j] = -1;
                selectedPos[i][j] = -1;
            }
        }
        
        
        // generate minimum VZA and time
        for (kscan = 0; kscan < shape[0]; kscan++) {
            if (this.orbitFlag[kscan] != selectedOrbitFlag) {
                continue;  // skip other orbit
            }
            tmpTm = timeArr.get(kscan);
            for (kpos = scanPos1; kpos <= scanPos2; kpos++) {
                tmpVza = vzaArr.get(kscan,kpos); // be careful with index
                krow = this.pixRow[kscan][kpos];
                kcol = this.pixCol[kscan][kpos];
                
                // check index range
                if ((krow<0)|(krow>=nrow)|(kcol<0)|(kcol>=ncol)){
                    String errorMsg = String.format("Error in AmsuFcdrGriddedProcessor:composeFcdrMinvza,"
                            + " index out of range  getting minimum VZA for %4d-%02d-%02d, "
                            + "kscan=%d, kpos=%d: krow=%d, kcol=%d, nrow=%d, ncol=%d"
                            + " ", year, month, day, kscan, kpos, krow, kcol, nrow, ncol);
                    fcdrErrorLogger.error(errorMsg);
                    continue; // skip pixels with bad geolocation
                }
                
                if (( tmpVza >= 0) &(tmpVza<90)) {
                    // valid VZA
                    if (pcount[krow][kcol]>0.5){
                        // update minimum VZA
                        if (tmpVza < pvza[krow][kcol]){
                            pvza[krow][kcol] = tmpVza;
                            ptime[krow][kcol] = tmpTm;
                            selectedScan[krow][kcol] = kscan;
                            selectedPos[krow][kcol] = kpos;                                                    
                        }
                    } else {
                        // first valid data
                        pvza[krow][kcol] = tmpVza;
                        ptime[krow][kcol] = tmpTm;
                        selectedScan[krow][kcol] = kscan;
                        selectedPos[krow][kcol] = kpos;                        
                    }
                    pcount[krow][kcol] ++;                    
                }
            }
        }
           
        for(i=0; i<nrow; i++) {
            for(j=0; j<ncol; j++){
                if (pvza[i][j]<90){
                    this.fcdrMinvza.minvza[i][j] = pvza[i][j];
                } else {
                    this.fcdrMinvza.minvza[i][j] =  AmsuFcdrGridConstant.FILLED_VALUE_INVALID_VZA;
                }
                this.fcdrMinvza.time_minvza[i][j] = ptime[i][j];
            }
        }
        
        // compose FCDR channel by channel
        for (k = 0; k < num_channel; k++) {
            channel = channels[k];
            // read  channel data, IMICA brightness temperature 
            String varName = AmsuFcdrL1cConstant.getFcdrL1cVarName(channel, AmsuFcdrL1cConstant.ID_FCDR_L1C_PRODUCT_IMICA);
            Variable dataVar = this.ncFileIn.findVariable(varName);
            attrScale = dataVar.findAttribute(AmsuFcdrL1cConstant.ATTRIBUTE_BRIGHTNESS_TEMPERATURE_SCALE_FACTOR);
            scaleFactor = (float)attrScale.getValue(0);
            attrRange = dataVar.findAttribute(AmsuFcdrL1cConstant.ATTRIBUTE_BRIGHTNESS_TEMPERATURE_VALID_RANGE);
            validBtRange[0] = (short)attrRange.getValue(0);
            validBtRange[1] = (short)attrRange.getValue(1);
            
            shape = dataVar.getShape();

            try {
                ArrayShort.D2 btArr = (ArrayShort.D2) dataVar.read(origin, shape);
                
                // use selected scan and position id to compose data
                for(i=0; i<nrow; i++){
                    for(j=0; j<ncol; j++){
                        mscan = selectedScan[i][j];
                        mpos = selectedPos[i][j];
                        if ((mscan>=0) &(mpos>=0)) {
                            // valid scan and position 
                            vtmp = btArr.get(mscan,mpos);
                            if ((vtmp >=validBtRange[0])&(vtmp <=validBtRange[1])) {
                                this.fcdrMinvza.fcdr_gridded_bt[i][j][k] = scaleFactor*((float)vtmp);
                            } else {
                                this.fcdrMinvza.fcdr_gridded_bt[i][j][k] = 
                                        AmsuFcdrGridConstant.FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE;                               
                            }
                        } else {
                            this.fcdrMinvza.fcdr_gridded_bt[i][j][k] = 
                                        AmsuFcdrGridConstant.FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE;
                        }
                    }
                }                
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
    
    
    /**
     * Compose gridded FCDR with block mean.
     *
     * @param selectedOrbitFlag, flag of selected orbit, 0: ascending, 1: descending.
     */
    private void composeFcdrBlockmean(short selectedOrbitFlag) {
        int k = 0;
        int i = 0;
        int j = 0;
        int kscan = 0;
        int kpos = 0;
        int krow = 0;
        int kcol = 0;
        float vtmp = 0.0f;
        int npixel = 0;

        int channel = 0;
        int ncol = AmsuFcdrGridConstant.GRID_NUM_LON;
        int nrow = AmsuFcdrGridConstant.GRID_NUM_LAT;

        int scanPos1 = AmsuFcdrGridConstant.INDEX_SCAN_POSITION_START - 1;
        int scanPos2 = AmsuFcdrGridConstant.INDEX_SCAN_POSITION_END - 1;

        int num_channel = AmsuFcdrL1cConstant.NUMBER_SELECTED_CHANNELS;
        int[] channels = AmsuFcdrL1cConstant.SELECTED_CHANNELS;
        this.fcdrBlockmean = new AmsuFcdrGriddedDailyDataBlockmean();
        float[][] pcount = new float[nrow][ncol];
        //float[][] psum = new float[nrow][ncol];
        float[][] pmean = new float[nrow][ncol];
        float[][] pstd = new float[nrow][ncol];
        Attribute attrScale = null;
        Attribute attrRange = null;
        float scaleFactor = 0.0f;
        short[] validBtRange = new short[2];
        float tmpBt = 0.0f;
        float tmpMeanOld = 0.0f;
        
        // compose FCDR channel by channel
        for (k = 0; k < num_channel; k++) {
            channel = channels[k];
            // initialize count and sum 
            for (i = 0; i < nrow; i++) {
                for (j = 0; j < ncol; j++) {
                    pcount[i][j] = 0.0f;
                    //psum[i][j] = 0.0f;
                    pmean[i][j] = AmsuFcdrGridConstant.FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE;
                    pstd[i][j]=AmsuFcdrGridConstant.FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE;
                }
            }
            // read  channel data, IMICA brightness temperature with limb correction
            String varName = AmsuFcdrL1cConstant.getFcdrL1cVarName(channel, AmsuFcdrL1cConstant.ID_FCDR_L1C_PRODUCT_IMICA_LIMBCORR);
            Variable dataVar = this.ncFileIn.findVariable(varName);
            attrScale = dataVar.findAttribute(AmsuFcdrL1cConstant.ATTRIBUTE_BRIGHTNESS_TEMPERATURE_SCALE_FACTOR);
            scaleFactor = (float)attrScale.getValue(0);
            attrRange = dataVar.findAttribute(AmsuFcdrL1cConstant.ATTRIBUTE_BRIGHTNESS_TEMPERATURE_VALID_RANGE);
            validBtRange[0] = (short)attrRange.getValue(0);
            validBtRange[1] = (short)attrRange.getValue(1);
            
            int[] shape = dataVar.getShape();
            int[] origin = new int[2];
            try {
                ArrayShort.D2 btArr = (ArrayShort.D2) dataVar.read(origin, shape);
                // process scan by scan
                for (kscan = 0; kscan < shape[0]; kscan++) {
                    // check orbit, if this scan doesn't belong to the selected ascending/descending orbit, ignore it.
                    if (this.orbitFlag[kscan] != selectedOrbitFlag) {
                        continue;
                    }

                    // only use selected scan positions: 8-23
                    for (kpos = scanPos1; kpos <= scanPos2; kpos++) {
                        // be careful with scan position index
                        krow = this.pixRow[kscan][kpos];
                        kcol = this.pixCol[kscan][kpos];
                        vtmp = btArr.get(kscan,kpos);
                        if ((vtmp >=validBtRange[0])&(vtmp <=validBtRange[1])) {
                            pcount[krow][kcol]++;
                            npixel = (int)pcount[krow][kcol];
                            tmpBt = scaleFactor*((float)vtmp);                              
                            if(npixel <2) {
                                // if first pixel, assign mean and std
                                pmean[krow][kcol] = tmpBt;
                                pstd[krow][kcol] = 0.0f;
                            } else {
                                // update mean and std
                                tmpMeanOld = pmean[krow][kcol];
                                pmean[krow][kcol] = ( tmpBt + tmpMeanOld*(npixel-1.0f))/npixel;
                                pstd[krow][kcol] = ((npixel-2.0f)/(npixel-1.0f))* pstd[krow][kcol] 
                                        + (tmpBt-tmpMeanOld)*(tmpBt-tmpMeanOld)/npixel;
                            }
                            
                            //psum[krow][kcol] = psum[krow][kcol] + scaleFactor*((float)vtmp);                            
                        }
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
                for (i = 0; i < nrow; i++) {
                    for (j = 0; j < ncol; j++) {
                        //psum[i][j] = AmsuFcdrGridConstant.FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE;
                        pmean[i][j] = AmsuFcdrGridConstant.FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE;
                        pstd[i][j] = AmsuFcdrGridConstant.FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE;
                        pcount[i][j] = 0.0f;
                    }
                }
            }

            for (i = 0; i < nrow; i++) {
                for (j = 0; j < ncol; j++) {
                    if (pcount[i][j] > 0.5) {
                        this.fcdrBlockmean.fcdr_gridded_bt[i][j][k] = pmean[i][j];
                        // change variance to std
                        if(pstd[i][j]>=0) {
                            this.fcdrBlockmean.fcdr_gridded_bt_std[i][j][k] = (float)Math.sqrt((double)pstd[i][j]);                     
                        }else {
                            this.fcdrBlockmean.fcdr_gridded_bt_std[i][j][k] = 
                                AmsuFcdrGridConstant.FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE;                            
                        }
                    } else {
                        this.fcdrBlockmean.fcdr_gridded_bt[i][j][k] = 
                                AmsuFcdrGridConstant.FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE;
                        this.fcdrBlockmean.fcdr_gridded_bt_std[i][j][k] = 
                                AmsuFcdrGridConstant.FILLED_VALUE_INVALID_BRIGHTNESS_TEMPERATURE;
                    }
                }
            }
        }
    }
    
    
    /**
     * Create directory of daily gridded FCDR files. If keep intermediate data files, save in folder rootFCDRL1d/sateName/year
     * @return true if created folder correctly, false if failed to create sub-directories.
     */
    private boolean createFolderL3Daily() {
        boolean ret = true;

            // if keep intermediate data files, create sub-directories.
            String dirRoot = this.rootOut;
            String fullpath = null;
            if (dirRoot.endsWith(File.separator)) {
                fullpath = String.format("%s%s%s%04d%s", dirRoot, this.satId, File.separator, this.year, File.separator);
            } else {
                fullpath = String.format("%s%s%s%s%04d%s", dirRoot, File.separator, 
                        this.satId, File.separator, this.year, File.separator);
            }
                        
            File folder = new File(fullpath);
            if (folder.exists()) {
            } else {
                try {
                    folder.mkdirs();
                } catch (Exception ioe) {
                    ret = false;
                    String msg = String.format("Error in processing %s, %4d-%02d-%02d, Failed to create folder + %s",
                            this.satId, this.year, this.month, this.day, fullpath);
                    fcdrErrorLogger.error(msg);
               }
           }
  
        return ret;
    }

    /**
     * Get name of Daily gridded FCDR  file.
     *
     * @param dirRoot, root directory of FCDR daily data.
     * @param satId, ID (short name) of satellite.
     * @param orbitId, Id for orbit, 'A': ascending, 'D': descending.
     * @param year, year.
     * @param month, month.
     * @param day, day.
     * @param dateTag, date for file creation.
     * @return full path of file for the specified date.
     */
    public static String getFullpathL3Daily(String dirRoot, String satId, char orbitId, int year, int month, int day, Date dateTag){
        String ret = null;
        String orbitTag = null;
        
        // tag for orbit
        if(orbitId==AmsuFcdrGridConstant.ORBIT_ASCENDING){
            orbitTag = AmsuFcdrGridConstant.NAMETAG_ORBIT_ASCENDING;
        } else {
            orbitTag = AmsuFcdrGridConstant.NAMETAG_ORBIT_DESCENDING;            
        }
        
        // tag for file creation time
        String strDateFormat = "yyyyMMdd";
        DateFormat dateFormat = new SimpleDateFormat(strDateFormat);
        dateFormat.setTimeZone(TimeZone.getTimeZone( "UTC" ) );
        String fileTimeTag= String.format("%s", dateFormat.format(dateTag));   

        // name FCDR file: "NESDIS-STAR_FCDR-GRID_AMSU-A_V01R00_%s_D%4d%02d%02d_%s_c%s.nc";
        // NESDIS-STAR_FCDR-GRID_AMSU-A_V[Version]_[platform]_D[date]_[tag_orbit]_c[tag_file_creation_date].nc
        String fileShotName = String.format(AmsuFcdrGridConstant.NAMING_CONVENTION_FILE_GRIDDED_FCDR, satId, year, month, day, 
               orbitTag,fileTimeTag);
        
        if (dirRoot.endsWith(File.separator)) {
            ret = String.format("%s%s%s%04d%s%s", dirRoot, satId, File.separator, year, File.separator, fileShotName);
        } else {
            ret = String.format("%s%s%s%s%04d%s%s", dirRoot, File.separator, satId, File.separator, year, 
                    File.separator, fileShotName);
        }
        return ret;
    }
    
}
