/*  Robodoc Markup Header
*@***h* AMSU-A_FCDR_STAR/amsu.fcdr.gridded.AmsuFcdrGriddedTask.java
*
* NAME
*   amsu.fcdr.gridded.AmsuFcdrGriddedTask.java
*
* PURPOSE
*   This is the main class to start tasks for producing AMSU-A daily gridded FCDR. 
* 
* DESCRIPTION
*   This class is  used to read configuration file and start tasks to generate AMSU-A daily gridded FCDR.
* AUTHOR
*   Xianjun Hao, George Mason University 
*
* COPYRIGHT 
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*
* REVISION HISTORY
*   12/08/2015 Initial Version prepared for NCEI
*
*@***** 
 */
package amsu.fcdr.gridded;

import java.io.File;
import java.util.ArrayList;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;


/**
 * This is the main class to start tasks for producing AMSU-A daily gridded FCDR. Read configuration file and start tasks to
 * generate AMSU-A daily gridded FCDR.
 *
 * @author Xianjun Hao, George Mason University
 */
public class AmsuFcdrGriddedTask {    
    /**
     * Default number of thread.
     */
    final public static int DEFAULT_NUMBER_OF_THREAD = 1;
    
    /**
     *  Maximum size of the thread pool.
     */
    final public static int  MAXIMUM_SIZE_THREAD_POOL =   64; 

    /**
     * Information logger for FCDR processing.
     */
    final private static Logger  fcdrErrorLogger = LogManager.getLogger("ErrorLog");

    /**
     * Information logger for FCDR processing.
     */
    final private static Logger  fcdrInfoLogger = LogManager.getLogger("InfoLog");    
    
    /**
     * Name of task description file (in XML format).
     */
    private String taskInfoFile = null;

    /**
     * Root directory of AMSU-A FCDR level 1c data (input).
     */
    private String rootDirFcdrL1c = null;

    /**
     * Root directory of AMSU-A gridded FCDR data (output).
     */
    private String rootDirFcdrGridded = null;

    /**
     * Root directory of quality controlled daily FCDR level 1c data (temporary).
     */
    private String rootDirFcdrAligned = null;

    /**
     * Number of threads for generating AMSU-A daily gridded FCDR.
     */
    private int numberOfThreads = 4;

    /**
     * Number of tasks for generating AMSU-A daily gridded FCDR.
     */
    private int numberOfTasks = 0;

    /**
     * List of task Ids.
     */
    private ArrayList taskIdList = new ArrayList();

    /**
     * List of satellite names for tasks.
     */
    private ArrayList satList = new ArrayList();

    /**
     * Array of start year of tasks.
     */
    private int[] yearStart = null;

    /**
     * Array of start month of tasks.
     */
    private int[] monthStart = null;

    /**
     * Array of start day of tasks.
     */
    private int[] dayStart = null;

    /**
     * Array of end year of tasks.
     */
    private int[] yearEnd = null;

    /**
     * Array of end month of tasks.
     */
    private int[] monthEnd = null;

    /**
     * Array of end day of tasks.
     */
    private int[] dayEnd = null;

    /**
     * Flag to indicate if keep intermediate data files (the quality controlled and temporally aligned daily level 1c data). Set it
     * to true to use intermediate data for testing or other purpose.
     */
    private boolean flagKeepIntermediateData = false;

    /**
     * Flag to indicate if task is ready for processing.
     */
    private boolean flagTaskReady = false;
    
    /**
     *  Running mode.  If verbose is true, show more detailed running information.
     */
    private boolean verbose = false;

    /**
     * Create task according to task description file in XML format.
     *
     * @param fileTaskInfo, XML file to define tasks.
     */
    public AmsuFcdrGriddedTask(String fileTaskInfo) {
        this.taskInfoFile = fileTaskInfo;
        this.flagKeepIntermediateData = false;
        // parse the XML file to get task information
        parseTaskInfo();
        // if parse task description correctly, further check availability and permission of data folders.
        if (this.flagTaskReady) {
            if (this.checkRootDirFcdrL1c() && this.checkRootDirFcdrGridded() && this.checkRootDirFcdrAligned()) {
                this.flagTaskReady = true;
            } else {
                this.flagTaskReady = false;
            }
        }
    }

    /**
     * Check if tasks are setup correctly.
     *
     * @return true if tasks are ready to start, false if there are problems.
     */
    public boolean isTaskReady() {
        return this.flagTaskReady;
    }

    /**
     * Parse task description file to get details of tasks.
     */
    private void parseTaskInfo() {
        int numErrors = 0;     // number of errors in task specification
        int numWarnings = 0;    // number of warning in task specification

        try {
            File taskFile = new File(this.taskInfoFile);
            if (taskFile.exists() && taskFile.isFile()) {
                DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
                DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
                Document doc = dBuilder.parse(taskFile);
                doc.getDocumentElement().normalize();
                // get root directories of input, output and temporary data files.
                NodeList dirList = doc.getElementsByTagName("RootDirectory");
                if (dirList.getLength() > 0) {
                    Node dirNode = dirList.item(0);
                    if (dirNode.getNodeType() == Node.ELEMENT_NODE) {
                        Element dirElem = (Element) dirNode;
                        this.rootDirFcdrL1c = dirElem.getElementsByTagName("RootDirL1c").item(0).getTextContent().trim();
                        this.rootDirFcdrGridded = dirElem.getElementsByTagName("RootDirGridded").item(0).getTextContent().trim();
                        this.rootDirFcdrAligned = dirElem.getElementsByTagName("RootDirTemp").item(0).getTextContent().trim();
                    } else {
                        numErrors++;
                        fcdrErrorLogger.error("Fatal error, failed to get root directories from task description file, program stopped.");
                    }
                } else {
                    numErrors++;
                    fcdrErrorLogger.error("Fatal error, failed to get root directories from task description file, program stopped.");
                }

                // get number of Thread, if undefined, use default value.
                NodeList tList = doc.getElementsByTagName("Thread");
                if (tList.getLength() > 0) {
                    Node tNode = tList.item(0);
                    if (tNode.getNodeType() == Node.ELEMENT_NODE) {
                        Element tElem = (Element) tNode;
                        String str = tElem.getElementsByTagName("NumberOfThread").item(0).getTextContent();
                        try {
                            this.numberOfThreads = Integer.parseInt(str.trim());
                            if (this.numberOfThreads < 1) {
                                numWarnings++;
                                this.numberOfThreads = this.DEFAULT_NUMBER_OF_THREAD;
                                fcdrErrorLogger.error("Error in specifying number of thread, use the dGefault value: "
                                        + this.DEFAULT_NUMBER_OF_THREAD);
                            }
                        } catch (Exception ne) {
                            numWarnings++;
                            this.numberOfThreads = this.DEFAULT_NUMBER_OF_THREAD;
                            fcdrErrorLogger.error("Error in specifying number of thread, use the default value: "
                                    + this.DEFAULT_NUMBER_OF_THREAD);
                        }
                    } else {
                        numWarnings++;
                        this.numberOfThreads = this.DEFAULT_NUMBER_OF_THREAD;
                        fcdrErrorLogger.error("Error in specifying number of thread, use the default value: "
                                + this.DEFAULT_NUMBER_OF_THREAD);
                    }
                } else {
                    // if number of threads is not specified, use default value;
                    numWarnings++;
                    fcdrErrorLogger.error("Number of thread undefined, use the default value: "
                            + this.DEFAULT_NUMBER_OF_THREAD);
                    this.numberOfThreads = this.DEFAULT_NUMBER_OF_THREAD;
                }              
                // get running mode
                NodeList rList = doc.getElementsByTagName("RunningMode");
                if (rList.getLength()>0) {
                    Node rNode = rList.item(0);
                    if (rNode.getNodeType() == Node.ELEMENT_NODE) {
                        Element rElem = (Element) rNode;
                        String str = rElem.getElementsByTagName("Verbose").item(0).getTextContent();
                        try {
                            int runMode = Integer.parseInt(str.trim());
                            if (runMode ==1) {
                                this.verbose = true;
                            } else {
                                this.verbose = false;
                            }
                        } catch (Exception e) {
                           fcdrErrorLogger.warn("Warning: Invalid setting of Verbose "
                                   + "in task description, use default mode for running.");
                           e.printStackTrace();
                           this.verbose = false; 
                        }
                    } else {
                        this.verbose = false;                        
                    }
                } else {
                    this.verbose = false;
                }

                // get tasks
                NodeList taskList = doc.getElementsByTagName("Task");
                int numTask = taskList.getLength();
                if (numTask < 1) {
                    // no task defined, skip 
                    fcdrInfoLogger.info("No task was specified, program stopped.");
                    numErrors++;
                } else {
                    // initialize arrays
                    this.yearStart = new int[numTask];
                    this.monthStart = new int[numTask];
                    this.dayStart = new int[numTask];
                    this.yearEnd = new int[numTask];
                    this.monthEnd = new int[numTask];
                    this.dayEnd = new int[numTask];
                    // add tasks to list
                    this.numberOfTasks = numTask;
                    for (int k = 0; k < numTask; k++) {
                        Node taskNode = taskList.item(k);
                        if (taskNode.getNodeType() == Node.ELEMENT_NODE) {
                            Element taskElem = (Element) taskNode;
                            // Task Id
                            this.taskIdList.add(taskElem.getAttribute("Id").trim());
                            // Satellite Name
                            this.satList.add(taskElem.getElementsByTagName("SatelliteName").item(0).getTextContent().trim());
                            // start and end date
                            String textStart = taskElem.getElementsByTagName("TaskStartDate").item(0).getTextContent().trim();
                            int[] dateArr = this.parseDate(textStart);
                            if (dateArr[0] < 0) { // error
                                numErrors++;
                                fcdrErrorLogger.error("Fatal error in specifying the start date of task " + taskElem.getAttribute("Id"));
                            } else {
                                this.yearStart[k] = dateArr[0];
                                this.monthStart[k] = dateArr[1];
                                this.dayStart[k] = dateArr[2];
                            }
                            String textEnd = taskElem.getElementsByTagName("TaskEndDate").item(0).getTextContent().trim();
                            dateArr = this.parseDate(textEnd);
                            if (dateArr[0] < 0) { // error
                                numErrors++;
                                fcdrErrorLogger.error("Fatal error in specifying the end date of task " + taskElem.getAttribute("Id"));
                            } else {
                                this.yearEnd[k] = dateArr[0];
                                this.monthEnd[k] = dateArr[1];
                                this.dayEnd[k] = dateArr[2];
                            }
                        } else {
                            numErrors++;
                        }
                    }
                }
            } else {
                fcdrErrorLogger.error("Fatal error, can't read task descrption file, program stopped.");
                numErrors++;
            }             
        } catch (Exception e) {
            // fatal error, can't start tasks  
            fcdrErrorLogger.error("Fatal error in reading task descrption file, program stopped.");
            numErrors++;
        }

        if (numErrors > 0) {
            this.flagTaskReady = false;
            fcdrErrorLogger.error("Fatal errors in task descrption file, program stopped.");
        } else {
            this.flagTaskReady = true;
        }

    }

    /**
     * Get root directory of AMSU-A FCDR level 1c data.
     *
     * @return the absolute path of the root directory of AMSU-A FCDR level 1c data.
     */
    public String getRootDirFcdrL1c() {
        return this.rootDirFcdrL1c;
    }

    /**
     * Get root directory of AMSU-A gridded FCDR data.
     *
     * @return the absolute path of the root directory of AMSU-A gridded FCDR data.
     */
    public String getRootDirFcdrGridded() {
        return this.rootDirFcdrGridded;
    }

    /**
     * Get root directory of quality controlled daily FCDR level 1c data (temporary).
     *
     * @return the absolute path of the root directory of quality controlled daily FCDR level 1c data (temporary).
     */
    public String getRootFcdrAligned() {
        return this.rootDirFcdrAligned;
    }

    /**
     * Check if the root directory of AMSU-A level 1c data is available.
     *
     * @return true if the root directory of AMSU-A level 1c data is available, otherwise false.
     */
    public boolean checkRootDirFcdrL1c() {
        File rootDir = new File(this.rootDirFcdrL1c);
        if (rootDir.exists() && rootDir.isDirectory()) {
            if (rootDir.canRead()) {
                return true;
            } else {
                fcdrErrorLogger.error("Fatal error, the root directory of AMSU-A FCDR l1c data is not readable, task stopped.");
                return false;
            }
        } else {
            fcdrErrorLogger.error("Fatal error, the root directory of AMSU-A FCDR l1c data doesn't exist, task stopped.");
            return false;
        }
    }

    /**
     * Check if the root directory of AMSU-A gridded FCDR is available, if not, create it.
     *
     * @return true if the root directory of AMSU-A level 1c data is available, otherwise false.
     */
    public boolean checkRootDirFcdrGridded() {
        File rootDir = new File(this.rootDirFcdrGridded);
        if (rootDir.exists() && rootDir.isDirectory()) {
            if (rootDir.canWrite()) {
                return true;
            } else {
                fcdrErrorLogger.error("Fatal error, the root directory of AMSU-A gridded FCDR data is not writable, task stopped.");
                return false;
            }
        } else {
            try {
                if (rootDir.mkdirs()) {
                    return true;
                } else {
                    fcdrErrorLogger.error("Fatal error, failed to create the root directory of AMSU-A gridded FCDR data, task stopped.");
                    return false;
                }
            } catch (Exception e) {
                fcdrErrorLogger.error("Fatal error, failed to create the root directory of AMSU-A gridded FCDR data, task stopped.");
                return false;
            }
        }
    }

    /**
     * Check if the root directory of AMSU-A daily aligned FCDR is available, if not, create it.
     *
     * @return true if the root directory of AMSU-A daily aligned FCDR is available, otherwise false.
     */
    public boolean checkRootDirFcdrAligned() {
        File rootDir = new File(this.rootDirFcdrAligned);
        if (rootDir.exists() && rootDir.isDirectory()) {
            if (rootDir.canWrite()) {
                return true;
            } else {
                fcdrErrorLogger.error("Fatal error, the root directory of aligned daily data is not writable, task stopped.");
                return false;
            }
        } else {
            try {
                if (rootDir.mkdirs()) {
                    return true;
                } else {
                    fcdrErrorLogger.error("Fatal error, failed to create the root directory of aligned FCDR data, task stopped.");
                    return false;
                }
            } catch (Exception e) {
                fcdrErrorLogger.error("Fatal error, failed to create the root directory of aligned FCDR data, task stopped.");
                return false;
            }
        }
    }

    /**
     * Parse data string in format yyyy-mm-dd to int array with year, month and day.
     *
     * @param str, date string in format yyyy-mm-dd.
     * @return array with year, month and day.
     */
    final private static int[] parseDate(String str) {
        int numTokens = 3;
        int[] dateArr = new int[numTokens];

        int k = 0;
        dateArr[0] = -1;

        String[] tokens = str.split("-");
        if (tokens.length == numTokens) {
            try {
                for (k = 0; k < numTokens; k++) {
                    dateArr[k] = Integer.parseInt(tokens[k]);
                }
            } catch (Exception e) {
                dateArr[0] = -1;
            }
        }
        // check if month data is correct
        if ((dateArr[1] < 1) || (dateArr[1] > 12)) {
            dateArr[0] = -1;
        }
        // simple check for day
        if ((dateArr[2] < 1) || (dateArr[2] > 31)) {
            dateArr[0] = -1;
        }
        return dateArr;
    }

    /**
     * Show task information from the description file.
     */
    public void showTaskInfo() {
        System.out.println("****************************************************************************************");
        System.out.println("Task description:");
        System.out.println("Root directory of FCDR L1c data files (input):" + this.rootDirFcdrL1c);
        System.out.println("Root directory of gridded FCDR data files (output):" + this.rootDirFcdrGridded);
        System.out.println("Root directory of temporary data files (temp):" + this.rootDirFcdrAligned);
        System.out.println("Number of threads:" + this.numberOfThreads);
        System.out.println("Number of tasks:" + this.numberOfTasks);
        for (int k = 0; k < this.numberOfTasks; k++) {
            System.out.println("--> Task " + this.taskIdList.get(k).toString());
            System.out.println("        Satellite Name:" + this.satList.get(k).toString());
            String str
                    = String.format("        Start Date: %1$04d-%2$02d-%3$02d", this.yearStart[k], this.monthStart[k], this.dayStart[k]);
            System.out.println(str);
            str = String.format("        Start Date: %1$04d-%2$02d-%3$02d", this.yearEnd[k], this.monthEnd[k], this.dayEnd[k]);
            System.out.println(str);
        }
        System.out.println("********************************************************************");
    }

    /**
     * Get flag to indicate if keep intermediate data ( quality controlled and temporally aligned daily level 1c data).
     *
     * @return true if keep intermediate data, false if ignore intermediate data.
     */
    public boolean getFlagKeepIntermediateData() {
        return this.flagKeepIntermediateData;
    }

    /**
     * Set the flag for intermediate data to specified value. True if keep the quality controlled and temporally aligned daily level
     * 1c data files for testing or other purpose, false if delete them after gridding.
     *
     * @param flag, the flag to indicate if keep intermediate data.
     */
    public void setFlagKeepIntermediateData(boolean flag) {
        this.flagKeepIntermediateData = flag;
    }

    /**
     * process tasks to generate AMSU-A daily gridded FCDR using thread pool.
     */
    public void processTasks() {
        int year, month, day;
        long timeStart = 0;
        long timeEnd = 0;
        int year1 = 0;
        int month1 = 0;
        int day1 = 0;
        int year2 = 0;
        int month2 = 0;
        int day2 = 0;
        int timeStep = 24 * 3600;
        int[] tmv;
        String satName = null;
        String logFile = null;
        
        if(this.numberOfTasks<1){
            return;
        }
                
        int k = 0;
        TimeUtility tu = new TimeUtility();
        for (k = 0; k < this.numberOfTasks; k++) {
            ExecutorService executor = Executors.newFixedThreadPool(this.numberOfThreads);
            // get start and and date of task
            year1 = this.yearStart[k];
            month1 = this.monthStart[k];
            day1 = this.dayStart[k];
            year2 = this.yearEnd[k];
            month2 = this.monthEnd[k];
            day2 = this.dayEnd[k];            
            timeStart = TimeUtility.getStartSecondSince78(year1, month1, day1) + 10;
            timeEnd = TimeUtility.getStartSecondSince78(year2, month2, day2) + 10;            
            
            for (long tm = timeStart; tm <= timeEnd; tm += timeStep) {
               tmv = tu.getTimeVec(tm);
               year = tmv[0];
               month = tmv[1];
               day = tmv[2];
               satName = (String)this.satList.get(k);               
               AmsuFcdrDailyProcessor taskProcessor = new AmsuFcdrDailyProcessor(satName, year, month, day, 
                this.rootDirFcdrL1c, this.rootDirFcdrAligned, this.rootDirFcdrGridded, this.flagKeepIntermediateData, this.verbose);
               executor.execute(taskProcessor);
            }
                        
            try {
                executor.shutdown();
                executor.awaitTermination(60, TimeUnit.SECONDS);                
            } catch (Exception e) {
                e.printStackTrace();
            } 
        }        
    }

////////////////////////////////////////////////////////////////////////////////////////////////////////
    /**
     * Main program.
     *
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        AmsuFcdrGriddedTask app = null;
        
        // check task desciption file
        if (args.length > 0) {
            app = new AmsuFcdrGriddedTask(args[0]);
        } else {
            app = new AmsuFcdrGriddedTask("./task.xml");
        }
        app.setFlagKeepIntermediateData(false);
        app.showTaskInfo();

        // run tasks if ready
        if (app.isTaskReady()) {
            // System.out.println("Task is ready to start");
            app.processTasks();            
        } else {
            fcdrErrorLogger.error("Fatal errors in task description, unable to run.");
        }
    }

}
