/*  Robodoc Markup Header
*@***h* AMSU-A_FCDR_STAR/amsu.fcdr.gridded.AmsuFcdrL1cIndex.java
*
* NAME
*   amsu.fcdr.gridded.AmsuFcdrL1cIndex.java
*
* PURPOSE
*   Set flag and index for each scan in level-1c files in quality control and temporal alignment. 
* 
* DESCRIPTION
*	This class are used to set flag and index for each scan in the quality control and temporal alignment step 
*   in generating gridded AMSU-A brightness temperature FCDR. There are 10800 time splots each day 
*   ( 8 seconds each, for 24 hours ). If a scan failed to pass quality control, the flag will be set to false. 
*   If a scan passes quality control, and is selected in generating gridded data, the index will be calculated based on scan time.
*   If a scan passes quality control, but the corresponding time slot has been assigned because of repetition, 
*   the flag will be set to false.
*
* AUTHOR
*   Xianjun Hao, George Mason University 
*
* COPYRIGHT 
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*
* REVISION HISTORY
*   12/08/2015 Initial Version prepared for NCEI
*
*@***** 
 */
package amsu.fcdr.gridded;

/**
 * This class are used to set flag and index for each scan in the quality
 * control and temporal alignment step in generating gridded AMSU-A brightness
 * temperature FCDR. There are 10800 time splots each day ( 8 seconds each, for
 * 24 hours ). If a scan failed to pass quality control, the flag will be set to
 * false. If a scan passes quality control, and is selected in generating
 * gridded data, the index will be calculated based on scan time. If a scan
 * passes quality control, but the corresponding time slot has been assigned
 * because of repetition, the flag will be set to false.
 *
 * @author Xianjun Hao (xianjun.hao@noaa.gov)
 */
public class AmsuFcdrL1cIndex {

    /**
     * Number of scans in an AMSU-A level-1c FCDR file
     */
    private int numScan = 0;

    /**
     * Index for each scan in temporal alignment. It is a position in time
     * splots (0-10799).
     */
    private int[] index = null;

    /**
     * Flag to each scan for quality control and repetition removement. If keep
     * a scan, set its flag to true; otherwie, set its flag to false.
     */
    private boolean[] flag = null;

    /**
     * Constructor. Create an instance with specified number of scans.
     *
     * @param numScan, number of scans.
     *
     */
    public AmsuFcdrL1cIndex(int numScan) {
        this.numScan = numScan;
        if (numScan > 0) {
            this.setNumScan(numScan);
        }
    }

    /**
     * Get number of scans.
     *
     * @return number of scans.
     */
    public int getNumScan() {
        return this.numScan;
    }

    /**
     * set number of scans.
     *
     * @param numScan, number of scans.
     */
    public void setNumScan(int numScan) {
        this.numScan = numScan;
        this.index = new int[numScan];
        this.flag = new boolean[numScan];
    }

    /**
     * Set flag at index k.
     *
     * @param k, index.
     * @param val, value.
     */
    public void setFlag(int k, boolean val) {
        this.flag[k] = val;
    }

    /**
     * Get flag at index k.
     *
     * @param k, array index.
     * @return valid flag at index k.
     */
    public boolean getFlag(int k) {
        return this.flag[k];
    }

    /**
     * Set flags to all elements.
     *
     * @param flag, flag array.
     */
    public void setFlag(boolean[] flag) {
        int k = 0;
        for (k = 0; k < this.numScan; k++) {
            this.flag[k] = flag[k];
        }
    }

    /**
     * Get all flags.
     *
     * @return flag array.
     */
    public boolean[] getFlag() {
        return this.flag;
    }

    /**
     * Set index of scan in daily FCDR L1c file.
     *
     * @param ind, index of each scan, from 0 to 10799 for valid scans. -1 for
     * invalid scans.
     */
    public void setIndex(int[] ind) {
        int k = 0;
        for (k = 0; k < numScan; k++) {
            this.index[k] = ind[k];
        }
    }

    /**
     * Set index of scan in daily FCDR L1c file.
     *
     * @param k, scan No.
     * @param ind, index of each scan, from 0 to 10799 for valid scans. -1 for
     * invalid scans.
     */
    public void setIndex(int k, int ind) {
        this.index[k] = ind;
    }

    /**
     * Get index of scans.
     *
     * @return index of each scan, from 0 to 10799 for valid scans. -1 for
     * invalid scans.
     */
    public int[] getIndex() {
        return this.index;
    }

}
