/*  Robodoc Markup Header
*@***h* AMSU-A_FCDR_STAR/amsu.fcdr.gridded.TimeUtility.java
*
* NAME
*   amsu.fcdr.gridded.TimeUtility.java
*
* PURPOSE
*   Utility class to handle date and time in generating gridded AMSU-A brightness temperature FCDR.
* 
* DESCRIPTION
*   This class provides tools to 
*     convert time in format of seconds since 1978 to a vector of year, month, day, hour, minute, and seconds;
*     get previous day of the specified year, month and day;
*     get next day of the specified year, month and day;
*     convert calendar day to Julian day;
*     convert Julian day to calendar day.
*
* AUTHOR
*   Xianjun Hao, George Mason University 
*
* COPYRIGHT 
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*
* REVISION HISTORY
*   12/08/2015 Initial Version prepared for NCEI
*
*@***** 
*/

package amsu.fcdr.gridded;

import java.util.Date;
import java.time.Instant;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoField;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * Utility class to process date and time.
 *   This class provides tools to 
 *     convert time in format of seconds since 1978 to a vector of year, month, day, hour, minute, and seconds;
 *     get previous day of the specified year, month and day;
 *     get next day of the specified year, month and day;
 *     convert calendar day to Julian day;
 *     convert Julian day to calendar day.
 * 
 * @author Xianjun Hao, George Mason University (xianjun.hao@noaa.gov)
*/
public class TimeUtility {
    /**
     * An static instance of Date.
     */	 
    final private static Date date=new Date();
	
	/**
	 * An Instant instance for the time  1978-01-01 T00:00:00Z.
	 */
    private Instant date78 = null;         

	/**
	 * Seconds of "1978-01-01 00:00:00Z" since 1970-01-01 00:00:00Z. 
	 */
    private long offset_in_seconds = 0;    // seconds of "1978-01-01 00:00:00Z" since 1970.
    
    /**
     * Construct an instance of TimeUnitity.
     */
    public TimeUtility(){
        DateTimeFormatter formatter = DateTimeFormatter.ISO_INSTANT;
        try {
            this.date78 = Instant.parse("1978-01-01T00:00:00Z");
            this.offset_in_seconds=this.date78.getEpochSecond();
        } catch (Exception e){
            e.printStackTrace();
            this.date78=null;
            this.offset_in_seconds=0;
        }
    }
                
    /**
     * Convert time in format of seconds since 1978 to a vector of year, month, day, hour, minute, and seconds.
     * @param secondsSince78, time in seconds since 1978-01-01 T00:00:00Z.
     * @return Time vector of year, month, day, hour, minute, and seconds.
     */
    public int[] getTimeVec(long secondsSince78){
        Instant ins = this.date78.plusSeconds(secondsSince78);
        LocalDateTime tm = LocalDateTime.ofInstant(ins, ZoneId.of("UTC"));
        int[] tmv = new int[6];
        tmv[0]=tm.get(ChronoField.YEAR);
        tmv[1]=tm.get(ChronoField.MONTH_OF_YEAR);
        tmv[2]=tm.get(ChronoField.DAY_OF_MONTH);
        tmv[3]=tm.get(ChronoField.HOUR_OF_DAY);
        tmv[4]=tm.get(ChronoField.MINUTE_OF_HOUR);
        tmv[5]=tm.get(ChronoField.SECOND_OF_MINUTE);
        return tmv;
    }
    
/**
 * Get the previous day of a calendar day specified with year, month and day.
 * @param year, year.
 * @param month, month.
 * @param day, day.
 * @return previous day specified with a vector of year, month and day.
 */
    public static int[] getPreviousDay(int year, int month, int day){
        int[] tmv = new int[3];
        //GregorianCalendar, month starts with 0
        Calendar cal = new GregorianCalendar(year, month-1, day);
        System.out.println("DOY:"+cal.get(Calendar.DAY_OF_YEAR));
        cal.add(Calendar.DAY_OF_YEAR,-1);
        tmv[0]=cal.get(Calendar.YEAR);
        tmv[1]=cal.get(Calendar.MONTH)+1;      // Calendar, month starts with 0, need to add 1 to correct it
        tmv[2]=cal.get(Calendar.DAY_OF_MONTH);        
        return tmv;
    }

/**
 * Get the next day of a calendar day specified with year, month and day.
 * @param year, year.
 * @param month, month.
 * @param day, day.
 * @return next day specified with a vector of year, month and day.
 */
    public static int[] getNextDay(int year, int month, int day){
        int[] tmv = new int[3];
        //GregorianCalendar, be careful that month starts with 0 in GregorianCalendar.
        Calendar cal = new GregorianCalendar(year, month-1, day);
        cal.add(Calendar.DAY_OF_YEAR,1);
        tmv[0]=cal.get(Calendar.YEAR);
		//  Month starts with 0 GregorianCalendar, need to add 1 to correct it for general use.
        tmv[1]=cal.get(Calendar.MONTH)+1;      
        tmv[2]=cal.get(Calendar.DAY_OF_MONTH);        
        return tmv;
    }
    
    /**
     * Convert calendar day to Julian day.
     * @param year, year.   
     * @param month, month.
     * @param day, day.
     * @return, Julian day of the year.
     */
    public static int getDayofYear(int year, int month, int day){
        int jday =0;
        Calendar cal = new GregorianCalendar(year, month-1, day);
        jday = cal.get(Calendar.DAY_OF_YEAR);        
        return jday;
    }
    
    /**
     * Convert Julian day to calendar day.
     * @param year, year.
     * @param jday, Julian day.
     * @return vector for calendar day.
     */
    public static int[] getCalDay(int year, int jday){
        int[] tmv=new int[3];
        Calendar cal = new GregorianCalendar(year, 0, 1);
        cal.add(Calendar.DAY_OF_YEAR,jday-1);
        tmv[0]=cal.get(Calendar.YEAR);
		// Month starts with 0 GregorianCalendar, need to add 1 to correct it for general use.
        tmv[1]=cal.get(Calendar.MONTH)+1;      
        tmv[2]=cal.get(Calendar.DAY_OF_MONTH);        
       
        return tmv;
    }
    
    /**
     * Get the start time of a day in seconds since 1978-01-01 T00:00:00Z.
     * @param year, year.
     * @param month, month.
     * @param day, day.
     * @return seconds starting from 01/01/1978.
     */
    public static long getStartSecondSince78(int year, int month, int day){
        long ret = 0;
        ZoneId zone;
        zone = ZoneId.of("UTC");
        ZonedDateTime zdt  = ZonedDateTime.of(year, month, day, 0, 0, 0, 0, zone);
        ZonedDateTime zdt0 = ZonedDateTime.of(1978, 1, 1, 0, 0, 0, 0, zone);
        ret = zdt.toEpochSecond() - zdt0.toEpochSecond();
        return ret;
    }
    
}
