/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.AMSUAAntennaPatternCorrection.java
* NAME
*   amsu.amsua.AMSUAAntennaPatternCorrection.java 
* 
* PURPOSE
*   To correct for the AMSU-A antenna pattern effect
* 
* DESCRIPTION
*   This class provides methods for loading antenna pattern correction coefficients 
*   from text file, and converting antenna temperature to brightness temperatures.
*   Coefficients were obtained from Dr. Tsan Mo at NOAA/NESDIS/STAR. 
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   10/30/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial version prepared for NCDC.
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
package amsu.amsua;


import java.io.*;
import java.util.*;
import pub.QualityControl;


public class AMSUAAntennaPatternCorrection implements AMSUA{
    static float[][][][] coeffsAntennaCorrection=new float[5][NUM_CHANNELS][NUM_SPOTS_PER_LINE][3];
    static float[][] etas=new float[5][NUM_CHANNELS];
    static int[] arrayIndex={-1,-1,-1,-1,-1,-1,-1,-1,-1,0,1,2,3,4,-1};
    static final int[] indexIntruments={2,2,1,1,1,0,0,1,0,0,0,0,0,0,0};
    
    public AMSUAAntennaPatternCorrection(){
        
    }
    
    static public int init(String dirCoefficients){
        int status=0;
        String[] filenames={"NOAAK_Ant_coeff.txt","NOAAL_Ant_coeff.txt",
                            "NOAAM_Ant_coeff.txt","NOAAN_Ant_coeff.txt",
                            "METOPA_Ant_coeff.txt"};
        String pathname;
        int instrument,channel,spot,i;
        for (instrument=0;instrument<5;instrument++){
            pathname=dirCoefficients+"/"+filenames[instrument];
            try{
                Scanner sc= new Scanner(new DataInputStream(new FileInputStream(pathname)));
               
                for (channel=0;channel<NUM_CHANNELS;channel++){
                    for (spot=0;spot<NUM_SPOTS_PER_LINE;spot++){            
                        for (i=0;i<3;i++){ 
                            float value=sc.nextFloat();
                            coeffsAntennaCorrection[instrument][channel][spot][i]=value;  
                        }
                    }
                }
                for (i=0;i<NUM_CHANNELS;i++){
                    etas[instrument][i]=sc.nextFloat();
                }
                    
                sc.close();  
            }catch (IOException e){
                status=-1;
                break;
            }
        }        
        
        return status;
    }

    static public int antennaTemperature2BrightnessTemperature(
            int internalSatelliteID,float[][] at,float[][]bt,float[] tempIntruments){
        //antennaCorrectionMethod-   >1 <=14  sensitivity mode, perform correction for one channel, for IMICA sensivity 
        //                           other values (except 0) : normal correction
        float tempIntrument;
        int i,j;
        float[] f;
        float[] eta;
        float neta;
        int index=arrayIndex[internalSatelliteID]; 
        
        //index=0; //for compare with ERAInterim obsvalue
        if (index<0 || index>4)
            return -1;
        
        eta=etas[index];      
        float bt0,temp;
        for (i=0;i<NUM_CHANNELS;i++){ 
            tempIntrument=tempIntruments[indexIntruments[i]];
            for (j=0;j<NUM_SPOTS_PER_LINE;j++){
                temp=at[i][j];
                if (QualityControl.isValidTemperature(temp)){
                    f=coeffsAntennaCorrection[index][i][j];
                    neta=(f[0]+eta[i]*f[1]+f[2]);    
                    bt[i][j]=(temp*neta-eta[i]*f[1]*tempIntrument-f[2]*2.73f)/(f[0]);
                } else{
                    bt[i][j]=QualityControl.missing_value;
                }
            }
        }
        return 0;
    }
    
    static public int antennaTemperature2BrightnessTemperature_SensitivityStudy(
            int internalSatelliteID,float[][] at,float[][]bt,float[] tempIntruments,
            int channelSensitivityStudy,int nPara){
        float tempIntrument;
        int i,j;
        float[] f;
        float[] eta;
        float neta;
        int index=arrayIndex[internalSatelliteID];
        if (index<0 || index>4)
            return -1;
        
        eta=etas[index];
        tempIntrument=tempIntruments[indexIntruments[channelSensitivityStudy]];
        for (i=channelSensitivityStudy;i<=channelSensitivityStudy/*NUM_CHANNELS*/;i++){            
            for(int indexRow=0;indexRow<nPara;indexRow++){
              for (j=0;j<NUM_SPOTS_PER_LINE;j++){
                f=coeffsAntennaCorrection[index][i][j];
                neta=(f[0]+eta[i]*f[1]+f[2]);    
                bt[indexRow][j]=(at[indexRow][j]*neta-eta[i]*f[1]*tempIntrument-f[2]*2.73f)/(f[0]);
              }
            }
        }
        return 0;
    }
}
