/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1B.AMSUALevel1BFile.java
* NAME
*   amsu.amsua.L1B.AMSUALevel1BFile.java 
* 
* PURPOSE
*   To provide a common interface to AMSUA leve1 1B file in different formats.
* 
* DESCRIPTION
*   This class is the super class for AMSUALevel1BFileNOAA and AMSUALevel1BFileAqua
*   NOAA and NASA EOS Aqua AMSU-A level 1b files are in two different formats:
*   (1) NOAA binary fromat
*   (2) NASA HDF format
*   This class hides the differences between the two formats and provides 
*   a common interface to the level 1c code.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   01/03/2012 
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial version prepared for NCDC
*   05/14/2013 - added coeffsIMICACalibration and coeffsLinearCalibration
*
*@*****/
 
 
package amsu.amsua.L1B;

import amsu.amsua.*;
import amsu.amsua.L1B.header.AMSUALevel1BHeader;
import io.*;
import pub.L1C.QualityControlL1C;
import util.*;

public abstract class AMSUALevel1BFile implements AMSUA {
    public QualityControlL1C[] qc; //quality flags
    
    public int[] scanlineNumber;   //scan line number
    public Time[] time;            //start time of each scan line
    public float[][] lat,lon;      //latitude, longitude
    public short[][][] xy;         //index of 2.5 x 2.5 degree grid cell
    public byte[][] landcovertype; //land type
    public float[][] landfrac;     //land fraction
    
    public float[][] solarZenith,sensorZenith,localAzimuth;  //geometry info
    public short[] spacecraftAltitude;   //spacecraft altitude
    public float[][] spacecraftAttitude; //spacecraft attitude
    
    public float[][][] countsScene;      //scene raw counts
    public int[][][] countsColdCal,      //cold space raw counts before averaging
                     countsWarmCal;      //warm target raw counts before averaging
    public float[][] countsColdCalAvg,   //averaged cold space counts
                     countsWarmCalAvg;   //averaged warm target raw counts
    public float[][] countsColdCalAvgConvolved, //smoothed averaged cold space counts
                     countsWarmCalAvgConvolved; //smoothed averaged warm target raw counts

    //these variable are for exploring only only
    public int[][] countsPRTsA1,countsPRTsA2; 
    public float[][] temperaturesPRTsA1,temperaturesPRTsA2,temperatureInstruments;

    
    public double[][][] coeffsOperationalCalibrationPrimary;  //data buffer for storing calibration coefficients from primary operational calibration coefficients
    public double[][][] coeffsOperationalCalibrationSecondary; //data buffer for storing calibration coefficients  from secondary operational calibration coefficients
    public double[][][] coeffsIMICACalibration;                //data buffer for storing calibration coefficients of IMICA calibration
    public double[][][] coeffsLinearCalibration;               //data buffer for storing calibration coefficients of linear calibration
    
    public float[][] avgWarmTargetTemp;// averaged warm target temperatures; //0-A1-1 1- A1-2 2- A2
    public int[][] nWarmTargetPRTUsed; // warm target PRT used for each antenna system 0-A1-1 1- A1-2 2- A2;    
    public float[][] wtTemperatureWarmLoadCorrected;// bias correction warm target temperatures;
    public float[][] wtConvertedFromRawCounts;
    
    protected float[][] warmTargetRadiance;  //warm target radiances
    protected float[][] coldSpaceRadiance;   //cold space radiances 
    protected float[][][] sceneRadiances;    //scene radiance for scanline,channel,spot

    ////////////
    //explore house keeping data
    public static boolean bReadHouseKeeping=true;
    public float[][] houseKeepingAMSUA1;      //used to store 8bits unsigned integer
    public int[][] houseKeepingAMSUA1QC;
    
    protected float[][][] at;    //antenna temperatures
    protected float[][][] bt;    //brightness temperatures    
    
    public short[][] flagsCalibrationQuality; //quality of raw counts
    
    
    public int nScans;      //number of scans
    public int satelliteID; //NOAA satellite ID
            
    public AMSUALevel1BHeader header=null; //initialize in subclass
    public AMSUALevel1BLine line=null;//initialize in subclass

    /**
     * Constructor 
     */
    public AMSUALevel1BFile(){
        qc=new QualityControlL1C[NUM_SCANLINE_MAX];
        
        
        scanlineNumber=new int[NUM_SCANLINE_MAX];
        time=new Time[NUM_SCANLINE_MAX];
        for (int i=0;i<NUM_SCANLINE_MAX;i++){
            time[i]=new Time();
            qc[i]=new QualityControlL1C(NUM_CHANNELS,NUM_SPOTS_PER_LINE);
        }
        lat=new float[NUM_SCANLINE_MAX][NUM_SPOTS_PER_LINE];
        lon=new float[NUM_SCANLINE_MAX][NUM_SPOTS_PER_LINE];
        xy=new short[NUM_SCANLINE_MAX][NUM_SPOTS_PER_LINE][2];
        landcovertype=new byte[NUM_SCANLINE_MAX][NUM_SPOTS_PER_LINE];
        landfrac=new float[NUM_SCANLINE_MAX][NUM_SPOTS_PER_LINE]; //for precipitation
        
        solarZenith=new float[NUM_SCANLINE_MAX][ NUM_SPOTS_PER_LINE];
        sensorZenith=new float[NUM_SCANLINE_MAX][ NUM_SPOTS_PER_LINE];
        localAzimuth=new float[NUM_SCANLINE_MAX][ NUM_SPOTS_PER_LINE];
        spacecraftAltitude=new short[NUM_SCANLINE_MAX];
        spacecraftAttitude=new float[NUM_SCANLINE_MAX][3];
        
        //rowcounts
        countsScene=new float[NUM_SCANLINE_MAX][NUM_CHANNELS][ NUM_SPOTS_PER_LINE];
        countsColdCal=new int[NUM_SCANLINE_MAX][NUM_CHANNELS][2];
        countsWarmCal=new int[NUM_SCANLINE_MAX][NUM_CHANNELS][2];
        countsColdCalAvg=new float[NUM_SCANLINE_MAX][NUM_CHANNELS];
        countsWarmCalAvg=new float[NUM_SCANLINE_MAX][NUM_CHANNELS];        
        countsColdCalAvgConvolved=new float[NUM_SCANLINE_MAX][NUM_CHANNELS];
        countsWarmCalAvgConvolved=new float[NUM_SCANLINE_MAX][NUM_CHANNELS];        
        
        countsPRTsA1=new int[NUM_SCANLINE_MAX][NUM_PRTS_A1];
        countsPRTsA2=new int[NUM_SCANLINE_MAX][NUM_PRTS_A2];

        temperaturesPRTsA1=new float[NUM_SCANLINE_MAX][NUM_PRTS_A1];
        temperaturesPRTsA2=new float[NUM_SCANLINE_MAX][NUM_PRTS_A2];        
        temperatureInstruments=new float[NUM_SCANLINE_MAX][3];
        
        warmTargetRadiance=new float[NUM_SCANLINE_MAX][NUM_CHANNELS];
        coldSpaceRadiance=new float[NUM_SCANLINE_MAX][NUM_CHANNELS];    
        sceneRadiances=new float[NUM_SCANLINE_MAX][NUM_CHANNELS][NUM_SPOTS_PER_LINE];
        
        at=new float[NUM_SCANLINE_MAX][NUM_CHANNELS][NUM_SPOTS_PER_LINE];
        bt=new float[NUM_SCANLINE_MAX][NUM_CHANNELS][NUM_SPOTS_PER_LINE];
        
        coeffsOperationalCalibrationPrimary=new double[NUM_SCANLINE_MAX][NUM_CHANNELS][3];
        coeffsOperationalCalibrationSecondary=new double[NUM_SCANLINE_MAX][NUM_CHANNELS][3];
        coeffsIMICACalibration=new double[NUM_SCANLINE_MAX][NUM_CHANNELS][3];
        coeffsLinearCalibration=new double[NUM_SCANLINE_MAX][NUM_CHANNELS][3];
        
        
        flagsCalibrationQuality=new short[NUM_SCANLINE_MAX][NUM_CHANNELS];
        
        avgWarmTargetTemp = new float[NUM_SCANLINE_MAX][3]; //0-A1-1 1- A1-2 2- A2
        nWarmTargetPRTUsed = new int[NUM_SCANLINE_MAX][3];    
        wtTemperatureWarmLoadCorrected = new float[NUM_SCANLINE_MAX][NUM_CHANNELS];
        wtConvertedFromRawCounts=new float[NUM_SCANLINE_MAX][NUM_CHANNELS];
        
        if (bReadHouseKeeping){
            houseKeepingAMSUA1=new float[NUM_SCANLINE_MAX][28];
            houseKeepingAMSUA1QC=new int[NUM_SCANLINE_MAX][1];
  
        }
    }

    /**
     * Get name of the level 1b file
     * @return  name of the level 1b file
     */
    public abstract String getName();
    
    /**
     * Common interface for open level 1b file
     * @param pathname - AMSU-A level 1b file name
     * @return 0 if sucessful, otherwise return -1;
     */
    public abstract int open(String pathname);    
    
    /**
     * Common interface for close level 1b file
     * @return 
     */
    public abstract int close();

    /**
     * Common interface for read level 1b file header information
     * @return AMSU-A level 1b header object
     */
    public abstract AMSUALevel1BHeader readHeader();
    
    /**
     * Common interface for get one scan line of data from level 1b file object
     * @param indexLine - index of scan line
     * @param line0 - AMUS-A level 1b scan line object
     * @return 0 if successful, otheriwse return -1.
     */
    public abstract int readLine(int indexLine,AMSUALevel1BLine line0);
    
    /**
     * Common interface for read all level 1b data into memory
     * @return 0 if successful, otherwise return -1;
     */
    public abstract int readData();
    
    /**
     * Common interface for smoothing level 1b calibration raw counts
     */
    public abstract void convolveCalibrationTargetCounts();
    
    
    /**
     * Write one line to a text file, for diagnosis purpose only
     * @param pathname - pathname of output file
     * @param index - index of scan line
     */
    public void writeLine2File(String pathname,int index){
        OutputFile fpOut=new OutputFile();
        String str;
        int channel,spot;
        float[] centralWVs=header.getCentralWVs();

        fpOut.open(pathname);
        
        str=this.getName()+" line="+(index+1)+"\r\n";
        fpOut.writeString(str);
        
        str="c1="+pub.physics.PlanckFunction.plcoef1+"\r\n";
        fpOut.writeString(str);

        str="c2="+pub.physics.PlanckFunction.plcoef2+"\r\n";
        fpOut.writeString(str);
        fpOut.writeString("\r\n");
        
        
        str="Channel     CentralWavNum    ifov   countScene         radiance       AT             a0             a1             a2             \r\n";
        fpOut.writeString(str);
        for (channel=0;channel<NUM_CHANNELS;channel++){         
            for (spot=0;spot<NUM_SPOTS_PER_LINE;spot++){
                str=String.format("%7d%15.7f%10d%10.0f%20.7e%15.7f%15.7e%15.7e%15.7e\r\n",
                           channel+1,
                           centralWVs[channel],
                           spot+1,
                           countsScene[index][channel][spot],
                           sceneRadiances[index][channel][spot],
                           at[index][channel][spot],
                           this.coeffsOperationalCalibrationPrimary[index][channel][2],
                           this.coeffsOperationalCalibrationPrimary[index][channel][1],
                           this.coeffsOperationalCalibrationPrimary[index][channel][0]   
                           );
                
                fpOut.writeString(str);
            }
        }        
        
        fpOut.close();
        
    }
    
}
