/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1B.AMSUALevel1BLine.java
* NAME
*   amsu.amsua.L1B.AMSUALevel1BLine.java 
* 
* PURPOSE
*   To provide a common interface to the AMSUA leve1 1B file scan line 
* 
* DESCRIPTION
*   This class is the super class for AMSULevel1BLineAqua and AMSULevel1BLineNOAA
*   NOAA and NASA EOS Aqua AMSU-A level 1b files are in different format
*   AMSUALevel1BFile provides a common interface for the level 1c code
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   01/04/2012 
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/14/2013 - added function that calculate warm target temperature using raw 
*                warm counts and calibration coefficients.
*
*@*****/
 

package amsu.amsua.L1B;

import amsu.amsua.*;
import amsu.amsua.L1B.header.AMSUALevel1BHeader;
import amsu.amsua.limbcorrection.*;
import grid.Grid;
import io.*;
import noaa.*;
import pub.L1C.QualityControlL1C;
import pub.QualityControl;
import pub.physics.PlanckFunction;
import ucar.ma2.Index2D;
import ucar.ma2.Index3D;
import util.*;
import amsu.amsua.L1C.AMSUALevel1CFileConfig;

public abstract class AMSUALevel1BLine extends NOAADataSetRecord implements AMSUA {

    //weights for warm target PRTs for different antenna systems
    protected static final byte[][] weigthPRTPrelaunch_n15 = {{0, 1, 1, 1, 1},
        {1, 1, 1, 1, 1},
        {1, 1, 1, 1, 1, 1, 1}
    };
    //number of warm target PRTs
    protected static final int[] nWarmTargets = {NUM_WT_PRTS_A1, NUM_WT_PRTS_A1, NUM_WT_PRTS_A2};
    //antanna system that each channel is located in.
    public static final int[] indexChannelWarmTarget = {2, 2, 1, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0};
    
    public AMSUALevel1BHeader header;  //AMSU-A level 1b header object
    protected boolean bFullScanMode;    //calibration coefficients for convert count to radiance

    //following member variables are pointed to their countparts in NOAALevel1BFile
    protected int scanlineNumber;      
    //0-2nd order 1-1st order 2-0 order
    protected double[][] coeffsOperationalCalibrationPrimary;
    protected double[][] coeffsOperationalCalibrationSecondary;
    protected double[][] coeffsIMICACalibration;
    protected double[][] coeffsLinearCalibration;
    protected double[][][] calcoefsAllCalibrationMethod;
    
    protected short spacecraftAltitude;  //in km  *10    
    protected float[] spacecraftAttitude;
    protected float[] lat,  lon; //space is allocated in AMSUALevel1bFile =new float[NUM_SPOTS_PER_LINE];        
    //viewing geometry
    protected float[] solarZenith,  sensorZenith,  localAzimuth; //=new float[ NUM_SPOTS_PER_LINE];
    protected short[][] xy;// = new short[NUM_SPOTS_PER_LINE][2];
    protected byte[] landcovertype;// = new byte[NUM_SPOTS_PER_LINE];
    protected float[] landfrac;
    protected static LandType landType = null;
    protected float[][] countsScene;
    protected int[][] countsColdCal,  countsWarmCal;
    protected float[] countsColdCalAvg,  countsWarmCalAvg;
    public float[] countsColdCalAvgConvolved,  countsWarmCalAvgConvolved; //scanline;    channel

    //currently for exploring only only
    protected int[] countsPRTsA1;
    protected int[] countsPRTsA2;
    protected float[] temperaturesPRTsA1;
    protected float[] temperaturesPRTsA2;
    protected float[] temperatureInstruments;//instrument temperatures for current scan line
    
    protected float[] avgWarmTargetTemp;// = new float[3]; //0-A1-1 1- A1-2 2- A2
    protected int[] nWarmTargetPRTUsed;// = new int[3];    
    protected float[] wtTemperatureWarmLoadCorrected;// = new float[NUM_CHANNELS];
    public float[] wtConvertedFromRawCounts; //NUM_CHANNELS
    
    protected float[] warmTargetRadiance;
    protected float[] coldSpaceRadiance;
    protected float[][] sceneRadiances;
    ;
    protected float[][] at = null;
    protected float[][] bt = null; //new float[NUM_CHANNELS][NUM_SPOTS_PER_LINE];
    protected float[][] temperaturesLimbCorrected = new float[NUM_CHANNELS][NUM_SPOTS_PER_LINE];
    protected float[][] temperatures;    //for limb correction when sensitivity is on
    protected QualityControlL1C qcLine;
    protected short[] deltaBlackbodyCounts = null;    //interpolated using instrument temperature
    protected static int flagSensitivityStudy = 0; //0-- normal mode 1--sensitivity for alpha 2--sensitivity for fcoeftime
    protected static int channelSensitivityStudy = -1;  //channel 7
    protected static int nParaSensitivityStudy = 14;  //14 groups of parameters
    protected short[] flagsCalibrationQuality;  //offset 33-64 unsigned, NUM_CHANNELS

    protected float[] houseKeepingAMSUA1;    
    protected int[] houseKeepingAMSUA1QC;

    protected int indexLine = 0;  //index of current scan line

    
    /**
     * Constructor
     */
    public AMSUALevel1BLine() {
        
    }

    /**
     * Read one scan line of level 1b data 
     * @param ind - index of scan line to be read
     * @param fileL1B - AMSU-A level 1b file
     */
    public void setLineIndex(int ind, AMSUALevel1BFile fileL1B) {
        qcLine = fileL1B.qc[ind];

        indexLine = ind;

        time = fileL1B.time[indexLine];
        lat = fileL1B.lat[indexLine];
        lon = fileL1B.lon[indexLine];
        xy=fileL1B.xy[indexLine];
        landcovertype=fileL1B.landcovertype[indexLine];
        landfrac=fileL1B.landfrac[indexLine];
        
        solarZenith = fileL1B.solarZenith[indexLine];
        sensorZenith = fileL1B.sensorZenith[indexLine];
        localAzimuth = fileL1B.localAzimuth[indexLine];

        sceneRadiances = fileL1B.sceneRadiances[indexLine];
        warmTargetRadiance = fileL1B.warmTargetRadiance[indexLine];
        coldSpaceRadiance = fileL1B.coldSpaceRadiance[indexLine];

        at = fileL1B.at[indexLine];
        bt = fileL1B.bt[indexLine];
        flagsCalibrationQuality = fileL1B.flagsCalibrationQuality[indexLine];
        
        spacecraftAltitude=fileL1B.spacecraftAltitude[indexLine];
        spacecraftAttitude=fileL1B.spacecraftAttitude[indexLine];
        
        avgWarmTargetTemp=fileL1B.avgWarmTargetTemp[ind];
        nWarmTargetPRTUsed=fileL1B.nWarmTargetPRTUsed[ind];       
        wtTemperatureWarmLoadCorrected=fileL1B. wtTemperatureWarmLoadCorrected[ind];    
        
        if (AMSUALevel1BFile.bReadHouseKeeping){
            houseKeepingAMSUA1=fileL1B.houseKeepingAMSUA1[indexLine];
            houseKeepingAMSUA1QC=fileL1B.houseKeepingAMSUA1QC[indexLine];
        }

        coeffsOperationalCalibrationPrimary=fileL1B.coeffsOperationalCalibrationPrimary[indexLine];
        coeffsOperationalCalibrationSecondary=fileL1B.coeffsOperationalCalibrationSecondary[indexLine];
        coeffsIMICACalibration=fileL1B.coeffsIMICACalibration[indexLine];
        coeffsLinearCalibration=fileL1B.coeffsLinearCalibration[indexLine];
 
        calcoefsAllCalibrationMethod=new double[NUM_CALIBRATION_METHODS][][];
        calcoefsAllCalibrationMethod[0]=getIMICACalibrationCoefficients();
        calcoefsAllCalibrationMethod[1]=getOperationalCalibrationCoefficients();
        calcoefsAllCalibrationMethod[2]=getLinearCalibrationCoefficients();

        wtConvertedFromRawCounts=fileL1B.wtConvertedFromRawCounts[indexLine];
    }

    /**
     * Get scanline number
     * @return scanline number
     */
    public int getScanlineNumber() {
        return scanlineNumber;
    }

    /**
     * Set land type data
     * @param landType0 - land type 
     */
    public static void setLandTypeMask(LandType landType0) {
        landType = landType0;
    }

    /**
     * Set level 1b header
     * @param h - level 1b header
     */
    public void setHeader(AMSUALevel1BHeader h) {
        header = h;
        int satidInternal = header.getSatelliteIDInternal();
        deltaBlackbodyCounts = AMSUALevel1BQualityControl.getDeltaBlackbodyCounts(satidInternal);
    }

    /**
     * Check whether the current scan line is good or not
     * @return true if current scan line is good, otherwise return false.
     */
    public boolean isGood() {
        return qcLine.getScanLineQC() == 0;
    }
    
    /**
     * Get Spacecraft altitude
     * @return spacecraft altitude in unit of km*10 
     */
    public short getSpacecraftAltitude() {
        return spacecraftAltitude;
    }
    
    /**
     * Get spacecraft attitude
     * @return spacecraft attitude
     */
    public float[] getSpacecraftAttitude(){
        return spacecraftAttitude;
    }
    
    /**
     * Get latitude for all pixels in a scan line
     * @return latitude (-90 - 90) degrees
     */
    public float[] getLat() {
        return lat;
    }

    /**
     * Get longitude all pixels in a scan line
     * @return longitude
     */
    public float[] getLon() {
        return lon;
    }

    /**
     * Get grid cell index for all pixels in a scan line
     * @return grid cell index
     */
    public short[][] getXY() {
        return xy;
    }

    /**
     * Get land cover types for all pixels in a scan line
     * @return land cover types
     */
    public byte[] getLandCoverType() {
        return landcovertype;
    }
    
    /**
     * Get land fraction for all pixels in a scan line
     * @return land fraction
     */
    public float[] getLandFraction(){
        return landfrac;
    }

    /**
     * Get one channel's cold space raw count
     * @param channel - channel
     * @return cold space raw count for this channel
     */
    public float getColdSpaceCount(int channel) {
        return countsColdCalAvg[channel];
    }

    /**
     * Get one channel's warm target raw count
     * @param channel - channel
     * @return warm target raw count for this channel
     */
    public float getWarmTargetCount(int channel) {
        return countsWarmCalAvg[channel];
    }

    /**
     * Get scene raw counts for one channels
     * @param channel - channel
     * @return scene raw counts for this channel
     */
    public float[] getSceneCount(int channel) {
        return countsScene[channel];
    }

    /**
     * Get cold space raw counts for all channels
     * @return cold space raw counts for all channels
     */    
    public float[] getColdSpaceCounts() {
        return countsColdCalAvg;
    }

    
    /**
     * Get warm target raw counts for all channels
     * @return warm target raw counts for all channels
     */    
    public float[] getWarmTargetCounts() {
        return countsWarmCalAvg;
    }

    /**
     * Get scene raw counts for all channels
     * @return scene raw counts for all channels
     */    
    public float[][] getSceneCounts() {
        return countsScene;
    }

    /**
     * Get calibration quality flags
     * @return calibration quality flags
     */
    public short[] getCalibrationQualityFlags(){
        return flagsCalibrationQuality;
    }


    /**
     * Get scene temperatures
     * @return scene temperatures
     */
    public float[][] getTemperatures() {
        if (temperatures == null) {
            temperatures = at;
            QualityControl.setFillValues(temperatures);
        }
        return temperatures;
    }

    /**
     * Get solar zenith angles
     * @return solar zenith angles
     */
    public float[] getSolarZenith() {
        return solarZenith;
    }

    /**
     * Get sensor zenith angles
     * @return sensor zenith angles
     */
    public float[] getSensorZenith() {
        return sensorZenith;
    }

    /**
     * Get local azimuth angles
     * @return local azimuth angles
     */
    public float[] getLocalAzimuth() {
        return localAzimuth;
    }

    /**
     * Get averaged warm target temperatures
     * @return averaged warm target temperatures
     */
    public float[] getAvgWarmTargetTemperatures() {
        return avgWarmTargetTemp;
    }

    /**
     * Get NOAA operational calibration coefficients
     * @return NOAA operational calibration coefficients
     * Note:
    //http://oiswww.eumetsat.org/WEBOPS/eps-pg/ATOVS-L1/ATOVSL1-PG-6ProdFormDis.htm
    //Calibration coefficients (slope, intercept and quadratic term coefficient) used 
    //for all pixels in that scan line, for each one of the 15 channels. 
    //Two sets of coefficients are given, corresponding to a Primary and a Secondary calibration. 
    //Those used operationally are the Primary calibration coefficients. 
    //The Secondary calibration coefficients are only used if no in-flight calibration can be carried out, in which case the product is flagged by setting up the corresponding flag in the field QUALITY_INDICATOR.
    */
    public double[][] getOperationalCalibrationCoefficients() {
        return coeffsOperationalCalibrationPrimary;
       //return coeffsOperationalCalibrationSecondary;
     }
    
    /**
     * Get IMICA calibration coefficients (0, 1, 2 orders)
     * @return IMICA calibration coefficients
     */
    public double[][] getIMICACalibrationCoefficients() {
        return coeffsIMICACalibration;
     }

    /**
     * Get Linear calibration coefficients (0, 1, 2 orders, 2nd order is set to zero)
     * @return linear calibration coefficients
     */
     public double[][] getLinearCalibrationCoefficients() {
        return coeffsLinearCalibration;
    }

    /**
     * Get RF Shelf temperature for an antenna system
     * @param indexInstrument - index of antenna system
     * @return RF Shelf temperature for an antenna system
     */
    public float getRFShelfTemperature(int indexInstrument) {
        int id = header.instrTempReference.getTemperatureSensorID(indexInstrument);
        if (indexInstrument == 0) {
            if (id == 0) {
                return temperaturesPRTsA1[32]; //A1-1
            } else {
                return temperaturesPRTsA1[4];
            }
        } else if (indexInstrument == 1) {
            if (id == 0) {
                return temperaturesPRTsA1[33]; //A1-2
            } else {
                return temperaturesPRTsA1[5]; //A1-2 
            }
        } else if (indexInstrument == 2) {
            if (id == 0) {
                return temperaturesPRTsA2[10]; //A2
            } else {
                return temperaturesPRTsA2[2];
            }
        } else {
            return QualityControl.missing_value;
        }
    }

    /**
     * Get RF shelf temperature for one channel
     * @param channel - channel
     * @return RF shelf temperature for the specified channel
     */
    public float getRFShelfTemperatureByChannel(int channel) {
        int indexInstrument = indexChannelWarmTarget[channel];
        return getRFShelfTemperature(indexInstrument);

    }

    /**
     * Get RF shelf temperatures for all antenna systems
     * @return RF shelf temperatures 
     */
    public float[] getRFShelfTemperatures() {
        for (int i = 0; i < 3; i++) {
            temperatureInstruments[i] = getRFShelfTemperature(i);
        }
        return temperatureInstruments;
    }

    /**
     * Parse viewing geometry
     * @param line - NOAA level 1b scan line buffer
     */
    void parseViewingGeometry(byte[] line) {
        int offset = 470;
        spacecraftAltitude = (short) ParserUtil.combineInteger2(line[offset], line[offset + 1]);

        offset = 472;
        int temp;
        for (int i = 0; i < NUM_SPOTS_PER_LINE; i++) {
            temp = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            solarZenith[i] = (float) (temp * 1.0e-2);

            offset += 2;
            temp = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            sensorZenith[i] = (float) (temp * 1.0e-2);
            if (i > 15) {
                sensorZenith[i] = -sensorZenith[i];            //not sure about the scale factor of local azimuth angle
            }
            offset += 2;
            temp = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            localAzimuth[i] = (float) (temp * 1.0e-2);

            offset += 2;
        }
    }

    /**
     * Calculate and set grid cell coordinates
     * @param idxSpot  - scan position
     * @return grid cell coordinates
     */
    //return 0 if current lat/lon is valid, otherwise return 1;
    protected int setGrid(int idxSpot) { //also used in AMSUALevel1BLineAqua

        if (lat[idxSpot] < -90 || lat[idxSpot] > 90 || lon[idxSpot] < -180 || lon[idxSpot] > 180) {
            System.out.println("AMSUA Level 1B: bad lat or lon! " + "lat=" + lat[idxSpot] + " lon=" + lon[idxSpot]);
            xy[idxSpot][0] = -9999;
            xy[idxSpot][1] = -9999;
            landcovertype[idxSpot] = LandType.UNDEFINED;
            return 1;
        } else {//caliculate grid xy           
            Grid.getGrid(lat[idxSpot], lon[idxSpot], xy[idxSpot]);
            //extract land type of each spot  
            if (landType != null) {
                landcovertype[idxSpot] = landType.getType1(xy[idxSpot][0], xy[idxSpot][1]);
                landfrac[idxSpot]=landType.getLandFraction(xy[idxSpot][0], xy[idxSpot][1]);
            } else {
                landcovertype[idxSpot] = LandType.UNDEFINED;
                landfrac[idxSpot]=-9999;
            }
            return 0;
        }

    }

 

  


    /**
     * Common interface for calibration
     * @param satIDInternal - internal satellite ID
     * @param calibrationMethod - calibration method
     * @param correctAntennaPattern - whether perform antenna pattern correction
     * @param indexMUGroup - index of IMICA calibration coefficients used
     * @return 0 if successful, otherwise return -1.
     */
    public abstract int calibrate(int satIDInternal, byte calibrationMethod, byte correctAntennaPattern,
            int indexMUGroup);
    
    /**
     * Calculate warm target temperature using raw warm counts and calibration coefficients
     * @param indexCalibration - index of calibration method
     */
    public void calculateWTConvertedFromRawCounts(int indexCalibration) {

        if (AMSUALevel1CFileConfig.isAqua())
            return;
        
        double[][] calcoeffs = calcoefsAllCalibrationMethod[indexCalibration];
        double[] coefs;
        if (calcoeffs == null) {
            return;
        }

        float cw;
        float radWT, wt;
        float[] centralWVs = header.getCentralWVs();

        for (int i = 0; i < NUM_CHANNELS; i++) {
            cw = countsWarmCalAvg[i];
            if (cw > 0) {
                coefs = calcoeffs[i];
                if (indexCalibration!=1){
                    radWT = (float) (coefs[0] + coefs[1] * cw + coefs[2] * cw * cw);
                }else{
                    radWT = (float) (coefs[2] + coefs[1] * cw + coefs[0] * cw * cw);                    
                }
                wt = AMSUAPlanckFunction.radiance2Temperature(centralWVs[i], radWT);

                if (!QualityControl.isValidWarmTargetTemperature(wt)) {
                    wt = QualityControl.missing_value;
                }
                wtConvertedFromRawCounts[i] = wt;
            }
        }
    }

    /**
     * Limb Correction
     * @return 
     */
    public int limbCorrection() {
        int month;

/*        //limb correction
        if (flagSensitivityStudy > 0) {
            //get channels that will be used for current sensitivity study channel
            AMSUALimbCorrection1Month.limbCorrectionSensitivity(temperatures,
                    antennaTemperaturesForLimbCorrection,
                    temperaturesLimbCorrected,
                    landcovertype, channelSensitivityStudy);

        } else {*/
            AMSUALimbCorrection.limbCorrection(temperatures, temperaturesLimbCorrected, landcovertype, time);
        //}
        temperatures = temperaturesLimbCorrected;

        return 0;

    }

}
