/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1B.aqua.AMSUALevel1BAquaHDFFile.java
* NAME
*   amsu.amsua.L1B.aqua.AMSUALevel1BAquaHDFFile.java 
* 
* PURPOSE
*   To support NASA EOS Aqua AMSU-A level 1b file in HDF format.
* 
* DESCRIPTION
*   This class provides methods for access NASA AQUA AMSU-A level 1b file.
*   Reference http://disc.sci.gsfc.nasa.gov/AIRS/documentation/v5_docs/AIRS_V5_Release_User_Docs/V5_Data_Release_UG.pdf
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   01/03/2012
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   06/04/2013 added methods readCenterFrequency() and getCenterFrequncy()
*   05/20/2013 Initial Version prepared for NCDC
*
*@*****/
 
 
package amsu.amsua.L1B.aqua;

import amsu.amsua.*;
import io.hdf.HDFDataFile;
import java.util.Vector;
import ncsa.hdf.object.*;
import util.*;


public class AMSUALevel1BAquaHDFFile extends HDFDataFile implements AMSUA {
    static final String swathName = "L1B_AMSU";    //swath name
    
    //data buffers for different variables in the HDF file
    double[] lat = null, lon = null;
    double[] time = null; //new double[NUM_SCANLINE_MAX][NUM_SPOTS_PER_LINE];
    byte[] qc = null;  //[NUM_SCANLINE_MAX][NUM_CHANNELS]; unsigned byte
    short[]qcScanLine=null;    
    float[]at = null; //new float[NUM_SCANLINE_MAX][NUM_SPOTS_PER_LINE][NUM_CHANNELS];
    float[] bt = null; //new float[NUM_SCANLINE_MAX][NUM_SPOTS_PER_LINE][NUM_CHANNELS];
    float[]sensorZenith=null; // = new float[NUM_SCANLINE_MAX][NUM_SPOTS_PER_LINE];
    float[]solarZenith =null;// new float[NUM_SCANLINE_MAX][NUM_SPOTS_PER_LINE];
    float[]sensorAzimuth = null; //new float[NUM_SCANLINE_MAX][NUM_SPOTS_PER_LINE];
    float[]solarAzimuth = null; //new float[NUM_SCANLINE_MAX][NUM_SPOTS_PER_LINE];
    float[][] wts=new float[3][];  //warm target temperatures for a1-1 a1-2 a2
    float[] satheight;  //spacecraft altitude
    float[] center_freq;
    
    Group gridsGeolocation = null; //geolocation grids
    Group gridsData = null; //data grides
    protected java.util.List listGridsGeo,  listGridsData;

    
    static LandType landType;  //LandType object
    int nScans; //number of scan lines
    
    /**
     * Constructor
     */
    public AMSUALevel1BAquaHDFFile() {
    }

    /**
     * Open a NASA level 1b HDF file
     * @param pathnameIn - pathname of NASA level 1b HDF file
     * @return NASA level 1b HDF file
     */
    @Override
    public int open(String pathnameIn) {
        if (super.open(pathnameIn) == 0) {
            gridsGeolocation = (Group) eosgrid.getMemberList().get(0);
            listGridsGeo = gridsGeolocation.getMemberList();
            gridsData = (Group) eosgrid.getMemberList().get(1);
            listGridsData = gridsData.getMemberList();
            return 0;
        } else {
            return -1;
        }
    }

   
    /**
     * Read time data from HDF file
     * @return 
     */
    public int readTime(){
        time=(double[])readGrid2D(listGridsGeo, "Time");
        //some Aqua level 1b files have format errors
        //AIRS.2010.01.01.135.L1B.AMSU_Rad.v5.0.0.0.G10004085354.hdf
        if (time!=null)  
            return 0;
        else
            return -1;     
    }
    
    /**
     * Read geolocation data from HDF file
     */
    public void readGeolocation() {
        lat=(double[])readGrid2D(listGridsGeo, "Latitude");
        lon=(double[])readGrid2D(listGridsGeo, "Longitude");
        
        

    }
    
    /**
     * Read channel center frequencies
     */
    public void readCenterFrequency(){
         Vector vec=(Vector)readGrid1D(listGridsData,"center_freq");
          Object obj=vec.get(0);
        center_freq=(float[])obj; 
    }
    /**
     * Read geometry variables from HDF file
     */
    public void readGeometry(){
        sensorZenith=(float[])readGrid2D(listGridsData,"satzen"); 
        solarZenith=(float[])readGrid2D(listGridsData,"solzen"); 
        sensorAzimuth=(float[])readGrid2D(listGridsData,"satazi"); 
        solarAzimuth=(float[])readGrid2D(listGridsData,"solazi"); 
                
        Vector vec=(Vector)readGrid1D(listGridsData,"satheight");  
        Object obj=vec.get(0);
        satheight=(float[])obj;
        
    }
    /**
     * qa_scanline
Bit field for each scanline (bit 0 set if sun glint in scanline; bit 1 set if costal
crossing in scanline, bit 2 set if some channels had excessive NeDT
estimated), dimension (45)
• qa_channel
Bit field by channel for each scanline (bit 0 set if all space view counts
bad; bit 1 set if space view counts marginal; bit 2 set if space view counts
could not be smoothed; bit 3 set if all blackbody counts bad; bit 4 set if
blackbody counts marginal; bit 5 set if blackbody counts could not be
smoothed; bit 6 set if unable to calculate calibration coefficients; bit 7 set if
excessive NeDT estimated), dimension (15,45)
     */
    public void readQC() {
        qc=(byte[])readGrid2D(listGridsData,"qa_channel");
        
        java.util.Vector v=(java.util.Vector)readGrid1D(listGridsData,"qa_scanline");
        Object obj=v.get(0);
        qcScanLine=(short[])obj; // not used; the information in user guide indicated if any bit is set 1, 
                      //it doesn't mean the scan line is bad
    }

    /**
     * Read antenna temperatures from HDF file
     */
    public void readAT() {
        at=(float[])readGrid3D(listGridsData,"antenna_temp");   
    }

    /**
     * Read brightness temperatures from HDF file
     */
    public void readBT() {
        bt=(float[])readGrid3D(listGridsData,"brightness_temp");
    }

    /**
     * Read warm target temperatures from HDF file
     */
    public void readWT(){
                
        Vector vec=(Vector)readGrid1D(listGridsData,"a11_target_temp");  
        Object obj=vec.get(0);
        wts[0]=(float[])obj;
        
        vec=(Vector)readGrid1D(listGridsData,"a12_target_temp");   
        wts[1]=(float[])vec.get(0);
        
        vec=(Vector)readGrid1D(listGridsData,"a2_target_temp");   
        wts[2]=(float[])vec.get(0);
    }
 

    /**
     * Get number of scan lines in a HDF file
     * @return number of scan lines in a HDF file
     */
    public int getNumOfScans() {
        return nScans;
    }
 
    /**
     * Get start observation time of current L1B file
     * @return start observation time of current L1B file
     */
    public Time getStartTime() {  
        Time t=new Time();
        for (int i = 0; i < time.length; i++) {
                if (time[i]> 1) {
                    t.setFromEpoch(time[i], (short)1993);
                    break;
                }
        }
        return t;
    }
    
    /**
     * Get end observation time of current L1B file
     * @return end observation time of current L1B file
     */
    public Time getEndTime() {
        Time t=new Time();
        for (int i = time.length-1; i >=0; i--) {
                if (time[i]> 1) {
                    t.setFromEpoch(time[i], (short)1993);
                    break;
                }
        }
        return t;
    }
     
    /**
     * To determine whether a scan line is good or bad
     * @param indexLine - index of scan line
     * @return 0 if a scan line is good, otherwise return -1.
     */
    public byte getQCScanline(int indexLine){
        int nBadChannel=0;
        for (int channel=0;channel<NUM_CHANNELS;channel++){
            if (getQC(indexLine,channel)!=0)
                nBadChannel++;
        }
        if (nBadChannel==NUM_CHANNELS)
            return 1;  //bad line, cannot be calibrated
        else
            return 0;
    }

    /**
     * Get QC data for a channel
     * @param indexLine - index of scan line
     * @param channel - channel
     * @return QC data
     */
    public byte getQC(int indexLine,int channel){
        int offset=indexLine*NUM_CHANNELS+channel;
        return qc[offset];
    }
    
    /**
     * Get channel center frequencies
     * @return channel center frequencies
     */
    public float[] getCenterFrequncy(){
        return center_freq;
    }
    /**
     * Set LandType data
     * @param landType0 - LandType object
     */
    public static void setLandTypeMask(LandType landType0){
        landType=landType0;
    }
    
   
  }
