/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1B.aqua.AMSUALevel1BFileAqua.java
* NAME
*   amsu.amsua.L1B.aqua.AMSUALevel1BFileAqua.java 
* 
* PURPOSE
*   To support NASA AQUA AMSU-A level 1b file.
* 
* DESCRIPTION
*   This class encapsulates HDF file (AMSUALevel1BAquaHDFFile) and provide a 
*   similar interface as the NOAA level 1b file.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   01/03/2012
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   06/04/2013 Modified readHeader() method to set center frequency (wavenumber)
*              information in header record.
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
package amsu.amsua.L1B.aqua;

import amsu.amsua.*;
import amsu.amsua.L1B.*;
import amsu.amsua.L1B.header.AMSUALevel1BHeader;

public class AMSUALevel1BFileAqua extends AMSUALevel1BFile implements AMSUA {
    public static final int NUM_SCANLINE_AQUA=45;//number of scan lines in each NASA AMSU-A level 1b HDF file
    AMSUALevel1BAquaHDFFile fpHDF;  //NASA Aqua AMSU-A level 1b HDF file object
    AMSUALevel1BLineAqua lineAqua;  //scan line object for the Aqua AMSU-A level 1b HDF file object
    
    /**
     * Constructor
     */
    public AMSUALevel1BFileAqua() {
         header=new AMSUALevel1BHeader();
         lineAqua=new AMSUALevel1BLineAqua();
         line=lineAqua;
    }

    /**
     * Open Aqua AMSU-A level 1b file
     * @param pathnameIn
     * @return 0 if successful, otherwise return -1
     */
    @Override
    public int open(String pathnameIn) {

        fpHDF=new AMSUALevel1BAquaHDFFile();
        if (fpHDF.open(pathnameIn) == 0) {  
            if (fpHDF.readTime()==0)
                    return 0;
            else{
                fpHDF.close();
                return -1;
            }
        } else {
            return -1;
        }
        
    }
    

    /**
     * Close Aqua AMSU-A level 1b
     * @return 0 if successful, otherwise return -1
     */
    @Override
    public int close(){
       fpHDF.close();
       return 0;
    }
    
    /**
     * Read global attribute of Aqua AMSU-A level 1b file
     * @return AMSU-A header object
     */
    @Override
    public AMSUALevel1BHeader readHeader(){
        header.setDataType((short)10);

        header.setSatelliteID("Aqua");
        
        header.setNumberOfRecords(NUM_SCANLINE_AQUA);
        header.setStartTime(fpHDF.getStartTime());
        header.setEndTime(fpHDF.getEndTime());
        
        fpHDF.readCenterFrequency();
        header.setCenterWavenumber(fpHDF.getCenterFrequncy());
        return header;
    }
   
    /**
     * Get one scan line of level 1b data
     * @param indexLine - index of scan line
     * @param line0 - AMSU-A scan line object
     * @return 0 if successful, otherwise return -1
     */
    @Override
    public int readLine(int indexLine,AMSUALevel1BLine line0){         
        //get one line of data
        AMSUALevel1BLineAqua line2=(AMSUALevel1BLineAqua)line0;
        line2.setHDFFile(fpHDF);
        int status=line2.parse();
        
        spacecraftAltitude[indexLine]=line2.getSpacecraftAltitude();

        return status;              
    }
    
    /**
     * Get AMSUALevel1BLine object
     * @return AMSUALevel1BLine object
     */
    public AMSUALevel1BLine getLineL1B(){
        return lineAqua;
    }
    
    /**
     * Read all data used for generating L1C file into memory
     * @return 0
     */
    public int readData(){
            fpHDF.readGeolocation();
            fpHDF.readGeometry();
            fpHDF.readQC();
            fpHDF.readAT();
            fpHDF.readBT();
            fpHDF.readWT();
            lineAqua.setHDFFile(fpHDF);
            return 0;
    }

    
    @Override
    public String getName(){
        return fpHDF.getName();
    }
    public int getNumberOfRecords(){
        return NUM_SCANLINE_MAX;
    }
    
    /**
     * Calibrate one scan line of data. Not really doing anything for AQUA
     * @param satIDInternal - internal satellite ID
     * @param calibrationMethod - calibration method
     * @param correctAntennaPattern - whether performs antenna correction
     * @param indexMUGroup - index of calibration coefficients used
     * @return 0 if calibration method is NOAA Operational Calibration, otherwise return -1.
     */
    public int calibrate(int satIDInternal,byte calibrationMethod, byte correctAntennaPattern, 
                         int indexMUGroup){
        
        if(calibrationMethod==CALIBRATION_OPERATIONAL){          
            return 0;
        }else{
            return -1;  
        }
    }
    
    /**
     * To smooth calibration counts. Not doing anything for Aqua
     */
    public void convolveCalibrationTargetCounts(){
        
    }
}
