/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1B.aqua.AMSUALevel1BLineAqua.java
* NAME
*   amsu.amsua.L1B.aqua.AMSUALevel1BLineAqua.java 
* 
* PURPOSE
*   To support Aqua AMSU-A level 1b file scan lines.
* 
* DESCRIPTION
*   This class provide methods to read and parse one line from Aqua level 1b file.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   01/03/2012
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   06/04/2013 Modified calibrate(...) method to support IMICA calibration for AQUA.
*   05/20/2013 Initial Version prepared for NCDC
* 
*
*@*****/
 
 

package amsu.amsua.L1B.aqua;

import amsu.amsua.*;
import amsu.amsua.L1B.AMSUALevel1BLine;
import pub.L1C.IMICACalibration;
import pub.L1C.IMICACoefficient;
import pub.physics.PlanckFunction;
import amsu.amsua.L1B.AMSUAPlanckFunction;
import pub.QualityControl;

public class AMSUALevel1BLineAqua extends AMSUALevel1BLine{
   AMSUALevel1BAquaHDFFile fpAqua=null; //Aqua level 1b HDF file object


    /**
     * Set level 1b HDF file object
     * @param fp - level 1b HDF file object
     */
    public void setHDFFile(AMSUALevel1BAquaHDFFile fp){
        fpAqua=fp;
    }
    
    /**
     * Get one scan line of Aqua level 1b data
     * @return 0 if successful, otherwise return -1.
     */
    public int parse(){
        //check quality control flag
       
        byte qc,qcScanline;
        
        qcScanline=fpAqua.getQCScanline(indexLine);
        qcLine.setScanLineQC(qcScanline==0);
        if (qcScanline == 0) {
            for (int ch = 0; ch < NUM_CHANNELS; ch++) {
                qc = fpAqua.getQC(indexLine, ch);
                if (qc == 0) {
                    qcLine.setScanLineQC(true);
                }
                flagsCalibrationQuality[ch]=qc;
            }
        }
              
        parseTime();
        //read other data for current line
        if (parseLatLon(null)!=0)
            return -1;
 

        parseGeometry();
        parseWTs();
        
        if (qcLine.isGoodScanLine())
            return -1;
        else
            return 0;
    }
    
    
    
    /**
     * Get time of a scan line
     */
    void parseTime(){//AQUA observation time is for each pixel, not scanline
       int index=indexLine*NUM_SPOTS_PER_LINE;
       time.setFromEpoch(fpAqua.time[index], (short)1993);
    }
    
    /**
     * Get one scan line of geolocation data
     * @param line - index of scan line
     * @return 0 if at least one pixel is good, otherwise return -1;
     */
    int parseLatLon(byte[] line){//line is not used, only for compatible with NOAA data
        int nBad=0;
       //read other data for current line
        int offset2D;

        offset2D=indexLine*NUM_SPOTS_PER_LINE;
        for (int i=0;i<NUM_SPOTS_PER_LINE;i++){
            lat[i]=(float)fpAqua.lat[offset2D+i];
            lon[i]=(float)fpAqua.lon[offset2D+i];
            nBad+=setGrid(i);
             
        }
   
        if (nBad==NUM_SPOTS_PER_LINE)
            return -1;
        else return 0;        
    }
    
    /**
     * Get one scan line of warm target temperature data
     */
    void parseWTs(){
        float temp;
        int i;
        try {
            for (i = 0; i < AMSUA.NUM_SUB_INSTRUMENTS; i++) {
                temp = fpAqua.wts[i][indexLine] + 273.15f;
                wtTemperatureWarmLoadCorrected[i] = temp;
                avgWarmTargetTemp[i] = temp;
            }
        } catch (java.lang.ArrayIndexOutOfBoundsException e) {
            temp=-9999;
        }
    }
    
    /**
     * Get one scan line of antenna temperature and brightness temperature data
     */
    void parseTemperatures(){
        int offset3D;
        offset3D = indexLine * NUM_SPOTS_PER_LINE*NUM_CHANNELS;

        try {
            for (int i = 0; i < NUM_SPOTS_PER_LINE; i++) {
                for (int ch = 0; ch < NUM_CHANNELS; ch++) {
                    at[ch][i] = fpAqua.at[offset3D];
                    bt[ch][i] = fpAqua.bt[offset3D];
                    offset3D++;
                }
            }
            
            
        } catch (java.lang.ArrayIndexOutOfBoundsException e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Get one scan line of geometry data
     */
    void parseGeometry(){
        int offset2D=indexLine*NUM_SPOTS_PER_LINE;
        for (int i=0;i<NUM_SPOTS_PER_LINE;i++){
            solarZenith[i]=(float)fpAqua.solarZenith[offset2D+i];
            sensorZenith[i]=(float)fpAqua.sensorZenith[offset2D+i];
            localAzimuth[i]=(float)fpAqua.solarAzimuth[offset2D+i]-
                            (float)fpAqua.sensorAzimuth[offset2D+i];  
            while (localAzimuth[i]<-180)
                localAzimuth[i]+=360;
            
            while (localAzimuth[i]>180)
                localAzimuth[i]-=360;
        }  
        
        spacecraftAltitude=(short)(fpAqua.satheight[indexLine]*10);
    }
    
    /**
     * Calibrate one scan line of data. If calibration method is IMICA,
     * adjust antenna temperature using the constant offset term from AQUA's 
     * IMICA coefficients.
     * 
     * @param satIDInternal - internal satellite ID
     * @param calibrationMethod - calibration method
     * @param correctAntennaPattern - whether to perform antenna correction
     * @param indexMUGroup - index of calibration coefficients
     * @return 0
     */
    public int calibrate(int satIDInternal,byte calibrationMethod, byte correctAntennaPattern, 
                         int indexMuGroup){
        parseTemperatures();
        temperatures = at;
        
        if (calibrationMethod==2){//IMICA cliabration
            float delta0,rad,temp,wv;            
            IMICACoefficient snoCoef;
            float[] centralWvs=header.getCentralWVs();
            for (int channel=0;channel<temperatures.length;channel++){
                snoCoef = IMICACalibration.coeffs[channel][indexMuGroup][satIDInternal-9]; 
                delta0=(float)snoCoef.delta0;
                wv=centralWvs[channel];
                for (int spot = 0; spot < NUM_SPOTS_PER_LINE; spot++) {
                    temp = temperatures[channel][spot];
                    if (!QualityControl.isMissingValue(temp)) {
                        rad = AMSUAPlanckFunction.temperature2Radiance(wv, temp);
                        rad += delta0;
                        temp = AMSUAPlanckFunction.radiance2Temperature(wv, rad);
                        temperatures[channel][spot] = temp;
                    }
                }
            }
        }
        return 0;
    }
    
}
