/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1B.header.AMSUACalibrationTargetFixedBias.java
* NAME
*   amsu.amsua.L1B.header.AMSUACalibrationTargetFixedBias.java 
* 
* PURPOSE
*    AMSU-A calibration target fixed bias correction (in header record)
* 
* DESCRIPTION
*    This class provides methods for 
*    1. parsing calibration target fixed bias correction from level 1b file
*    2. supporting level 1c file generation.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   11/10/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
package amsu.amsua.L1B.header;

import amsu.amsua.AMSUA;
import util.*;


public class AMSUACalibrationTargetFixedBias implements AMSUA {

    //warm target and cold space fixed biases
    protected float[][] warmTargetFixedBias=new float[NUM_CHANNELS][3];  //1. min 2. norminal 3. max RF Shelf temperautre
    protected float[][] warmTargetFixedBiasPPLO2=new float[NUM_CHANNELS][3]; //similar to warmTargetFixedBias, but for PPLO#2
                                                                   //only channel 9-14 used
    protected float[] coldSpaceFixedBias=new float[NUM_CHANNELS];  //cold space fixed biases
    
    /**
     * Parse calibration target fixed biases
     * @param buffer - buffer of level 1b header
     */
    public void parse(byte[] buffer){
       //////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////
        //warm target cold space fixed biases correction coefficients
        //for each channel 3 for WT at different RF temperautre + 1 for cold space
        int temp4;      
        int offset=264;       
        short temp2;
        float f2;
        for (int i=0;i<NUM_CHANNELS;i++){
            for (int j=0;j<3;j++){
                temp4=ParserUtil.combineInteger2(buffer[offset],buffer[offset+1]);
                temp2=(short)temp4;
                f2=temp2*0.001f;
                warmTargetFixedBias[i][j]=f2;  //min,norminal,max
                offset+=2;
            }
            
            temp4=ParserUtil.combineInteger2(buffer[offset],buffer[offset+1]);
            temp2=(short)temp4;
            coldSpaceFixedBias[i]=temp2*0.001f; 
            offset+=2;
        }

        //CHANNEL 9-14 PLLO#2 warm target bias correction, not used at this time
        offset=384;
        for (int i=8;i<14;i++){
            for (int j=0;j<3;j++){
                temp4=ParserUtil.combineInteger2(buffer[offset],buffer[offset+1]);
                warmTargetFixedBiasPPLO2[i][j]=temp4*0.001f;
                offset+=2;
            }
        }
        //END WT COLD SPACEfixed bias correction
        
    }
    
    /**
     * Get warm target fixed biases
     * @param channel - AMSU-A channel
     * @param bPPLO2 - true if used PPLO2
     * @return warm target fixed biases
     */
    public float[] getWarmTargetFixedBias(int channel,boolean bPPLO2){
        if (channel>=8 && channel<=13 && bPPLO2)
            return warmTargetFixedBiasPPLO2[channel];
        else
            return warmTargetFixedBias[channel];
    }

    /**
     * Get cold space fixed biases for all channels
     * @return cold space fixed biases
     */
    public float[] getColdSpaceFixedBiases(){
        return coldSpaceFixedBias;
    }
    
    /**
     * Get cold space fixed bias for one channel
     * @param channel - AMSU-A channel
     * @return cold space fixed biases for the specified channel
     */
    public float getColdSpaceFixedBias(int channel){
        return coldSpaceFixedBias[channel];
    }
}
