/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1B.header.AMSUALevel1BHeader.java
* NAME
*   amsu.amsua.L1B.header.AMSUALevel1BHeader.java 
* 
* PURPOSE
*   To parse AMSU-A level 1B header record.
* 
* DESCRIPTION
*   This class is the supper class of AMSUALevel1BHeaderNOAA and AMSUALevel1BHeaderAqua
*   It provides definitions of different data items in L1B header, as well as methods
*   for access these data items .
*   Unfinished functions:
*       1. PPLO#1 PPLO#2 switch 
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   09/23/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   01/27/2012: add navigation info (AMSUANavigation)
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 
package amsu.amsua.L1B.header;

import amsu.amsua.*;
import noaa.*;
import util.*;



public class AMSUALevel1BHeader extends NOAADataSetHeader implements AMSUA {
    

    //Temperature - Radinace Conversion start offset: 688
    //1. central wave number, 2. constant 1, 3. constant 2 - slope
    //only central wave number is used currently
    // constant 1 =0 constant 2=1.0
    protected float[][] temperatureRadinaceConversion=new float[NUM_CHANNELS][3];
    protected float[] centralWVs=new float[NUM_CHANNELS]; //derived from temperatureRadinaceConversion[][0]
    
            
            
    
    //A1 A2 Digital Conversion coefficients
    //currently not used in calibration
    //based on pp32, Mo, calibration of the advanced microwave sounding unit-a for NOAA-K
    protected double[][] coeffsPRTCounts2TemperatureA1=new double[45][4];             
    protected double[][] coeffsPRTCounts2TemperatureA2=new double[19][4]; 
    //a subset in digital conversion
    //0- A1-1 5 warm targets
    //1- A1-2 5 warm targets 
    //2- A2 7 warm targets
    protected double[][][] coeffsWarmTargetCount2Temp=new double[3][NUM_WT_PRTS_A2][4];
    
       
    protected int nScansCalibratedGeolocated;  //number of scan lines that are calibration
    protected String datasetCreationSiteID;    //dataset creation site ID
    protected short dataType;                  //data type, should be 10 for AMSU-A    
    
    
    public  AMSUAInstrumentStatus instrumentStatusA1, //instrument status object for A1
                  instrumentStatusA2;                 //instrument status object for A2
    public AMSUALevel1BInstrumentTemperatureReference instrTempReference;  //instrument temperature reference object   
    public AMSUACalibrationTargetFixedBias biasCorrection;  //calibration target fix bias object             
    
    public AMSUALevel1BHeader(){
              
    }
    
    
    /**
     * Get # of scans that are calibrated and geolocated
     * @return # of scans that are calibrated and geolocated 
     */
    public int getNumberOfScansCalibratedGeolocated(){
        return nScansCalibratedGeolocated;
    }

    /**
     * Get temperature radiance conversion coefficients
     * @return parameters (only central wave number used) for temperature radiance conversion
     */
    public float[][] getTemperatureRadianceConversion(){
        return temperatureRadinaceConversion;
    }
    
    /**
     * Get central wave numbers for all channels
     * @return central wave numbers for all channels
     */
    public float[] getCentralWVs(){
        return centralWVs;
    }
    
    /**
     * Set Data Type
     * @param type - data type
     */
    public void setDataType(short type){ 
        dataType=type;
    }
    
    /**
     * Get Data Type
     * @return data type
     */
    public short getDataType(){
        return dataType;
    }
    
    /**
     * Get count to temperature conversion coefficients for warm target PRTs
     * @return temperature conversion coefficients for warm target PRTs
     */
    public double[][][] getWarmTargetCount2TempCoefficients(){
        return coeffsWarmTargetCount2Temp;
    }
    
    
    /**
     * Set orbit start observation time
     * @param t - time
     */
    public void setStartTime(Time t){
        timeStart.set(t);
    }

    /**
     * Set orbit end observation time
     * @param t - time
     */
    public void setEndTime(Time t){
        timeEnd.set(t);
    }
    
    /**
     * Set satellite IDs from satellite name
     * @param satName - satellite name
     */
    public void setSatelliteID(String satName){
        satelliteID=noaa.ObservationPlatforms.NOAASatelliteName2NOAASatelliteID(satName);
        satidInternal=noaa.ObservationPlatforms.NOAASatelliteName2InternalSatelliteID(satName);
    }
    
    /**
     * Set channel center wavelength using center frequencies
     * @param center_freq - channel center frequencies
     */
    public void setCenterWavenumber(float[] center_freq){
        float lamda;
        for (int i=0;i<NUM_CHANNELS;i++){
            lamda=299792458/(center_freq[i]*10000000);
            centralWVs[i]=1/lamda;
        }
    }
    /**
     * Get reference instrument temperatures
     * @param indexInstr - index of reference instrument
     * @param bPLLO2 - whether PLLO2 is used
     * @return  reference instrument temperatures
     */
    public float[] getInstrumentTemperatureReference(int indexInstr,boolean bPLLO2){
        return instrTempReference.getInstrumentTemperatureReference(indexInstr,bPLLO2);
    }
    
}
