/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1B.header.AMSUALevel1BInstrumentTemperatureReference.java
* NAME
*   amsu.amsua.L1B.header.AMSUALevel1BInstrumentTemperatureReference.java 
* 
* PURPOSE
*   To parse AMSU-A Instrument reference temperatures.
* 
* DESCRIPTION
*   This class parses and manages instrument reference temperatures.
*   AMSUALevel1BInstrumentTemperatureReference parses which instrument are used as 
*   reference for  A1-1 A1-2 and A2:
*      0 - RF Shelf 
*      1 - RF Mux
*   for A1-1, instrument reference temperature also depends on either PLLO1 OR PLLO2 status
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   10/07/2010
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
* 
*
*@*****/
 
 
package amsu.amsua.L1B.header;

import util.*;

public class AMSUALevel1BInstrumentTemperatureReference {
    //instrument reference temperautre
    //which reference tempeature is used as instrument temperature  
    short[] instrumentTempeatureSensorID=new short[3];   //only used in this class
    //RF Shelf, RF Mux reference temperature 
    //RF Shelf a1-1 (PLLO#1),a1-2,a2,a1-1 (PLLO#2)
    //RF Mux a1-1 (PLLO#1),a1-2,a2,a1-1 (PLLO#2)
    //max,nominal,min
    float[][] referenceTemperatureRF=new float[8][3]; //only used in this class
    //A1-1 A1-2 A2 instrument reference temperature max,normal,min
    //opposite order as those in L1B DATA (MIN,NOR,MAX)
//    float[][] instrumentTemperatureReferences=new float[3][]; //derived from instrumentTempeatureSensorID & referenceTemperatureRF

    static final int[][] index111={{0,4},{1,5},{2,6},{3,7}}; //for dealing with deal with PLLO#1 and PLLO#2
                //0- 
    
    
    public AMSUALevel1BInstrumentTemperatureReference(){
        
    }
    
    /**
     * Parse instrument temperature reference information from NOAA L1B file
     * @param buffer - buffer of NOAA AMSU-A L1B header record 
     */ 
    public void parseNOAAL1B(byte[] buffer){
        int offset;
        int temp4;
        
        offset=210;   
        int id;
        //which reference tempeature is used as instrument temperature  
        for (int i=0;i<3;i++){
           id=ParserUtil.combineInteger2(buffer[offset],buffer[offset+1]);
           instrumentTempeatureSensorID[i]=(short)id;           
           offset+=2;
        }
        /////////////////////////////////////////////////////////////
        //parse instrument reference temperature

        offset=216;
        for (int i=0;i<8;i++){//a1-1 pllo1,a1-2,a1,a1-1 pllo2                       
            for (int j=0;j<3;j++){//min,norminal,max
                temp4=ParserUtil.combineInteger2(buffer[offset],buffer[offset+1]);
                //referenceTemperatureRF[i][2-j]=temp4*0.01f;
                referenceTemperatureRF[i][j]=temp4*0.01f;
                offset+=2;
            }
        }                              


        
    }
    
    /**
     * Get temperature sensor ID
     * @param indexInstr  - instrument index
     * @return temperature sensor ID
     */
    public int getTemperatureSensorID(int indexInstr){
        return instrumentTempeatureSensorID[indexInstr];
    }
    
    /**
     * Get instrument reference temperature
     * @param indexInstr - index of instrument
     * @param bPPLO2 - true if PPL02 is turned on
     * @return reference temperature of instrument
     */
    public float[] getInstrumentTemperatureReference(int indexInstr, boolean bPPLO2){
           int id= instrumentTempeatureSensorID[indexInstr];
              
           float[] ref;
           if (indexInstr==0 && bPPLO2){ //A1-1 
                ref=referenceTemperatureRF[index111[3][id]];
           } else {
                ref=referenceTemperatureRF[index111[indexInstr][id]];
           }

            return  ref;
    }
}
