/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1B.noaa.AMSUALevel1BFileNOAA.java
* NAME
*   amsu.amsua.L1B.noaa.AMSUALevel1BFileNOAA.java 
* 
* PURPOSE
*   To support AMSU-A level 1b file in NOAA format.
* 
* DESCRIPTION
*   This class supports methods for parse and access AMSU-A level 
*   1b file in the NOAA binary format.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   09/23/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
package amsu.amsua.L1B.noaa;

import amsu.amsua.L1B.AMSUALevel1BFile;
import amsu.amsua.L1B.AMSUALevel1BLine;
import amsu.amsua.L1B.header.AMSUALevel1BHeader;
import io.*;
import pub.QualityControl;

public class AMSUALevel1BFileNOAA extends AMSUALevel1BFile {
    FileRandom inputFile=new FileRandom();    //input file object for read NOAA level 1b file
    byte[] buffer=new byte[L1B_RECORD_LEN_MAX];    //NOAA level 1b file scan line buffer
    
    AMSUALevel1BLineNOAA lineNOAA;     //line object
    AMSUALevel1BHeaderNOAA headerNOAA; //header object
    
    //variables for smoothing raw counts
    static int rawcountsCovolvingN=-1; //window size
    static int rawcountsCovolvingWindow;
    static float[] rawcountsCovolvingWeights; //weights for smoothing
    
    /**
     * Construction
     */
    public AMSUALevel1BFileNOAA(){
        headerNOAA=new AMSUALevel1BHeaderNOAA();
        lineNOAA=new AMSUALevel1BLineNOAA();
        super.line=(AMSUALevel1BLine)lineNOAA;
        super.header=(AMSUALevel1BHeader)headerNOAA;
        
    }
    
    /**
     * Initialize member variables that are related to smoothing raw calibration counts
     * @param n - window size for smoothing
     */
    public static void initRawCountsConvolving(int n){
        rawcountsCovolvingN=n;
        rawcountsCovolvingWindow=rawcountsCovolvingN*2+1;
        rawcountsCovolvingWeights=new float[rawcountsCovolvingWindow];
        
        for (int i=-rawcountsCovolvingN;i<=rawcountsCovolvingN;i++){
            rawcountsCovolvingWeights[i+rawcountsCovolvingN]=1-((float)Math.abs(i))/(rawcountsCovolvingN+1.0f);
        }
    }
    
    /**
     * Get NOAA level 1b file pathname
     * @return NOAA level 1b file pathname
     */
    public String getName(){
        return inputFile.getName();
    }
    
    /**
     * Open NOAA level 1b file
     * @param pathname - pathname of NOAA level 1b file
     * @return 0 if successful, otherwise return -1.
     */
    public int open(String pathname){        
        if (inputFile.open(pathname,"r")!=0){
           System.out.println("Inputfile "+pathname+" open failed");
           return -1;       
        }else {
        }
        return 0;
    }
    
    /**
     * Close NOAA level 1b file
     * @return 0
     */
    public int close(){
        inputFile.close();
        return 0;
    }

    /**
     * Read all scan lines into memory
     * @return 0 if at least one scan line is read, otherwise turn -1;
     */
    @Override
    public int readData(){
        int status=-1;
        //read level 1B raw data into internal buffer in fileL1B
        for (int i = 0; i < nScans; i++) {
            lineNOAA.setLineIndex(i, this);
            if (readLine(i, lineNOAA)==0){
                status=0;
            }
        }
        return status;
    }

    /**
     * Read one scan line of NOAA level 1b data
     * @param indexLine - index of scan line
     * @param line0 - AMSU-A level 1b scan line object
     * @return 0 if successful, otherwise return -1;
     */
    @Override
    public int readLine(int indexLine,AMSUALevel1BLine line0){
        int status;
        status = inputFile.read(buffer); 
        if (status!=0)
            return -1;
         
        AMSUALevel1BLineNOAA line1=(AMSUALevel1BLineNOAA)line0;
        status=line1.parse(buffer,satelliteID);

        scanlineNumber[indexLine]=line1.getScanlineNumber();
        spacecraftAltitude[indexLine]=line1.getSpacecraftAltitude();
        
        return status;     
    }
    
    /**
     * To read and parse NOAA AMSU-A level 1b file header
     * @return - NOAA AMSU-A level 1b file header object
     */
    @Override
    public AMSUALevel1BHeader readHeader(){
        
        int status=0;
    
        //read header
        status=inputFile.read(buffer);
        if (status!=0){
            System.out.println("    AMSUALevel1BHeader::readHeader read input file failed");
            return null;
        }
        
        status=headerNOAA.parseHeader(buffer);
        if (status!=0){
            System.out.println("    AMSUALevel1BHeader::readHeader parse header failed");
            return null;
        }
        
        line.setHeader(header);
        nScans=header.getNumberOfRecords();
        return header;
    }

//end of operational    
 
    
    ////////////////////////////////////////////////////////////////////////////////////////
    //functions to support operational calibration from scrath
    /**
     * To smooth cold space and warm target calibration raw counts
     */
    public void convolveCalibrationTargetCounts(){
        convolveCalibrationTargetCounts(countsColdCalAvg,countsColdCalAvgConvolved);
        convolveCalibrationTargetCounts(countsWarmCalAvg,countsWarmCalAvgConvolved);
    }
    
    
    /**
     * To smooth one type of calibration raw counts
     * @param counts - raw counts to be smoothed
     * @param countsConvolved - for returning smoothed raw counts
     */
    void convolveCalibrationTargetCounts(float[][] counts, float[][] countsConvolved) {
        int idxLineChunkStart, idxLineChunkEnd;
        float count;
        float sum, sumW;
        int nGood = 0, nGap;
        int ireplace, ireplaceLeft, ireplaceRight;
        int idxLine,index;
        
        //initialize output
        for (int channel = 0; channel < NUM_CHANNELS; channel++) {
            for (idxLine = 0; idxLine < nScans; idxLine++) {
                countsConvolved[idxLine][channel] = QualityControl.missing_value;
            }
        }

        for (int channel = 0; channel < NUM_CHANNELS; channel++) {
            idxLine = 0;
            nGood = 0;
            nGap = 0;
            while (idxLine < nScans) {
                sum = 0;
                sumW = 0;
                if (idxLine==845)
                    index=845;
                for (int i = -rawcountsCovolvingN; i <= rawcountsCovolvingN; i++) {
                    index=i+idxLine;
                    if (index < 0 || index >= nScans) {
                        continue;
                    }

                    lineNOAA.setLineIndex(index, this);

                    count = counts[idxLine + i][channel];
                    if (lineNOAA.isGoodRawCounts(channel) && !QualityControl.isMissingValue(count)) {
                            sum += count * rawcountsCovolvingWeights[i + rawcountsCovolvingN];
                            sumW += rawcountsCovolvingWeights[i + rawcountsCovolvingN];
                      } else if (i == 0) {//middle count is invalid
                        sumW = -1;
                        break;
                    }

                }
                if (sumW > 0) {
                    countsConvolved[idxLine][channel] = sum / sumW;
                } else {
                    countsConvolved[idxLine][channel] = QualityControl.missing_value;
                }
                idxLine++;
            }//end for current chunk

  
        }
    }
    
}
