/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1B.noaa.AMSUALevel1BHeaderNOAA.java
* NAME
*   amsu.amsua.L1B.noaa.AMSUALevel1BHeaderNOAA.java 
* 
* PURPOSE
*   To parse AMSU-A level 1B header record in the NOAA binary format.
* 
* DESCRIPTION
*   This class supports methods for parse and access the header of a AMSU-A level 
*   1b file in NOAA format.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   09/23/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   10/28/2011 - add AMSUANolinearity class 
*   10/07/2011 - separated instrument temperature reference data and method to a class.
*   10/07/2010 - add AMSUAInstrumentStatus for A1 and A2
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*   * Unfinished code:
*   1. PPLO#1 PPLO#2 switch 
* 
*@*****/

 
package amsu.amsua.L1B.noaa;

import amsu.amsua.*;
import amsu.amsua.L1B.*;
import noaa.*;
import util.*;

public class AMSUALevel1BHeaderNOAA extends amsu.amsua.L1B.header.AMSUALevel1BHeader implements AMSUA {

    public AMSUAAnalogTelemetryConversion anologTelemetryConvresion=new AMSUAAnalogTelemetryConversion(); //Analog telemetry conversion object
   
    public AMSUALevel1BHeaderNOAA(){
         instrTempReference=new amsu.amsua.L1B.header.AMSUALevel1BInstrumentTemperatureReference(); 
         instrumentStatusA1=new amsu.amsua.L1B.header.AMSUAInstrumentStatus(); //offset 133 
         instrumentStatusA2=new amsu.amsua.L1B.header.AMSUAInstrumentStatus() ; //offset 121
         biasCorrection=new amsu.amsua.L1B.header.AMSUACalibrationTargetFixedBias();
    }
        
    /**
     * Parse header
     * @param buffer - header buffer for the NOAA level 1b data
     * @return 0 if successful, otherwise return -1.
     */
    public int parseHeader(byte[] buffer){
        //data set creation site ID
        char[] str=new char[3];
        str[0]=(char)buffer[0];
        str[1]=(char)buffer[1];
        str[2]=(char)buffer[2];
        datasetCreationSiteID=String.copyValueOf(str);
        
        //check dataset creation site ID
        if (!(datasetCreationSiteID.equals("DSS") ||
            datasetCreationSiteID.equals("NSS") ||
            datasetCreationSiteID.equals("UKM")))
            return -1;
        
        if (buffer[3]!=' ' && buffer[3]!=0)
            return -1;
        
        //Version Number
        short versionNumber=(short)ParserUtil.combineInteger2(buffer[4],buffer[5]);
        short versionYear=(short)ParserUtil.combineInteger2(buffer[6],buffer[7]);
        short versionDayOfYear=(short)ParserUtil.combineInteger2(buffer[8],buffer[9]);
        
        //satellite id
        satelliteID= (short)ParserUtil.combineInteger2(buffer[72],buffer[73]);
        if (satelliteID<2 || (satelliteID>8 && satelliteID!=12)) //metopa=12
            return -1;
        
        //instrument iD;
        int instrumentID_A2, instrumentID_A1;
        instrumentID_A2=buffer[74];
        instrumentID_A1=buffer[75];
        
        dataType=(short)(ParserUtil.combineInteger2(buffer[76],buffer[77]));
        if (dataType!=10)
            return -1;
        
        //set start ending time -- year,jday,millisecondOfDay
        short year,day;
        int milliseconds;
        year=(short)ParserUtil.combineInteger2(buffer[84],buffer[85]);        
        if (year<1998 || year>2100)
            return -1;
        timeStart.setYear(year);
        day=(short)ParserUtil.combineInteger2(buffer[86],buffer[87]);
        if (day<1 || day>366)
            return -1;
        timeStart.setDayOfYear(day);
        milliseconds=ParserUtil.combineInteger4(buffer[88],buffer[89],buffer[90],buffer[91]);
        timeStart.setMillisecondOfDay(milliseconds);
        timeEnd.setYear((short)ParserUtil.combineInteger2(buffer[96],buffer[97]));
        timeEnd.setDayOfYear((short)ParserUtil.combineInteger2(buffer[98],buffer[99]));
        timeEnd.setMillisecondOfDay(ParserUtil.combineInteger4(buffer[100],buffer[101],buffer[102],buffer[103]));
        
        //internal satellite ID
        satidInternal=ObservationPlatforms.NOAASatelliteID2InternalID(satelliteID,timeStart.getYear(),
                timeStart.getDayOfYear()); 
        
        //number of scans
        nRecords=ParserUtil.combineInteger2(buffer[144],buffer[145]);
        nScansCalibratedGeolocated=ParserUtil.combineInteger2(buffer[146],buffer[147]);
        
        instrumentStatusA1.parseA1(buffer);
        instrumentStatusA2.parseA2(buffer);
        
        //parse temperature radiance conversion coefficients
        //0 - central wave number 1 - constant 1 (always 0) 2- cosntant 2 slope (always 1.0)
        int offset=688;
        long[][] temp=new long[15][3];
        for (int i=0;i<NUM_CHANNELS;i++){
            for (int j=0;j<3;j++){
                temp[i][j]=ParserUtil.combineInteger4(buffer[offset],buffer[offset+1],
                                     buffer[offset+2],buffer[offset+3]);
                temperatureRadinaceConversion[i][j]=(float)(temp[i][j]*1.0e-06);
                offset+=4;
            }
            centralWVs[i]=temperatureRadinaceConversion[i][0];
            
            if (Math.abs(temperatureRadinaceConversion[i][1])>0 || 
                Math.abs(temperatureRadinaceConversion[i][2]-1)>0 ){
                System.out.println("AMSU-A parse header: attention to temperature radiance conversion bytes!!!");
            }
        }

        //end of instrument reference temperatures
        instrTempReference.parseNOAAL1B(buffer);
        biasCorrection.parse(buffer);
        //////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////        
        //coeffsPRTCounts2Temperature for debug only currently
        //coefficients for converting all kind of PRT counts to temperatures for A1
        //AMSU-A1 Digital Conversion
        int temp4;
        offset=976;
        double[] scalesCoeffsPRTCounts2Temperature={1.e-4,1.e-9,1.e-16,1.e-20};
        for (int i=0;i<45;i++){
            for (int j=0;j<4;j++){
                temp4=ParserUtil.combineInteger4(buffer[offset],buffer[offset+1],
                                     buffer[offset+2],buffer[offset+3]);
                coeffsPRTCounts2TemperatureA1[i][j]=(float)(temp4*scalesCoeffsPRTCounts2Temperature[j]);
                offset+=4;
            }
            
        }
        //A2 digital conversion
        offset=1920;        
        for (int i=0;i<19;i++){
            for (int j=0;j<4;j++){
                temp4=ParserUtil.combineInteger4(buffer[offset],buffer[offset+1],
                                     buffer[offset+2],buffer[offset+3]);
                coeffsPRTCounts2TemperatureA2[i][j]=(float)(temp4*scalesCoeffsPRTCounts2Temperature[j]);
                offset+=4;
            }
            
        }
        
        //END PARSE A1 A2 Digital Conversion
        ////////////////////////////////////////////////////////////////////////////////
        //read coefficients for converting A1-1 A1-2 warm target counts to temperatures 
        // subset for the previous digital conversion coefficients
        offset=1536;
        double[] scales={1.e-4,1.e-9,1.e-16,1.e-20};
        for (int k=0;k<2;k++){
          for (int i=0;i<NUM_WT_PRTS_A1;i++){
            for (int j=0;j<4;j++){
                temp4=ParserUtil.combineInteger4(buffer[offset],buffer[offset+1],
                                     buffer[offset+2],buffer[offset+3]);
                coeffsWarmTargetCount2Temp[k][i][j]=(float)(temp4*scales[j]);
                offset+=4;
            }
          }
        }
        //A2
        //read coefficients for converting A2 warm target counts to temperatures 
        offset=2112;   
        for (int k=2;k<3;k++){
          for (int i=0;i<NUM_WT_PRTS_A2;i++){
            for (int j=0;j<4;j++){
                temp4=ParserUtil.combineInteger4(buffer[offset],buffer[offset+1],
                                     buffer[offset+2],buffer[offset+3]);
                coeffsWarmTargetCount2Temp[k][i][j]=(float)(temp4*scales[j]);
                offset+=4;
            }
          }
        }
        
        anologTelemetryConvresion.parseHeader(buffer);
                      
        return 0;
    }

    
    public double[][] getPRTCounts2TemperatureCoefficientsA1(){
        return coeffsPRTCounts2TemperatureA1;
    }             
   
    public double[][] getPRTCounts2TemperatureCoefficientsA2(){
        return coeffsPRTCounts2TemperatureA2;
    } 

    public float calWarmTargetFixedBias(int channel,float tempInstr,boolean bPLLO2){          
        //get instrument id 0-A1-1 1-A1-2 2-A2
        int indexInstr=AMSUALevel1BLine.indexChannelWarmTarget[channel];
        
        //get fixed bias at 3 instrument temperatures for current channel
        float[] bias=biasCorrection.getWarmTargetFixedBias(channel,bPLLO2);
        
        //get reference instrument temperature 
        float[] refs=instrTempReference.getInstrumentTemperatureReference(indexInstr,bPLLO2);//need 
        float t, t1,t2,b1,b2;
        int index1,index2;

        if (tempInstr<refs[1]){
            index1=0;
        }else
            index1=1;
        index2=index1+1;
        
        t1=refs[index1];
        t2=refs[index2];
        b1=bias[index1];
        b2=bias[index2];

        t=util.InterpolationUtil.linearInterpolation(tempInstr, t1, t2, b1, b2);
        return t;
    }
 

}
