/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1B.noaa.AMSUALevel1BLineNOAA.java
* NAME
*   amsu.amsua.L1B.noaa.AMSUALevel1BLineNOAA.java 
* 
* PURPOSE
*   To support NOAA AMSU-A level 1b scan line.
* 
* DESCRIPTION
*   This class supports methods for parse and access one scan line of a AMSU-A level 
*   1b file in NOAA format.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   01/23/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/07/2013 - corrected a error in parsing local azimuth angle.
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 
package amsu.amsua.L1B.noaa;

import amsu.amsua.L1B.AMSUAPlanckFunction;
import amsu.amsua.*;
import amsu.amsua.L1B.AMSUALevel1BFile;
import amsu.amsua.L1B.AMSUALevel1BLine;
import amsu.amsua.L1C.*;
import pub.QualityControl;
import pub.physics.*;
import util.*;

public class AMSUALevel1BLineNOAA extends AMSUALevel1BLine implements AMSUA {

    static final byte QC_RAWCOUNTS_COLDSPACE_BAD=0x10;   //bad cold space counts flag
    static final byte QC_RAWCOUNTS_WARMTARGET_BAD=0x20;  //bad warm target counts flag
    
    //tempsWT is derived from PRT temperatures for A1 and A2
    float[][] tempsWT = new float[3][NUM_WT_PRTS_A2];
    float[][] tempsWTPrev = new float[3][NUM_WT_PRTS_A2]; //PRT temperature for previous scan line
    byte flagPLLO;


    public AMSUALevel1BLineNOAA() {
        initWarmTargetTemperatures();

    }

    /**
     * Initialize warm target variables
     */
    public void initWarmTargetTemperatures() {
        //warm target temperatures is a part of PRT temperatures
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < NUM_WT_PRTS_A2; j++) {
                tempsWT[i][j] = QualityControl.missing_value;
                tempsWTPrev[i][j] = QualityControl.missing_value;
            }
        }
    }

    
    public boolean isPLLOPrimaryOn() {
        return flagPLLO == 1;
    }

    /**
     * Initialize data buffer of different L1B variables
     * @param ind - index of scan line
     * @param fileL1B - level 1b file object
     */
    @Override
    public void setLineIndex(int ind, AMSUALevel1BFile fileL1B) {
        super.setLineIndex(ind, fileL1B);

        countsScene = fileL1B.countsScene[ind];
        countsColdCal = fileL1B.countsColdCal[ind];
        countsWarmCal = fileL1B.countsWarmCal[ind];
        countsColdCalAvg = fileL1B.countsColdCalAvg[ind];
        countsWarmCalAvg = fileL1B.countsWarmCalAvg[ind];
        countsColdCalAvgConvolved = fileL1B.countsColdCalAvgConvolved[ind];
        countsWarmCalAvgConvolved = fileL1B.countsWarmCalAvgConvolved[ind];

        coeffsOperationalCalibrationPrimary = fileL1B.coeffsOperationalCalibrationPrimary[ind];
        coeffsOperationalCalibrationSecondary = fileL1B.coeffsOperationalCalibrationSecondary[ind];

        //temperature sensors
        countsPRTsA1 = fileL1B.countsPRTsA1[ind];
        countsPRTsA2 = fileL1B.countsPRTsA2[ind];
        temperaturesPRTsA1 = fileL1B.temperaturesPRTsA1[ind];
        temperaturesPRTsA2 = fileL1B.temperaturesPRTsA2[ind];
        temperatureInstruments = fileL1B.temperatureInstruments[ind];
        flagsCalibrationQuality = fileL1B.flagsCalibrationQuality[ind];
        
    }

    public static void setSensivityStudy(int flagSensitivity, int channel) {
        flagSensitivityStudy = flagSensitivity;
        channelSensitivityStudy = channel;
    }

    public static void setLandTypeMask(LandType landType0) {
        landType = landType0;
    }


    static void setFillValues(float[][] temps) {
        for (int i = 0; i < NUM_CHANNELS; i++) {
            for (int j = 0; j < NUM_SPOTS_PER_LINE; j++) {
                temps[i][j] = QualityControl.missing_value;
            }
        }
    }

    /**
     * Initialize scan line data buffers
     */
    public void init() {
        int i, j;
        for (i = 0; i < NUM_SPOTS_PER_LINE; i++) {
            lat[i] = QualityControl.missing_value;
            lon[i] = QualityControl.missing_value;
            spacecraftAltitude = QualityControl.missing_value_short;
            solarZenith[i] = QualityControl.missing_value;
            sensorZenith[i] = QualityControl.missing_value;
            localAzimuth[i] = QualityControl.missing_value;

            for (j = 0; j < NUM_CHANNELS; j++) {
                at[j][i] = QualityControl.missing_value;
                bt[j][i] = QualityControl.missing_value;
//                temperaturesLimbCorrected[j][i] = missing_value;

                countsScene[j][i] = QualityControl.missing_value;
            }
        }

        for (j = 0; j < NUM_CHANNELS; j++) {
            countsColdCalAvg[j] = QualityControl.missing_value;
            countsWarmCalAvg[j] = QualityControl.missing_value;
        }

        for (i = 0; i < 3; i++) {
            avgWarmTargetTemp[i] = QualityControl.missing_value;
        }
    }


    /**
     * Parse one scan line of data
     * @param line - data record buffer
     * @param satID - not used
     * @return 0 if successful, otherwise return -1.
     */
    public int parse(byte[] line, int satID) {
        init();
        scanlineNumber = ParserUtil.combineInteger2(line[0], line[1]);

        //parse time no matter good or bad
        time.setYear((short) ParserUtil.combineInteger2(line[2], line[3]));
        time.setDayOfYear((short) ParserUtil.combineInteger2(line[4], line[5]));
        time.setMillisecondOfDay(ParserUtil.combineInteger4(line[8], line[9], line[10], line[11]));

        //get lat,lon no matter good or bad
        parseLatLon(line);


        //parse scan quality indicator
        boolean isGood = parseScanQualityBytes(line);
        qcLine.setScanLineQC(isGood);
        if (!isGood)
            return -1;



        //FOR ERABiasAnalysis
    /*    int year = time.getYear();
        int dayOfYear = time.getDayOfYear();
        int hour = time.getHour();
        int min = time.getMinute();
        int sec = time.getSecond();
        int spot = 3;

        if (year == 2005 && dayOfYear == 40 &&
                Math.abs(lat[spot] - 76.94) < 0.1 && Math.abs(lon[spot] - 167.80) < 0.1) {
            flagERABias = 1;
        }
*/
        parsePLLOStatus(line);
        //parse calibration coefficients
        parseCalibrationCoefficients(line);

        //parse viewing geometry
        parseViewingGeometry(line);

        //parse scene count
        parseCounts(line);


        
        //parse counts related to local occilator , for channel 6 exploration
        AMSUALevel1BHeaderNOAA headerNOAA=(AMSUALevel1BHeaderNOAA)header;
        headerNOAA.anologTelemetryConvresion.parseLine(line, houseKeepingAMSUA1);
      
        return 0;
    }

    /**
     * Parse PPLO status
     * @param line - data record buffer
     * ftp://193.17.11.142/pub/EUM/out/LEO/AMSU%20A1%20SN105%20EIDP%20Updated%2025-8-10/9.%20Instrument%20Operations%20and%20Maintenance%20Manuals/AE-26771.pdf
     *AMSU-A1 has two PLLOs: PLLO #1 (Pr imary) an d PLLO#2 (Redundant ). Only one PLLO will be
    ON at any t ime since th e toggle posit ion of a magnet ic latching r elay det ermines which PLLO is
    supplied power . Comman d and command ver ifica t ion (Digital A and Digital B) is processed via the
    AMSU-A1 int erna l micr oprocessor . Th e microprocessor gen er a tes a Digit al B signa l indicat ing
    which PLLO is being commanded ON and provides a signal to th e latchin g r elay. Dur ing init ia l
    turn-on and oth er abn ormal modes such a s emergency shut off (module power disconnect ), out of
    sequence commanding or less than command sequ ence delay t ime of 18 secon ds, abn orma l statu s of
    the PLLO is possible. Th e Digital B signal may sh ow PLLO#1 ON and the analog t elemet ry da ta
    may show that PLLO#2 is ON. The a nalog data is der ived dir ect ly from th e PLLOs wher ea s th e
    Digital B is gener at ed by the micr oprocessor . Th is situa t ion will n ot damage th e in st rument but
    does cau se an abn orma l inst rument configur at ion sta tus. To avoid this situ at ion, follow th e
    pr escr ibed command an d delay t ime. Similar ly, out of sequence commanding and less than
    command sequ ence delay t ime of 18 seconds may cause an abn orma l in st rument configur at ion
    dur ing select ion of cold ca l posit ion. 
     */

    void parsePLLOStatus(byte[] line) {
        //AMSU-A1 DIGITAL B TELEMETRY
        int offset, temp4;
        int invalidWordBitFlagPLLO;

        int debug = 0;
        int PLLOPrimaryLockDetect, PLLORedundantLockDetect;
        offset = 2140;
        temp4 = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
        invalidWordBitFlagPLLO = (temp4 >> 3) & 0x1;
        if (invalidWordBitFlagPLLO == 1) {//not updated during most recent monitor frame
        } else {
            offset = 2142;
            temp4 = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            flagPLLO = (byte) ((temp4 >> 3) & 0x01);

            if (flagPLLO == 1) {
                debug = 1;
            }
            offset = 2152 + 24 * 2;
            temp4 = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            PLLOPrimaryLockDetect = temp4;
            offset += 2;
            temp4 = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            PLLORedundantLockDetect = temp4;
        }
        debug = 0;

    }


    /**
     * Parse geometry data
     * @param line - data record buffer
     */
    void parseViewingGeometry(byte[] line) {
        int offset = 470;
        spacecraftAltitude = (short) ParserUtil.combineInteger2(line[offset], line[offset + 1]);
        //System.out.println("alt="+spacecraftAltitude);
        
        offset = 472;
        int temp;
        short temp1;
        for (int i = 0; i < NUM_SPOTS_PER_LINE; i++) {
            temp = (short) ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            temp1=(short)(temp & 0x0ffff);
            solarZenith[i] = (float) (temp1 * 1.0e-2);

            offset += 2;
            temp = (short) ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            temp1=(short)(temp & 0x0ffff);
            sensorZenith[i] = (float) (temp1 * 1.0e-2);
           // if (i > 15) {
           //     sensorZenith[i] = -sensorZenith[i];           
           // }
            offset += 2;
            
            temp = (short) ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            temp1=(short)(temp & 0x0ffff);
            localAzimuth[i] = (float) (temp1 * 1.0e-2);
            offset += 2;
        }
    
        offset=464;
        float[] angles=spacecraftAttitude; //roll,pitch,yaw;
        for (int i=0;i<3;i++){
            temp = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            temp1=(short)(temp & 0x0ffff);
            angles[i]=0;
            angles[i] = (float) (temp1 * 1.0e-3);
            offset+=2;
        }
    }

    
    /**
     * Parse raw count
     * @param line - data record buffer
     */
    void parseCounts(byte[] line) {
        int channel, i, offset;
        int temp;

        bFullScanMode = (line[901] & 0x2) != 0;


        //channel 3-15 scene count
        offset = 904;//scene count
        for (int pos = 0; pos < NUM_SPOTS_PER_LINE; pos++) {
            offset += 8; //skip reflector positions           
            for (channel = 2; channel < NUM_CHANNELS; channel++) {
                temp = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
                countsScene[channel][pos] = temp;
                offset += 2;
            }
        }

        //channel 1-2
        offset = 2192;
        for (int pos = 0; pos < NUM_SPOTS_PER_LINE; pos++) {
            offset += 4; //skip reflector positions            
            for (channel = 0; channel < 2; channel++) {
                temp = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
                countsScene[channel][pos] = temp;
                offset += 2;
            }
        }

        //for debug linear calibration        
        //if (calibrationMethod==CALIBRATION_OPERATIONAL)
        //    return;

        //////////////////////////////////////////////////////////////////////
        //not needed for operational calibration
        offset = 1924; //cold calibration telemetry
        offset += 8; //skip reflector positions           
        for (i = 0; i < 2; i++) {
            for (channel = 2; channel < NUM_CHANNELS; channel++) {
                temp = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
                countsColdCal[channel][i] = temp;
                offset += 2;
            }
        }

 
        //read A1 PRT counts 
        offset = 1984; //PRT temperature counts           
        for (int j = 0; j < AMSUA.NUM_PRTS_A1; j++) {//A1
            temp = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            countsPRTsA1[j] = temp;
            offset += 2;
        }
        offset = 2444; //PRT temperature counts           
        for (int j = 0; j < AMSUA.NUM_PRTS_A2; j++) {//A2   
            temp = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            countsPRTsA2[j] = temp;
            offset += 2;
        }


        //read warm cal counts for channels 3-15, two readings for each channel
        offset = 2076; //warm calibration telemetry
        offset += 8; //skip reflector positions           
        for (i = 0; i < 2; i++) {
            for (channel = 2; channel < NUM_CHANNELS; channel++) {
                temp = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
                countsWarmCal[channel][i] = temp;
                offset += 2;
            }
        }



        offset = 2432; //cold calibration telemetry channel 1-2
        offset += 4; //skip reflector positions           
        for (i = 0; i < 2; i++) {
            for (channel = 0; channel < 2; channel++) {
                temp = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
                countsColdCal[channel][i] = temp;
                offset += 2;
            }
        }
 

        offset = 2484; //cold calibration telemetry
        offset += 4; //skip reflector positions           
        for (i = 0; i < 2; i++) {
            for (channel = 0; channel < 2; channel++) {
                temp = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
                countsWarmCal[channel][i] = temp;
                offset += 2;
            }
        }

        //average two readings for all channels       
        int flag = 0;
        int deltaBlackbodyCountCur;
        int cc1,cc2,cw1,cw2,cdiff;
        for (i = 0; i < NUM_CHANNELS; i++) { //i here is always the real channel 
            deltaBlackbodyCountCur = deltaBlackbodyCounts[i];
            cc1=countsColdCal[i][0];
            cc2=countsColdCal[i][1];
            cdiff=Math.abs(cc1-cc2);
            if (cdiff>=deltaBlackbodyCountCur){
                setRawCountsCalibrationQualityFlag(i, QC_RAWCOUNTS_COLDSPACE_BAD);
               // System.out.println("discard cold space counts ch="+(i+1)+" c1="+cc1+" c2="+cc2);
                countsColdCalAvg[i]=QualityControl.missing_value;
            }else
                countsColdCalAvg[i] = (cc1 + cc2) / 2.0f;
            
            cw1=countsWarmCal[i][0];
            cw2=countsWarmCal[i][1];
            cdiff=Math.abs(cw1-cw2);
            //check the difference of two readings, if too large, should not be used
            if (//(i == 5) && //channel 4-10
                cdiff>= deltaBlackbodyCountCur) {
                setRawCountsCalibrationQualityFlag(i, QC_RAWCOUNTS_WARMTARGET_BAD);
                countsWarmCalAvg[i] = QualityControl.missing_value;
 /*               String satname = NOAAUtil.getSatelliteNameUsingInternalSatelliteID(header.getSatelliteIDInternal());
                System.out.println("discard wt counts sat=" + satname + " ch=" + (i + 1) + 
                                   " cw1=" + cw1 + " cw2=" + cw2 + " " +
                                   cdiff + "> threshold=" + deltaBlackbodyCountCur);*/
                
            } else {
                countsWarmCalAvg[i] = (cw1+cw2) / 2.0f;
            }
        }

        calculateWarmTargetTemperatureRadiance();
        calculateColdSpaceRadinace();

    }

  

    /**
     * Calculate cold space radiance
     */
    void calculateColdSpaceRadinace() {
        float[] centralWVs = header.getCentralWVs();
        float[] biases = header.biasCorrection.getColdSpaceFixedBiases();
        float temp = 4.78f; //2.73f;
        double wv;
        for (int i = 0; i < NUM_CHANNELS; i++) {
            wv = centralWVs[i];  //get central wave number for this channel
            temp=4.78f; //not 2.73 K, assigned by Dr. Zou
            //temp = 2.73f + biases[i]; // bias not used
            coldSpaceRadiance[i] = PlanckFunction.temperature2Radiance(wv, temp);
        }
    }
 
    /**
     * Calculate warm target radiance and temperatures
     */
    void calculateWarmTargetTemperatureRadiance() {
        AMSUALevel1BHeaderNOAA headerNOAA = (AMSUALevel1BHeaderNOAA) header;
        double[][][] fAll = headerNOAA.getWarmTargetCount2TempCoefficients();
        double[] f = null;
        double value;
        float fvalue;
        double c, c2, c3;

        //save warm target PRT temperatures of previous scan for quality control purpose
        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < NUM_WT_PRTS_A2; j++) {
                tempsWTPrev[i][j] = tempsWT[i][j];
            }
        }


        //PRT temperature counts -->temperature 
        //2 convert individual warm target PRT counts to temperature
        double[][] coeffs = headerNOAA.getPRTCounts2TemperatureCoefficientsA1();
        for (int j = 0; j < AMSUA.NUM_PRTS_A1-1; j++) {
            f = coeffs[j];
            c = countsPRTsA1[j];
            c2 = c * c;
            c3 = c2 * c;
            value = f[0] + f[1] * c + f[2] * c2 + f[3] * c3;
            fvalue=(float)value;
            temperaturesPRTsA1[j] = fvalue;
            if (j>=35 && j<=39)//A1-1 warm target temperatures
                tempsWT[0][j-35]=fvalue;
            if (j>=40 && j<=44) //A1-2 warm target temperature
                 tempsWT[1][j-40]=fvalue;
        }
//        System.out.println("channel 5,6,7 LO T=,"+temperaturesPRTsA1[18]+","+temperaturesPRTsA1[19]+","+temperaturesPRTsA1[20]);
        
        coeffs = headerNOAA.getPRTCounts2TemperatureCoefficientsA2();
        for (int j = 0; j < AMSUA.NUM_PRTS_A2-1; j++) {
            f = coeffs[j];
            c = countsPRTsA2[j];
            c2 = c * c;
            c3 = c2 * c;
            value = f[0] + f[1] * c + f[2] * c2 + f[3] * c3;
            fvalue=(float)value;
            temperaturesPRTsA2[j] =fvalue;
            if (j>=12 && j<=18)//warm target temperatures for A2
                tempsWT[2][j-12]=fvalue;
        }




        int n = 0;
        float sum = 0, avg = 0;

        for (int i = 0; i < 3; i++) {//loop for A1-1 A1-2
            avgWarmTargetTemp[i] = QualityControl.missing_value;
            nWarmTargetPRTUsed[i] = 0;
            /*if (i==debugPRT){
            str=""+wtIndex; //i="+i;
            
            ftimeDay=time.getMillisecondOfDay()/24.0/60.0/60000.0;
            ftimeYear=time.getYear()+(time.getDayOfYear()+ftimeDay)/366.0;
            str+=","+time.getYear()+","+time.getDayOfYear()+","+time.getMillisecondOfDay()+","+ftimeYear+","+ftimeDay;
            }*/

            n = 0;
            sum = 0;
            avg = 0;
            int satid = header.getSatelliteID();
            float tempCur, tempPrev, diff;
            boolean bTempCurValid, bTempPrevValid;
            for (int j = 0; j < nWarmTargets[i]; j++) {
                //str+=","+tempsPRT[i][j];
                tempCur = tempsWT[i][j];
                tempPrev = tempsWTPrev[i][j];
                bTempCurValid = QualityControl.isValidWarmTargetTemperature(tempCur); //tempCur > 200 && tempCur < 350;
                bTempPrevValid =QualityControl.isValidWarmTargetTemperature(tempPrev); //tempPrev > 200 && tempPrev < 350;
                diff = Math.abs(tempPrev - tempCur);
                if (!bTempCurValid || //bad current PRT temperautre
                        (bTempPrevValid && diff > 0.2)) {//the difference between cur and prev is too big
                    if (!(i == 0 && j == 0 && satid == 4)) {
/*                        String satname = NOAAUtil.getSatelliteNameUsingInternalSatelliteID(header.getSatelliteIDInternal());
                        System.out.println("discard sat=" + satname + " wtInstr.=" + wtInstruments[i] +
                                " prt=" + j + " tempCur=" + tempsWT[i][j] + " tempPrevLine=" + tempsWTPrev[i][j]);
*/
                    }
                } else {//accepted
                    switch (satid) {
                        case 4: //NOAA-15
                            sum += tempsWT[i][j] * weigthPRTPrelaunch_n15[i][j];
                            n += weigthPRTPrelaunch_n15[i][j];
                            break;
                        default:
                            sum += tempsWT[i][j];
                            n++;
                            break;
                    }

                }
            }

            if (n > 0) {
                avg = sum / n;
                avgWarmTargetTemp[i] = avg;
                nWarmTargetPRTUsed[i] = n;
            }
        }



        //convert warm target tmepeature to radiance for each channel
        //adjust biases
        //float[][] biases=header.getWarmTargetFixedBias();
        float biasWT;
        float[] centralWVs = header.getCentralWVs();
        double wv;
        float temp;
        int debug=0;
        for (int i = 0; i < NUM_CHANNELS; i++) {
            wv = centralWVs[i];
            biasWT = headerNOAA.calWarmTargetFixedBias(i,
                    getRFShelfTemperatureByChannel(i), !isPLLOPrimaryOn());
            //System.out.println(wv);
            temp = avgWarmTargetTemp[indexChannelWarmTarget[i]]; 

            if (Math.abs(biasWT) < 1.0f) {
                //temp += biasWT;
            }
            
            warmTargetRadiance[i] = pub.physics.PlanckFunction.temperature2Radiance(wv, temp);
            wtTemperatureWarmLoadCorrected[i] = temp;

        }
    }

 
    /**
     * Parse geolocation data
     * @param line - data record buffer
     * @return 0 if successful, otherwise return -1.
     */
    int parseLatLon(byte[] line) {
        int offset = 652;
        int temp;
        int nBad = 0;
        int flagdebug = 0;

        for (int i = 0; i < NUM_SPOTS_PER_LINE; i++) {
            temp = ParserUtil.combineInteger4(line[offset], line[offset + 1], line[offset + 2], line[offset + 3]);
            lat[i] = (float) (temp * 1.0e-4);
            offset += 4;

            temp = ParserUtil.combineInteger4(line[offset], line[offset + 1], line[offset + 2], line[offset + 3]);
            lon[i] = (float) (temp * 1.0e-4);
            offset += 4;
            // System.out.println(temp);

            nBad += setGrid(i); //set x, y grid value
        }

        if (nBad == NUM_SPOTS_PER_LINE) {
            return -1;
        } else {
            return 0;
        }
    }
    
    
    /**
     * Parse calibration coefficients
     * @param line - data record buffer
     */
    void parseCalibrationCoefficients(byte[] line) {

        double[] scales = {1.0e-19, 1.0e-13, 1.0e-09};
        //primary
        int offset = 80;
        int temp;
        for (int i = 0; i < NUM_CHANNELS; i++) {
            for (int j = 0; j < 3; j++) {
                temp = ParserUtil.combineInteger4(line[offset], line[offset + 1],
                        line[offset + 2], line[offset + 3]);
                coeffsOperationalCalibrationPrimary[i][j] = temp * scales[j];

                offset += 4;
            }
        }
//                System.out.println("f2="+coeffsCalibration[4][0]);
        offset = 260;
        for (int i = 0; i < NUM_CHANNELS; i++) {
            for (int j = 0; j < 3; j++) {
                temp = ParserUtil.combineInteger4(line[offset], line[offset + 1],
                        line[offset + 2], line[offset + 3]);
                coeffsOperationalCalibrationSecondary[i][j] = temp * scales[j];

                offset += 4;
            }
        }
    }
    
    /**
     * Parse quality control info in a data record
     * @param line - data record buffer
     * @return true if a scan line is good, otherwise return false.
     */
    boolean parseScanQualityBytes(byte[] line) {
        boolean bGood, bGood1, bGoodCal;

        bGood = line[24] == 0 && line[25] == 0 && line[26] == 0 && line[27] == 0;
        //if (line[24]!=0){
            //
        //    System.out.println("line[24]="+line[24]);
        //}
        //byte 29
        //int b30=line[30]& 0x7f);
        bGood1 = line[28] == 0 && //zero filled
                line[29] == 0 && // bit 7-4 used
                (line[30] & 0x0b3) == 0 && //calibration problem code //used to be 0xf3, changed to 0xb3 due to too many bad line in metopa
                //try 0f7 next time
                //               (line[30]&0x50)==0 && //calibration problem
                //               line[30]==0 && //calibration problem
                line[31] == 0; //earth location problem

        //calibration, at least one channel is good
        bGoodCal = false;
        int offset = 32;
        int word;
        for (int i = 0; i < NUM_CHANNELS; i++) {
            word = ParserUtil.combineInteger2(line[offset], line[offset + 1]);
            flagsCalibrationQuality[i] = (short) (word & 0x0ffff);
            bGoodCal = bGoodCal || word == 0;
            offset += 2;
        }
        //System.out.println("line[30]="+line[30]+"line[31]="+line[31]);
        return bGood & bGood1 && bGoodCal;
    }

    /**
     * Check whether raw counts data are good
     * @param channel - channel
     * @return true if raw counts data are good, otherwise return false.
     */
    public boolean isGoodRawCounts(int channel) {
        return flagsCalibrationQuality[channel] == 0;
    }
    
    /**
     * Set raw counts calibration quality flag
     * @param channel - channel
     * @param flag - quality flag
     */
    void setRawCountsCalibrationQualityFlag(int channel,byte flag){
         flagsCalibrationQuality[channel]=flag;
    }
    


    /**
     * NO limb correction version , used by netCDF concurrent version
     * @param satIDInternal
     * @param calibrationMethod
     * @param correctAntennaPattern
     * @param indexMUGroup
     * @return
     */
    @Override
    public int calibrate(int satIDInternal, byte calibrationMethod, byte correctAntennaPattern,
            int indexMUGroup) {
        //counts to radiance
        int satIDAMSU;
        float ftime;
        double[][] coeffsOperationalCalibration = getOperationalCalibrationCoefficients();
        switch (calibrationMethod) {
            case CALIBRATION_LINEAR: //linear
                //consponding warm target cold space parameters have been calculated in parse
                AMSUAL1CCalibrator.count2RadianceLinear(coldSpaceRadiance,
                warmTargetRadiance,
                countsColdCalAvg,countsWarmCalAvg,countsScene,sceneRadiances,
                coeffsLinearCalibration);
   /*             AMSUAL1CCalibrator.count2RadianceOperationalV2(
                        coldSpaceRadiance, warmTargetRadiance,
                        countsColdCalAvgConvolved, countsWarmCalAvgConvolved, countsScene,
                        coeffsNolinearityCur,
                        sceneRadiances, coeffsOperationalCalibration);*/
                break;
            case CALIBRATION_SNO:
                //get current calibration coefficients for current satellite
                satIDAMSU = satIDInternal - 9;  //n15-0 
                float[] countsCold,countsWarm;
                if (AMSUALevel1CFileConfig.bUseConvolvedCalibrationCounts){
                    countsCold=countsColdCalAvgConvolved;
                    countsWarm=countsWarmCalAvgConvolved;
                }else{
                    countsCold=countsColdCalAvg;
                    countsWarm=countsWarmCalAvg;                    
                }
                AMSUAL1CCalibrator.count2RadianceSNO(coldSpaceRadiance, warmTargetRadiance,
                        countsCold, countsWarm, countsScene, time,
                        indexMUGroup, satIDAMSU,
                        sceneRadiances,coeffsIMICACalibration);
                break;
//           case CALIBRATION_OPERATIONAL_V2:
/*                AMSUAL1CCalibrator.count2RadianceOperationalV2(coldSpaceRadiance,warmTargetRadiance,
            countsColdCalAvg,countsWarmCalAvg,countsScene, 
            sceneRadiances);               */
//                break;
            case CALIBRATION_OPERATIONAL: //operational
            default:
                AMSUAL1CCalibrator.count2RadianceOperational(countsScene,
                        coeffsOperationalCalibration, sceneRadiances);

                break;
        }

        AMSUAPlanckFunction.radiance2Temperature(sceneRadiances, at,
                header.getCentralWVs());
        temperatures = at;

        

        if (correctAntennaPattern != 0) {
            float[] tempIntruments = this.getRFShelfTemperatures();
            if (flagSensitivityStudy > 0) {
                AMSUAAntennaPatternCorrection.antennaTemperature2BrightnessTemperature_SensitivityStudy(
                        satIDInternal,
                        at,
                        bt,
                        tempIntruments,
                        channelSensitivityStudy,
                        nParaSensitivityStudy);
            } else {
                AMSUAAntennaPatternCorrection.antennaTemperature2BrightnessTemperature(
                        satIDInternal,
                        at,
                        bt,
                        tempIntruments);
            }

            temperatures = bt;
        } 
        
        return 0;
    }

    /**
     * Parse house keeping data
     * @param line - data record buffer
     */
    void parseHouseKeeping(byte[] line){
        int offset=2148;
        int temp = ParserUtil.combineInteger4(line[offset], line[offset + 1],line[offset+2], line[offset + 3]);
        houseKeepingAMSUA1QC[0]=temp;
         offset = 2152;        
         for (int i=0;i<28;i++){
             houseKeepingAMSUA1[i]=(short)(line[offset+i]&0x0ff);
             
         }
    }
 
}
