/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1C.AMSUAL1CFileIndex.java
* NAME
*   amsu.amsua.L1C.AMSUAL1CFileIndex.java 
* 
* PURPOSE
*   To generate indices for level 1c radiance netcdf files.
* 
* DESCRIPTION
*   Preprocessing level 1b files and generate AMSU-A level 1c index:
*   1. Define relationships between level 1b files and level 1c files that are need
*       to be processed.
*   2. Calculate number of scan lines for each level 1c file.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   01/03/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   06/27/2013 Modifed level 1c filename based on the NCDC DRAFT_SA_AMSU-MSU-L1C-FCDR
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 
package amsu.amsua.L1C;

import amsu.amsua.L1B.aqua.*;
import amsu.amsua.L1B.header.*;
import amsu.amsua.L1B.noaa.*;
import noaa.ObservationPlatforms;
import pub.attribute.CDRGlobalAttributes;
import pub.attribute.GlobalAttributesWWANG;
import util.*;

public class AMSUAL1CFileIndex {
    //Aqua has 240 level 1b files per day
    //Maximum number of Aqua level 1b files that are put into one level 1c file 
    static final int MAX_NUM_L1B_FILES_PER_L1C_FILE_AQUA=20; //=1 for debug;  //20
    //Number of Aqua level 1c file per day
    static final int MAX_NUM_L1C_FILES_PER_DAY_AQUA=240/MAX_NUM_L1B_FILES_PER_L1C_FILE_AQUA;
    
    String dirRoot;
    String[][] filenamesL1B;   //level 1b file names for each level 1c file
    int[] nFilesL1B;           //number of level 1b files for each level 1c file  1-for NOAA >1 for NASA Aqua
    int[] nLinesL1C;           //number of scan lines for each level 1c file
    int nFilesL1C;             //total number of level 1c files that will be generated
    Time[] timeStart,timeEnd;  //star and ending time for each level 1c file
    short[] years,dayOfYears;  // year and day of year info. of all L1B files
    String[] filenamesL1C;     //level 1c filenames
    
    
    /**
     * Get total number of level 1c files that are needed to be generated
     * @return total number of level 1c files
     */
    public int getNumberOfFiles(){
        return nFilesL1C;
    }
    
    /**
     * Get number of level 1b files that are needed to be processed.
     * @param index -- level 1c file index
     * @return number of level 1b files that are used to generate this level 1c file
     */
    public int getNumberOfL1BFiles(int index){
        return nFilesL1B[index];
    }
    
    /*
     * Get level 1b file name(s) that correspond to a given levle 1c file 
     * @param index - level l1c file index
     * @return level 1b file name(s) that correspond to a given levle 1c file
     */
    public String[] getL1BFilenames(int index){       
        return filenamesL1B[index];
    }
    
    /**
     * total number of scan lines in this level 1c file
     * @param index -- level 1c file index
     * @return  total number of scan lines in this level 1c file
     */
    public int getNumberOfLines(int index){
        return nLinesL1C[index];
    }
    
    /**
     * Start start time
     * @param index - level 1c file index
     * @return start time
     */
    public Time getTimeStart(int index){
        return timeStart[index];
    }
    
    /**
     * Get end time
     * @param index - level 1c file index
     * @return end time
     */
    public Time getTimeEnd(int index){
        return timeEnd[index];
    }
    
    /**
     * Get filename
     * @param index - level 1c file index
     * @return level 1c file name
     */
    public String getFilenameL1C(int index){
        return filenamesL1C[index];
    }
    
      
    /**
     * Generate level 1c file name
     * @param index - index of level 1c file
     * @param globalAttrs - global attributes of the current level 1c file
     */
    public void generateL1CFilename(int index,AMSUALevel1CGlobalAttributes globalAttrs){
        Time timeS=timeStart[index];
        Time timeE=timeEnd[index];
        //Time timeCreated=globalAttrs.timeCreated;        
        //String sensor=(String)globalAttrs.getValue(GlobalAttributesWWANG.GA_NAME_WWANG_SENSORNAME);
        int satidInternal=AMSUALevel1CFileConfig.satIdInternal;
        String platform=ObservationPlatforms.platformsSTAR[satidInternal];

        String filenameL1C="NESDIS-STAR_FCDR"+
                "_AMSU-A_"+
                CDRGlobalAttributes.productVersionFormat2.toUpperCase()+"_"+
                platform+"_"+
                "D"+timeS.getYear()+StringUtil.toString2(timeS.getMonth())+StringUtil.toString2(timeS.getDay())+"_"+
                "S"+StringUtil.toString2(timeS.getHour())+StringUtil.toString2(timeS.getMinute())+"_"+
                "E"+StringUtil.toString2(timeE.getHour())+StringUtil.toString2(timeE.getMinute())+
                ".nc";
                
       filenamesL1C[index]=filenameL1C;
    }
    
    /**
     * Generate level 1c file index for NOAA level 1b inputs
     * @param root -- directory of level 1c files
     * @param filenamesL1BAll -- level 1c file names
     */
    void genreateIndexNOAA(String root,String[] filenamesL1BAll){
        int n = filenamesL1BAll.length;
        
        System.out.println("Generating L1C file index ... ...nFilesIn="+n);
        
        AMSUALevel1BFileNOAA fileL1B = new AMSUALevel1BFileNOAA();
        AMSUALevel1BHeader header;

        filenamesL1B = new String[n][1];
        nFilesL1B = new int[n];
        nLinesL1C = new int[n];
        timeStart = new Time[n];
        timeEnd = new Time[n];
        filenamesL1C = new String[n];        
//        navigationAvgs=new AMSUANavigation[n];
        
        int index=0;
        for (int i = 0; i < n; i++) {

            String pathname = root + "/" + filenamesL1BAll[i];
            if (fileL1B.open(pathname) == 0) {
                header = fileL1B.readHeader();
                if (header != null) {//excluding non L1B files
                    timeStart[index] = new Time();
                    timeEnd[index] = new Time();
                    filenamesL1B[index][0] = filenamesL1BAll[i];
                    nFilesL1B[index] = 1;
                   // filenamesL1C[index] = generateL1CFilename(index, false);

                    nLinesL1C[index] = header.getNumberOfRecords();
                    timeStart[index].set(header.getTimeStart());
                    timeEnd[index].set(header.getTimeEnd());
                    
                    index++;
                   // System.out.println(pathname);
                }else{
                     System.out.println("Non AMSU-A L1B file:"+pathname);
                }
                fileL1B.close();
            }
            
            if (i%100==0){
                System.out.print(" "+i);
            }
        }
        System.out.println(n);
        
        nFilesL1C = index;
        System.out.println("Generating L1C file index done nFilesL1C="+nFilesL1C);

    }

    
    /**
     * Generate level l1c file index for Aqua level 1b inputs
     * @param root -- aqua level 1b file directory
     * @param filenamesL1BAll -- aqua level 1b file names
     */
    void genreateIndexAqua(String root,String[] filenamesL1BAll){
        //all files should be in one year, not check in this function
        //seperate files into days, --> 12 L1C files per day
        int i,n = filenamesL1BAll.length;

        dirRoot=root;
        nFilesL1C=366*MAX_NUM_L1C_FILES_PER_DAY_AQUA;
        filenamesL1B = new String[nFilesL1C][MAX_NUM_L1B_FILES_PER_L1C_FILE_AQUA];
        nFilesL1B = new int[nFilesL1C];
        nLinesL1C = new int[nFilesL1C];
        timeStart = new Time[nFilesL1C];
        timeEnd = new Time[nFilesL1C];
        years=new short[nFilesL1C];
        dayOfYears=new short[nFilesL1C];
        
        filenamesL1C = new String[nFilesL1C];  
        for (i=0;i<nFilesL1C;i++){
            nFilesL1B[i]=0;
            nLinesL1C[i]=0;     
            timeStart[i]=new Time();
            timeEnd[i]=new Time();
        }

        AMSUALevel1BFileAqua fileL1B = new AMSUALevel1BFileAqua();
        AMSUALevel1BHeader header;  

        
        Time tStart,tEnd;
        String filename;
        short year,month,dayOfMonth,indexInDayL1B,indexInDayL1C,dayOfYear;        
        short yearPrev=-1,indexInDayL1CPrev=-1;
        int index;
        for (i=0;i<n;i++){
            filename=filenamesL1BAll[i];
            
            year=Short.parseShort(filename.substring(5,9));
            if (i>0 && year!=yearPrev){
                System.out.println("genreateIndexAqua wrong year="+year+" yearPrev="+yearPrev);
                System.exit(-1);
            }
            month=Short.parseShort(filename.substring(10,12));
            dayOfMonth=Short.parseShort(filename.substring(13,15));
            dayOfYear=Time.monthDay2DayOfYear(year, month,dayOfMonth);
            indexInDayL1B=Short.parseShort(filename.substring(16,19));                                    
            indexInDayL1C=(byte)((indexInDayL1B-1)/MAX_NUM_L1B_FILES_PER_L1C_FILE_AQUA);
            
            index=(dayOfYear-1)*MAX_NUM_L1C_FILES_PER_DAY_AQUA+indexInDayL1C;
            filenamesL1B[index][ nFilesL1B[index]]=filename;
            nFilesL1B[index]++;

            yearPrev=year;
        }
                
    }
   
    /**
     * for Aqua L1C file only
     * generate netcdf level 1c file header information before processing each file/scan line
     * moved for AMSUAL1CFileIndex::void genreateIndexAqua(..)
     * purpose: shorten user waiting time for generating first level 1c file from level 1c files
     * NOAA format level 1b files don't need this function, because 1 L1B file --> 1 L1C file
     * @param indexL1C - index of level 1c file
     */
    public synchronized void generateAquaL1CFileHeader(int indexL1C){
        AMSUALevel1BFileAqua fileL1B = new AMSUALevel1BFileAqua();
        AMSUALevel1BHeader header;  

        
        Time tStart,tEnd;
        String filename;
        int n= nFilesL1B[indexL1C];
        for (int i=0;i<n;i++){
            filename=filenamesL1B[indexL1C][i];
            String pathname = dirRoot + "/" + filename;

            try {
                if (fileL1B.open(pathname)!=0){
                    continue;
                }
                System.out.println(pathname);
                header = fileL1B.readHeader();

                nLinesL1C[indexL1C] += header.getNumberOfRecords();

                tStart = header.getTimeStart();
                tEnd = header.getTimeEnd();
                if (i==0) {
                    timeStart[indexL1C].set(tStart);
                    timeEnd[indexL1C].set(tEnd);
                } else {
                    if (tStart.compare1(timeStart[indexL1C]) < 0) {
                        timeStart[indexL1C].set(tStart);
                    }
                    if (tEnd.compare1(timeEnd[indexL1C]) > 0) {
                        timeEnd[indexL1C].set(tEnd);
                    }
                }
                fileL1B.close();
            } catch (java.lang.ArrayIndexOutOfBoundsException e) {
                e.printStackTrace();
            }
           
        }
        //generate L1C filename
        //generate level 1c file name
        //NL.D02001.S0043.E0237.B0658182.WI.L1C.nc
        String strYear2,strDayOfYear,strHourMinStart,strHourMinEnd;
        if (nFilesL1B[indexL1C]>0){
                tStart=timeStart[indexL1C];
                tEnd=timeEnd[indexL1C];
                strYear2=tStart.getYearString2();
                strDayOfYear=tStart.getDayOfYearString();
                strHourMinStart=tStart.getHourMinuteString();
                strHourMinEnd=tEnd.getHourMinuteString();
                filenamesL1C[indexL1C]="AQ.D"+strYear2+strDayOfYear+".S"+strHourMinStart+".E"+strHourMinEnd+".L1C.nc";
         }
    }     
    
    
    /**
     * Generate level 1c file index for both NOAA for NASA Aqua AMSU-A 
     * @param root -- level 1b file directory
     * @param filenamesL1BAll -- level 1b file names
     * @param bAqua  -- true if is NASA Aqua
     */
    public void generateIndex(String root,String[] filenamesL1BAll,boolean bAqua){
        if (!bAqua){
            genreateIndexNOAA(root,filenamesL1BAll);
        }else{
            genreateIndexAqua(root,filenamesL1BAll);
        }
        
    }
   
}
