/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1C.AMSUAL1CThreadNetCDF.java
* NAME
*   amsu.amsua.L1C.AMSUAL1CThreadNetCDF.java 
* 
* PURPOSE
*    To generate one level 1c radiance file.
* 
* DESCRIPTION
*   This is the thread for generate one AMSU-A level 1c file.
*   Support generate multiple level 1c files concurrently
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   09/23/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/10/2013 - modified the code to generate level 1c file names based on the NCDC rules.
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 
package amsu.amsua.L1C;

import amsu.amsua.*;
import static amsu.amsua.AMSUA.nedtsN15;
import amsu.amsua.L1B.*;
import amsu.amsua.L1B.aqua.*;
import amsu.amsua.L1B.header.AMSUALevel1BHeader;
import amsu.amsua.L1B.noaa.*;
import amsu.amsua.precipitation.*;
import io.*;
import pub.L1C.QualityControlL1C;
import pub.QualityControl;
import pub.variable.VariableControl;




public class AMSUAL1CThreadNetCDF extends pub.L1C.L1CThread implements AMSUA {
    static AMSUAL1CFileIndex indexL1C;
    int indexL1CFile;
    
    //netcdf format AMSUA level 1c file
    AMSUALevel1CFileNetCDF ncfile = null;     //output ncfile
    
    //precipitation screening
    PrecipitationAMSUA precipitation = new PrecipitationAMSUA(); 
    
    //detailed level 1c quality control for each scan line, for 3 different calibration method
    QualityControlL1C[] qc = new QualityControlL1C[3];        

    /**
     * Constructor
     * Initialize AMSUA level 1c file
     * Initialize quality control
     */
    public AMSUAL1CThreadNetCDF() {
        ncfile = new AMSUALevel1CFileNetCDF();
        for (int i = 0; i < 3; i++) {
            qc[i] = new QualityControlL1C(NUM_CHANNELS,NUM_SPOTS_PER_LINE);
        }
    }
 
    /**
     * Set level 1c file Index
     * @param idxL1C -- level 1c file index, containing info. that are needed 
     *                  to generate this level 1c file, such as level 1b file 
     *                  names.
     */
    public static void setL1CFileIndex(AMSUAL1CFileIndex idxL1C){
        indexL1C=idxL1C;
    }
    public void setL1CFileIndex(int index){
        indexL1CFile=index;
    }
    
    /**
     * Processing AMSU-A level 1b file(s) to generate one level 1c file
     */
    @Override
    public void run() {//this for generate amsua level 1c product       
        String str;
        

        AMSUALevel1BFile fileL1B;
        AMSUALevel1BHeader headerL1B;
        AMSUALevel1BLine lineL1B;
        boolean bAqua=AMSUALevel1CFileConfig.isAqua();
        
        //get log file
        fpLog = AMSUALevel1CFileConfig.fpLog;
   
 
        //get level 1b input file(s)
        if (bAqua){
            fileL1B=new amsu.amsua.L1B.aqua.AMSUALevel1BFileAqua();
            
            //genrate netcdf level 1c file header information before processing each file/scan line
            //moved for AMSUAL1CFileIndex::void genreateIndexAqua(..)
            //purpose: shorten user waiting time for generating first level 1c file from level 1c files
            indexL1C.generateAquaL1CFileHeader(indexL1CFile);   
        }else{
            fileL1B=new AMSUALevel1BFileNOAA();
            AMSUALevel1BFileNOAA.initRawCountsConvolving(3);
        }
        
     
        
        
        //init AMSU-A level 1c file to be written
        AMSUALevel1CFileNetCDF fileL1C = new AMSUALevel1CFileNetCDF();
        int nInputFiles=indexL1C.getNumberOfL1BFiles(indexL1CFile);
        String[] filenamesL1B=indexL1C.getL1BFilenames(indexL1CFile);
        indexL1C.generateL1CFilename(indexL1CFile, fileL1C.globalAttributesAMSUA);
        
        //get level 1c file name and check if it exists
        String nameL1C = indexL1C.getFilenameL1C(indexL1CFile);
        String pathL1C = AMSUALevel1CFileConfig.dirOut + "/" + nameL1C;
        if (!AMSUALevel1CFileConfig.bOverwrite){
            java.io.File file=new java.io.File(pathL1C);
            if (file.exists()){
                fpLog.writeLog(nameL1C+" already exists",FileLog.PRIORITY_MUST_WRITE_MSG);
                return;
            }
        }
        //create a new netCDFFile and variables
        if (fileL1C.create(pathL1C, indexL1C,indexL1CFile) == null) { 
            fileL1B.close();
            return;
        }

        
        //processing all level 1b file into a level 1c file
        int nCalibrationMethodsUsed = VariableControl.getNumCalibrationMethodsUsed();
        int nGoodLines = 0;
        int nAll=0;
        int nPrecipTotal = 0;
        for (int idxFile = 0; idxFile < nInputFiles; idxFile++) {

            //name of level 1b file needed to be processed            
            String pathIn = AMSUALevel1CFileConfig.dirIn + "//" + filenamesL1B[idxFile];

            //open level 1b file and check header record first
            if (fileL1B.open(pathIn) != 0) {
                break;        //check whether level 1b satellite id match the intended satellite ID
            }
            
            //get NOAA satellite id
            short satid = noaa.ObservationPlatforms.NOAASatelliteName2NOAASatelliteID(
                    AMSUALevel1CFileConfig.satelliteName);

            //read level 1B header
            headerL1B = fileL1B.readHeader();
            if (headerL1B == null) {
                System.out.println("headerL1B is null");
                return;
            }

            //level 1b consistency test
            if (satid != headerL1B.getSatelliteID() || headerL1B.getDataType() != 10) {
                str = "satid=" + satid + "   satid in file=" + headerL1B.getSatelliteID();
                fpLog.writeLog(str, FileLog.PRIORITY_MUST_WRITE_MSG);
                System.out.println(str);

                fileL1B.close();
                str = "satellite id not match " + satid + " vs. " + headerL1B.getSatelliteID();
                fpLog.writeLog(str, FileLog.PRIORITY_MUST_WRITE_MSG);
                System.out.println(str);
                return; //not match
            }

            //init level 1b line
            if (!bAqua) {
                lineL1B = new AMSUALevel1BLineNOAA();
            } else {
                lineL1B = ((AMSUALevel1BFileAqua)fileL1B).getLineL1B();
            }
            lineL1B.setHeader(headerL1B);

            //get intersatellite id, number of scan lines
            int satIDInternal = headerL1B.getSatelliteIDInternal();
            int i, n = headerL1B.getNumberOfRecords();
            
            int  nPrecip=0;
            int nBadPixels = 0;
            float[][][] temperatures6 = new float[6][NUM_CHANNELS][NUM_SPOTS_PER_LINE];
            float[][] temperatures;
          
            
            //read level 1B raw data into internal buffer in fileL1B
            fileL1B.readData();
            if (!bAqua) {
                fileL1B.convolveCalibrationTargetCounts();//average calibration counts for NOAA level 1b file

            } else {
                for (i = 0; i < n; i++) {
                    lineL1B.setLineIndex(i, fileL1B);
                    fileL1B.readLine(i, lineL1B);
                }
            }
            
            //calibration
            boolean bGenerateT, bGenerateTLimb;
            int indexAT,indexAT_LIMB; //BT are not used currently, these index are consistent in L1CFileNetCDF definitions
            int[] indexT2CalibrationMethod={2,1,0}; //this indexT range from 0 -2 0-SNOt
            double[][] calcoefOPR,calcoefIMICA,calcoefLinear;
            for (i = 0; i < n; i++) {
                
                //if (i==140)
                //    i=140;
                lineL1B.setLineIndex(i, fileL1B);

                calcoefOPR=lineL1B.getOperationalCalibrationCoefficients(); //get.get.getOPRCalCoef();
                calcoefIMICA=lineL1B.getIMICACalibrationCoefficients();
                calcoefLinear=lineL1B.getLinearCalibrationCoefficients();
                        
                //Calculating temperatures using different calibration methods
                //this indexT is different as the INDEX_T in Variable Control
                //0 - T_SNO T_SNO_LIMB //BT_SNO BT_SNO_LIMB in the future
                //1 - T_OPR T_OPR_LIMB
                //2 - T_LINEAR T_LINEAR_LIMB
                for (byte indexT = 0; indexT < 3; indexT++) { 
                    indexAT=indexT*2;
                    indexAT_LIMB=indexAT+1;
                    bGenerateT = VariableControl.isWriteT(indexAT);
                    bGenerateTLimb = VariableControl.isWriteT(indexAT_LIMB); 
                    if (!bGenerateT && !bGenerateTLimb) {
                        continue;
                    }

                    qc[indexT].init(); //set all qc flags to be good
                    if (lineL1B.isGood() ) {

                        lineL1B.calibrate(satIDInternal,
                                (byte)(indexT2CalibrationMethod[indexT]), /*AMSUALevel1CFileConfig.calibrationMethod,*/
                                AMSUALevel1CFileConfig.correctAntennaPattern,
                                AMSUALevel1CFileConfig.indexMUGroup);
                    }
                    
                    qc[indexT].setScanLineQC(lineL1B.isGood());
                    float[] lat = lineL1B.getLat();
                    float[] lon = lineL1B.getLon();
                    temperatures = lineL1B.getTemperatures();
                    nBadPixels = (qc[indexT]).qualityControlGeolocation(lat, lon, temperatures);
                    if (lineL1B.isGood() ) {
                         //check geolocation
                        short[] flagCalibration = lineL1B.getCalibrationQualityFlags();
                        nBadPixels += (qc[indexT]).qaulityControlCalibration(temperatures, flagCalibration,
                                lineL1B.getSceneCounts(),lineL1B.getColdSpaceCounts(),lineL1B.getWarmTargetCounts());

                        //check precipitation
                        float[] sensorViewZenith = lineL1B.getSensorZenith();
                        float[] landfrac=lineL1B.getLandFraction();
                        nPrecip = 0;
                        if (nBadPixels < NUM_SPOTS_PER_LINE*NUM_CHANNELS) {
                            nPrecip = precipitation.checkPrecipitation(temperatures, sensorViewZenith, landfrac,lat);
                            if (nPrecip > 0) {
                                nPrecipTotal += nPrecip;
                                //set precipitation spots with missing_value
                                (qc[indexT]).qualityControlPrecipitation(precipitation, temperatures);
                            }
                        }

                        if (bGenerateT) {
                            fileL1C.bufferSceneTemperatureVariable(indexAT, temperatures);                    //limb correction
                        }
                        if (bGenerateTLimb) {
                            lineL1B.limbCorrection();
                            temperatures = lineL1B.getTemperatures();
                            fileL1C.bufferSceneTemperatureVariable(indexAT_LIMB, temperatures);
                        }
                        
                        //calculate wt using calibration coefficients
                        //lineL1B.calculateWTConvertedFromRawCounts(indexT);
                        //fileL1C.bufferWTConvertedFromRawCounts(indexT,lineL1B.wtConvertedFromRawCounts);
                        
                        nGoodLines++;
                    } else {
                        temperatures = lineL1B.getTemperatures();
                        (qc[indexT]).setScanLineQC(false);
                        (qc[indexT]).qualityControlL1B(temperatures);
                        if (bGenerateT) {
                            fileL1C.bufferSceneTemperatureVariable(indexAT, temperatures);
                        }
                        if (bGenerateTLimb) {
                            fileL1C.bufferSceneTemperatureVariable(indexAT_LIMB, temperatures);
                        }
                    }

                    //save qc flags to buffer
                    
                    fileL1C.bufferQC(indexT, qc[indexT]);

                }//end for calculating temperatures using different calibration methods

                /////////////////////////////////////////////////////////////////////////////
                //write other variables to buffer one level 1c scan line   
                fileL1C.varsTime.bufferTimeVariables(fileL1C.getIndexLine(),lineL1B.getTime());
                fileL1C.bufferGeolocationVariables(lineL1B.getLat(), lineL1B.getLon());
                fileL1C.bufferGeometryVariables(lineL1B.getSolarZenith(),
                        lineL1B.getSensorZenith(), lineL1B.getLocalAzimuth());
 
                float[] wts=lineL1B.getAvgWarmTargetTemperatures();
                QualityControlL1C.qualityControlWT(wts, QualityControl.TEMPERATURE_MIN_WT,  QualityControl.TEMPERATURE_MAX_WT);
                fileL1C.bufferWTTemperatureVariables(wts);

                fileL1C.bufferRawCountVariables(lineL1B.getSceneCounts(),
                        lineL1B.getColdSpaceCounts(), lineL1B.getWarmTargetCounts());
                
                short altitude=lineL1B.getSpacecraftAltitude();
                altitude=QualityControlL1C.qualityControlSpacecraftAltitude(
                        altitude,QualityControl.ALTITUDE_MIN,QualityControl.ALTITUDE_MAX);
                fileL1C.bufferAltitudeVariable(altitude);
                
                fileL1C.bufferAttitudeVariable(lineL1B.getSpacecraftAttitude());

                fileL1C.bufferCalCoefAMSUA(calcoefIMICA, calcoefOPR, calcoefLinear);
                        
                fileL1C.nextLine();

            }

            //consistency check for # of level 1b scan line
            if (i < n) {
                str = "AMSUL1BFile nRecords does not match that in the header";
                fpLog.writeLog(str, FileLog.PRIORITY_MUST_WRITE_MSG);
                System.out.println(str);
            }

            
            //close level 1b file
            fileL1B.close();

            
            //update total number of scan lines, good scan lines, precipitation pixels
            nAll+=n;
            nPrecipTotal+=nPrecip;
            
            if (nCalibrationMethodsUsed>0){
                nGoodLines /= nCalibrationMethodsUsed;
                str="";
            }else{
                str="No Temperature Product Generated\r\n";
            }
            str += filenamesL1B[idxFile] + " done. " + "#of total/good lines:" + n + "/" + nGoodLines;
            fpLog.writeLog(str, FileLog.PRIORITY_MUST_WRITE_MSG);
            
            
            System.out.println("    nPrecipTotal="+nPrecipTotal+" nTotalPixels="+n*NUM_SPOTS_PER_LINE+
                        " "+100*nPrecipTotal/(n*NUM_SPOTS_PER_LINE)+"%");
        }
        
        
        //generate ascending/descending flag
        fileL1C.generateAscendingDescendingFlagV1();
        
        //calculate on-board NEDT for each channel
        fileL1C.calculateNEDT();
        
        //generate LECT 
        fileL1C.generateLECT();
        
        //write data buffers to the netCDF file
        fileL1C.writeVariables();
        
        //create global attributes
        fileL1C.updateGlobalAttributes();
      
        //close level 1c file
        fileL1C.close();

        //for debug only
        updateScanlineCounter(nAll, nGoodLines, nBadRawCounts, nPrecipTotal / NUM_SPOTS_PER_LINE);
    }


}
