/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1C.AMSUALevel1CFileConcurrentNetCDF.java
* NAME
*   amsu.amsua.L1C.AMSUALevel1CFileConcurrentNetCDF.java 
* 
* PURPOSE
*   To convert a group of AMSU-A level 1b files to recalibrated level 1c radiance netcdf files.
* 
* DESCRIPTION
*   This is the main class of the AMSU-A radiance FCDR. 
*   It supports concurrent generating of level l1c radiance netcdf files using multi-threading technique. 
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   09/23/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   06/04/2013 Modified main(...) so that the AQUA AMSU-A radiance FCDR files will 
*              have the same FCDR variables as NOAA AMSU-A.
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*   01/24/2010 add AMSUALevel1cFileConfig to dedicate to system configuration
* INPUTS
*   args -- input control parameters for the main program. 
*           Please refer to AMSULevel1CFileConfig.java for the definition of 
*           different input parameters.
* OUTPUTS
*   level 1c radiance netcdf files. 
*
*@*****/
 
 
package amsu.amsua.L1C;

import amsu.amsua.*;
import amsu.amsua.L1B.*;
import amsu.amsua.limbcorrection.*;
import io.*;
import java.util.Arrays;
import pub.L1C.IMICACalibration;
import pub.variable.VariableControl;
import util.*;


public class AMSUALevel1CFileConcurrentNetCDF implements AMSUA{   

    static AMSUAL1CThreadNetCDF[] threadsL1C=null; //AMSU-A level 1c threads, each thread processes one level 1B file.
    static FileLog fpLog=null;                     //log file object
    
    /**
     * Constructor
     */
    public AMSUALevel1CFileConcurrentNetCDF(){
    }
    

    /**
     * Function: Initialize AMSU-A level 1c threads
     * @param n - # of level 1c files that can be process concurrently.
     * @param bAqua - TRUE means NASA EOS Aqua AMSU-A 
     *                FALSE means NOAA AMSU-A
     */
    static void initThreads(int n, boolean bAqua){        
        threadsL1C=new AMSUAL1CThreadNetCDF[n];
        for (int i=0;i<n;i++){
            threadsL1C[i]=new AMSUAL1CThreadNetCDF();
        }
    }
 
 
    /**
     * Main function for the AMSU-A level 1c FCDR
     * @param args  -- input parameters
     * @throws InterruptedException 
     */
     
    public static void main(String[] args) throws InterruptedException {

        //1.Set-up which variables will be generated in the level 1c netcdf file
        //VariableControl.setPredefinedControl(VariableControl.controlL1CDefault);
        VariableControl.setAMSUALevel1R2OControl(); 

        //2. customize AMSU-A FCDR variable names
        AMSUALevel1CFileNetCDF.customizeVariableNames();
        
        //3. processing program input parameters
        int n=args.length;                                           
        System.out.println("Number of Input Parameters: "+n);
        for (int i=0;i<n;i++){
            System.out.println("arg "+i+"= "+args[i]);
        }
        AMSUALevel1CFileConfig.initStaticVariables(args);             
       
        //4. set control flags for Aqua AMSU-A
        boolean bAqua=AMSUALevel1CFileConfig.isAqua();
        
        //5 Read Ancillary Data
        //5.1load 2.5 x 2.5 degree gridded land type data
        LandType landType=LandType.createLandType(AMSUALevel1CFileConfig.dirCoefficients);
        AMSUALevel1BLine.setLandTypeMask(landType);

        //5.2 initialize AMSU LIMB Correction 
        AMSUALimbCorrection.init(AMSUALevel1CFileConfig.limbCorrectionScheme, 
                                    AMSUALevel1CFileConfig.dirCoefficients, 
                                    AMSUALevel1CFileConfig.satIdInternal);
        
        //5.3 Read in antenna pattern correction
        if ( AMSUALevel1CFileConfig.correctAntennaPattern!=0){
            AMSUAAntennaPatternCorrection.init(AMSUALevel1CFileConfig.dirCoefficients);
        }
        
        //5.4 Read in IMICA calibration coefficients
        IMICACalibration.initSNOCalibration(AMSUALevel1CFileConfig.dirCoefficients+"//SNO//normal//",
                 NUM_CHANNELS,6); //6 means that there are totally AMSU-A intruments: NOAA-15 through -18, Metop-A, Aqua
        
        
        //6. preprocessing input level 1b files
        String[] files;       
        java.io.File fileDirIn=new java.io.File(AMSUALevel1CFileConfig.dirIn);  
        if (AMSUALevel1CFileConfig.filelistIn==null){
            files=fileDirIn.list(); 
        }else{
            //filelistIn obtained from input parameter
            files=AMSUALevel1CFileConfig.readInputFiles(AMSUALevel1CFileConfig.filelistIn);
        }
        if (files==null || files.length==0){
            System.out.println("Zero Input Level 1B files in "+AMSUALevel1CFileConfig.dirIn);
            System.exit(-1);
        }
        System.out.println("Number of Input Level 1B files = "+files.length);             
        Arrays.sort(files);        
        
        
        //7. Initialize level 1c file indices
        //For NOAA AMSU-A, one level 1b file will be converted to 1 level 1c file
        //For NASA Aqua AMSU-A, multiple level 1c files will be combined to generate 1 level 1c file
        AMSUAL1CFileIndex indexL1C=new AMSUAL1CFileIndex();
        indexL1C.generateIndex(AMSUALevel1CFileConfig.dirIn,files, bAqua);
        int nFilesL1C=indexL1C.getNumberOfFiles();
        AMSUAL1CThreadNetCDF.setL1CFileIndex(indexL1C); 

        
        //8.Initialize log file
        fpLog=AMSUALevel1CFileConfig.getFileLog();
        fpLog.writeLog("\r\n\r\n"+AMSUALevel1CFileConfig.dirOut, FileLog.PRIORITY_MUST_WRITE_MSG);
        
        //9. init threads for parallel processing
        initThreads(AMSUALevel1CFileConfig.nThreadMax,bAqua);
        int nIdles=AMSUALevel1CFileConfig.nThreadMax;
        int index=0;
        
        //10. Start level 1b to level 1c conversion concurrently
        while (index<nFilesL1C || nIdles<AMSUALevel1CFileConfig.nThreadMax){
            nIdles=0;
            for (int i = 0; i < AMSUALevel1CFileConfig.nThreadMax; i++) {
                if ( threadsL1C[i].isAlive()){
                   
                }else {//not alive can run new files
                    if (index < nFilesL1C){ 
                        if (indexL1C.getNumberOfL1BFiles(index)>0){
                            if ( threadsL1C[i].getState()==Thread.State.TERMINATED){
                                 threadsL1C[i]=new AMSUAL1CThreadNetCDF();                                                        
                            }
                            threadsL1C[i].setL1CFileIndex(index);
                            threadsL1C[i].start();
                        }
                        index++;
                    }
                    
                     
                    if (!threadsL1C[i].isAlive()){
                        nIdles++;                                               
                    }  
            
                }
            }
        }
        
        
        //11. Processing statistics
        String str;       
        str="nFiles="+files.length+" index="+index+" nIdles="+nIdles+
            " nThreadMax="+AMSUALevel1CFileConfig.nThreadMax;
        fpLog.writeLog(str+"\r\n", FileLog.PRIORITY_MUST_WRITE_MSG);
        float nScanlineTotal=AMSUAL1CThreadNetCDF.getTotalScanlines();
        float nScanlineTotalGood=AMSUAL1CThreadNetCDF.getTotalGoodScanline();
        float percentGood=100.0f*nScanlineTotalGood/nScanlineTotal;
        str="nScanlineTotal="+nScanlineTotal+" nGood="+nScanlineTotalGood+" "+percentGood+"% good";
        fpLog.writeLog(str, FileLog.PRIORITY_MUST_WRITE_MSG);      
 
        //close log file
        AMSUALevel1CFileConfig.closeLogFile();        
    } 
    
}
