/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1C.AMSUALevel1CFileConfig.java
* NAME
*   amsu.amsua.L1C.AMSUALevel1CFileConfig.java 
* 
* PURPOSE
*   To set up global configuration flags for AMSU-A level 1c processing.
* 
* DESCRIPTION
*   This class supports AMSU-A radiance FCDR processing system configuration.
*   It specifies:
*   (1) input directories for level 1b data and ancillary data;
*   (2) output root directory for level 1c netCDF files;
*   (3) what variables will be generated;
*   (4) other configuration parameters to control the running of the main 
*       program.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   01/24/2010
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
* 
*@*****/
 
 
package amsu.amsua.L1C;

import amsu.amsua.*;
import config.*;
import pub.variable.VariableControl;


public class AMSUALevel1CFileConfig extends ConfigPub implements AMSUA {
    static final String[] namesCalibrationMethod={"Linear","Operational","SNO"}; //names of calibration methods
    
    public static String satelliteName="n15";  //satellite name
    public static int year=2010;               //year of satellite observation
    public static String dirIn="";             //input directory for level 1b files
    public static String filelistIn=null;      //pathname of filelist for level 1b files, support monthly update
    public static String dirOut="";            //output directory for level 1c netcdf files
    public static String dirCoefficients="";   //directory for finding ancillary data
    static boolean bOverwrite=true; //wether to generate a level 1c file is a file already exist in the output folder    
    public static byte checkPrecipitation=1;   //1- check precipitation 0 - not check
    public static byte correctAntennaPattern=0; //whether perform antenna pattern correction
    public static int indexMUGroup=1;           //index of SNO calibration coefficients that will be used
    static public boolean bUseConvolvedCalibrationCounts=false; //whether averging calibration counts over multiple scan lines, default false
    public static byte limbCorrectionScheme=1;   //limb correction scheme
    public static int nThreadMax=1;              //# of level 1c files that can generated concurrently

   
    public static int satIdInternal=-1; //internal satellite id, not from input parameter, but set using satelliteName
    static boolean bLinux=false;  //run under linxu or windows, for debug only
    
    static public boolean isAqua(){
        return satelliteName.compareToIgnoreCase("Aqua")==0;
    }
    
    /**
     * Set all AMSU-A configuration flags:
     *      default flags;
     *      flags from input parameters;
     * Processing input parameters
     * @param args 
     */
    static public void initStaticVariables(String[] args) {
        int n = args.length;
         
        filelistIn=null;
        satelliteName="metopa";             
        year=2007;
        if (bLinux){//set default linux directory for debuging purpose 
                //dirIn="/net/orbit001x/disk008/pub/amsu-a/aqua/2010/";
                dirIn="/net/orbit186l/home/wwang/test/data/aqua/2011/";
                //dirOut="/net/orbit235l/disk1/pub/wwang/amsua/proc_level1c_netcdf/debug/2010/";
                dirOut="/net/orbit186l/home/wwang/test/data/";

                //dirCoefficients="/net/orbit235l/disk1/pub/wwang/amsua/proc_level1c_netcdf/bin/";
                dirCoefficients="/data/data040/wwang/code/amsua/proc_level1c_netcdf/bin/";
        }else{
                //set default Windows directory for debuging purpose
                String rootData="C:\\Users\\wwang\\my.research\\data\\amsua\\";
                dirIn=rootData+"L1B\\"+satelliteName+"\\"+year;
                dirOut=rootData+"L1C\\"+satelliteName+"\\"+year+"\\";
                //dirCoefficients="d:\\data\\amsu\\coeffs\\wwang"; 
                dirCoefficients=rootData+"coefficients\\";
        }
 
        //parse and set control flags using input parameters
        setCommandLineInputParameters(args);
        
        //set internal satellite ID 
        satIdInternal=noaa.ObservationPlatforms.NOAASatelliteName2InternalSatelliteID(satelliteName);
        
        //list contral flags
        printConfig();

        //open log file, duplicated?
        String pathnameLog=dirOut+"//..//"+satelliteName+"."+"l1c.log.txt";
        openLogFile(pathnameLog);
 
    }
    
    
    /**
     * Set one configuration flag.
     * @param strName  -- input parameter name
     * @param strValue -- input parameter value
     * @return 0-success other values-unrecognized input parameter
     */
    static public int setInputParameter(String strName,String strValue) {
        int flag = 0; //0 -- parsed other values -- failed
        
        if (ConfigPub.setInputParameter(strName, strValue)==0){
            //process in super class
        }else if (strName.compareToIgnoreCase("dirCoefficients") == 0) {
            dirCoefficients = strValue;
        } else if (strName.compareToIgnoreCase("dirOut") == 0) {
            dirOut = strValue;
        } else if (strName.compareToIgnoreCase("dirIn") == 0) {
            dirIn = strValue;
        } else if (strName.compareToIgnoreCase("satelliteName") == 0) {
            satelliteName =strValue;
        } else if (strName.compareToIgnoreCase("correctAntennaPattern") == 0) {
            correctAntennaPattern = Byte.parseByte(strValue);
        } else if (strName.compareToIgnoreCase("limbCorrectionScheme") == 0) {           
            limbCorrectionScheme= Byte.parseByte(strValue);
        } else if (strName.compareToIgnoreCase("checkPrecipitation") == 0) {
            //using latitude Min,Max            
            checkPrecipitation=Byte.parseByte(strValue);
        } else if (strName.compareToIgnoreCase("indexMUGroup") == 0) {
            indexMUGroup=Byte.parseByte(strValue);
        } else if (strName.compareToIgnoreCase("nThreadMax") == 0) {
             nThreadMax=Byte.parseByte(strValue);
             if (nThreadMax<=0){
                fpLog.writeLog("Error in nThreaxMax="+nThreadMax+" nThreadMax is set to 1", fpLog.PRIORITY_OPTINAL_MSG);
                nThreadMax=1;
             }
        } else if (strName.compareToIgnoreCase("filelistIn") == 0) {
            filelistIn=strValue;
        } else if (strName.compareToIgnoreCase("bGenerateGeometryVariables") == 0) {
           //bGenerateGeometryVariables = Boolean.parseBoolean(strValue);
            boolean b=Boolean.parseBoolean(strValue);
            VariableControl.setControl(VariableControl.CONTROL_GEOMETRY, b);
        } else if (strName.compareToIgnoreCase("bGenerateRawCountProduct") == 0) {
            boolean b=Boolean.parseBoolean(strValue);
            VariableControl.setControl(VariableControl.CONTROL_RAWCOUNTS, b);
            
          // bGenerateRawCountProduct = Boolean.parseBoolean(strValue);
        } else if (strName.compareToIgnoreCase("bGenerateOPRProduct") == 0) {
//           bGenerateOPRProduct = Boolean.parseBoolean(strValue);
            boolean b=Boolean.parseBoolean(strValue);
            VariableControl.setControl(VariableControl.CONTROL_T_OPR, b);
            VariableControl.setControl(VariableControl.CONTROL_T_OPR_LIMB, b);
            
        } else if (strName.compareToIgnoreCase("bGenerateSNOProduct") == 0) {
           //bGenerateSNOProduct = Boolean.parseBoolean(strValue);
            boolean b=Boolean.parseBoolean(strValue);
            VariableControl.setControl(VariableControl.CONTROL_T_SNO, b);
            VariableControl.setControl(VariableControl.CONTROL_T_SNO_LIMB, b);
            
        } else if (strName.compareToIgnoreCase("bGenerateLinearCalProduct") == 0) {
           boolean b=Boolean.parseBoolean(strValue);
            VariableControl.setControl(VariableControl.CONTROL_T_LINEAR, b);
            VariableControl.setControl(VariableControl.CONTROL_T_LINEAR_LIMB, b);
           //bGenerateLinearCalProduct = Boolean.parseBoolean(strValue);
        }else if (strName.compareToIgnoreCase("bLinux") == 0) {
           bLinux = Boolean.parseBoolean(strValue);
        }else if (strName.compareToIgnoreCase("bOverwrite") == 0) {
           bOverwrite = Boolean.parseBoolean(strValue);
        }else if (strName.compareToIgnoreCase("bUseConvolvedCalibrationCounts")==0){
            bUseConvolvedCalibrationCounts=Boolean.parseBoolean(strValue);
        }else{
            flag = -1;
        }
        return flag;
    }

    /**
     * Parse and input level 1c processing configuration flags from input parameters
     * @param args - raw input parameters from main program  
     */    
    public static void setCommandLineInputParameters(String[] args) {
        String strName="", strValue="";
        String[] strsNameValue = {"", ""};        int nargs = args.length;
        System.out.println("Num of command line Parameters=" + nargs);
        for (int i=0;i<nargs;i++){
            seperateInputParameterNameValue(args[i], strsNameValue);
            strName = strsNameValue[0];
            strValue = strsNameValue[1];
    
            if (strName.compareToIgnoreCase("") == 0) {
                //annotation, not parsed
            }else  if (setInputParameter(strName,strValue)==0){
            } else{
               System.out.println("ConfigKernel:: Invalid input parameter="+args[i]);
            }
        }
        printConfig();

    }

    /**
     * Print configuration flags to screen 
     */
    public static void printConfig(){
        System.out.println("NOAA/NESDIS/STAR AMSU-A Recalibrated Level 1C Product");
        System.out.println("Satellite: "+satelliteName);
        System.out.println("satIdInternal: "+satIdInternal);
        System.out.println("dirIn: "+dirIn);
        System.out.println("dirOut: "+dirOut);
        System.out.println("dirCoefficients= "+dirCoefficients);
        System.out.println("filelistIn="+filelistIn);

        System.out.println("correctAntennaPattern= "+ correctAntennaPattern);
        System.out.println("checkPrecipitation= "+checkPrecipitation);
        System.out.println("IndexMUGroup (for SNO only)="+ indexMUGroup);   
                  
        System.out.println("limbCorrectionScheme= "+  limbCorrectionScheme);
        System.out.println("bLinux="+bLinux);
        System.out.println("bOverwrite="+bOverwrite);
        System.out.println("bUseConvolvedCalibrationCounts="+bUseConvolvedCalibrationCounts);
        System.out.println("nThreads="+nThreadMax);
 
        VariableControl.printStatus();
     }  

}

