/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1C.AMSUALevel1CFileNetCDF.java
* NAME
*   amsu.amsua.L1C.AMSUALevel1CFileNetCDF.java 
* 
* PURPOSE
*   To generate AMSU-A level 1c radiance file in netCDF format.
* 
* DESCRIPTION
*   This class provides methods for creating or modifying global attributes and 
*   variables of level 1c netcdf file.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   09/23/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   06/20/2013 Modified variable attriubtes of pixel level quality flags. 
*   06/05/2013 Overrided methods in super classes to generate FCDR variables that meet
*              NCDC's requirement. Following methods were overrided:
*              createDimensions(...)
*              getCDRVariableNames()
*              writeSceneTemperatureVariables()
*              createQCVariables()
*              createSceneTemperatureVariables()
*   05/20/2013 Initial Version prepared for NCDC
*   05/14/2013 1. Added generateUncertaintyData() to generate uncertainty global 
*                   attribute 
*              2. Added NOAA-15 NEdT constant (nedtsN15) from Mo 1999 to generate 
*                   uncertainty global attribute modified updateGlobalAttributes() 
*                   to update the uncertainty global attribute
*   04/30/2013 Added AMSUALevel1CGlobalAttributes
*   10/22/2011 Added updateTimeGlobalAttributes() method to modify global time 
*              attribute in the end of a L1C file generation
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 
package amsu.amsua.L1C;

import amsu.amsua.*;
import pub.L1C.*;
import pub.QualityControl;
import pub.attribute.CDRVariableAttributes;
import pub.attribute.GlobalAttributesWWANG;
import pub.attribute.VariableAttributesCF;
import pub.attribute.VariableAttributesWWANG;
import pub.variable.CDRVariablesL1C;
import pub.variable.VariableControl;
import ucar.ma2.Array;
import ucar.ma2.DataType;
import ucar.ma2.Index2D;
import ucar.ma2.Index3D;
import ucar.nc2.*;
import util.*;


public class AMSUALevel1CFileNetCDF extends L1CFileNetCDFWritable implements AMSUA{
   static int INDEX_IMICA_CHANNEL_START=3;
   static int INDEX_IMICA_CHANNEL_END=14;
   
        //NEdt from Mo's papers
   //standard names for fcdr variables
   static final String[] namesTAMSUA={
            "fcdr_brightness_temperature_IMICA",
            "fcdr_brightness_temperature_IMICA_limbcorr",
            "fcdr_brightness_temperature_OPR",
            "fcdr_brightness_temperature_OPR_limbcorr",
            "fcdr_brightness_temperature_LINEAR",
            "fcdr_brightness_temperature_LINEAR_limbcorr",
        };
   static final String strLimbCorrection= ", with limb correction using Goldberg (2001) method";
   //long names for fcdr variables
   static final String[] longNamesTAMSUA = {
            "IMICA recalibrated brightness temperatures at scan positions",
            "IMICA recalibrated brightness temperatures at scan positions"+strLimbCorrection,
            "NOAA operational calibrated brightness temperatures at scan positions",
            "NOAA operational calibrated brightness temperatures at scan positions"+strLimbCorrection,
            "linear calibrated brightness temperatures at scan positions",
            "linear calibrated brightness temperatures at scan positions"+strLimbCorrection,
        };
   //standard names for scan level quality flags
   static final String[] standardNamesCFlags={
            "quality_flags_scan_IMICA",
            "quality_flags_scan_OPR",
            "quality_flags_scan_LINEAR",
        };
   //standard names for pixel level quality flags
   static final String[] standardNamesCFlagPixels={
            "quality_flags_pixel_IMICA",
            "quality_flags_pixel_OPR",
            "quality_flags_pixel_LINEAR",
        };

   
   
    AMSUALevel1CLineNetCDF lineL1C=new AMSUALevel1CLineNetCDF();  //class for level 1c scan line       
    AMSUALevel1CGlobalAttributes globalAttributesAMSUA=null;  //global attribute objects
    
    
    /**
     * Constructor
     */
    public AMSUALevel1CFileNetCDF() {
        super(NUM_SPOTS_PER_LINE, NUM_CHANNELS);
                
        indexLine = 0;
        dataCFlags = new Array[VAR_CFLAGs.length];
        dataCFlagPixel = new Array[VAR_CFLAGs.length];
        dataTsNew = new Array[VAR_Ts.length];
      
        globalAttributesAMSUA=new AMSUALevel1CGlobalAttributes();
        globalAttributes=globalAttributesAMSUA;
    }
    
    /**
     * Set variable names and long names for AMSU-A radiance FCDR
     */
    static public void customizeVariableNames(){
        VAR_YEAR="time_year";
        VAR_DAYOFYEAR="time_day_of_year";
        VAR_MILLISECONDOFDAY="time_millisecond_of_day";
        
        VAR_LATITUDE="latitude";
        VAR_LONGITUDE="longitude";
        
        VAR_SOLARZENITH = "solar_zenith_angle";
        VAR_SENSORZENITH = "sensor_zenith_angle";
        VAR_LOCALAZIMUTH = "local_azimuth_angle";

        for (int i=0;i<standardNamesCFlags.length;i++){
            VAR_CFLAGs[i]=standardNamesCFlags[i];
            VAR_CFLAGPIXELs[i]=standardNamesCFlagPixels[i];
        }
        
        for (int i=0;i<namesTAMSUA.length;i++){
            VAR_Ts[i]=namesTAMSUA[i];
            longnamesVAR_Ts[i]=longNamesTAMSUA[i];
        } 
        
        VAR_WARMTARGETTEMPERATURE="warm_target_temperatures";
        
        VAR_ALTITUDE="spacecraft_altitude";
    }  
    
    
     /* 
     * Modify global time attributes of level 1c file
     */
    public void updateGlobalAttributes() {
        globalAttributes.updateGeospatialAttributes(ncfileWritable,
                statLongitude.min, statLongitude.max, statLatitude.min, statLatitude.max);

        globalAttributes.updateTimeConverageAttributes(ncfileWritable, timeStart, timeEnd);
        
        globalAttributes.setValue(GlobalAttributesWWANG.GA_NAME_WWANG_Uncertainty_IMICA,generateUncertaintyData(0) );
        globalAttributes.setValue(GlobalAttributesWWANG.GA_NAME_WWANG_Uncertainty_OPR,generateUncertaintyData(1) );
        globalAttributes.update(GlobalAttributesWWANG.GA_NAME_WWANG_Uncertainty_IMICA, ncfileWritable);
        globalAttributes.update(GlobalAttributesWWANG.GA_NAME_WWANG_Uncertainty_OPR, ncfileWritable);
   }

    /**
     * Generate values of uncertainty global attributes
     * @param indexCalibration - index of calibration methods
     * @return a string for values of uncertainty global attributes
     */
    String generateUncertaintyData(int indexCalibration){
       
        String str="";
        float[] nedt=statNEDT[indexCalibration];
        int chStart=0,chEnd=14;
        if (indexCalibration==0){
            chStart=3;
            chEnd=13;
        }
        for (int channel=chStart;channel<=chEnd;channel++){
            str+="ch"+(channel+1)+": ";
//            str+=String.format("%5.3f", nedt[channel]+0.5)+" K";
            str+=String.format("%5.3f", nedtsN15Specified[channel]+0.5)+"K";
            if (channel!=chEnd)
                str+=" ";
        }

        return str;
    }
    
    /**
     * Generate AMSU-A level 1c netcdf file
     * @param pathname -- full path name for a AMSU-A level 1c file
     * @param indexL1C -- AMSUA level 1c file Index Data
     * @param index -- index of current level 1c file 
     * @return AMSUALevel1CFileNetCDF object
     */
    public AMSUALevel1CFileNetCDF create(String pathname, AMSUAL1CFileIndex indexL1C,int index){
    
        nLines = indexL1C.getNumberOfLines(index);
        timeStart = indexL1C.getTimeStart(index);
        timeEnd = indexL1C.getTimeEnd(index);
        int satidInternal = AMSUALevel1CFileConfig.satIdInternal;
        System.out.println("nLine="+nLines);

        try{
            indexLine=0;
    
            //create netcdf file
            ncfileWritable=NetcdfFileWriteable.createNew(pathname,false);

            //create global attributes, move to before closing file
             globalAttributesAMSUA.setGlobalAttributes(this,indexL1C,index);
             globalAttributes.create(ncfileWritable);
            
            //create dimensions
            createDimensions(NUM_SPOTS_PER_LINE,nLines,NUM_CHANNELS,3);
            
            
            varsTime.create(ncfileWritable,yDim,timeISO8601Dim); //createTimeVariables();
            createGeolocationVariables();
            createQCVariables();
            createSceneTemperatureVariables();
            createGeometryVariables();
            createRawCountVariables();
            createWTVariables();
            createASDVariable();
            createAltitudeVariable();
            createAttitudeVariable();
            createLECTVariable();
            createCalibrationCoefficientsVariablesAMSUA();
            
            ncfileWritable.create();
        }catch(java.io.IOException e){
            System.out.println("Create new netcdf l1c file failed: "+pathname);
            return null;
        }
 
        return this;
    }
    
    /**
     * 
     * @return the current level 1c file scan line
     */
    public AMSUALevel1CLineNetCDF getLine(){
        return lineL1C;
    }

     /**
     * Support alternative geolocation data. for research only
     * @param pathnameL1C - 
     * @param pathnameNewGeolocation
     * @return 0 if successful, otherwise return -1.
     */
    public int replaceGeolocation(String pathnameL1C, String pathnameNewGeolocation){        
        io.FileRandom fpNewGeo=new io.FileRandom();
        int nLines;
        fpNewGeo.open(pathnameNewGeolocation,"r");        
        
        openExistingForUpdate(pathnameL1C);
        readVariables0();
        
        //read Attitude 
        int iroll,ipitch,iyaw,inscan;
        byte[] buf=new byte[16];
        int offset=0;
        fpNewGeo.read(buf);
        ipitch=ParserUtil.combineInteger4(buf[offset+3], buf[offset+2],buf[offset+1],buf[offset+0]);
        offset+=4;        
        iroll=ParserUtil.combineInteger4(buf[offset+3], buf[offset+2],buf[offset+1],buf[offset+0]);
        offset+=4;
        iyaw=ParserUtil.combineInteger4(buf[offset+3], buf[offset+2],buf[offset+1],buf[offset+0]);
        offset+=4;
        inscan=ParserUtil.combineInteger4(buf[offset+3], buf[offset+2],buf[offset+1],buf[offset+0]);
        
        nLines = getNumberOfRecords();
        int ilat, ilon, isensorZenith, isca;
        float lat,lon,zenithangle;
        float f=1e-5f;
        
        int[] shape2D={nLines,NUM_SPOTS_PER_LINE};
        Index2D index2D=new Index2D(shape2D);
        //int[] shape2D1={nLines,3};
        //Index2D index2D1=new Index2D(index2D1);
        for (int i = 0; i < nLines; i++) {
            for (int ifov = 0; ifov < NUM_SPOTS_PER_LINE; ifov++) {
                index2D.set(i,ifov);
                fpNewGeo.read(buf);
                offset=0;
                ilat = ParserUtil.combineInteger4(buf[offset+3], buf[offset+2],buf[offset+1],buf[offset+0]);
                offset+=4;
                ilon = ParserUtil.combineInteger4(buf[offset+3], buf[offset+2],buf[offset+1],buf[offset+0]);
                offset+=4;
                isensorZenith = ParserUtil.combineInteger4(buf[offset+3], buf[offset+2],buf[offset+1],buf[offset+0]);
                offset+=4;
                isca =ParserUtil.combineInteger4(buf[offset+3], buf[offset+2],buf[offset+1],buf[offset+0]);
                
                lat=f*ilat;
                lon=f*ilon;
                zenithangle=f*isensorZenith; 
                if (ifov<15)
                    zenithangle=-zenithangle;
                
                dataLatitude.setFloat(index2D,lat);
                dataLongitude.setFloat(index2D,lon);
                dataSensorZenith.setFloat(index2D,zenithangle);
//                this.dataLatitude.setFloat()
            }
        }
        
        try{
            ncfileWritable.write(pub.variable.CDRVariablesL1C.VAR_LATITUDE, dataLatitude);
            ncfileWritable.write(pub.variable.CDRVariablesL1C.VAR_LONGITUDE, dataLongitude);
            ncfileWritable.write(VAR_SENSORZENITH, dataSensorZenith);
            
        }catch (java.io.IOException e){
            
        }catch (ucar.ma2.InvalidRangeException e){
            
        }

        close();
        fpNewGeo.close();
        return 0;
    }

    /**
     * Define dimensions that are used to define different variables in AMSU-A level 1c
     * radiance FCDR netCDF file. 
     * @param NUM_SPOTS_PER_LINE - number of scans in each scan line
     * @param nLines - total number of scan lines
     * @param nChannels - total number of channels
     * @param nWTs - total number of warm target temperatures
     */
    @Override
    protected void createDimensions(int NUM_SPOTS_PER_LINE, int nLines, int nChannels, int nWTs) {
        nSpotsPerLine = NUM_SPOTS_PER_LINE;
        this.nChannels = nChannels;
        this.nLines = nLines;
        this.nWTs = nWTs;

        //init dimensions
        xDim = ncfileWritable.addDimension("pixelDim", NUM_SPOTS_PER_LINE);
        yDim = ncfileWritable.addDimension("nscanDim", nLines);
        channelDim=new Dimension("channelDim",nChannels);
        //channelDim = ncfileWritable.addDimension("channelDim", nChannels);
        wtDim = ncfileWritable.addDimension("wtDim", nWTs);
        if (VariableControl.isWriteSatelliteAttitude()){
            attitudeDim = ncfileWritable.addDimension("attitudeDim", 3);
        }else{
            attitudeDim=null;
        }
        if (VariableControl.isWriteCalibrationCoefficients()){
            calcoefDim = ncfileWritable.addDimension("calcoefDim", DIM_CALIBRATION_COEFFICIENTS);
        }else{
            calcoefDim=null;
        }
        timeISO8601Dim=ncfileWritable.addDimension("timeISO8601Dim", 20);

        dims1D.add(yDim);

        dims2D.add(yDim);
        dims2D.add(xDim);

        dims3D.add(yDim);
        dims3D.add(xDim);
        dims3D.add(channelDim);

        shape1D = new int[1];
        shape1D[0] = yDim.getLength();

        shape2D = new int[2];
        shape2D[0] = yDim.getLength();
        shape2D[1] = xDim.getLength();
        index2D = new Index2D(shape2D);

        shape2DWT = new int[2];
        shape2DWT[0] = yDim.getLength();
        shape2DWT[1] = wtDim.getLength();
        index2DWT = new Index2D(shape2DWT);
/*
//        if (VariableControl.isWriteCalibrationCoefficients()) {
            shape3DCalCoef = new int[3];
            shape3DCalCoef[0] = yDim.getLength();
            shape3DCalCoef[1] = DIM_CALIBRATION_COEFFICIENTS;
            shape3DCalCoef[2] = channelDim.getLength();
            index3DCalCoef = new Index3D(shape3DCalCoef);
//        }
        shape2DRawCount = new int[2];
        shape2DRawCount[0] = yDim.getLength();
        shape2DRawCount[1] = channelDim.getLength();
        index2DRawCount = new Index2D(shape2DRawCount);

*/
        shape3D = new int[3];
        shape3D[0] = yDim.getLength();
        shape3D[1] = xDim.getLength();
        shape3D[2] = channelDim.getLength();
        index3D = new Index3D(shape3D);

        //for calculating NEDT
        wtConvertedFromRawCounts=new float[NUM_CALIBRATION_METHODS][nLines][nChannels];
    }

 
    /**
     * To generate variable name for individual channels, based on NCDC's
     * requirement.
     * 
     * @param name0 - variable name for STAR
     * @param channel - channel index
     * @return variable name for individual channels
     */
    String getVaraibleNameNCDC(String name0,int channel){
        return name0+"_ch"+(channel+1);
    }
    
    /**
     * Crate scene temperature variables for AMSU-A radiance FCDR,
     * one channel in one netCDF variable based on NCDC's requirement
    */
    @Override
    protected void createSceneTemperatureVariables() {
        String varName;
        dataTsNew = new Array[VAR_Ts.length];
        for (int i = 0; i < VAR_Ts.length; i++) {
            if (VariableControl.isWriteT(i)) { //varControl & controlTs[i]) !=0){
                //create fcdr variable
                for (int channel = INDEX_IMICA_CHANNEL_START; channel < INDEX_IMICA_CHANNEL_END; channel++) {
                    varName=getVaraibleNameNCDC(VAR_Ts[i],channel);

                    ncfileWritable.addVariable(varName, dataTypeTemperature, dims2D);

                    //set variable attributes
                    CDRVariableAttributes attr = new CDRVariableAttributes();
                    attr.setLongName(VariableAttributesCF.longName0 + 
                                    "AMSU-A channel "+(channel+1)+" "+
                                    pub.variable.CDRVariablesL1C.longnamesVAR_Ts[i]);
                    attr.setStandardName("brightness_temperature");
                    attr.setUnits("kelvin");
                    if (dataTypeTemperature == DataType.SHORT) {
                        attr.modifyRequiredFlag(VariableAttributesWWANG.VA_NAME_WWANG_SCALE_FACTOR, true);
                        attr.setScaleFactor(1.0f / scalingFactorTemperature);
                        short[] validRange = {(short) (QualityControl.TEMPERATURE_RANGE_MIN * scalingFactorTemperature),
                            (short) (QualityControl.TEMPERATURE_RANGE_MAX * scalingFactorTemperature)};

                        attr.setValidMax(validRange[1]);
                        attr.setValidMin(validRange[0]);
                        attr.setValidRange(validRange);
                        attr.setFillValue(QualityControl.missing_value_short);
                        attr.setMissingValue(QualityControl.missing_value_short);


                    } else {
                        float[] validRange = {QualityControl.TEMPERATURE_RANGE_MIN, QualityControl.TEMPERATURE_RANGE_MAX};
                        attr.setValidMax(validRange[1]);
                        attr.setValidMin(validRange[0]);
                        attr.setValidRange(validRange);
                        attr.setFillValue(QualityControl.missing_value);
                        attr.setMissingValue(QualityControl.missing_value);
                    }

                    attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_COORDINATES, true);
                    attr.setValue(VariableAttributesCF.VA_NAME_CF_COORDINATES, CDRVariablesL1C.VAR_LONGITUDE + " " + CDRVariablesL1C.VAR_LATITUDE);

                    attr.create(ncfileWritable, varName);
                }

            }//end indiviudal channel

            //allocate data buffer for the temperature variable
            dataTsNew[i] = Array.factory(dataTypeTemperature, shape3D);
        }
    }

    /**
     * Create level 1c scan line and pixel level quality control variables
     * for AMSU-A radiance FCDR
     */
    @Override
    protected void createQCVariables() {
        String[] longnamesScan = {
            "scan-line quality flags for the IMICA recalibrated brightness temperatures",
            "scan-line quality flags for the NOAA operational calibrated brightness temperatures",
            "scan-line quality flags for the linear calibrated brightness temperatures",
            };
       String[] longnamesPixel = {
            "pixel quality flags for the IMICA recalibrated brightness temperatures",
            "pixel quality flags for the NOAA operational calibrated brightness temperatures",
            "pixel quality flags for the linear calibrated brightness temperatures",
            };
       String strNotePixelLevel="Definition of pixel quality flags:\n"+
               "bit 0 - bad pixel due to bad scan line;\n"+
               "bit 1 - bad pixel due to bad geolcation;\n"+
               "bit 2 - bad pixel due to precipitation;\n"+
               "bit 3 - bad pixel due to calibration counts;\n"+
               "bits 4-7: not used.";
       
        dataCFlags = new Array[3];
        dataCFlagPixel = new Array[3];
        String varName;
        byte[] flagValuesScan={0,1};
        byte[] flagValuesPixel={0,1,2,4,8};
        byte[] validRangePixel={0,15};
        
        for (int i = 0; i < 3; i++) {
            if (VariableControl.isWriteT(i * 2) || VariableControl.isWriteT(i * 2 + 1)) {
                varName=VAR_CFLAGs[i];
                ncfileWritable.addVariable(varName, DataType.BYTE, dims1D);
               //set variable attributes
                CDRVariableAttributes attr = new CDRVariableAttributes();
                attr.setLongName(longnamesScan[i]);
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, false);
                //attr.setStandardName(varName);
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_UNITS, false);
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FLAG_VALUES,true);
                attr.setValue(VariableAttributesCF.VA_NAME_CF_FLAG_VALUES, flagValuesScan);

                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FLAG_MEANINGS,true);
                attr.setValue(VariableAttributesCF.VA_NAME_CF_FLAG_MEANINGS,QualityControlL1C.getL1CQCStringLine());
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FILLVALUE,true);
                attr.setValue(VariableAttributesCF.VA_NAME_CF_FILLVALUE,QualityControl.missing_value_byte);
                
                attr.create(ncfileWritable, varName);
             
                for (int channel = INDEX_IMICA_CHANNEL_START; channel < INDEX_IMICA_CHANNEL_END; channel++) {
                    //create pixel level quality flags variable
                    varName = getVaraibleNameNCDC(VAR_CFLAGPIXELs[i],channel); 
                    ncfileWritable.addVariable(varName, DataType.BYTE, dims2D);
                    //set variable attributes
                    attr = new CDRVariableAttributes();
                    attr.setLongName("AMSU-A channel "+(channel+1)+" "+longnamesPixel[i]);
                    attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, false);
                    attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_UNITS, false);
                    
                    //attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FLAG_VALUES, true);
                    //attr.setValue(VariableAttributesCF.VA_NAME_CF_FLAG_VALUES, flagValuesPixel);
                    //attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FLAG_MEANINGS, true);
                    //attr.setValue(VariableAttributesCF.VA_NAME_CF_FLAG_MEANINGS, QualityControlL1C.getL1CQCStringChannelPixel());

                    attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_VALID_RANGE,true);
                    attr.setValidRange(validRangePixel);
                    attr.modifyRequiredFlag(VariableAttributesWWANG.VA_NAME_WWANG_NOTE,true);
                    attr.setValue(VariableAttributesWWANG.VA_NAME_WWANG_NOTE, strNotePixelLevel);
                            
                    //attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FLAG_VALUES, true);
                    //attr.setValue(VariableAttributesCF.VA_NAME_CF_FLAG_VALUES, flagValuesPixel);
                    
                    
                    attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_COORDINATES, true);
                    attr.setValue(VariableAttributesCF.VA_NAME_CF_COORDINATES, CDRVariablesL1C.VAR_LONGITUDE + " " + CDRVariablesL1C.VAR_LATITUDE);
                    attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FILLVALUE, true);
                    attr.setValue(VariableAttributesCF.VA_NAME_CF_FILLVALUE, QualityControl.missing_value_byte);

                    attr.create(ncfileWritable, varName);
                }
                
                dataCFlags[i] = Array.factory(DataType.BYTE, shape1D);
                dataCFlagPixel[i] = Array.factory(DataType.BYTE, shape3D);
            }
        }


    }

    /*
     * Wrtie sence temperature variables for AMSU-A radiance FCDR,
     * one channel in one netCDF variable.
     */
    @Override
    public int writeSceneTemperatureVariables() {
        int status = 0;
        String varName;
        boolean[] bCFlagWritten = {false, false, false};
        Array arr2D;
        int[] org={0,0,0};
        int[] shape=(int[])shape3D.clone();
        
        try {
            for (int i = 0; i < VAR_Ts.length; i++) {
                if (VariableControl.isWriteT(i)) {
                    for (int channel = INDEX_IMICA_CHANNEL_START; channel < INDEX_IMICA_CHANNEL_END; channel++) {
                        varName = getVaraibleNameNCDC(VAR_Ts[i], channel);
                        org[2]=channel;
                        shape[2]=1;
                        arr2D=dataTsNew[i].section(org,shape);
                        ncfileWritable.write(varName, arr2D);                                                
                    }
                    
                    //modify here later
                    int indexCFlag = i / 2;
//                    if (i%2==0){//T_SNO,T_OPR,T_LINEAR
                    if (!bCFlagWritten[indexCFlag]) {
                        ncfileWritable.write(VAR_CFLAGs[indexCFlag], dataCFlags[indexCFlag]);
                        for (int channel = INDEX_IMICA_CHANNEL_START; channel < INDEX_IMICA_CHANNEL_END; channel++) {
                            varName=getVaraibleNameNCDC(VAR_CFLAGPIXELs[indexCFlag],channel);
                            org[2]=channel;
                            shape[2]=1;
                            arr2D=dataCFlagPixel[indexCFlag].section(org,shape);
                            ncfileWritable.write(varName, arr2D);
                        }

                        if (VariableControl.isWriteCalibrationCoefficients()) {
                            if (dataCalCoefAMSUA != null && dataCalCoefAMSUA[indexCFlag] != null) {
                                ncfileWritable.write(VAR_CALCOEF_AMSUA[indexCFlag], dataCalCoefAMSUA[indexCFlag]);
                            }
                        }

                        bCFlagWritten[indexCFlag] = true;
                    }
//                    }
                }
            }
        } catch (java.io.IOException e) {
            e.printStackTrace();
            return -1;
        } catch (ucar.ma2.InvalidRangeException e) {
            e.printStackTrace();
            return -1;
        }
        return status;
    }
    
    /**
     * Get CDR variable names for AMSU-A radiance FCDR
     * @return CDR variable names
     */
    @Override
    public String getCDRVariableNames() {
        String res = "";
        String sep = ",";
        String varName;
        boolean bFirst = true;
        for (int i = 0; i < 6; i++) {
            if (VariableControl.isWriteT(i)) {
                for (int channel = INDEX_IMICA_CHANNEL_START; channel < INDEX_IMICA_CHANNEL_END; channel++) {
                    varName = getVaraibleNameNCDC(VAR_Ts[i], channel);;
                    if (bFirst) {
                        res = varName;
                        bFirst = false;
                    } else {
                        res += sep + varName;
                    }
                }
            }
        }


        return res;
    }
 }
