/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.L1C.AMSUALevel1CGlobalAttributes.java
* NAME
*   amsu.amsua.L1C.AMSUALevel1CGlobalAttributes.java 
* 
* PURPOSE
*   To support AMSU-A level 1c FCDR global attributes.
* 
* DESCRIPTION
*   This class is a subclass of CDRGlobalAttirubtes. Besides metadata defined 
*   in CDRGlobalAttirubtes, it also support self-defined global attributes for 
*   level 1c radiance FCDRs.
*   It also provides method for setting the values of various global attributes 
*   for AMSU-A level 1c radiance FCDR.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/01/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   06/04/2013 Modified setGlobalAttributes(...) method to make NOAA and NASA AMSU-A have
*              the same global attributes.
*   05/20/2013 Initial Version prepared for NCDC
*   05/04/2013 added methods for setting and updating different global attributes.
*
*@*****/
 
 

package amsu.amsua.L1C;

import amsu.amsua.AMSUA;
import amsu.amsua.limbcorrection.AMSUALimbCorrection;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import pub.L1C.IMICACoefficient;
import pub.attribute.CDRGlobalAttributes;
import pub.attribute.GlobalAttributesCDR;
import pub.attribute.GlobalAttributesCF;
import pub.attribute.GlobalAttributesGDS;
import pub.attribute.GlobalAttributesUnidataACDD;
import pub.attribute.GlobalAttributesWWANG;
import pub.attribute.GlobalAttributesWWANGL1C;
import pub.variable.VariableControl;
import util.Time;

public class AMSUALevel1CGlobalAttributes extends CDRGlobalAttributes implements AMSUA{
    //self-defined level 1c radiance FCDR global attributes
    protected GlobalAttributesWWANGL1C gaWWANGL1C=new GlobalAttributesWWANGL1C();
    public Time timeCreated=new Time();    
    
    public AMSUALevel1CGlobalAttributes(){
        gaAll.add(gaWWANGL1C);
    }


    /**
     * Set the values of various global attributes for AMSU-A level 1c radiance FCDR 
     * @param fileL1C - AMSU-A level 1c file
     * @param indexL1C - level l1c file index for level 1c files to be processed
     * @param index - index of current level 1c file
     */  
    public void setGlobalAttributes(AMSUALevel1CFileNetCDF fileL1C,AMSUAL1CFileIndex indexL1C,int index){
        CDRGlobalAttributes.productVersionFormat1="Version 1.0";
        CDRGlobalAttributes.productVersionFormat2="v01r00";
         
         //1. set self-defined global attriubtes for all CDR products
        int satidInternal=AMSUALevel1CFileConfig.satIdInternal;
        String satname=noaa.ObservationPlatforms.getSatelliteNameUsingInternalSatelliteID(satidInternal);
        int satidNOAA=noaa.ObservationPlatforms.InternalSatelliteID2NOAAInternalID(satidInternal);

        setValue(GlobalAttributesWWANG.GA_NAME_WWANG_SATELLITENAME,satname);
        setValue(GlobalAttributesWWANG.GA_NAME_WWANG_SATELLITEIDINTERNAL,satidInternal);
        setValue(GlobalAttributesWWANG.GA_NAME_WWANG_SATELLITEID,satidNOAA);
        
        setTimeConverageAttributes(fileL1C.getStartTime(), fileL1C.getEndTime());
 
        //2.set self-defined level 1c attributes
        setValue(GlobalAttributesWWANGL1C.GA_NAME_WWANG_L1C_NSCANLINES, fileL1C.nLines);        
        //set SNO calibration coefficients
        if (VariableControl.bUseSNOCalibration()) {
            int index1 = AMSUALevel1CFileConfig.indexMUGroup;
            int indexSat = satidInternal - 9;
            IMICACoefficient snoCoef;
            String str = "\n";
            for (int i = 3; i < NUM_CHANNELS-1; i++) {
                snoCoef = pub.L1C.IMICACalibration.coeffs[i][index1][indexSat];
                str += snoCoef.toString(i);
                if (i < NUM_CHANNELS - 2) {
                    str += "\n";
                }
            }
            setValue(GlobalAttributesWWANGL1C.GA_NAME_WWANG_L1C_SNOCOEFFICIENTS, str);
        }else{
            modifyRequiredFlag(GlobalAttributesWWANGL1C.GA_NAME_WWANG_L1C_SNOCOEFFICIENTS,false);
        }
       
        if (VariableControl.bUseLimbCorrection()){
            setValue(GlobalAttributesWWANGL1C.GA_NAME_WWANG_L1C_LIMBCORRECTIONSCHEME, 
                     AMSUALimbCorrection.getLimbCorrectionSchemeName());
        }
        
        setValue(GlobalAttributesWWANGL1C.GA_NAME_WWANG_L1C_ANTENNAPATTERNCORRECTION, 
                AMSUALevel1CFileConfig.correctAntennaPattern == 1 ? "Yes" : "No");
        
        //set CF Metadata Convention
        setValue(GlobalAttributesCF.GA_NAME_CF_TITLE,"STAR "+productVersionFormat1+" AMSU-A Radiance FCDR");

        String sourceL1Bs="";
        String[] inputFilenames = indexL1C.getL1BFilenames(index);
        int nInputFiles = indexL1C.getNumberOfL1BFiles(index);
         for (int i=0;i<nInputFiles;i++){
            sourceL1Bs+=inputFilenames[i];
            if (i<nInputFiles-1)
                sourceL1Bs+="\n";
        }
        setValue(GlobalAttributesCF.GA_NAME_CF_SOURCE,sourceL1Bs);

        
        //set CDR attributes
        setValue(GlobalAttributesCDR.GA_NAME_CDR_VARIABLE,fileL1C.getCDRVariableNames());
        
        //set Unidata ACDD
        String timeStamp = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss").format(Calendar.getInstance().getTime());
        timeCreated.setUsingISO8601String(timeStamp);
        setValue(GlobalAttributesUnidataACDD.GA_NAME_ACDD_DATE_CREATED,timeCreated.toStringISO8601());
        
        setValue(GlobalAttributesUnidataACDD.GA_NAME_ACDD_ID,indexL1C.filenamesL1C[index]);
        setValue(GlobalAttributesUnidataACDD.GA_NAME_ACDD_SUMMARY,
                    "NOAA/NESDIS/STAR "+productVersionFormat1+" Advanced Microwave Sounding Unit A (AMSU-A) level 1c radiance Fundamental Climate Data Record (FCDR);"+
                 " inter-calibrated channels 4-14 brightness temperatures using the Integrated Microwave Inter-Calibration Approach (IMICA, formerly known as simultaneous nadir overpass method, Zou and Wang 2011) and with quality flags, from 1998 - present; "+
                 "NOAA operational calibrated brightness temperatures (channels 4-14), as well as limb effect corrected IMICA and operational calibrated brightness temperatures using Goldberg (2001) method, are also provided."
                );
        setValue(GlobalAttributesUnidataACDD.GA_NAME_ACDD_CONTRIBUTOR_NAME,"Cheng-Zhi.Zou, Wenhui Wang");
        
        setValue(GlobalAttributesUnidataACDD.GA_NAME_ACDD_CONTRIBUTOR_ROLE,
                "Principle Investigator and the architect of the IMICA approach, "+
                "the author of the Java-based AMSU-A Radiance FCDR processing code and the support scientist who derived new calibration coefficients for AMSU-A channels 4-14 and helped improve the IMICA approach.");
        
        //GDS 
        String strPlatform=noaa.ObservationPlatforms.getPlatform(satidInternal); 
        setValue(GlobalAttributesGDS.GA_NAME_GDS_PLATFORM,strPlatform);
        setValue(GlobalAttributesGDS.GA_NAME_GDS_PRODUCT_VERSION,productVersionFormat2);
        setValue(GlobalAttributesGDS.GA_NAME_GDS_METADATA_LINK,"gov.noaa.ncdc:C00787");
     }
    
  }
