/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.limbcorrection.AMSUALimbCorrection.java
* NAME
*   amsu.amsua.limbcorrection.AMSUALimbCorrection.java 
* 
* PURPOSE
*   To adjust a satellite observation to nadir sensor incidence angle (limb correction).
* 
* DESCRIPTION
*   This class is the entry point for limb correction both for level 1c processing 
*   (if limb correction flag is set) and level 3 processing (if limb correction 
*    flag is set). It specifies current limb correction scheme used, loads coefficients
*    from files, and adjusts satellite observations to nadir.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/30/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   10/05/2010 Tried other limb correction methods 2 & 3. 
*                Goldberg 2001 is still be best so far.
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 


package amsu.amsua.limbcorrection;
import util.Time;

public class AMSUALimbCorrection {
    //public static AMSUALimbCorrectionMonthly limbCorr=null;  
    static String[] namesLimbCorrectionScheme={"no limb correction",
        "Goldberg 2001",
        "12 month scheme from Zhaohui Cheng",
        "1 month scheme Wang 2010"
    };
    
    static public int limbCorrectionScheme=0;  //0- no limb correction 
                                               //1- 1month scheme goldberg 2001
                                               //2 - 12month scheme from zhaohui
                                               //3 - 1month scheme wang 2010
    static public boolean bLimbCorrection=false; //whether to perform limb correction
    
    /**
     * Get the name of current limb current scheme
     * @return the name of current limb current scheme
     */
    static public String getLimbCorrectionSchemeName(){
        return namesLimbCorrectionScheme[limbCorrectionScheme];
    }
    
    /**
     * Initialize limb correction, loading correction coefficients 
     * @param scheme - which limb correction scheme will be used
     * @param dirCoefficients - directory to find find limb correction coefficients
     * @param satIdInternal - internal satellite ID
     */
    static public void init(int scheme,String dirCoefficients,int  satIdInternal){
            limbCorrectionScheme= scheme;
            dirCoefficients+="/limb_corr/";
            switch (limbCorrectionScheme){
                case 0:
                    System.out.println("No limb correction will be applied!");
                    bLimbCorrection=false;
                    break;
                case 1: //Goldberg 2001 coefficients
                    System.out.println("Goldberg 2001 1 month limb correction scheme will be applied!");
                    AMSUALimbCorrectionGoldberg2001.initLimbCorrection(dirCoefficients+"/goldberg2001/"); 
                    bLimbCorrection=true;
                    break;
                /*case 3://Wang 2010 coefficients derived from global mean temeprature time series
                   // System.out.println("Wang 2010 1 month limb correction scheme will be applied!");
                   // AMSUALimbCorrectionGoldberg2001.initLimbCorrection(dirCoefficients+"/wang2010/");                    
                  //  bLimbCorrection=true;
                    break;
                case 2: //monthly
                  //  System.out.println("Cheng 12-month limb correction scheme will be applied!");
                  //  limbCorr=new AMSUALimbCorrectionMonthly();
                 //   limbCorr.initLimbCorrection(dirCoefficients+"/monthly/", satIdInternal);
                  //  bLimbCorrection=true;
                    break;*/
                default:
                    System.out.println("Limb Correction scheme = "+ limbCorrectionScheme+" not correct !!! ");
                    System.out.println("   No limb correction will be applied!");
                    bLimbCorrection=false;
                    limbCorrectionScheme=0;
                    break;
           }
    }
    
    /**
     * Calculate limb effect corrected brightness temperatures
     * @param temps - channel brightness temperature with limb correction
     * @param tempsLimbCorrected - used for storing brightness temperatures with limb correction
     * @param landcovertype - land cover type of each pixels in a scan line
     * @param time - observation time of a scan line
     */
    static public void limbCorrection(float[][] temps, float[][] tempsLimbCorrected, byte[] landcovertype, Time time){
       
            if (limbCorrectionScheme==1 || limbCorrectionScheme==3){
                //one set of coefficients
                AMSUALimbCorrectionGoldberg2001.limbCorrection(temps, tempsLimbCorrected, landcovertype,time);
            }else if (limbCorrectionScheme==2){
                //monthly
                //int month=time.getMonth();
                //AMSUALimbCorrection.limbCorr.limbCorrection(temps, tempsLimbCorrected, landcovertype, month);
            }
  
    }
    
}
