/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.limbcorrection.AMSUALimbCorrectionGoldberg2001.java
* NAME
*   amsu.amsua.limbcorrection.AMSUALimbCorrectionGoldberg2001.java 
* 
* PURPOSE
*   To read AMSU-A limb correction coefficients (Goldberg 2001) from file to memory.
* 
* DESCRIPTION
*   This class loads limb correction coefficients derived using the Goldberg 2001 
*   method. The Goldberg 2001 method corrects limb effect in a particular 
*   channel using adjacent channels whose weighting functions peak up and/or 
*   below the target channel. 
*   
*   reference:
*   Goldberg, M. D., D. S. Crosby, and L. Zhou, 2001: The limb adjustment of 
*   AMSU-A observations: methodology and validation. J. Appl. Meteorol., 40, 70-83.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/30/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 
package amsu.amsua.limbcorrection;

import amsu.amsua.*;
import java.io.*;
import java.util.*;
import pub.QualityControl;
import util.*;


public class AMSUALimbCorrectionGoldberg2001 implements AMSUA{
    public static int[][] channelsForLimbCorrectionSensitivity={
        {0,1},{0,1}, //chanel 1,2
        {2,3,4},{2,3,4},{3,4,5},  //channel 3, 4, 5, 
        {4,5,6},{5,6,7},{6,7,8},  //channel 6,7,8
        {7,8,9},{8,9,10},{9,10,11}, //channel 9,10,11
        {10,11,12},{11,12,13},{11,12,13},{1,14} }; //12,13,14,15
        
    
    //limb correction coefficents and related information for ocean
    static float[][][] coefLimbCorrectionOcean=new float[NUM_CHANNELS][NUM_SPOTS_PER_LINE][NUM_CHANNELS];
    static float[][][] ameanOcean=new float[NUM_CHANNELS][NUM_SPOTS_PER_LINE][15];
    static float[] dmeanOcean=new float[NUM_CHANNELS];
    static int[][] indexChannelsUsedOcean=new int[NUM_CHANNELS][NUM_SPOTS_PER_LINE]; 
    static int[] nChannelsUsedOcean=new int[NUM_CHANNELS];
  
    //limb correction coefficents and related information for land
    static float[][][] coefLimbCorrectionLand=new float[NUM_CHANNELS][NUM_SPOTS_PER_LINE][NUM_CHANNELS];
    static float[][][] ameanLand=new float[NUM_CHANNELS][NUM_SPOTS_PER_LINE][15];
    static float[] dmeanLand=new float[NUM_CHANNELS];
    static int[][] indexChannelsUsedLand=new int[NUM_CHANNELS][NUM_SPOTS_PER_LINE];          
    static int[] nChannelsUsedLand=new int[NUM_CHANNELS];
       
 
    public AMSUALimbCorrectionGoldberg2001 (){
        
    }


    /**
     * Load one set of limb correction coefficients, for land or ocean
     * @param pathname - pathname of limb correction coefficients file
     * @param coefLimbCorrection - used for storing the coefficients loaded from file
     * @param amean - 
     * @param dmean
     * @param nChannelsUsed - number of channels used for each target channel
     * @param indexChannelsUsed - index of channels used for each target channel
     * @return return 0 if the coefficients are loaded successfully, otherwise return -1.
     */
    static int initLimbCorrectionKernel(String pathname,float[][][] coefLimbCorrection,
            float[][][] amean, float[]dmean,int[] nChannelsUsed,int[][] indexChannelsUsed){
        int i,j,k,spot;
        int status=0;
        
        float error;
        int channel,indexChannel;
          
        try{
            Scanner sc= new Scanner(new DataInputStream(new FileInputStream(pathname)));
                     
            for (i=0;i<NUM_CHANNELS;i++){

                channel=sc.nextInt();
                nChannelsUsed[i]=sc.nextInt();
                dmean[i]=sc.nextFloat();
 
                for (k=0;k<nChannelsUsed[i];k++){
                    indexChannelsUsed[i][k]=sc.nextInt()-1;  //-1 convert 1-15 representation to 0-15 channel representation
                }
                
                for (j=0;j<NUM_SPOTS_PER_LINE;j++){
                    channel=sc.nextInt();
                    spot=sc.nextInt();
                    
                    for (k=0;k<nChannelsUsed[i];k++){
                        indexChannel=indexChannelsUsed[i][k];
                        coefLimbCorrection[i][j][indexChannel]=sc.nextFloat();                              
                    }
                    for (k=0;k<nChannelsUsed[i];k++){
                        indexChannel=indexChannelsUsed[i][k];
                        amean[i][j][indexChannel]=sc.nextFloat();
                    }
                    error=sc.nextFloat();
                }
           
            }
            sc.close();
        }catch (IOException e){
            System.out.println("Limb correction coefficients "+ pathname+" not found!!!");
            status=-1;
        }
        
        return status; 
       
    }

    /**
     * Load land and ocean limb correction coefficients from files 
     * @param dirCoeffs - directory of limb correction coefficients
     * @return 0 if the coefficients are loaded successfully, otherwise return -1.
     */
     public static int initLimbCorrection(String dirCoeffs){
         int status;
         
         String nameOcean=dirCoeffs+"/amsualimb_sea.txt"; 
         String nameLand=dirCoeffs+"/amsualimb_land.txt"; 

         
         status=initLimbCorrectionKernel(nameOcean,coefLimbCorrectionOcean,
                 ameanOcean,dmeanOcean,nChannelsUsedOcean,indexChannelsUsedOcean);
         if (status!=0)
             return -1;
         
         status=initLimbCorrectionKernel(nameLand,coefLimbCorrectionLand,
                 ameanLand,dmeanLand,nChannelsUsedLand,indexChannelsUsedLand);
         
           //load grided land type data        
         System.out.print("One set limb correction parameter loaded...from:\r\n");
         System.out.print("    "+nameOcean+"\r\n");
         System.out.print("    "+nameLand+"\r\n");
         return status;
     }
     
     /**
      * Limb correction using Goldberg 2001
      * @param t - scene temperatures for before the limb correction
      * @param tCorrected - used for storing scene temperatures at nadir
      * @param landcovertype - land cover types for each pixel
      * @param time - time (not used here)
      * @return 0 if successful, otherwise return -1.
      */
     public static int limbCorrection(float[][] t,float[][] tCorrected, byte[] landcovertype,Time time){
        //check the validity of input and output data
        if (t==null || tCorrected==null || 
            t.length!=NUM_CHANNELS || t[0].length!=NUM_SPOTS_PER_LINE ||
            tCorrected.length!=NUM_CHANNELS || tCorrected[0].length!=NUM_SPOTS_PER_LINE){
            return -1;
        }
        
        float[][][] coefLimbCorrection;
        float[][][] amean;
        float[]dmean;
        int[] nChannelsUsed;
        int[][] indexChannelsUsed;
        
        
        int nChannels=t.length;
        int nSpots=t[1].length;
        int spot,k,index;
        float sum;
        for (spot=0;spot<nSpots;spot++){
            if (landcovertype[spot]==LandType.OCEAN){
                coefLimbCorrection=coefLimbCorrectionOcean;
                amean=ameanOcean;
                dmean=dmeanOcean;
                nChannelsUsed=nChannelsUsedOcean;
                indexChannelsUsed=indexChannelsUsedOcean;
            }else{
                coefLimbCorrection=coefLimbCorrectionLand;
                amean=ameanLand;
                dmean=dmeanLand;            
                nChannelsUsed=nChannelsUsedLand;
                indexChannelsUsed=indexChannelsUsedLand;
            }
 
            float temp;
            for (int ch=0;ch<nChannels;ch++){
                
                
                if (landcovertype[0]==LandType.UNDEFINED || 
                    !AMSUAQualityControl.isValidTemperature(t[ch][spot])){
                //if antnenna temp in this channel is not valid
                //not correction
                    tCorrected[ch][spot]=QualityControl.missing_value;
                }else {
                  sum=0;
                  for (k=0;k<nChannelsUsed[ch];k++){
                    index=indexChannelsUsed[ch][k];
                      
                    temp=t[index][spot];
                      
                    if (AMSUAQualityControl.isValidTemperature(temp)){
                         sum+=coefLimbCorrection[ch][spot][index]*
                            (temp- amean[ch][spot][index]); 
                    }
                  }
                  tCorrected[ch][spot]=sum+dmean[ch];
                  if (tCorrected[ch][spot]<QualityControl.TEMPERATURE_RANGE_MIN ||
                        tCorrected[ch][spot]>QualityControl.TEMPERATURE_RANGE_MAX){
                      tCorrected[ch][spot]=QualityControl.missing_value;
                  }  
                }
            }
        }
        return 0;         
     }

     public static int limbCorrectionSensitivity(float[][] tt,float[][] tAncillary,
             float[][] tCorrected, byte[] landcovertype,int channelSensitivity){
        
       if (tt==null || tCorrected==null || tAncillary==null || 
            tt.length!=NUM_CHANNELS || tt[0].length!=NUM_SPOTS_PER_LINE ||
            tCorrected.length!=NUM_CHANNELS || tCorrected[0].length!=NUM_SPOTS_PER_LINE)
            return -1;
        
        float[][][] coefLimbCorrection;
        float[][][] amean;
        float[]dmean;
        int[] nChannelsUsed;
        int[][] indexChannelsUsed;
        
        
        int nChannels=tt.length;
        int nSpots=tt[1].length;
        int spot,k,index;
        float sum;
        for (spot=0;spot<nSpots;spot++){
            if (landcovertype[spot]==LandType.OCEAN){
                coefLimbCorrection=coefLimbCorrectionOcean;
                amean=ameanOcean;
                dmean=dmeanOcean;
                nChannelsUsed=nChannelsUsedOcean;
                indexChannelsUsed=indexChannelsUsedOcean;
            }else{
                coefLimbCorrection=coefLimbCorrectionLand;
                amean=ameanLand;
                dmean=dmeanLand;            
                nChannelsUsed=nChannelsUsedLand;
                indexChannelsUsed=indexChannelsUsedLand;
            }
 
            float[][] t;
            float temp;
            for (int ch=0;ch<nChannels;ch++){//not real amasu a channel, 
                if (landcovertype[0]==LandType.UNDEFINED){
                //if antnenna temp in this channel is not valid
                //not correction
                    tCorrected[ch][spot]=QualityControl.missing_value;
                }else {
                  sum=0;
   
                  for (k=0;k<nChannelsUsed[channelSensitivity];k++){                      
                    index=indexChannelsUsed[channelSensitivity][k];
                    if (index==channelSensitivity){
                        temp=tt[ch][spot];
                    }else
                        temp=tAncillary[index][spot];                                      
                    
                    if (temp>100 && temp<300){
                        //if a related channel is not valid, not used
                            sum+=coefLimbCorrection[channelSensitivity][spot][index]*
                                (temp- amean[channelSensitivity][spot][index]); 
                    }                                                                
                  }
                  tCorrected[ch][spot]=sum+dmean[channelSensitivity];
                }
            }
        }
        return 0;         
     }

    
     public static void main(String[] args) {       
         initLimbCorrection("d:\\data\\amsu\\");
     }

}
