/*@***h* AMSU-A_FCDR_STAR/amsu.amsua.precipitation.PrecipitationAMSUA.java
* NAME
*    amsu.amsua.precipitation.PrecipitationAMSUA.java 
* 
* PURPOSE
*   To provide AMSU-A precipitation screening algorithms.
* 
* DESCRIPTION
*   This class provides multiple precipitation algorithms. The first 3 methods are based on
*      EUMETSAT ATOS Level 2 Product Generation Specification
*      http://www.eumetsat.int/Home/Main/DataProducts/Resources/index.htm?l=en
* 
*      Test1: ???
*      Test2: based on Crosby, Ferraro and Wu, Journal of applied Met. 1995, 34, 11
*      Test3: based on Grody, 1991, J. Geophys. res., vol 96, pp 7423 - 7435
*      Test4: Grody 1999
*   Currently Grody 1999 algorithm is used.
*
*   Potential Testing data
*       Applications of AMSU for obtaining water vapor, cloud liquid water,
*       precipitation, snow cover and sea ice concentration
*       AMSU/NOAA15 data, 2003.12.02, 17:08 UTC  
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   03/20/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   Oct. 13, 2010 -- added Grody 1999 method.
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*@*****/
 
 
package amsu.amsua.precipitation;

import amsu.amsua.AMSUA;
import pub.QualityControl;

public class PrecipitationAMSUA extends PrecipitationPub implements AMSUA {
    //for test1
    final static double[][] bs={
        {-179.588,      0.55091080,     -0.24632506,    1.61315530},
        {-11.828385,	-0.79630378,	0.70550387,	-0.09248847},
        {-79.47264,	0.82639551,	-0.78458517,	0.41939181},
        {7.7797457,	-0.25387738,	0.26119687,	-0.05278192}};
    double[] as=new double[4];
    //for test2
    static final float c1=10.5f,c2=0.184f,c3=-0.221f;
    //for test3
    static final float g0=38,g1=0.88f;
    

    /**
     * Check precipitation affected pixels
     * @param temperatures  -  brightness temperatures for 15 channels, before limb correction
     * @param sensorViewZenith - sensor view zenith angle
     * @param landfrac -  land fraction data
     * @param lat - latitude
     * @return total number of precipitation pixels
     */
    public int checkPrecipitation(float[][] temperatures, float[] sensorViewZenith,                               
                                  float [] landfrac,float[] lat){    
    
        int spot;
        float t1,t2,t3,t15;
        boolean bPrecip,bPrecip1,bPrecip2,bPrecip3;
        
        nPrec=0; 
        
        for (spot=0;spot<NUM_SPOTS_PER_LINE;spot++){
 
 
            t1=temperatures[0][spot];
            t2=temperatures[1][spot];
            t3=temperatures[2][spot];
            t15=temperatures[14][spot];
            
            if (!QualityControl.isMissingValue(t1) && !QualityControl.isMissingValue(t2) &&
                    !QualityControl.isMissingValue(t3) && !QualityControl.isMissingValue(t15)){
                bPrecip=precipTestGrody1999(t1,t2,t3,t15,lat[spot],sensorViewZenith[spot],landfrac[spot]);
            }else{
                bPrecip=false;
            }
            
    /*        bPrecip=false;
            if (landcovertype[spot]==LandType.OCEAN ){
                bPrecip1=precipTest1Ocean(t1,t2,t3,t15,sensorViewZenith[spot]);
            }else{
                bPrecip1=precipTest1Land(t1,t15);
            }
            
            bPrecip2=precipTest2(t1,t15);
            
            //bPrecip3=precipTest3(t1,t2);
 
            bPrecip=bPrecip1; //|| bPrecip2; //|| bPrecip3;*/
            //save results
            precipFlag[spot]=bPrecip;
            if (bPrecip)
                nPrec++;
        }
        
        return nPrec;
    }

    //from EUMETSAT 
    boolean precipTest1Ocean(float t1,float t2,float t3,float t15Obs,float sensorViewZenith){
        double theta=sensorViewZenith*Math.PI/180.0;
        double x=1/Math.cos(theta)-1;
        double x2=x*x,
               x3=x2*x;
        double[] xs={1,x,x2,x3};
       
        
        float t15Est;
        
        for (int i=0;i<4;i++){
            as[i]=bs[0][i]+bs[1][i]*x+bs[2][i]*x2+bs[3][i]*x3;
        }
        
        t15Est=(float)(as[0]+as[1]*t1+as[2]*t2+as[3]*t3);
        
        float Isc=t15Est-t15Obs;
        return Isc>10 || Isc<-10;
    }

    /*
     *P4.25 EVALUATION OF THE AMSU LAND SURFACE TEMPERATURE ALGORITHM FOR SKIN AND SHELTERAIR
TEMPERATURE RETRIEVALS
Cezar Kongoli 12, Paul Pellegrino12, Fuzhong Weng2, Charlie Dean12 and Ralph Ferraro2
1QSS Group, Inc., Lanham, MD
2NOAAA/NESDIS, Camp Springs, MD 
     */    
    boolean precipTest1Land(float t1,float t15){
        return t1>=262 && t1-t15>=4;       
    }
    
    //Crosby 1995
    boolean precipTest2(float t1,float t15){
        float f;
        double Iprec;
        f=c1+c2*t1*c3*t15;
        Iprec=1/(1+Math.exp(-f));
        return Iprec>0.05;
    }
    
    //adapted from Grody 1991
    boolean precipTest3(float t1,float t2){
        float t1Est=g0+g1*t2;
        return t1<t1Est;
    }

    //Grody 1999
    //http://books.google.com/books?ei=oll8T-rcAaXt0gG5tcTVCw&id=IwIIhhMxcJwC&dq=Microwave+Radiometry+and+Remote+Sensing+of+the+Earth%E2%80%99s+Surface+and+Atmosphere&q=Grody#v=snippet&q=Grody&f=false
//Grody, N. C., F. Weng, and R. R. Ferraro, 1999: Application of AMSU for Obtaining Hydrological
//Parameters, Microwave Radiometry and Remote Sensing of the Earth’s Surface
//and Atmosphere. VNU Science Press, Brill Academic Publishers, Inc., 339-351.
    boolean precipTestGrody1999(float T23,float T31,float T50,float T89,
            float lat, float sensorViewZenith,float landfrac){
        boolean bRain=false;
        float df1=2.85f+0.020f*T23-0.028f*T50;
        float df2=5.10f+0.078f*T23-0.096f*T50;
        float df3=10.2f+0.036f*T23-0.074f*T50;
        float theta=(float)(sensorViewZenith*Math.PI/180.0f);
        float cosZ=(float)Math.cos(theta);
        //byte landtype;
        float threshold=0.5f;
        //land/ocean interface are not checked
        if (landfrac<threshold){
            if (!(Math.abs(lat)>50 && df1>0)){
                float a=8.240f-(2.622f-1.846f*cosZ)*cosZ;
                float b=0.754f;
                float c=-2.265f;
                float d=(float)(a+b*Math.log(285-T23)+c*Math.log(285-T31));
                float liq=d*cosZ; //CLOUD LIQUD WATER
                float siw=(float)(-113.2+(2.41-0.0049*T23)*T23+0.454*T31-T89); //SCATTER INDEX OVER WATER
                if (liq>0.3f || siw>9)
                    bRain=true;

            }
            
        }else if (landfrac>1-threshold) {//land
            float tt=168+0.49f*T89;
            float sil=T23-T89;
            if (sil>=3)
                bRain=true;
            if (T23<=261 && T23<tt)
                bRain=false;
            if (T89>273 || df2<0.6)
                bRain=false;
        } 
        return bRain;
    }

 
}
