/*@***h* AMSU-A_FCDR_STAR/config.ConfigPub.java
* NAME
*   config.ConfigPub.java 
* 
* PURPOSE
*   To define a super class for CDR processing global configuration parameters.
* 
* DESCRIPTION
*   This class is the super class for the MSU,AMSU-A,SSU CDR processing global 
*   configuration subclasses, including AMSUALevel1CFileConfig.
*   It also support system log file management, and main program input parameter 
*   processing.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   01/24/2010
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/09/2013 - moved readInputFiles() from AMSUALevel1CFileConfig to this class
*                to support better code sharing.
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 

package config;

import io.FileLog;
import io.InputParameterFile;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

public class ConfigPub {
    public static FileLog fpLog;  //log file object
    public static String pathnameLog; //name of log file
    
    /**
     * Open log file
     * @param pathname - pathname of log file
     * @return log file object
     */
    static public FileLog openLogFile(String pathname){
        pathnameLog=pathname; 
        fpLog=new FileLog();
        fpLog.open(pathnameLog);
        return fpLog;
        
    }
    
    /**
     * Get log file object
     * @return log file object
     */
    static public FileLog getFileLog(){
        return fpLog;
    }

    /**
     * Close log file
     */
    public static void closeLogFile(){
        fpLog.close();
    }
    
    //will be override in subclass
    public static void setCommandLineInputParameters(String[] args){
        
    }

    /**
     * Set system configuration parameters from a text file
     * @param strName - parameter name
     * @param strValue - parameter value
     * @return 0 if successful, otherwise return 1.
     */
    static public int setInputParameter(String strName,String strValue){
       if (strName.compareToIgnoreCase("parameterFile") == 0) {
            setInputParameterFromFile(strValue);
            return 0;
        }else{
            return -1;
       }
    }

    /**
     * Read system configuration parameters from a text file
     * @param pathname - pathname of the system configuration parameters text file
     */
    public static void setInputParameterFromFile(String pathname) {
        String line, strName="", strValue="";
        String[] strsNameValue = {"", ""};
        InputParameterFile fpPara = new io.InputParameterFile();

        try{
        fpPara.open(pathname);
        while ((line = fpPara.readLine()) != null) {
            seperateInputParameterNameValue(line, strsNameValue);
            strName = strsNameValue[0];
            strValue = strsNameValue[1];

            if (strName.compareToIgnoreCase("") == 0) {
                //annotation, not parsed
            }else  if (setInputParameter(strName,strValue)==0){
            } else{
                      System.out.println("ConfigKernel:: Invalid input parameter="+line);
            }

        }
        
        fpPara.close();

        }catch (java.lang.NumberFormatException e){
            System.out.println("ConfigPub:: parseInputParameterFile NumberFormatException strName="+strName+" strValue="+strValue);
            System.exit(-1);
        }
    }
    
    static public void printConfig(){
        System.out.print("printConfig should be implemented in subclass");
    }
  
    /**
     * Separate configuration parameter name and value
     * @param line - a string that contains parameter name and value
     * @param strNameValue - String array of 2 used for returning name and value separated
     * @return  if succeed, return string array of 2 used for returning name and value separated;
     *          otherwise, return null.
     */
    static public String[] seperateInputParameterNameValue(String line,String[] strNameValue){
        String name,str1,str2;
       
        int i,n=line.length();
        
        if (line.length()==0 || line.charAt(0)=='#'){ //annotation line, not parsed
            strNameValue[0]="";
            strNameValue[1]="";                    
            return strNameValue;
        }
        
        for (i=0;i<n;i++){
            if (line.charAt(i)=='=')
                break;
        }
        
        if (i<n-1){
            str1=line.substring(0,i);
            str2=line.substring(i+1,n);
            strNameValue[0]=str1;
            strNameValue[1]=str2;
            return strNameValue;
        }
        
        return null;
    }
    
    /**
     * Function: Read in a list of AMSU-A level 1b file names that need to be 
     *           processed.
     *           It was used to support monthly update of AMSU-A level 1c radiance FCDRs
     * @param pathnameL1BFilelist -  pathname for a text file 
     *                               that contains level 1b files pathnames that are
     *                               needed to be processed into level 1c files
     * @return a String array that contain Level 1b pathnames 
     */
    static public String[] readInputFiles(String pathnameL1BFilelist){
        String[] files=null;
        String line;
        int n=0;
        RandomAccessFile fp;
        
        try {
            fp = new RandomAccessFile(pathnameL1BFilelist, "r");

            //count number of files
            while (true) {
                line = fp.readLine();
                if (line != null) {
                    n++;
                } else {
                    break;
                }
            }

            //actually read file names
            files = new String[n];

            fp.seek(0);
            for (int i = 0; i < n; i++) {
                files[i] = fp.readLine();
            }

            fp.close();
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
            String str="Input level 1b filelist not found - "+pathnameL1BFilelist;
            util.Error.fatalError(str,fpLog);
        } catch (IOException ex) {
            ex.printStackTrace();
            String str="Input level 1b filelist exception - "+pathnameL1BFilelist;
            util.Error.fatalError(str,fpLog);
        }
       
        return files;
    }
}

