/*@***h* AMSU-A_FCDR_STAR/io.hdf.HDFDataFile.java
* NAME
*   io.hdf.HDFDataFile.java 
* 
* PURPOSE
*   To support reading of data in HDF4 format.
* 
* DESCRIPTION
*   This class provide methods for read NASA EOS Aqua AMSU-A level 1b file,
*   which is in HDF4 format.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*    05/20/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
* 
*
*@*****/
 
 
package io.hdf;

import ncsa.hdf.object.*; 
import ncsa.hdf.object.h4.*;


public class HDFDataFile {
    protected String pathname=null;
    protected String filename=null;  //no dir
    protected String dir=null;
    
    
    protected H4File file=null;
    protected Group root=null;
    java.util.List list=null;
    protected Group eosgrid=null;
    protected java.util.List listGrids;

    public HDFDataFile(){
        pathname=null;
        file=null;
    }

    public String getName(){
        return filename;
    }

    public boolean isOpen(String pathnameIn){
        if (pathname==null)
            return false;
        if (pathnameIn.compareTo(pathname)==0){
            //already open
            return true;
        }
        return false;
    }
    

    
    public int open(String pathnameIn){
        if (pathname!=null && pathnameIn.compareTo(pathname)==0){
            //already open
            return 0;
        }

        // create a new file with a given file name.
        file = new H4File(pathnameIn, H4File.READ);//,HDFConstants.DFACC_READ);//   .DFACC_READ); 
        if (!file.isReadOnly()){
            return -2;
        }
        if (file == null)
        {
            System.err.println("Failed to open file:"+pathnameIn);
            return -1;
        }
        try{            
            int status=file.open();
            root = (Group)((javax.swing.tree.DefaultMutableTreeNode)file.getRootNode()).getUserObject();
            list=root.getMemberList();
            //1. read pressure levels            
            eosgrid = (Group) list.get(0);
            Group datasets = (Group) eosgrid.getMemberList().get(0);
            listGrids = datasets.getMemberList();
            pathname=pathnameIn;
            filename=io.File.getFilename(pathname);
            dir=io.File.getDirectory(pathname);
                        
        }catch (java.io.IOException e){
            e.printStackTrace();
            return -1;
        }catch (java.lang.Exception e){
            e.printStackTrace();
            return -1;
            
        }
        return 0;
    }
    
    public int close(){
        try{
            if (file!=null){
                file.close();
                file=null;
                pathname=null;
            }
        }catch (ncsa.hdf.hdflib.HDFException e){
             e.printStackTrace();
            return -1;           
        }
        return 0;
    }
    
    /**
     * Read a subset of data from a HDF data object
     * @param dataset - HDF data object
     * @param start0 - 
     * @param stride0
     * @param size0
     * @return
     * @throws Exception 
     */
    protected static Object readData(Dataset dataset, long[] start0,long[]stride0,long[]size0) throws Exception{
        
        int rank=dataset.getRank();
        long[] n=dataset.getDims();
        long[] selected= dataset.getSelectedDims();
        long[] start = dataset.getStartDims();
        long[] stride=dataset.getStride();
        for (int i=0;i<rank;i++){
            if (start0!=null)
                start[i]=start0[i];
            else
                start[i]=0;
            
            if (stride0!=null)
                stride[i]=stride0[i];
            else 
                stride[i]=1;
            
            if (size0!=null)
                selected[i]=size0[i];
            else
                selected[i]=1;
        }
        Object obj=dataset.read();
        return obj;
    }
    static public float linearIntepolation(float x,float x1,float x2,float y1,float y2){
        float y;
	y=y1+(y2-y1)*(x-x1)/(x2-x1);
	return (y);
    }
    protected static int findVariableIndex(java.util.List listGrids,String varName){
        int n=listGrids.size();
        String name;
        for (int i=0;i<n;i++){
            Dataset  dataset = (Dataset) listGrids.get(i);  //"qa_channel"
            name=dataset.getName();
            if (varName.compareTo(name)==0){
                return i;
            }
        }
        return -1;
    }

    protected Object readGrid1D(java.util.List listGrids,String varName){
        try {
            int index=findVariableIndex(listGrids, varName);
            Dataset dataset = (Dataset) listGrids.get(index);
            dataset.init();
            long[] dims = dataset.getDims();

            long[] start = {0};
            long[] size = {dims[0]};
            long[] stripe = {1};
            Object obj = readData(dataset, start, stripe, size); //Latitude
            return obj;

        } catch (java.lang.Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    
    protected Object readGrid2D(java.util.List listGrids,String varName){
        try {
            int index=this.findVariableIndex(listGrids, varName);
            if (index<0){
                return null;
            }
            Dataset dataset = (Dataset) listGrids.get(index);
            dataset.init();
            long[] dims = dataset.getDims();

            long[] start = {0, 0};
            long[] size = {dims[0], dims[1]};
            long[] stripe = {1, 1};
            Object obj = readData(dataset, start, stripe, size); //Latitude
            return obj;

        }catch (java.lang.ArrayIndexOutOfBoundsException e){
            e.printStackTrace();
            return null;
            
        } catch (java.lang.Exception e) {
            e.printStackTrace();
            return null;
        }
    }
    protected Object readGrid3D(java.util.List listGrids,String varName) {
        try {
            int index=findVariableIndex(listGrids,varName);
            Dataset  dataset = (Dataset) listGrids.get(index);              
            dataset.init();
            long[] dims = dataset.getDims();
             
            long[] start = {0, 0,0};
            long[] size = {dims[0],dims[1],dims[2]};
            long[] stripe = {1, 1,1};
            Object obj = readData(dataset, start, stripe, size);
            return obj;
        } catch (java.lang.Exception e) {
            e.printStackTrace();
            return null;
        }        
    }
        
}
