/*@***h* AMSU-A_FCDR_STAR/noaa.ObservationPlatforms.java
* NAME
*    noaa.ObservationPlatforms.java 
* 
* PURPOSE
*   To define satellite observation platform (satellite) names and 
*   their identification numbers.
* 
* DESCRIPTION
*   This class defines names of data observation platforms, satellite identifiers 
*   defined by NOAA and the CDR processing code. 
*   It also provides functions for satellite identifier and name conversion.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   12/15/2008
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/10/2013 Added NOAA GCMD platform names
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 

package noaa;


public class ObservationPlatforms {
    public static final int SATELLITE_ID_AQUA=99;   //selfdefined by wwang
     
    static final int NUM_SATELLITES=16;
    static final String[] satelliteNames={"NTN","N6","N7","N8","N9",
           "N10","N11","N12","N14","N15","N16","N17","N18","METOPA","AQUA","N19"};
    static final String[] platformsNOAAGCMD={"TIROS-N","NOAA-6","NOAA-7","NOAA-8","NOAA-9","NOAA-10","NOAA-11","NOAA-12","NOAA-14",
        "NOAA-15","NOAA-16","NOAA-17","NOAA-18","METOP-A","AQUA"};
    static public final String[] platformsSTAR={"TN","N06","N07","N08","N09","N10","N11","N12","N14",
        "N15","N16","N17","N18","M02","AQUA"};
    
    static final short[] satelliteIDInternal={0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
    static final short[] satelliteIDNOAA=    {1,2,4,6,7,8,1,5,3,4, 2, 6, 7,12,SATELLITE_ID_AQUA,8};
    public static String getSatelliteNameUsingInternalSatelliteID(int satelliteIDInternal){
        return satelliteNames[satelliteIDInternal].toLowerCase();
    }
    
    public static short NOAASatelliteID2InternalID(int idNOAA, int year, int jday){
        short idInternal=-1;
        switch(idNOAA){
            case 1:
                if (year>=1978 && year<=1981)
                    idInternal=0;//NTN
                else if (year>=1988 && year<2006)
                    idInternal=6;//N10
                
                break;
            case 2:
                if (year>=1979 && year<1987)
                    idInternal=1;
                else if (year>1987) //more work
                    idInternal=10;
                break;
            case 3:
                idInternal=8;
                break;
            case 4:
                if (year>=1981 && year<1986)
                    idInternal=2;
                else 
                    idInternal=9;
                break;
            case 5:
                if (year>=1984 && year<1989)
                    idInternal=7;
                else if (year>=2005)
                     idInternal=12;
                break;
            case 6:
                if (year>=1983 && year<1986)
                    idInternal=3; //NOAA8
                else
                    idInternal=11; //NOAA17
                break;
            case 7:
                if (year>=2005)
                    idInternal=12;
                else
                    idInternal=4;
                break;
            case 8:
                if (year<2009)
                    idInternal=5;  //N10
                else
                    idInternal=15; //N19
                break;
            case 12:
                idInternal=13;
                break;
 
        }
    
        return idInternal;
    }

    public static short InternalSatelliteID2NOAAInternalID(int idInternal){
        return satelliteIDNOAA[idInternal];
        
    }
    
    public static short NOAASatelliteName2NOAASatelliteID(String name){
        int i=0;
        while (i<NUM_SATELLITES){
            if (name.equalsIgnoreCase(satelliteNames[i]))
                return satelliteIDNOAA[i];
            i++;
        }
        return -1;
    }
    
    public static short NOAASatelliteName2InternalSatelliteID(String name){
        int i=0;
        while (i<NUM_SATELLITES){
            if (name.equalsIgnoreCase(satelliteNames[i]))
                return satelliteIDInternal[i];
            i++;
        }
        return -1;
    }  
    
    public static String getPlatform(int satidInternal){
        return platformsNOAAGCMD[satidInternal];
    }
}
