/*@***h* AMSU-A_FCDR_STAR/pub.L1C.IMICACalibration.java
* NAME
*   pub.L1C.IMICACalibration.java 
* 
* PURPOSE
*   To support the IMICA calibration.
* 
* DESCRIPTION
*   This class provides variables and methods for loading and storing SNO calibration
*   coefficients.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   03/20/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
* 
*
*@*****/
 
 

package pub.L1C;

import java.io.*;
import java.util.*;
import java.util.regex.Pattern;


public class IMICACalibration {
    
    static public int N_MAX_COEFFICIENTS=14, //maximum groups of SNO coefficients supported
                      nInstruments,  //number of instruments (same type)
                      nChannels; //number of channels
    
    static public IMICACoefficient[][][] coeffs=null; //SNO calibration coefficients for multiple instruments
        //dimensions: channel/N_MAX_COEFFICIENTS/nInstruments
   /**
    * Load SNO calibration coefficients for different instruments/channels from text files
    * @param dirCoeffs - directory for calibration coefficient text files
    * @param nChannels - number of channels
    * @param nInstruments - number of instruments
    */
    static public void initSNOCalibration(String dirCoeffs,int nChannels,int nInstruments){      
        System.out.println("Init SNO coefficients ...");

        IMICACalibration.nChannels=nChannels;
        IMICACalibration.nInstruments=nInstruments;
        coeffs=new IMICACoefficient[nChannels][N_MAX_COEFFICIENTS][nInstruments];
        int status;
        for (int channel=0;channel<nChannels;channel++){

            for (int i=0;i<N_MAX_COEFFICIENTS;i++){
                for (int j=0;j<nInstruments;j++){
                    coeffs[channel][i][j]=new IMICACoefficient();
                }
            }
            //read coefficients for one channel
            status=readCoefficients(dirCoeffs,channel); 
              //if (status==0)
              //  printSNOCoefficients(channel);
        }
         System.out.println("Init SNO coefficients ... finished");
    }

    /**
     * Load calibration coefficients for one channel from an text file
     *
     * @param dirCoeffs - directory for calibration coefficient text files
     * @param channel - target channel
     * @return data reading status: 0-successful; other values-failed
     */
    static public int readCoefficients(String dirCoeffs,int channel){
        int status=0;
        String filename;
        float value=0;
        int i,j;
        int n=0;
        Byte b;
        
        Pattern p=Pattern.compile(".");
        filename=dirCoeffs+"/"+"coefs.ch"+(channel+1)+".txt";         
        String line;
        try {
            BufferedReader in= new BufferedReader(new FileReader(filename));                        
            Scanner sc;
            
            //read mu0
            for (i = 0; i <N_MAX_COEFFICIENTS; i++) {
                line=in.readLine();
                sc=new Scanner(line);
                for (j = 0; j < nInstruments; j++) {
                    value = sc.nextFloat();
                    coeffs[channel][i][j].setMu0(value);
                }
                sc.close();
            }
            line=in.readLine();
            
            //read delta0
            for (i = 0; i < 14; i++) {
                line=in.readLine();
                sc=new Scanner(line);
                for (j = 0; j < nInstruments; j++) {
                    value = sc.nextFloat();
                     coeffs[channel][i][j].setDelta0(value*0.00001f);
                }
                sc.close();
            }
            line=in.readLine();
             
            //kappa
            for (i = 0; i < 14; i++) {
                line=in.readLine();
                sc=new Scanner(line);
                for (j = 0; j < nInstruments; j++) {
                    value = sc.nextFloat();
                    coeffs[channel][i][j].setKappa(value);                   
                }
                sc.close();
            }
            line=in.readLine();
            
            
            //read lamda 
            for (i = 0; i < 14; i++) {
                line=in.readLine();
                sc=new Scanner(line);
                for (j = 0; j < nInstruments; j++) {
                    value = sc.nextFloat();
                     coeffs[channel][i][j].setLamda(value);
                }
                sc.close();
            }
            
            in.close();
        } catch (IOException e) {
            System.out.println("Read SNO Coefficients for Channel "+(channel+1)+" Failed");
            status=-1;
        } catch (NoSuchElementException e){
            System.out.println("Read SNO Coefficients for Channel "+(channel+1)+" Failed");            
            status=-1;            
        }
        
        if (status==0){
            System.out.println("Read SNO Coefficients for Channel "+(channel+1)+" Sucess");      
        }
        return status;
    }

        
      
    
    /**
     * Display calibration coefficients for a channel
     * @param channel - target channel
     */
    static public void printSNOCoefficients(int channel){
        String str;
       
        for (int i=0;i<N_MAX_COEFFICIENTS;i++){
            for (int j=0;j<nInstruments;j++){
                str=coeffs[channel][i][j].toString();
                System.out.println("Ch="+(channel+1)+": "+str);
            }
            
        }
    }
    
    
    /**
     * For debug only
     * @param args - not used
     */
    public static void main(String[] args) {
        int nChannels=15;
        int nInstruments=5;
            initSNOCalibration("D:\\data\\amsu\\",nChannels,nInstruments);
    }
    
}
