/*@***h* AMSU-A_FCDR_STAR/pub.L1C.L1CCalibrator.java
* NAME
*   pub.L1C.L1CCalibrator.java 
* 
* PURPOSE
*   To support linear and IMICA calibration.
* 
* DESCRIPTION
*   This class provides methods for performing linear calibration of one orbit or
*   one scan lines of data. 
*   It also provide function for the IMICA calibration.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   04/03/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 
package pub.L1C;

import pub.QualityControl;
import util.Time;


public class L1CCalibrator {
    static final int WARM_COLD_COUNT_DISTANCE=100;
    /**
     * Linear calibration (one orbit)
     * @param rCold - cold space radiance
     * @param rWarm - warm target radiance
     * @param countsCold - cold space counts
     * @param countsWarm - warm target counts
     * @param countsScene - scene counts
     * @param radiances - buffer for returning calibrated radiances
     * @param calcoefs - used to return linear calibration coefficients
     */
    public static void count2RadianceLinear(float[][] rCold, float[][] rWarm,
            float[][] countsCold, float[][] countsWarm, float[][][] countsScene,
            float[][][] radiances, double[][][] calcoefs) {
        int idxLine;
        int nScans = countsScene.length;

        for (idxLine = 0; idxLine < nScans; idxLine++) {
            count2RadianceLinear(rCold[idxLine], rWarm[idxLine],
                    countsCold[idxLine], countsWarm[idxLine], countsScene[idxLine],
                    radiances[idxLine],calcoefs[idxLine]);
        }


    }

    /**
     * Linear calibration (one scan line)
     * @param rCold - cold space radiance
     * @param rWarm - warm target radiance
     * @param countsCold - cold space counts
     * @param countsWarm - warm target counts
     * @param countsScene - scene counts
     * @param radiances - buffer for returning calibrated radiances
     */
    public static void count2RadianceLinear(float[] rCold, float[] rWarm,
            float[] countsCold, float[] countsWarm, float[][] countsScene,
            float[][] radiances, double[][] calcoefs) {
        int i, j;
        double temp;
        float count;
        float radCold, radWarm;
        float cCold, cWarm;
        float Q, gain;
        int nChannels = countsScene.length;
        int nSpots = countsScene[0].length;

        for (i = 0; i < nChannels; i++) {
            radCold = rCold[i];
            radWarm = rWarm[i];
            cCold = countsCold[i];
            cWarm = countsWarm[i];
            gain = (radWarm - radCold) / (cWarm - cCold);

            //not tested
            calcoefs[i][0]=radCold-gain*cCold;
            calcoefs[i][1]=gain;
            calcoefs[i][2]=0;
            
            for (j = 0; j < nSpots; j++) {
                count = countsScene[i][j];
                temp = (count - cCold) * gain + radCold;
                radiances[i][j] = (float) temp;
            }
        }
    }


    /**
     * SNO calibration (one scan line)
     * @param rCold - cold space radiance
     * @param rWarm - warm target radiance
     * @param countsCold - cold space counts
     * @param countsWarm - warm target counts
     * @param countsScene - scene counts
     * @param time - satellite observation time
     * @param indexMuGroup - index of calibration coefficients
     * @param indexSatellite - index of satellite
     * @param radiances - output calibrated radiances
     * @param calcoefs - used for returning calibration coefficients for one scan line, all channels
     */
     public static void count2RadianceSNO(float[] rCold, float[] rWarm,
            float[] countsCold, float[] countsWarm, float[][] countsScene,
            Time time, int indexMuGroup, int indexSatellite,
            float[][] radiances,double[][] calcoefs) {

        int i, j;
        double temp,tempTest=0;
        float count;
        float radCold, radWarm;
        float cCold, cWarm;
        float gain=0;
        IMICACoefficient snoCoef;
        double delta;
        int nChannels = countsScene.length;
        int nSpots = countsScene[0].length;
        float ftime;
        double coef0,coef1,coef2;
        
        
        ftime = time.getYear();//+time[idxLine].g
        int debug = 0;
        for (i = 0; i < nChannels; i++) {
            snoCoef = IMICACalibration.coeffs[i][indexMuGroup][indexSatellite];

            radCold = rCold[i];
            radWarm = rWarm[i];
            cCold = countsCold[i];
            cWarm = countsWarm[i];

            if (Math.abs(cWarm - cCold) > WARM_COLD_COUNT_DISTANCE) {
                gain = (radWarm - radCold) / (cWarm - cCold);

                //calculate 0,1st, 2nd order of calibration coefficients
                double ctemp = (snoCoef.mu0 + snoCoef.lamda * (ftime - IMICACoefficient.time1)) * gain * gain;
                coef0 = radCold - cCold * gain + snoCoef.delta0 + snoCoef.kappa * snoCoef.fkappa * (ftime - IMICACoefficient.time0)
                        + ctemp * cCold * cWarm;
                coef1 = gain - (cWarm + cCold) * ctemp;
                coef2 = ctemp;
            } else {
                coef0 = QualityControl.missing_value;
                coef1 = QualityControl.missing_value;
                coef2 = QualityControl.missing_value;

            }
            calcoefs[i][0]=coef0;
            calcoefs[i][1]=coef1;
            calcoefs[i][2]=coef2;
            
            for (j = 0; j < nSpots; j++) {
                count = countsScene[i][j];

                if (count > cCold && !QualityControl.isMissingValue((float)coef0)) {

                    temp = (count - cCold) * gain + radCold; //linear

                    delta = snoCoef.delta0 + snoCoef.kappa * snoCoef.fkappa * (ftime - IMICACoefficient.time0)
                            + (snoCoef.mu0 + snoCoef.lamda * (ftime - IMICACoefficient.time1)) * (count - cCold) * (count - cWarm) * gain * gain;

                    temp += delta;
                    //tempTest=coef0+coef1*count+coef2*count*count;
                    
                } else {
                    temp = QualityControl.missing_value;
                }
                //radiances[i][j] = (float) temp;
               radiances[i][j] = (float) temp;

            }
        }
    }
}
