/*@***h* AMSU-A_FCDR_STAR/pub.L1C.L1CFileNetCDF.java
* NAME
*   pub.L1C.L1CFileNetCDF.java 
* 
* PURPOSE
*   To spport Level 1c netCDF file.
* 
* DESCRIPTION
*   This class defines: 
*       (1) global attributes names of l1c netCDF file
*       (2) methods for creating, opening, and closing l1c netCDF file
*       (3) methods for loading variables from l1c netCDF file
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   07/30/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/02/2013-05/09/2013 - relocated CDR variable name definitions to 
*                         pub.variable.CDRVariableNameL1C.java
*   05/02/2013 add log file support
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 
package pub.L1C;

import io.FileLog;
import java.io.IOException;
import pub.QualityControl;
import pub.attribute.CDRGlobalAttributes;
import pub.statistics.StatMinMaxFloat;
import pub.variable.CDRVariablesL1C;
import pub.variable.VariableControl;
import ucar.ma2.Array;
import ucar.ma2.Section;
import ucar.nc2.*;
import util.*;

public class L1CFileNetCDF extends CDRVariablesL1C {
    
    public static int NCFILE_L1C = 0;           //main level 1c file
    public static int NCFILE_LIMBCORR = 1;      //pixel level limb correction term for TCDR
    public static int NCFILE_N15CH6FREQ = 2;    //pixel level frequency shift correction term for TCDR
    public static int NCFILE_DIURCORR = 1;      //pixel level diurnal correction term for TCDR
    public static int NCFILE_MSUEQU = 1;        //pixel level MSUEquivalentAMSUA correction term for TCDR
    protected String[] pathnames = null;  //0- main level 1c file   1-limbcorr 2-n15ch6freq 3-diurcorr 4-msuequ (for AMSU-A)
    protected NetcdfFile[] ncfiles = {null, null, null, null, null}; //netCDF file objects for different level 1c files  (read only)
    protected NetcdfFileWriteable ncfileWritable = null; //NetcdfFileWriteable object for creating new l1c netCDF file


    //global attribute related
    protected StatMinMaxFloat statLatitude=new StatMinMaxFloat();  //minimum and maximum latitude
    protected StatMinMaxFloat statLongitude=new StatMinMaxFloat(); //minimum and maxiumu longitude
    protected Time timeStart, timeEnd;  //starting and ending time for current level 1c file
    protected float[][] statNEDT = new float[NUM_CALIBRATION_METHODS][];  //channel NEDT for each calibration method
    protected float[][][] wtConvertedFromRawCounts; 
    
    //fields that are related to variable dimensions
    public static final int DIM_CALIBRATION_COEFFICIENTS = 3; //offset, 1st order, 2nd order
    public  int nSpotsPerLine = -1,     //number of pixels per line
            nChannels = -1,             //number of channels
            nLines,                     //number of scan lines
            nWTs;                       //number of wart target temperatures
    protected Dimension xDim = null;
    protected Dimension yDim = null;
    protected Dimension channelDim = null;
    protected Dimension wtDim = null;
    protected Dimension attitudeDim = null;
    protected Dimension calcoefDim = null;
    protected Dimension timeISO8601Dim=null;
    
    
    protected CDRGlobalAttributes globalAttributes=null; //global attribute object
                                                         //will be substantialized in subclass 

    public FileLog fpLog=null;  //log file object
    protected int indexLine;    // index of scan line of a level 1c file
    float[] tlt = new float[2]; //for lower tropospher temperautre; 
    
    public L1CFileNetCDF(int nSpotsPerLine, int nChannels) {
        this.nSpotsPerLine = nSpotsPerLine;
        this.nChannels = nChannels;
        indexLine = 0;
        
    }

    /**
     * System log file 
     * @param fpLog -- log file
     */
    public void setLogFile(FileLog fpLog){
        this.fpLog=fpLog;
    }
    /**
     * Create a new l1c netCDF file
     */
    public void create() {
        try {
            ncfileWritable.create();
        } catch (java.io.IOException e) {

            System.out.println("Fatal error!!! \r\nCreate new netcdf l1c file failed. ");
            System.out.println("Please check whether there is enough disk space!!!");
            System.exit(-1);
        }

    }

    /**
     * Open existing l1c netcdf file for updating
     * @param pathname - pathname of l1c netcdf file
     * @return 0 if successful, otherwise return -1.
     */
    public int openExistingForUpdate(String pathname) {
        try {
            this.pathnames[0] = pathname;
            ncfileWritable = NetcdfFileWriteable.openExisting(pathname);
            if (ncfileWritable == null) {
                System.out.println("AMSUALevel1CFileNetCDF " + pathname + " open failed");
            }
            ncfiles[0] = ncfileWritable;
            readGlobalAttributes();
            return 0;
        } catch (IOException e) {
            System.out.println("AMSUALevel1CFileNetCDF " + pathname + " open failed");
            System.exit(-1);
            return -1;
        }


    }

    /**
     * Open l1c netCDF file
     * @param pathname - pathname for the l1c pathname
     * @return 0 if successful, otherwise return -1.
     */
    public int open(String pathname) {
        String[] pathnames = {pathname};
        return open(pathnames);

    }

    /**
     * Open l1c netcdf file, as well as associated files of correction terms if 
     * pathnames are provided.
     * @param pathnames - pathnames[0]: l1c netcdf file
     *                    pathnames[1]-[n-1]: files for correction terms
     * @return 0 if successful, otherwise return -1.
     */
    public int open(String[] pathnames) {
        this.pathnames = pathnames;
        String pathname="";
        try {
            for (int i = 0; i < pathnames.length; i++) {
                pathname = pathnames[i];
                if (pathname != null && pathname.substring(0, 5).compareTo("dummy") != 0) {
                    java.io.File fileTemp = new java.io.File(pathname);
                    if (!fileTemp.exists()) {
                        String str="Level1CFileNetCDF FILE" + pathname + " doesn't exist";
                        System.out.println(str);
                        fpLog.writeLog(str, FileLog.PRIORITY_MUST_WRITE_MSG, true);
                        if (i == 0)//main l1c file missing
                        {
                            System.exit(-1);
                        }
                    }
                    ncfiles[i] = NetcdfFile.open(pathname);
                    if (ncfiles[i] == null) {
                        String str="Level1CFileNetCDF " + pathname + " open failed";
                        System.out.println(str);
                        if (i == 0) {
                            System.exit(-1);
                        }
                    } else {
                        //System.out.println("AMSUALevel1CFileNetCDF "+pathname+" opened");
                    }

                } else {
                    ncfiles[i] = null;
                }
            }

            readGlobalAttributes();
            return 0;
        } catch (IOException e) {
            System.out.println("Cannot open" + pathname);
            //System.exit(-1);
            return -1;
        }

    }

    /**
     * Close l1c netcdf file, as well as associated files for correction terms.
     */
    public void close() {
        try {
            for (int i = 0; i < ncfiles.length; i++) {
                if (ncfiles[i] != null) {
                    ncfiles[i].close();
                    ncfiles[i] = null;
                }
            }

            if (ncfileWritable != null) {
                ncfileWritable.close();
                ncfileWritable = null;
            }
        } catch (java.io.IOException e) {
            System.out.println("AMSUA level 1c netcdf file close failed: ");

        }
    }

    /**
     * Write all global attributes from l1c netCDF file
     */
    public void readGlobalAttributes() {
        timeStart = new Time();
        timeEnd = new Time();

        globalAttributes.readStartTimeEndTime(ncfiles[NCFILE_L1C],timeStart,timeEnd);
        nLines=globalAttributes.gaWWANG.readNumberOfScanlines(ncfiles[NCFILE_L1C]);
    }

    /**
     * Read time, geolication, geometry variables to l1c file data buffer
     * for CRTM simulation
     */
    public void readVariables0() {
        try {
            Variable var;
            int[] org = {0};
            int[] shape = {nLines};

            for (int i = 0; i < VAR_CFLAGs.length; i++) {
                var = ncfiles[NCFILE_L1C].findVariable(VAR_CFLAGs[i]);
                if (var != null) {
                    //System.out.println("read variable "+VAR_CFLAGs[i]);
                    dataCFlags[i] = var.read(); //org, shape);
                    var = ncfiles[NCFILE_L1C].findVariable(VAR_CFLAGPIXELs[i]);
                    dataCFlagPixel[i] = var.read();
                }
            }

            varsTime.read(ncfiles[NCFILE_L1C],null,null);


            var = ncfiles[NCFILE_L1C].findVariable(VAR_ALTITUDE);
            dataAltitude = var.read(org, shape);

            int[] org2 = {0, 0};
            int[] shape2 = {nLines, nSpotsPerLine};
            var = ncfiles[NCFILE_L1C].findVariable(pub.variable.CDRVariablesL1C.VAR_LATITUDE);
            dataLatitude = var.read(org2, shape2);
            var = ncfiles[NCFILE_L1C].findVariable(pub.variable.CDRVariablesL1C.VAR_LONGITUDE);
            dataLongitude = var.read(org2, shape2);
            var = ncfiles[NCFILE_L1C].findVariable(VAR_SENSORZENITH);
            if (var != null) {
                dataSensorZenith = var.read(org2, shape2);
            } else {
                dataSensorZenith = null;
            }

            var = ncfiles[NCFILE_L1C].findVariable(VAR_ATTITUDE);
            if (var != null) {
                int[] stride2 = {1, 1};
                shape2[0] = nLines;
                shape2[1] = 3;
                Section sec = new Section(org2, shape2, stride2);
                dataAttitude = var.read(sec);

            }
        } catch (IOException e) {
            System.out.println("readVariables IOException");
            e.printStackTrace();
        } catch (ucar.ma2.InvalidRangeException e) {
            System.out.println("readVariables invalid data range");
            e.printStackTrace();
        }
    }

    /**
     * Reset all l1c variable data buffers to null
     */
    public void deallocate() {
        varsTime.deallocate();
        
        dataLatitude = null;
        dataLongitude = null;
        dataAltitude = null;
        dataSensorZenith = null;
        dataASDFlag = null;
        dataLECT = null;
        for (int i = 0; i < VAR_CFLAGs.length; i++) {
            dataCFlags[i] = null;
            dataCFlagPixel[i] = null;
        }

        for (int i = 0; i < VAR_Ts.length; i++) {
            dataTsNew[i] = null;
        }
        dataWarmTargetTemperature = null;
        dataSceneCounts = null;
        dataColdSpaceCounts = null;
        dataWarmTargetCounts = null;
    }

    public Time getStartTime() {
        return timeStart;
    }

    public Time getEndTime() {
        return timeEnd;
    }

    public int getNumberOfRecords() {
        return nLines;
    }

    /**
     * 
     * @param indexT - index of scene temperature
     * @param indexLine - index of scan line
     * @return true if the current scan line is good, otherwise return false.
     */
    public boolean isGoodScanline(int indexT, int indexLine) {
        Array qc = dataCFlags[indexT];
        byte value = qc.getByte(indexLine);
        return value == 0; //for MSU =1 IS ALSO GOOD
    }

    /**
     * Get local equator crossing time
     * @return local equator crossing time
     */
    public float getLECT() {
        if (dataLECT == null) {
            return QualityControl.missing_value;
        }
        return dataLECT.getFloat(0);
    }

    /**
     *Not finished
     * @param latc - new latitude, indexSatInstr(0-2)*scanlines*spots (30);
     * @param lonc - new longitude
     */
    public void modifyGeolocation(float[][][] latc, float[][][] lonc) {
    }

    /**
     * Not finished
     * @param zenithAngleC 
     */
    public void modifyGeometry(float[][][] zenithAngleC) {
    }
    


    /**
     * Get CDR variable names
     * @return CDR variable names
     */
    public String getCDRVariableNames(){    
        String res="";
        String sep=",";
        boolean bFirst=true;
        for (int i=0;i<6;i++){
            if (VariableControl.isWriteT(i)){
                if (bFirst){
                    res=VAR_Ts[i];
                    bFirst=false;
                }else{
                    res+=sep+VAR_Ts[i];
                }
            }
        }  

        
        return res;
    }
    
    /**
     * Get line index
     * @return line index
     */
    public int getIndexLine(){
        return indexLine;
    }
}
