/*@***h* AMSU-A_FCDR_STAR/pub.L1C.L1CFileNetCDFWritable.java
* NAME
*   pub.L1C.L1CFileNetCDFWritable.java 
* 
* PURPOSE
*   To create different variables in level 1c netcdf file and to buffer scan 
*   line level data for 1c file variables.
* 
* DESCRIPTION
*   This class provide level 1c file data buffers and methods for create different 
*   level 1c netcdf file variables shared by the level 1c CDR files for MSU, SSU, 
*   ASMU-A level 1c radiance FCDRs.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   08/03/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/14/2013 - added bufferWTConvertedFromRawCounts to support NEDT calculation
*   05/13/2013 - added calculateNEDT()
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry. 
*
*@*****/
 
 

package pub.L1C;

import io.netcdf.*;
import java.util.*;
import pub.QualityControl;
import pub.attribute.*;
import pub.variable.CDRVariablesL1C;
import pub.variable.VariableControl;
import ucar.ma2.Array;
import ucar.ma2.DataType;
import ucar.ma2.Index;
import ucar.ma2.Index2D;
import ucar.ma2.Index3D;
import ucar.nc2.*;
import util.Time;
import vector.VectorFloatAverager;
/**
 *
 * @author wwang
 */
public class L1CFileNetCDFWritable extends L1CFileNetCDF {
    static final String strDirectReadFromL1B=" from AMSU-A level 1b product";
    
    public ArrayList dims1D=new ArrayList();
    protected ArrayList dims2D = new ArrayList();
    protected ArrayList dims3D = new ArrayList();

    protected int[] shape1D,shape2D,shape2DWT,shape3D,shape3DCalCoef;
    protected int[] shape2DRawCount;
    protected Index2D index2D=null,index2DWT=null,index2DRawCount=null;
    protected Index3D index3D=null,index3DCalCoef=null;
    
    //data type for temperature variables
    static protected DataType dataTypeTemperature=DataType.SHORT; //DataType.SHORT or FLOAT
    static protected DataType dataTypeRawCounts=DataType.SHORT; //DataType.SHORT or FLOAT
    protected static int scalingFactorTemperature=100; //USED only when dataTypeTemperture=DataType.SHORT
    static protected DataType dataTypeCalCoef=DataType.DOUBLE;
    
    
    protected boolean bSSU=false;
    protected boolean bHIRS=false;
    
    public L1CFileNetCDFWritable(int nSpotsPerLine, int nChannels) {
        super(nSpotsPerLine, nChannels);

        //for storing NEDT for each calibration method
        for (int i = 0; i < statNEDT.length; i++) {
            statNEDT[i] = new float[nChannels];
        }

    }

    /**
     * Get temperature variable scaling factor
     * @return temperature variable scaling factor
     */
    public static int getTemperatureVariableScalingFactor() {
        if (dataTypeTemperature == DataType.FLOAT) {
            return 1;
        } else {
            return scalingFactorTemperature;
        }
    }

    /**
     * Define dimensions that are used to define different variables in level 1c
     * netcdf file. 
     * @param NUM_SPOTS_PER_LINE - number of scans in each scan line
     * @param nLines - total number of scan lines
     * @param nChannels - total number of channels
     * @param nWTs - total number of warm target temperatures
     */
    protected void createDimensions(int NUM_SPOTS_PER_LINE, int nLines, int nChannels, int nWTs) {
        nSpotsPerLine = NUM_SPOTS_PER_LINE;
        this.nChannels = nChannels;
        this.nLines = nLines;
        this.nWTs = nWTs;

        //init dimensions
        xDim = ncfileWritable.addDimension("pixelDim", NUM_SPOTS_PER_LINE);
        yDim = ncfileWritable.addDimension("nscanDim", nLines);
        channelDim = ncfileWritable.addDimension("channelDim", nChannels);
        wtDim = ncfileWritable.addDimension("wtDim", nWTs);
        if (VariableControl.isWriteSatelliteAttitude()){
            attitudeDim = ncfileWritable.addDimension("attitudeDim", 3);
        }else{
            attitudeDim=null;
        }
        if (VariableControl.isWriteCalibrationCoefficients()){
            calcoefDim = ncfileWritable.addDimension("calcoefDim", DIM_CALIBRATION_COEFFICIENTS);
        }else{
            calcoefDim=null;
        }
        timeISO8601Dim=ncfileWritable.addDimension("timeISO8601Dim", 20);

        dims1D.add(yDim);

        dims2D.add(yDim);
        dims2D.add(xDim);

        dims3D.add(yDim);
        dims3D.add(xDim);
        dims3D.add(channelDim);

        shape1D = new int[1];
        shape1D[0] = yDim.getLength();

        shape2D = new int[2];
        shape2D[0] = yDim.getLength();
        shape2D[1] = xDim.getLength();
        index2D = new Index2D(shape2D);

        shape2DWT = new int[2];
        shape2DWT[0] = yDim.getLength();
        shape2DWT[1] = wtDim.getLength();
        index2DWT = new Index2D(shape2DWT);

//        if (VariableControl.isWriteCalibrationCoefficients()) {
            shape3DCalCoef = new int[3];
            shape3DCalCoef[0] = yDim.getLength();
            shape3DCalCoef[1] = DIM_CALIBRATION_COEFFICIENTS;
            shape3DCalCoef[2] = channelDim.getLength();
            index3DCalCoef = new Index3D(shape3DCalCoef);
//        }
        shape2DRawCount = new int[2];
        shape2DRawCount[0] = yDim.getLength();
        shape2DRawCount[1] = channelDim.getLength();
        index2DRawCount = new Index2D(shape2DRawCount);


        shape3D = new int[3];
        shape3D[0] = yDim.getLength();
        shape3D[1] = xDim.getLength();
        shape3D[2] = channelDim.getLength();
        index3D = new Index3D(shape3D);

        //for calculating NEDT
        wtConvertedFromRawCounts=new float[NUM_CALIBRATION_METHODS][nLines][nChannels];
    }

 
    /**
     * Create geolocation variables
     */
    protected void createGeolocationVariables() {
        //GEOLOCATION
        Variable var=ncfileWritable.addVariable(VAR_LATITUDE, DataType.FLOAT, dims2D);
        CDRVariableAttributes attr=new CDRVariableAttributes();
        attr.setLongName("Latitude" + strDirectReadFromL1B);

        attr.setStandardName(VAR_LATITUDE);
        attr.setUnits("degrees_north");
        attr.setFillValue(QualityControl.missing_value);
        attr.setMissingValue(QualityControl.missing_value);        
        float[] validRangeLat={-90.0f,90.0f};
        attr.setValidMax(validRangeLat[1]);
        attr.setValidMin(validRangeLat[0]);
        attr.setValidRange(validRangeLat);               
        attr.create(ncfileWritable,VAR_LATITUDE);
                
        ncfileWritable.addVariable(VAR_LONGITUDE, DataType.FLOAT, dims2D);
        attr=new CDRVariableAttributes();
        attr.setLongName("Longitude" +strDirectReadFromL1B);
        attr.setStandardName(VAR_LONGITUDE);
        attr.setUnits("degrees_east");
        attr.setFillValue(QualityControl.missing_value);
        attr.setMissingValue(QualityControl.missing_value);

        float[] validRangeLon={-180.0f,180.0f};
        attr.setValidMax(validRangeLon[1]);
        attr.setValidMin(validRangeLon[0]);
        attr.setValidRange(validRangeLon);
        
        attr.create(ncfileWritable,VAR_LONGITUDE);

        
        int[] shape = {yDim.getLength(), xDim.getLength()};
        dataLatitude = Array.factory(DataType.FLOAT, shape);
        dataLongitude = Array.factory(DataType.FLOAT, shape);

    }
    
    /**
     * Create geometry variables
     */
    protected void createGeometryVariables() {
        if (VariableControl.isWriteGeometry()) {
            ncfileWritable.addVariable(VAR_SENSORZENITH, DataType.FLOAT, dims2D);
            CDRVariableAttributes attr = new CDRVariableAttributes();
            attr.setLongName("Sensor zenith angle"+strDirectReadFromL1B);
            attr.setStandardName(VAR_SENSORZENITH);
            attr.setUnits("degrees");
            attr.setFillValue(QualityControl.missing_value);
            attr.setMissingValue(QualityControl.missing_value);
            float[] validRange={0,90.0f};
            attr.setValidMax(validRange[1]);
            attr.setValidMin(validRange[0]);
            attr.setValidRange(validRange);
            
            attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, false);
            
            attr.create(ncfileWritable, VAR_SENSORZENITH);            
            dataSensorZenith = Array.factory(DataType.FLOAT, shape2D);


            if (!bSSU && !bHIRS) {
                ncfileWritable.addVariable(VAR_SOLARZENITH, DataType.FLOAT, dims2D);
                attr = new CDRVariableAttributes();
                attr.setLongName("Solar zenith angle"+strDirectReadFromL1B);
                attr.setStandardName(VAR_SOLARZENITH);
                attr.setUnits("degrees");
                attr.setFillValue(QualityControl.missing_value);
                attr.setMissingValue(QualityControl.missing_value);
                
                validRange[0]=0.0f;
                validRange[1]=180.0f;
                attr.setValidMax(validRange[1]);
                attr.setValidMin(validRange[0]);
                attr.setValidRange(validRange);
                
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, false);

                attr.create(ncfileWritable, VAR_SOLARZENITH);
 
                ncfileWritable.addVariable(VAR_LOCALAZIMUTH, DataType.FLOAT, dims2D);
                attr = new CDRVariableAttributes();
                attr.setLongName("Local azimuth angle" +strDirectReadFromL1B);
                attr.setStandardName(VAR_LOCALAZIMUTH);
                attr.setUnits("degrees");
                attr.setFillValue(QualityControl.missing_value);
                attr.setMissingValue(QualityControl.missing_value);
                validRange[0]=-180.0f;
                validRange[1]=180.0f;
                attr.setValidMax(validRange[1]);
                attr.setValidMin(validRange[0]);
                attr.setValidRange(validRange);
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, false);
    
                attr.create(ncfileWritable, VAR_LOCALAZIMUTH);
 
                dataSolarZenith = Array.factory(DataType.FLOAT, shape2D);
                dataLocalAzimuth = Array.factory(DataType.FLOAT, shape2D);
            }
        }
    }

    /**
     * Crate scene temperature variables
     */
    protected void createSceneTemperatureVariables() {
 
        dataTsNew = new Array[VAR_Ts.length];
        for (int i = 0; i < VAR_Ts.length; i++) {
            if (VariableControl.isWriteT(i)) { //varControl & controlTs[i]) !=0){
                //create fcdr variable
                ncfileWritable.addVariable(VAR_Ts[i], dataTypeTemperature, dims3D);
              
                //set variable attributes
                CDRVariableAttributes attr = new CDRVariableAttributes();
                attr.setLongName(VariableAttributesCF.longName0+pub.variable.CDRVariablesL1C.longnamesVAR_Ts[i]);
                attr.setStandardName("brightness_temperature");
                attr.setUnits("kelvin");
                if (dataTypeTemperature == DataType.SHORT) {
                    attr.modifyRequiredFlag(VariableAttributesWWANG.VA_NAME_WWANG_SCALE_FACTOR, true);
                    attr.setScaleFactor(1.0f / scalingFactorTemperature);
                    short[] validRange={(short) (150.0f * scalingFactorTemperature),
                                        (short) (300.0f * scalingFactorTemperature)};
                    
                    attr.setValidMax(validRange[1]);
                    attr.setValidMin(validRange[0]);
                    attr.setValidRange(validRange);
                    attr.setFillValue(QualityControl.missing_value_short);
                    attr.setMissingValue(QualityControl.missing_value_short);
                    

                } else {
                    float[] validRange={150,300};                    
                    attr.setValidMax(validRange[1]);
                    attr.setValidMin(validRange[0]);
                    attr.setValidRange(validRange);
                    attr.setFillValue(QualityControl.missing_value);
                    attr.setMissingValue(QualityControl.missing_value);
                }

                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_COORDINATES, true);
                attr.setValue(VariableAttributesCF.VA_NAME_CF_COORDINATES, CDRVariablesL1C.VAR_LONGITUDE+" "+CDRVariablesL1C.VAR_LATITUDE);
                
                attr.create(ncfileWritable, VAR_Ts[i]);
               
                //allocate data buffer for the temperature variable
                dataTsNew[i] = Array.factory(dataTypeTemperature, shape3D);
            }
        }
    }

    
    /**
     * Create warm target temperature variable
     */
    protected void createWTVariables() {
        if (VariableControl.isWriteWT()) {
            ArrayList dimsWT = new ArrayList();
            dimsWT.add(yDim);
            dimsWT.add(wtDim);
            ncfileWritable.addVariable(VAR_WARMTARGETTEMPERATURE, dataTypeTemperature, dimsWT);
             
            //set variable attributes
            CDRVariableAttributes attr = new CDRVariableAttributes();
            attr.setLongName("AMSU-A multi-PRT averaged warm target temperature for antenna systems A1-1, A1-2, and A2");
            attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, false);
            //attr.setStandardName(VAR_WARMTARGETTEMPERATURE);
            attr.setUnits("K");
            if (dataTypeTemperature == DataType.SHORT) {
                    attr.modifyRequiredFlag(VariableAttributesWWANG.VA_NAME_WWANG_SCALE_FACTOR, true);
                    attr.setScaleFactor(1.0f/scalingFactorTemperature);
                    
                    short[] validRange={(short)(250*scalingFactorTemperature),
                                        (short)(32767/**scalingFactorTemperature*/)};
                    attr.setValidMax(validRange[1]);
                    attr.setValidMin(validRange[0]);
                    attr.setValidRange(validRange);
                    
                    attr.setFillValue(QualityControl.missing_value_short);
                    attr.setMissingValue(QualityControl.missing_value_short);

            }else{
                    float[] validRange={250.0f,327.67f};
                    attr.setValidMax(validRange[1]);
                    attr.setValidMin(validRange[0]);
                    attr.setValidRange(validRange);

                    attr.setFillValue(QualityControl.missing_value);
                    attr.setMissingValue(QualityControl.missing_value);
            }
            attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, false);
            attr.create(ncfileWritable, VAR_WARMTARGETTEMPERATURE);

            //allocate data buffer for the variable
            dataWarmTargetTemperature = Array.factory(dataTypeTemperature, shape2DWT);
        }
    }

    /**
     * Create level 1c scan line and pixel level quality control variables
     */
    protected void createQCVariables() {
        String[] longnamesScan = {
            "Scan-line quality flags for the IMICA recalibrated brightness temperatures",
            "Scan-line quality flags for the NOAA operational calibrated brightness temperatures",
            "Scan-line quality flags for the linear calibrated brightness temperatures",
            };
       String[] longnamesPixel = {
            "Pixel quality flags for the IMICA recalibrated brightness temperatures",
            "Pixel quality flags for the NOAA operational calibrated brightness temperatures",
            "Pixel quality flags for the linear calibrated brightness temperatures",
            };

        dataCFlags = new Array[3];
        dataCFlagPixel = new Array[3];
        String varName;
        byte[] flagValuesScan={0,1};
        byte[] flagValuesPixel={0,1,2,4,8};

        for (int i = 0; i < 3; i++) {
            if (VariableControl.isWriteT(i * 2) || VariableControl.isWriteT(i * 2 + 1)) {
                varName=VAR_CFLAGs[i];
                ncfileWritable.addVariable(varName, DataType.BYTE, dims1D);
               //set variable attributes
                CDRVariableAttributes attr = new CDRVariableAttributes();
                attr.setLongName(longnamesScan[i]);
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, false);
                //attr.setStandardName(varName);
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_UNITS, false);
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FLAG_VALUES,true);
                attr.setValue(VariableAttributesCF.VA_NAME_CF_FLAG_VALUES, flagValuesScan);

                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FLAG_MEANINGS,true);
                attr.setValue(VariableAttributesCF.VA_NAME_CF_FLAG_MEANINGS,QualityControlL1C.getL1CQCStringLine());
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FILLVALUE,true);
                attr.setValue(VariableAttributesCF.VA_NAME_CF_FILLVALUE,QualityControl.missing_value_byte);
                
                attr.create(ncfileWritable, varName);
                
                //create pixel level quality flags variable
                varName=VAR_CFLAGPIXELs[i];
                ncfileWritable.addVariable(varName, DataType.BYTE, dims3D);
               //set variable attributes
                attr = new CDRVariableAttributes();
                attr.setLongName(longnamesPixel[i]);
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, false);
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_UNITS, false);
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FLAG_VALUES,true);
                attr.setValue(VariableAttributesCF.VA_NAME_CF_FLAG_VALUES, flagValuesPixel);
                
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FLAG_MEANINGS, true);
                attr.setValue(VariableAttributesCF.VA_NAME_CF_FLAG_MEANINGS,QualityControlL1C.getL1CQCStringChannelPixel());
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_COORDINATES,true);
                attr.setValue(VariableAttributesCF.VA_NAME_CF_COORDINATES,CDRVariablesL1C.VAR_LONGITUDE+" "+CDRVariablesL1C.VAR_LATITUDE);
                attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_FILLVALUE,true);
                attr.setValue(VariableAttributesCF.VA_NAME_CF_FILLVALUE,QualityControl.missing_value_byte);

                attr.create(ncfileWritable, varName);

                dataCFlags[i] = Array.factory(DataType.BYTE, shape1D);
                dataCFlagPixel[i] = Array.factory(DataType.BYTE, shape3D);
            }
        }


    }

    /**
     * Create satellite altitude variable
     */
    protected void createAltitudeVariable() {
        float ALTITUDE_SCALE_FACTOR=100.f;
        
        String varName = VAR_ALTITUDE;
        ncfileWritable.addVariable(varName, DataType.SHORT, dims1D);


        dataAltitude = Array.factory(DataType.SHORT, shape1D);
        //set variable attributes
        CDRVariableAttributes attr = new CDRVariableAttributes();
        attr.setLongName("Spacecraft altitude above reference ellipsoid"+strDirectReadFromL1B);
        attr.setStandardName(varName);
        attr.setUnits("m");
        attr.modifyRequiredFlag(VariableAttributesWWANG.VA_NAME_WWANG_SCALE_FACTOR, true);
        attr.setScaleFactor(ALTITUDE_SCALE_FACTOR);
        
        short[] validRange={(short)QualityControl.ALTITUDE_MIN,
                            (short)QualityControl.ALTITUDE_MAX};
        attr.setValidMax(validRange[1]);
        attr.setValidMin(validRange[0]);
        attr.setValidRange(validRange);
        
        attr.setMissingValue(QualityControl.missing_value_short);
        attr.setFillValue(QualityControl.missing_value_short);
        attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, false);
        attr.create(ncfileWritable, varName);

    }
    
    
    /**
     * Create raw counts variables
     */
    protected void createRawCountVariables() {
        if (VariableControl.isWriteRawCount()) {
            ArrayList dimsColdSpaceCounts = new ArrayList();
            dimsColdSpaceCounts.add(yDim);
            dimsColdSpaceCounts.add(channelDim);

            ncfileWritable.addVariable(VAR_SCENECOUNTS, dataTypeRawCounts, dims3D);
            NetcdfFileWriteableAttributeUtil.addVariableAttributes(ncfileWritable, VAR_SCENECOUNTS, "Scene Raw Counts", "0 - 32768", "-9999");
            dataSceneCounts = Array.factory(dataTypeRawCounts, shape3D);

            if (!bSSU && !bHIRS) {
                ncfileWritable.addVariable(VAR_COLDSPACECOUNTS, DataType.SHORT, dimsColdSpaceCounts);
                NetcdfFileWriteableAttributeUtil.addVariableAttributes(ncfileWritable, VAR_COLDSPACECOUNTS, "Cold Space Raw Counts", "0 - 32768", "-9999");
                ncfileWritable.addVariable(VAR_WARMTARGETCOUNTS, DataType.SHORT, dimsColdSpaceCounts);
                NetcdfFileWriteableAttributeUtil.addVariableAttributes(ncfileWritable, VAR_WARMTARGETCOUNTS, "Warm Target Raw Counts", "0 - 32768", "-9999");
                dataColdSpaceCounts = Array.factory(dataTypeRawCounts, shape2DRawCount);
                dataWarmTargetCounts = Array.factory(dataTypeRawCounts, shape2DRawCount);
            }
        }
    }

    /**
     * Create calibration coefficient variable for SSU and HIRS
     */
    protected void createCalibrationCoefficientsVariables() {
        String[] names = {"OPR calibration coefficients",
            "Linear calibration coefficients"
        };
        if (VariableControl.isWriteCalibrationCoefficients()) {
            ArrayList dims = new ArrayList();
            dims.add(yDim);
            dims.add(calcoefDim);
            dims.add(channelDim);
            dataCalCoef = new Array[2];
            for (int i = 0; i < VAR_CALCOEF.length; i++) {
                ncfileWritable.addVariable(VAR_CALCOEF[i], dataTypeCalCoef, dims);
                NetcdfFileWriteableAttributeUtil.addVariableAttributes(ncfileWritable, VAR_CALCOEF[i], names[i], "", "-9999.0", "");
                dataCalCoef[i] = Array.factory(dataTypeCalCoef, shape3DCalCoef);
            }
        }
    }

       /**
     * Create calibration coefficient variable for AMSUA
     */    
    protected void createCalibrationCoefficientsVariablesAMSUA() {
        if (VariableControl.isWriteCalibrationCoefficients()) {
            String[] names = {
                "IMICA calibration coefficients",
                "OPR calibration coefficients",
                "Linear calibration coefficients"
            };
            ArrayList dims = new ArrayList();
            dims.add(yDim);
            dims.add(calcoefDim);
            dims.add(channelDim);
            dataCalCoefAMSUA = new Array[NUM_CALIBRATION_METHODS];
            for (int i = 0; i < VAR_CALCOEF_AMSUA.length; i++) {
                if (VariableControl.isWriteT(i * 2) || VariableControl.isWriteT(i * 2 + 1)) {
                    if (VariableControl.isWriteCalibrationCoefficients()) {
                        ncfileWritable.addVariable(VAR_CALCOEF_AMSUA[i], dataTypeCalCoef, dims);
                        NetcdfFileWriteableAttributeUtil.addVariableAttributes(ncfileWritable, VAR_CALCOEF_AMSUA[i], names[i], "", "-9999.0", "");
                    }
                    dataCalCoefAMSUA[i] = Array.factory(dataTypeCalCoef, shape3DCalCoef);
                }
            }
        }
    }

 
    /**
     * Create ascending/descending flag variable
     */
    protected void createASDVariable() {
        if (VariableControl.isWriteASDFlag()) {
            ncfileWritable.addVariable(VAR_ASDFLAG, DataType.BYTE, dims1D);
            NetcdfFileWriteableAttributeUtil.addVariableAttributes(ncfileWritable, VAR_ASDFLAG, "Ascending Descending Flag", "0 - ascending; 1 - descending; -1 - undefined", "-9999");
            dataASDFlag = Array.factory(DataType.BYTE, shape1D);
        }
    }


    /**
     * Create satellite attitude variable
     */
    protected void createAttitudeVariable() {
        if (VariableControl.isWriteSatelliteAttitude()) {
            ArrayList dimsAttitude = new ArrayList();
            dimsAttitude.add(yDim);
            dimsAttitude.add(attitudeDim);
            int[] shapeAttitude = {yDim.getLength(), attitudeDim.getLength()};
            ncfileWritable.addVariable(VAR_ATTITUDE, DataType.FLOAT, dimsAttitude);
            NetcdfFileWriteableAttributeUtil.addVariableAttributes(ncfileWritable, VAR_ALTITUDE, "", " ", "-9999.000", "degrees", "");
            dataAttitude = Array.factory(DataType.FLOAT, shapeAttitude);
        }
    }

    /**
     * Create local equator crossing time variable
     */
    protected void createLECTVariable() {
        if (VariableControl.isWriteLECT()) {
            ArrayList dims = new ArrayList();
            Dimension dim = ncfileWritable.addDimension("0Dim", 1);
            dims.add(dim);
            int[] shape = {1};
            ncfileWritable.addVariable(VAR_LECT, DataType.FLOAT, dims);
            NetcdfFileWriteableAttributeUtil.addVariableAttributes(ncfileWritable, VAR_LECT,
                    "Local Equator Crossing Time", " ", "-9999", "Hour", "");
            dataLECT = Array.factory(DataType.FLOAT, shape);
        }
    }

    /**
     * Write satellite altitude of current scan line to l1c file data buffer
     * @param altitude - satellite altitude
     */
    public void bufferAltitudeVariable(short altitude) {
        dataAltitude.setShort(indexLine, altitude);
    }

    /**
     * Write one scan line's geolocation data to l1c data buffer
     * @param lat - latitude
     * @param lon -longitude
     */
    public void bufferGeolocationVariables(float[] lats, float[] lons) {
        float lat,lon;
        
        for (int x = 0; x < nSpotsPerLine; x++) {
            lat=lats[x];
            lon=lons[x];
            
            index2D.set(indexLine, x);
            dataLatitude.setFloat(index2D, lats[x]);
            dataLongitude.setFloat(index2D, lons[x]);
            
            //for global attribute
            if (QualityControl.isValidGeolocation(lat, lon)){
                statLatitude.update(lat);
                statLongitude.update(lon);
             }
        }
    }

    /**
     * Write one scan line's geometry data to l1c data buffer
     * @param solarZenith - solar zenith angle
     * @param sensorZenith - sensor zenith angle
     * @param localAzimuth - local azimuth angle
     */
    public void bufferGeometryVariables(float[] solarZenith, float[] sensorZenith, float[] localAzimuth) {
        if (VariableControl.isWriteGeometry()) {
            for (int x = 0; x < nSpotsPerLine; x++) {
                index2D.set(indexLine, x);
                dataSensorZenith.setFloat(index2D, sensorZenith[x]);
                if (!bSSU && !bHIRS) {
                    dataSolarZenith.setFloat(index2D, solarZenith[x]);
                    dataLocalAzimuth.setFloat(index2D, localAzimuth[x]);
                }
            }
        }
    }

    /**
     * Write one scan line's raw counts data to l1c data buffers
     * @param countsScene - scene counts
     * @param countsColdSpace - cold space counts
     * @param countsWT - warm target counts
     */
    public void bufferRawCountVariables(float[][] countsScene, float[] countsColdSpace, float[] countsWT) {
        if (!VariableControl.isWriteRawCount()) {
            return;
        }
        for (int ch = 0; ch < channelDim.getLength(); ch++) {
            index2DRawCount.set(indexLine, ch);
            if (!bSSU && !bHIRS) {
                dataColdSpaceCounts.setShort(index2DRawCount, (short) (countsColdSpace[ch] + 0.5));
                dataWarmTargetCounts.setShort(index2DRawCount, (short) (countsWT[ch] + 0.5));
            }
            for (int x = 0; x < xDim.getLength(); x++) {
                index3D.set(indexLine, x, ch);
                if (dataTypeRawCounts == DataType.SHORT) {
                    dataSceneCounts.setShort(index3D, (short) (countsScene[ch][x] + 0.5));
                } else if (dataTypeRawCounts == DataType.FLOAT) {
                    dataSceneCounts.setFloat(index3D, countsScene[ch][x]);
                }
            }
        }
    }

    /**
     * Write one temperature value to a temperature data buffer 
     * @param data - l1c temperature data buffer
     * @param ind - index of the temperature data buffer
     * @param v0 - temperature value
     */
    protected void bufferTemperatureKernel(Array data, Index ind, float v0) {
        short sv = 0;
        if (dataTypeTemperature == DataType.SHORT) {
            int v;
            if (!QualityControl.isMissingValue(v0)) {
                v = (int) (v0 * scalingFactorTemperature /*+ 0.5*/);
            } else {
                v = (int) v0;
            }
            sv = (short) (v & 0x0ffff);
            data.setShort(ind, sv);
            //sv=data.getShort(ind);
            //System.out.println("sv="+sv);
        } else {
            data.setFloat(ind, v0);
        }
    }

    /**
     * Write one scan line's quality control data to l1c file data buffer
     * @param indexT - which scene temperature 's quality control data
     * @param qc - quality control data
     */
    public void bufferQC(int indexT, QualityControlL1C qc) {
        Array dataQC = dataCFlags[indexT];
        if (dataQC != null) {
            dataQC.setByte(indexLine, qc.getScanLineQC());
            for (int ch = 0; ch < channelDim.getLength(); ch++) {
                for (int x = 0; x < xDim.getLength(); x++) {
                    index3D.set(indexLine, x, ch);
                    dataCFlagPixel[indexT].setByte(index3D, qc.getPixelLevelQC(ch, x));
                }
            }
        }
    }

    /**
     * Write one scan line's warm target temperature data to l1c file data buffer
     * @param wts - warm target temperatures
     */
    public void bufferWTTemperatureVariables(float[] wts) {
        if (wts != null) {
            for (int i = 0; i < nWTs; i++) {
                index2DWT.set(indexLine, i);
                bufferTemperatureKernel(dataWarmTargetTemperature, index2DWT, wts[i]);
            }
        }
    }

    /**
     * Save one scan line of wt converted from raw counts
     * @param wts - wt converted from raw counts
     */
    public void bufferWTConvertedFromRawCounts(int indexCalibration,float[] wts){
       if (wts != null) {
            for (int i = 0; i < wts.length; i++) {
                wtConvertedFromRawCounts[indexCalibration][indexLine][i]=wts[i];
            }
        }
        
    }
    /**
     * Write one scan line's satellite attitude data to l1c file data buffer
     * @param angles - attitude roll, pitch, yaw angles
     */
    public void bufferAttitudeVariable(float[] angles) {
        if (VariableControl.isWriteSatelliteAttitude() && angles != null) {
            for (int i = 0; i < 3; i++) {
                index2DWT.set(indexLine, i);
                dataAttitude.setFloat(index2DWT, angles[i]);
            }
        }
    }

    /*
     * not used
     */
    public void bufferSceneTemperatureVariables(float[][][] temperatures) {
        for (int i = 0; i < VAR_Ts.length; i++) {
            bufferSceneTemperatureVariable(i, temperatures[i]);
        }
    }

    /**
     * Write one scan line's scene temperature data to netcdf data buffer
     * @param indexT - which scene temperature
     * @param temperatures - scene temperatures
     */
    public void bufferSceneTemperatureVariable(int indexT, float[][] temperatures) {
        if (VariableControl.isWriteT(indexT)) {//(varControl & controlTs[indexT]) != 0) {
            Array data = dataTsNew[indexT];
            for (int ch = 0; ch < nChannels; ch++) {
                for (int x = 0; x < nSpotsPerLine; x++) {
                    index3D.set(indexLine, x, ch);
                    float v0 = temperatures[ch][x];
                    bufferTemperatureKernel(data, index3D, v0);
                }
            }

        }
    }

    /**
     * Write one scan lien of calibration coefficients to netCDF data buffer
     * current only used by AMSUA FCDR, will be used in other FCDRs later
     * @param calcoeIMICA - IMICA calibration coefficients
     * @param calcoefOPR  - OPR calibration coefficients
     * @param calcoefLinear  - LINEAR calibration coefficients
     */
    public void bufferCalCoefAMSUA(double[][] calcoefIMICA, double[][] calcoefOPR, double[][] calcoefLinear) {
        if (VariableControl.isWriteCalibrationCoefficients()) {
            int[] shape3 = null;
            for (int i = 0; i < 3; i++) {
                if (dataCalCoefAMSUA[i] != null) {
                    shape3 = dataCalCoefAMSUA[i].getShape();
                }
            }
            if (shape3 == null) {
                return;
            }

            Index3D index3 = new Index3D(shape3);

            for (int i = 0; i < nChannels; i++) {
                for (int order = 0; order < 3; order++) {

                    index3.set(indexLine, order, i);
                    if (dataCalCoefAMSUA[0] != null && calcoefIMICA != null) {
                        dataCalCoefAMSUA[0].setDouble(index3, calcoefIMICA[i][order]);
                    }

                    if (dataCalCoefAMSUA[1] != null && calcoefOPR != null) {
                        dataCalCoefAMSUA[1].setDouble(index3, calcoefOPR[i][2 - order]);
                    }
                    if (dataCalCoefAMSUA[2] != null && calcoefLinear != null) {
                        dataCalCoefAMSUA[2].setDouble(index3, calcoefLinear[i][order]);
                    }
                }
            }
        }
    }

    /**
     * Advanced to next scan line
     */
    public void nextLine() {
        indexLine++;
    }

    /**
     * calculate onboard NEDTs
     */
    public void calculateNEDT() {
        if (dataCalCoefAMSUA==null)
            return;
        
        VectorFloatAverager averager = new VectorFloatAverager(nLines);
        float wt;

        for (int indexT = 0; indexT < dataCalCoefAMSUA.length; indexT++) {
            if (dataCalCoefAMSUA[indexT] == null) {
                continue;
            }

            for (int i = 0; i < nChannels; i++) {
                averager.initAverager();
                for (int indexLine1 = 0; indexLine1 < nLines; indexLine1++) {
                    wt = wtConvertedFromRawCounts[indexT][indexLine1][i];
                    if (QualityControl.isValidWarmTargetTemperature(wt)) {
                        averager.put(wt);
                    }
                }
                float nedt = averager.calculateStddev();
                statNEDT[indexT][i] = nedt;
            }
        }
    }

    /**
     * Calculate local equator crossing time for the current orbit.
     */
    public void generateLECT() {
        if (VariableControl.isWriteLECT()) {
            float lat, latNext, lon, lonNext;
            int ip1;
            float signnum;
            Time t1 = new Time(), t2 = new Time();
            float localsolartime1, localsolartime2, localsolartime;
            float sum = 0;
            int n = 0;
            Array dataYear=varsTime.getYearData();
            Array dataDayOfYear=varsTime.getDayOfYearData();
            Array dataMillisecondOfDay=varsTime.getMillisecondOfDayData();
 
            for (int i = 0; i < nLines - 1; i++) {
                index2D.set(i, nSpotsPerLine / 2);
                lat = dataLatitude.getFloat(index2D);
                lon = dataLongitude.getFloat(index2D);
                ip1 = i + 1;
                index2D.set(ip1, nSpotsPerLine / 2);
                latNext = dataLatitude.getFloat(index2D);
                lonNext = dataLongitude.getFloat(index2D);

                signnum = Math.signum(lat * latNext);
                if (Math.abs(lat) < 20 && Math.abs(latNext) < 20 && signnum < 0) {//cross equator
                    t1.set(dataYear.getShort(i), dataDayOfYear.getShort(i), dataMillisecondOfDay.getInt(i));
                    t2.set(dataYear.getShort(ip1), dataDayOfYear.getShort(ip1), dataMillisecondOfDay.getInt(ip1));
                    localsolartime1 = Time.GMT2LocalSolarTime(t1.year, t1.getFtimeOfDay() * 24, lon);
                    localsolartime2 = Time.GMT2LocalSolarTime(t2.year, t2.getFtimeOfDay() * 24, lonNext);
                    localsolartime = util.InterpolationUtil.linearInterpolation(0.0f, lat, latNext, localsolartime1, localsolartime2);

                    if (localsolartime > 12) {
                        localsolartime -= 12;
                    }

                    sum += localsolartime;
                    n++;
                }
            }

            if (n > 0) {
                dataLECT.setFloat(0, sum / n);
            } else {
                dataLECT.setFloat(0, QualityControl.missing_value);
            }
        }
    }

    /**
     * Generate ascending/descending flag for all scan lines using geolocation
     * data.
     */
    public void generateAscendingDescendingFlagV1() {
        if (VariableControl.isWriteASDFlag()) {
            byte flag;
            float lat, latNext;
            byte[] asdFlag = new byte[nLines];

            index2D.set(0, nSpotsPerLine / 2);
            lat = dataLatitude.getFloat(index2D);
            asdFlag[0] = -1;
            for (int i = 1; i < nLines; i++) {
                index2D.set(i, nSpotsPerLine / 2);
                latNext = dataLatitude.getFloat(index2D);
                if (latNext > 90 || latNext < -90 || lat > 90 || lat < -90) {
                    flag = ASD_UNDEFINED;
                } else {
                    if (latNext > lat) {
                        flag = ASD_ASCENDING;
                    } else {
                        flag = ASD_DESCENDING;
                    }
                }
                asdFlag[i] = flag;
                lat = latNext;
            }

            for (int i = 0; i < nLines; i++) {
                index2D.set(i, nSpotsPerLine / 2);
                lat = dataLatitude.getFloat(index2D);
                if (asdFlag[i] == ASD_UNDEFINED && lat < 90 && lat > -90 && i < nLines - 1) {
                    asdFlag[i] = asdFlag[i + 1];
                }
                dataASDFlag.setByte(i, asdFlag[i]);
            }
        }
    }

    /**
     * not used.
     */
    public void generateAscendingDescendingFlagV0() {
        try {
            Variable varLat = ncfileWritable.findVariable(pub.variable.CDRVariablesL1C.VAR_LATITUDE);
            int[] org = {0, 0};
            int[] shape = {nLines, 1};
            byte[] asdFlag = new byte[nLines];
            ucar.ma2.Array data = varLat.read(org, shape);
            byte flag;
            float lat0, lat, latNext;
            int debug = 1;

            lat = data.getFloat(0);
            lat0 = lat;
            asdFlag[0] = -1;
            for (int i = 1; i < nLines; i++) {
                if (i == 79) {
                    debug = 0;
                }

                latNext = data.getFloat(i);
                if (latNext > 90 || latNext < -90 || lat > 90 || lat < -90) {
                    flag = ASD_UNDEFINED;
                } else {
                    if (latNext > lat) {
                        flag = ASD_ASCENDING;
                    } else {
                        flag = ASD_DESCENDING;
                    }
                }
                asdFlag[i] = flag;
                lat = latNext;
            }

            for (int i = 0; i < nLines; i++) {
                lat = data.getFloat(i);
                if (asdFlag[i] == ASD_UNDEFINED && lat < 90 && lat > -90 && i < nLines - 1) {
                    asdFlag[i] = asdFlag[i + 1];
                }
            }

            netCDFUtil.writeByteVariable1D(ncfileWritable, VAR_ASDFLAG, asdFlag);

        } catch (java.io.IOException e) {
        } catch (ucar.ma2.InvalidRangeException e) {
        }
    }
///////////////////////////////////////////////////////////////////    
    /**
     * Write level 1c variables to netcdf file
     * @return 0 if all variables are written successfully, otherwise return -1.
     */
    public int writeVariables() {
        try {
            varsTime.writeDataToFile(ncfileWritable);

            ncfileWritable.write(pub.variable.CDRVariablesL1C.VAR_LATITUDE, dataLatitude);
            ncfileWritable.write(pub.variable.CDRVariablesL1C.VAR_LONGITUDE, dataLongitude);

            if (VariableControl.isWriteGeometry()) {
                ncfileWritable.write(VAR_SENSORZENITH, dataSensorZenith);
                if (!bSSU && !bHIRS) {
                    ncfileWritable.write(VAR_SOLARZENITH, dataSolarZenith);
                    ncfileWritable.write(VAR_LOCALAZIMUTH, dataLocalAzimuth);
                }
            }

            ncfileWritable.write(VAR_ALTITUDE, dataAltitude);

            if (VariableControl.isWriteASDFlag()) {
                ncfileWritable.write(VAR_ASDFLAG, dataASDFlag);
            }


            if (VariableControl.isWriteLECT()) {
                ncfileWritable.write(VAR_LECT, dataLECT);
            }
            if (VariableControl.isWriteSatelliteAttitude()) {
                ncfileWritable.write(VAR_ATTITUDE, dataAttitude);
            }

            //write scene temperature variables
            writeSceneTemperatureVariables();

            if (VariableControl.isWriteWT()) {
                ncfileWritable.write(VAR_WARMTARGETTEMPERATURE, dataWarmTargetTemperature);
            }

            if (VariableControl.isWriteRawCount()) {
                ncfileWritable.write(VAR_SCENECOUNTS, dataSceneCounts);
                if (!bSSU && !bHIRS) {
                    ncfileWritable.write(VAR_COLDSPACECOUNTS, dataColdSpaceCounts);
                    ncfileWritable.write(VAR_WARMTARGETCOUNTS, dataWarmTargetCounts);
                }
            }

            

        } catch (java.io.IOException e) {
            e.printStackTrace();
            return -1;
        } catch (ucar.ma2.InvalidRangeException e) {
            e.printStackTrace();
            return -1;
        }
        return 0;
    }

    public int writeSceneTemperatureVariables() {
        int status = 0;
        boolean[] bCFlagWritten = {false, false, false};
        try {
            for (int i = 0; i < VAR_Ts.length; i++) {
                if (VariableControl.isWriteT(i)) {
                    ncfileWritable.write(VAR_Ts[i], dataTsNew[i]);

                    int indexCFlag = i / 2;
//                    if (i%2==0){//T_SNO,T_OPR,T_LINEAR
                    if (!bCFlagWritten[indexCFlag]) {
                        ncfileWritable.write(VAR_CFLAGs[indexCFlag], dataCFlags[indexCFlag]);
                        ncfileWritable.write(VAR_CFLAGPIXELs[indexCFlag], dataCFlagPixel[indexCFlag]);

                        if (VariableControl.isWriteCalibrationCoefficients()) {
                            if (dataCalCoefAMSUA != null && dataCalCoefAMSUA[indexCFlag] != null) {
                                ncfileWritable.write(VAR_CALCOEF_AMSUA[indexCFlag], dataCalCoefAMSUA[indexCFlag]);
                            }
                        }

                        bCFlagWritten[indexCFlag] = true;
                    }
//                    }
                }
            }
        } catch (java.io.IOException e) {
            e.printStackTrace();
            return -1;
        } catch (ucar.ma2.InvalidRangeException e) {
            e.printStackTrace();
            return -1;
        }
        return status;
    }
    
    
    /**
     * Update starting and ending time of a level 1c file
     * @param tStart - staring observation time
     * @param tEnd - ending observations time
     */
    public void updateTime(Time tStart, Time tEnd) {
        if (tStart.compare1(timeStart) < 0) {
            timeStart.set(tStart);
        }
        if (tEnd.compare1(timeEnd) > 0) {
            timeEnd.set(tEnd);
        }

    }

    /**
     * Not used
     * @param year0 
     */
    public void qualityControlTime(int year0) {
        short year, dayOfYear;
        int millisecondOfDay;
        Array dataYear=varsTime.getYearData();
        Array dataDayOfYear=varsTime.getDayOfYearData();
        Array dataMillisecondOfDay=varsTime.getMillisecondOfDayData();
        
        for (int i = 0; i < nLines; i++) {
            year = dataYear.getShort(i);
            dayOfYear = dataDayOfYear.getShort(i);
            millisecondOfDay = dataMillisecondOfDay.getInt(i);

            if (Math.abs(year - year0) > 1
                    || dayOfYear < 1 || dayOfYear > 366
                    || millisecondOfDay < 0 || millisecondOfDay > Time.MILLISECONDS_PER_DAY) {
                dataYear.setShort(i, QualityControl.missing_value_short);
                dataDayOfYear.setShort(i, QualityControl.missing_value_short);
                dataMillisecondOfDay.setInt(i, QualityControl.missing_value_short);


            }
        }

    }
}
