/*@***h* AMSU-A_FCDR_STAR/pub.L1C.L1CLineNetCDF.java
* NAME
*   pub.L1C.L1CLineNetCDF.java 
* 
* PURPOSE
*   To support reading scan line of level 1c file. 
* 
* DESCRIPTION
*   This class defines variables and methods for one scan line of level 1c file.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   07/30/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 
package pub.L1C;

import grid.*;
import noaa.*;
import pub.QualityControl;
import ucar.ma2.*;
import util.*;


public class L1CLineNetCDF extends NOAADataSetRecord {
    protected L1CFileNetCDF fileL1C;
    protected static LandType landType=null;    
    
    protected byte[][][] cFlagsPixel; //3*NUM_CHANNELS*NUM_SPOTS_PER_LINE   
    protected byte[] cFlagsScanline;

    
    byte asdFlag;
    float[] lat=null; //new float[NUM_SPOTS_PER_LINE];
    float[] lon=null; //new float[NUM_SPOTS_PER_LINE];  
    short[][] xy=null; //new short[NUM_SPOTS_PER_LINE][2];
    byte[] landcovertype=null; //new byte[NUM_SPOTS_PER_LINE];
    
    
    //can be antenna temp, brightness temp, limb correct antenna temp, or limb corrected brightness temp
    //can also be scene counts
    float[][][] temperatures=null; //new float[4][NUM_CHANNELS][NUM_SPOTS_PER_LINE];  //4 calibration types + 4 channels + spots
 
    float[] temperaturesWT=null; //new float[NUM_SUBINSTRUMENTS];
    
    //scene, cold ,warm counts for one specified channel
    float[][] countsScene=null; //new float[NUM_CHANNELS][NUM_SPOTS_PER_LINE];  
    float[] countsColdSpace=null, //new float[NUM_CHANNELS],
             countsWarmTarget=null; //new float[NUM_CHANNELS];
    short satelliteAltitude; //because short is enough for satellite altitude
    float[] satelliteAttitudes; //roll,pitch, yaw
    short ect;
    
    int nChannels,nSpotsPerLine,nSubInstruments,nCalibrationMethods;
    
    public L1CLineNetCDF(int NUM_CHANNELS, int NUM_SPOTS_PER_LINE, int NUM_SUBINSTRUMENTS, int NUM_CALIBRATIONS) {
        nChannels = NUM_CHANNELS;
        nSpotsPerLine = NUM_SPOTS_PER_LINE;
        nSubInstruments = NUM_SUBINSTRUMENTS;
        nCalibrationMethods = NUM_CALIBRATIONS;

        cFlagsPixel = new byte[NUM_CALIBRATIONS][NUM_CHANNELS][NUM_SPOTS_PER_LINE];
        cFlagsScanline = new byte[NUM_CALIBRATIONS];
        lat = new float[NUM_SPOTS_PER_LINE];
        lon = new float[NUM_SPOTS_PER_LINE];
        xy = new short[NUM_SPOTS_PER_LINE][2];
        landcovertype = new byte[NUM_SPOTS_PER_LINE];
        countsScene = new float[NUM_CHANNELS][NUM_SPOTS_PER_LINE];
        countsColdSpace = new float[NUM_CHANNELS];
        countsWarmTarget = new float[NUM_CHANNELS];
        temperaturesWT = new float[NUM_SUBINSTRUMENTS];
        temperatures = new float[L1CFileNetCDF.VAR_Ts.length][NUM_CHANNELS][NUM_SPOTS_PER_LINE];  //4 calibration types + 4 channels + spots    
        satelliteAttitudes=new float[3];
    }

    public void setL1CFile(L1CFileNetCDF file){
        fileL1C=file;
    }

    public boolean isGoodScanline(){
        for (int i=0;i<nCalibrationMethods;i++){
            if (isGoodScanline(i))
                return true;
        }
        return false;
    }
    
    //MSU OR AMSU-A may have different standard, need to override this function in sub class
    public boolean isGoodScanline(int indexT){
        return cFlagsScanline[indexT]==0;    
    }
    
    public boolean isGoodPixel(int indexCFlag,int channel,int spot){
        byte flag=cFlagsPixel[indexCFlag][channel][spot];
        return flag==0;
    }
    
    public boolean isPrecipitationPixel(int indexCFlag,int channel,int spot){
        boolean bPrecip=(cFlagsPixel[indexCFlag][channel][spot] & QualityControlL1C.QC_PRECIPITATION)!=0;
        return bPrecip;
    }
    
     public byte getASDFlag(){
        return asdFlag;
    }
    public static void setLandType(LandType landType0){
        landType=landType0;
    }

    public short getSatelliteAltitude(){
        return satelliteAltitude;
    }
    public float[] getSatelliteAttitude(){
        return satelliteAttitudes;
    }
    
    public float[] getLat(){
        return lat;
    }

    public float[] getLon(){
        return lon;
    }

    public short[][] getXY(){
        return xy;
    }
    
    public byte[] getLandCoverType(){
        return landcovertype;
    }

    ////////////////////////////////////////////////////////////////////
    public float[][][] getTemperatures(){
        return temperatures;
    }
    public float[][] getTemperatures(int indexT){
        return temperatures[indexT];
    }
 
    
    public float[] getTemperatureWT(){
        return temperaturesWT;
    }
 
    public short getEquatorCrossingTime(){
        return ect;
    }

    public float[] getColdSpaceCounts(){
        return countsColdSpace;
    }
    public float[] getWarmTargetCounts(){
        return countsWarmTarget;
    }
    public float[][] getSceneCounts(){
        return countsScene;
    }
   
   
    
    public void readScanLineQualityControl(int indexLine){
          for (int iCal = 0; iCal < nCalibrationMethods; iCal++) {
                if (fileL1C.dataCFlags[iCal] != null) {
                    cFlagsScanline[iCal] = fileL1C.dataCFlags[iCal].getByte(indexLine);
                } else {
                    cFlagsScanline[iCal] = pub.QualityControl.missing_value_byte;
                }
            }
         
    }

    /**
     * Read one scan line of data from a level 1c file
     * @param indexLine - line to be read
     * @return 0 if successful, otherwise return a non-zero value
     */
    public int readLine(int indexLine) {
        int i=0, j=0;
        float temp=0;
        if (fileL1C == null) {
            return -1;
        }
        try {
            //READ QUALITY CONTROL
            if (fileL1C.dataASDFlag!=null)
                asdFlag = fileL1C.dataASDFlag.getByte(indexLine);
            else
                asdFlag=QualityControl.missing_value_byte;
            
            if (fileL1C.dataAltitude!=null)
                satelliteAltitude = (short) fileL1C.dataAltitude.getFloat(indexLine);
            else
                satelliteAltitude=QualityControl.missing_value_short;
            

            int nScanLines = fileL1C.getNumberOfRecords();
            int[] shape2 = {nScanLines, nSpotsPerLine};
            Index2D index2D = new Index2D(shape2);
            for (i = 0; i < nSpotsPerLine; i++) {
                index2D.set(indexLine, i);
                lat[i] = fileL1C.dataLatitude.getFloat(index2D);
                lon[i] = fileL1C.dataLongitude.getFloat(index2D);
            }
            
            
            //calculate grid cooridnate and landcovertype of each spot
            for (i = 0; i < nSpotsPerLine; i++) {
                if (lat[i] < -90 || lat[i] > 90 || lon[i] < -180 || lon[i] > 180) {
                    xy[i][0] = -9999;
                    xy[i][1] = -9999;
                    landcovertype[i] = LandType.UNDEFINED;
                } else {//caliculate grid xy           
                    Grid.getGrid(lat[i], lon[i], xy[i]);
                    //extract land type of each spot   
                    if (landType != null) {
                        landcovertype[i] = landType.getType1(xy[i][0], xy[i][1]);
                    }
                }
            }

            //read a line of qc
            readScanLineQualityControl(indexLine);
            //read channel pixel level qc
            int[] shape3D = {nScanLines, nSpotsPerLine, nChannels};
            Index3D index3D = new Index3D(shape3D);
               for (int iCal = 0; iCal < nCalibrationMethods; iCal++) {
               for (i = 0; i < nChannels; i++) {
                    for (j = 0; j < nSpotsPerLine; j++) {
                        index3D.set(indexLine, j, i);
                        if (fileL1C.dataCFlagPixel[iCal] != null) {
                            cFlagsPixel[iCal][i][j] = fileL1C.dataCFlagPixel[iCal].getByte(index3D);
                        } else {
                            cFlagsPixel[iCal][i][j] = pub.QualityControl.missing_value_byte;
                        }                        
                    }
                }
            }

            //read temperatures
           
            float scalingT = L1CFileNetCDFWritable.getTemperatureVariableScalingFactor();
            for (int iTemp = 0; iTemp < fileL1C.VAR_Ts.length; iTemp++) {
                for (i = 0; i < nChannels; i++) {
                    for (j = 0; j < nSpotsPerLine; j++) {
                        index3D.set(indexLine, j, i);
                        if (fileL1C.dataTsNew[iTemp] != null) {
                            temp = fileL1C.dataTsNew[iTemp].getFloat(index3D);
                            if (!pub.QualityControl.isMissingValue(temp)) {
                                temperatures[iTemp][i][j] = temp / scalingT;
                            } else {
                                temperatures[iTemp][i][j] = pub.QualityControl.missing_value;
                            }
                        } else {
                            temperatures[iTemp][i][j] = pub.QualityControl.missing_value;
                        }
                    }
                }
            }

          
            //read warm target temperature
            int[] shape1 = {nScanLines, nSubInstruments};
            Index2D index2 = new Index2D(shape1);
            if (fileL1C.dataWarmTargetTemperature != null) {
                for (i = 0; i < nSubInstruments; i++) {
                    index2.set(indexLine, i);
                    temp = fileL1C.dataWarmTargetTemperature.getFloat(index2);
                    if (!pub.QualityControl.isMissingValue(temp)) {
                        temperaturesWT[i] = temp / scalingT;
                    } else {
                        temperaturesWT[i] = pub.QualityControl.missing_value;
                    }
                }
            }
     

            //read one line of raw counts
            if (fileL1C.dataSceneCounts != null) {
                for (i = 0; i < nChannels; i++) {
                    for (j = 0; j < nSpotsPerLine; j++) {
                        index3D.set(indexLine, j, i);
                        countsScene[i][j] = (float) (fileL1C.dataSceneCounts.getShort(index3D));
                    }
                }
            }
            int[] shape22 = {nScanLines, nChannels};
            Index2D index2D2 = new Index2D(shape22);
            if (fileL1C.dataColdSpaceCounts != null) {
                for (i = 0; i < nChannels; i++) {
                    index2D2.set(indexLine, i);
                    countsColdSpace[i] = (float) (fileL1C.dataColdSpaceCounts.getShort(index2D2));
                }
            }
            if (fileL1C.dataWarmTargetCounts != null) {
                for (i = 0; i < nChannels; i++) {
                    index2D2.set(indexLine, i);
                    countsWarmTarget[i] = (float) (fileL1C.dataWarmTargetCounts.getShort(index2D2));
                }
            }

            if (fileL1C.dataAttitude != null) {
                index2 = new Index2D(fileL1C.dataAttitude.getShape());
                for (i = 0; i < 3; i++) {
                    //System.out.println("indexLine=" + indexLine + " i=" + i);
                    index2.set(indexLine, i);
                    satelliteAttitudes[i] = fileL1C.dataAttitude.getFloat(index2);
                }
            }
        } catch (java.lang.ArrayIndexOutOfBoundsException e) {
             System.out.println("Level 1C Read line filed");
             System.out.println("indexLine=" + indexLine + " i=" + i+" j="+j);
             System.exit(-1);
        }
        return 0;
    }

  
 }
