/*@***h* AMSU-A_FCDR_STAR/pub.L1C.QualityControlL1C.java
* NAME
*   pub.L1C.QualityControlL1C.java 
* 
* PURPOSE
*   To support level 1C quality controls.
* 
* DESCRIPTION
*   This class define constants, variables, and methods for level 1c file scan line
*   and pixel level quality control flags
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   04/10/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 
package pub.L1C;

import amsu.amsua.precipitation.*;
import pub.QualityControl;


public class QualityControlL1C {

    int NUM_CHANNELS, NUM_SPOTS_PER_LINE;
    public static final byte QC_GOOD = 0;
    public static final byte QC_L1B_FAILED = 0x01;
    public static final byte QC_GEOLOCATION_FAILED = 0x02;
    public static final byte QC_PRECIPITATION = 0x04;
    public static final byte QC_BAD_CALIBRATION_COUNTS = 0x08;
    byte[][] qcDetailed = null;  //pixel level quality control
    byte qcLine = 0;  //quality indicator for the entire line

    
    public QualityControlL1C(int nChannels, int nSpotsPerLine) {
        NUM_CHANNELS = nChannels;
        NUM_SPOTS_PER_LINE = nSpotsPerLine;
        qcDetailed = new byte[NUM_CHANNELS][NUM_SPOTS_PER_LINE];
    }

    /**
     * Get pixel level quality control
     * @return pixel level quality control
     */
    public byte[][] getQC() {
        return qcDetailed;
    }

    /**
     * Initialize scan line and pixel level quality control flags for a scan line
     */
    public void init() {
        qcLine = 0;
        for (int i = 0; i < qcDetailed.length; i++) {
            for (int j = 0; j < qcDetailed[0].length; j++) {
                qcDetailed[i][j] = 0;
            }
        }
    }

    /**
     * Set scan line quality control flag
     * @param good - whether a scan line is good or not
     */
    public void setScanLineQC(boolean good) {
        if (good){
            qcLine = 0;
        } else{
            qcLine=1;
        }
    }

    public void setScanlineQCSSU(boolean good,boolean bCalibrationLine){
        if (good){
            qcLine=(byte)(bCalibrationLine?2:0);
        }else{
            qcLine=1;
        }
    }
    
    public void setScanlineQCHIRS(boolean bGood, byte scanType){
        int flag=0;
        if (bGood){
            if (scanType>0){
                flag=1;
            }
            qcLine=scanType;
        }else{
            qcLine=0x7F;
        }
    }
    
    /**
     * Get scan line quality control byte
     * @return scan line quality control byte
     */
    public byte getScanLineQC() {
        return qcLine;

    }

    /**
     * Check if a scan line is good (at least one pixel is good)
     * @return true if a scan line is good.
     */
    public boolean isGoodScanLine() {
        return qcLine==0;
    }

    /**
     * Get pixel level quality control for a channel/scan 
     * @param channel - channel
     * @param x - scan position
     * @return pixel level quality control for a channel/scan 
     */
    public byte getPixelLevelQC(int channel, int x) {
        return qcDetailed[channel][x];
    }
    

    /**
     * Set all temperatures in a scan line bad
     * @param temperatures - scene temperature for a scan line
     */
    public void qualityControlL1B(float[][] temperatures) {
        for (int spot = 0; spot < NUM_SPOTS_PER_LINE; spot++) {
            for (int channel = 0; channel < NUM_CHANNELS; channel++) {
                temperatures[channel][spot] = QualityControl.missing_value;
                qcDetailed[channel][spot] |= QC_L1B_FAILED;
            }
        }
    }

    /**
     * Check whether all geolocation data in a scan line are zeros
     * @param lat - latitude
     * @param lon - longitude
     * @return true if all geolocation data in a scan line are zeros,
     * otherwise return false.
     */
    boolean isAllGeolocationZero(float[] lat, float[] lon) {
        for (int i = 0; i < lat.length; i++) {
            if (Math.abs(lat[i]) > 1.E-7) {
                return false;
            }
        }
        for (int i = 0; i < lon.length; i++) {
            if (Math.abs(lon[i]) > 1.E-7) {
                return false;
            }
        }
        return true;
    }
   
    /**
     * Pixel level quality control using geolocation information
     * 
     * @param lat - latitude
     * @param lon - longitude
     * @param temperatures - scene temperatures for a scan line
     * @return number of bad pixels detected using geolocation information.
     */

    public int qualityControlGeolocation(float[] lat, float[] lon, float[][] temperatures) {
        int nBad = 0;

        boolean bAllZero = isAllGeolocationZero(lat, lon);

        for (int i = 0; i < NUM_SPOTS_PER_LINE; i++) {
            for (int channel = 0; channel < NUM_CHANNELS; channel++) {
                if (bAllZero || lat[i] < -90 || lat[i] > 90 || lon[i] < -180 || lon[i] > 180) {
                    temperatures[channel][i] = QualityControl.missing_value;;
                    lat[i]=QualityControl.missing_value;
                    lon[i]=QualityControl.missing_value;
                    if (qcDetailed != null) {
                        qcDetailed[channel][i] |= QC_GEOLOCATION_FAILED;
                    }
                    nBad++;
                }
            }
        }
        return nBad;
    }

    /**
     * Set calibration quality control flags for a scan line using level lb 
     * calibration flag (based target raw counts)
     * @param temperatures  - scene temperatures for a scan line
     * @param flagCalibration - calibration quality control flags
     * @param sceneCounts - scene raw counts
     * @param coldCounts - cold space raw counts
     * @param wtCounts - warm target raw counts, not used currently, but may be used in the future
     * @return number of bad pixels
     */
    public int qaulityControlCalibration(float[][] temperatures, short[] flagCalibration,
            float[][] sceneCounts, float[] coldCounts, float[] wtCounts) { 
        int nBad = 0;
        int i, channel;
        float t;
        boolean bBadAtmT;
        for (i = 0; i < NUM_SPOTS_PER_LINE; i++) {
            for (channel = 0; channel < NUM_CHANNELS; channel++) {
                t = temperatures[channel][i];
 
                //NK.D98301.S2348.E0134.B0239395.WI, line 510/spot 10  (all start from 0)
                //may be caused by inormal scene counts
                //surface channels are not considered at this time
                //channel contrain removed due to processing ssu data Jan 25, 2013
                bBadAtmT = (channel>=3 && channel<=13 && (t < QualityControl.TEMPERATURE_MIN
                        || t > QualityControl.TEMPERATURE_MAX));
                //bBadAtmT= (t<QualityControl.TEMPERATURE_MIN ||                    
                //                                     t>QualityControl.TEMPERATURE_MAX); 
                if (bBadAtmT) {
                    nBad++;
                }
                if (flagCalibration[channel] != 0 ||
                  (sceneCounts != null && coldCounts != null && 
                        (sceneCounts[channel][i] < coldCounts[channel] || 
                        Math.abs(coldCounts[channel]-wtCounts[channel])<pub.L1C.L1CCalibrator.WARM_COLD_COUNT_DISTANCE)) || //scene counts must be larger than cold count
                          bBadAtmT) {  //scene counts error

                    temperatures[channel][i] = QualityControl.missing_value; // -9999;
                    nBad++;
                    if (qcDetailed != null) {
                        qcDetailed[channel][i] |= QC_BAD_CALIBRATION_COUNTS;
                    }
               }
            }
        }
        return nBad;

    }

    static public void qualityControlWT(float[] wts, float min, float max){
        float t;
        for (int i=0;i<wts.length;i++){
            t=wts[i];
            if (!QualityControl.isMissingValue(t)){
                if (t < min || t > max) {
                    wts[i] = QualityControl.missing_value;
                }
            }
        }
    }
    
    static public short qualityControlSpacecraftAltitude(short alt, float min, float max) {
        if (!QualityControl.isMissingValue(alt)) {

            if (alt < min || alt > max) {
                return QualityControl.missing_value_short;
            } else {
                return alt;
            }
        } else {
            return alt;
        }
        /*
        float alt;
        for (int i=0;i<altitude.length;i++){
            alt=altitude[i];
            if (alt<min || alt>max){
                altitude[i]=QualityControl.missing_value;
            }
        }*/
    }

    /**
     * Set pixel level quality control flag for precipitation
     * @param precip - precipitation flag
     * @param temperatures - scene temperatures for one scan line
     */
    public void qualityControlPrecipitation(PrecipitationPub precip, float[][] temperatures) {
        int channel;
        boolean[] precipFlag = precip.getPrecipitationFlag();

        for (int spot = 0; spot < NUM_SPOTS_PER_LINE; spot++) {
            if (precipFlag[spot]) {
                //set missing_value for channels affected by precipitation 
                for (int i = 0; i < PrecipitationPub.channelsAffectedByPrecip.length; i++) {
                    channel = PrecipitationPub.channelsAffectedByPrecip[i];
                    //temperatures[channel][spot] = missing_value;
                    qcDetailed[channel][spot] |= QC_PRECIPITATION;
                }
            }
        }
    }

    /**
     * Get scan line quality control metadata
     * @return scan line quality control metadata
     */
    public static String getL1CQCStringLine() {
//        return "0-at_least_1_channel/pixel_is_good 1-bad_scan_line";
        return "at_least_1_channel/pixel_is_good bad_scan_line";
    }

  /**
     * Get pixel level quality control metadata
     * @return pixel level quality control metadata
     */
    public static String getL1CQCStringChannelPixel() {
        String str;
//        str = "0-good pixel; 1-bad pixel due to bad scan line; 2-bad pixel due to bad geolcation; 4-bad pixel due to precipitation; 8-bad pixel due to calibration counts";
        str = "good_pixel "+
              "bad_pixel_due_to_bad_scan_line "+
              "bad_pixel_due_to_bad_geolcation "+
              "bad_pixel_due_to_precipitation "+
              "bad_pixel_due_to_calibration_counts";

        
        return str;
    }
    
       //quality control using level lb calibration flag (based target raw counts)
    public int qaulityControlCalibrationSSU(boolean bCalibrationLine,float[][] temperatures, float[][] sceneCounts,
            float minColdCount[],float maxWarmCount[]) { 
        int nBad = 0;
        int i, channel;
        float t,count;
        boolean bBadAtmT,bBadCount;
        for (i = 0; i < NUM_SPOTS_PER_LINE; i++) {
            for (channel = 0; channel < NUM_CHANNELS; channel++) {
                bBadCount=false;
                bBadAtmT=false;
                t = temperatures[channel][i];
                count=sceneCounts[channel][i];
                
                 bBadAtmT = ((t < QualityControl.TEMPERATURE_MIN || t > QualityControl.TEMPERATURE_MAX));
                 
                if (bCalibrationLine){
                    bBadCount=(i<4 && count<minColdCount[channel]) || (i>4 && count>maxWarmCount[channel]);
                     
                }else{
                        bBadCount=count<maxWarmCount[channel] || count>minColdCount[channel];
                }
                
                if (bBadAtmT || bBadCount) {  //scene counts error

                    temperatures[channel][i] = QualityControl.missing_value; // -9999;
                    nBad++;
                    if (qcDetailed != null) {
                        qcDetailed[channel][i] |= QC_BAD_CALIBRATION_COUNTS;
                    }


                }
            }
        }
        return nBad;

    }

    
       //quality control using level lb calibration flag (based target raw counts)
    public int qaulityControlCalibrationHIRS(float[][] temperatures, short[] flagCalibration,
            float[][] rawcounts , byte scanType) { //wtCounts not used currently, but may be used in the future
        int nBad = 0;
        int i, channel;
        float t;
        boolean bBadT=false;
        for (i = 0; i < NUM_SPOTS_PER_LINE; i++) {
            for (channel = 0; channel < NUM_CHANNELS; channel++) {
                t = temperatures[channel][i];
                if (scanType==0 || scanType==2 ){ //scene or internal cold target
                    bBadT = ((t < QualityControl.HIRS_TEMPERATURE_MIN
                                || t > QualityControl.HIRS_TEMPERATURE_MAX));
                }else if (scanType==1){//cold space view
                    if (i<8)
                        bBadT=true;
                    else
                        bBadT=false;
                }else if (scanType==3){//warm target view
                    bBadT = !QualityControl.isValidWarmTargetTemperature(t);
                }
                
                if (bBadT) {
                    nBad++;
                }
                if (flagCalibration[channel] != 0 || bBadT) {  //scene counts error

                    temperatures[channel][i] = QualityControl.missing_value; // -9999;
                    nBad++;
                    if (qcDetailed != null) {
                        qcDetailed[channel][i] |= QC_BAD_CALIBRATION_COUNTS;
                    }
                    if (rawcounts!=null){
                        rawcounts[channel][i]= QualityControl.missing_value;
                    }

                }
            }
        }
        return nBad;

    }

}
