/*@***h* AMSU-A_FCDR_STAR/pub.QualityControl.java
* NAME
*   pub.QualityControl.java 
* 
* PURPOSE
*   To define constants and methods for quality control of different satellite 
*   instrument measurements.
* 
* DESCRIPTION
*   This class defines constants for missing value, minimum, maximum values of 
*   scene temperature or radiance, warm target temperatures, and satellite altitude. 
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   01/23/2009
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
package pub;

import grid.Grid;

public class QualityControl {

    
    public static final float missing_value = -9999;
    public static final short missing_value_short = -9999;
    public static final double missing_value_double = -9999.0;
    public static final String missing_value_string = "-9999";
    public static final byte missing_value_byte = (byte) 0xff;
    public static final float TEMPERATURE_MAX = 300;
    public static final float TEMPERATURE_MIN = 170;
    public static final float TEMPERATURE_RANGE_MAX = 327.67f;
    public static final float TEMPERATURE_RANGE_MIN = 150.0f;
    public static final float HIRS_TEMPERATURE_MAX = 350;
    public static final float HIRS_TEMPERATURE_MIN = 170;
    public static final float SSU_RADIANCE_MIN = 10;
    public static final float SSU_RADIANCE_MAX = 100;
    public static final float TEMPERATURE_MAX_WT = 350;
    public static final float TEMPERATURE_MIN_WT = 250;
    public static final float ALTITUDE_MAX = 10000;   //KM*10
    public static final float ALTITUDE_MIN = 6000;    //KM*10

    static public boolean bPrintInvalidAltitude = false;
    static public boolean bPrintInvalidGridXY = false;
    static public boolean bPrintInvalidGeolocation = false;

    /**
     * To check whether a radiance value is a valid SSU channel radiance
     *
     * @param rad - radiance
     * @return true if it is valid, otherwise return false.
     */
    static public boolean isValidSSURadiance(float rad) {
        return rad > SSU_RADIANCE_MIN && rad < SSU_RADIANCE_MAX;
    }

    /**
     * To check whether a radiance value is a valid HIRS channel radiance
     *
     * @param rad - radiance
     * @return true if it is valid, otherwise return false.
     */
    static public boolean isValidHIRSRadiance(float rad) {
        return rad > SSU_RADIANCE_MIN && rad < SSU_RADIANCE_MAX;
    }

    /**
     * To check whether a temperature value is a valid scene temperature
     *
     * @param temp - temperature
     * @return true if it is valid, otherwise return false.
     */
    static public boolean isValidTemperature(float temp) {
        return temp > TEMPERATURE_MIN && temp < TEMPERATURE_MAX;
    }

    /**
     * To check whether a temperature value is a valid warm target temperature
     *
     * @param temp - temperature
     * @return true if it is valid, otherwise return false.
     */
    static public boolean isValidWarmTargetTemperature(float temp) {
        return temp > TEMPERATURE_MIN_WT && temp < TEMPERATURE_MAX_WT;
    }

    /**
     * To check whether a given value is the missing value (-9999)
     *
     * @param v - value to be checked
     * @return true if it is missing value, otherwise return false.
     */
    static public boolean isMissingValue(float v) {
        return Math.abs(v - missing_value) < 0.0001;
    }

 
    /**
     * To check whether geolocation data is valid
     *
     * @param lat - latitude
     * @param lon - longitude
     * @return true if geolocation data is valid, otherwise return false.
     */
    static public boolean isValidGeolocation(float lat, float lon) {
        return lat >= -90.0f && lat <= 90.0f && lon >= -180.0f && lon <= 180.0f;
    }

    /**
     * To check whether a pair of level 3 grid coordinates is valid
     *
     * @param x - x coordinate of a grid
     * @param y - y coordinate of a grid
     * @return true if the coordinate is valid, otherwise return false.
     */
    static public boolean isValidGridXY(int x, int y) {
        return x >= 0 && x < Grid.GRID_X_MAX && y >= 0 && y < Grid.GRID_Y_MAX;
    }

    /**
     * Quality control of latitude information. If a given latitude is out of
     * limit, the closest valid latitude is return
     *
     * @param xxlat - raw latitude before quality control
     * @return the latitude given if it is valid, otherwise return a valid
     * latitude value that is closet to it.
     */
    static public float qcLatitude(float xxlat) {
        if (xxlat < -90) {
            xxlat = -90;
        } else if (xxlat > 90) {
            xxlat = 90;
        }
        return xxlat;
    }

    /**
     * Quality control of longitude information. If a given longitude is out of
     * limit, the closest valid longitude is return
     *
     * @param xxlon - raw longitude before quality control
     * @return the longitude given if it is valid, otherwise return a valid
     * longitude value that is closet to it.
     */
    static public float qcLongitude(float xxlon) {
        if (xxlon < -180) {
            xxlon = -180;
        } else if (xxlon > 180) {
            xxlon = 180;
        }
        return xxlon;
    }

    /**
     * Set the value of 2D float array to filling value
     * @param temps - 2D float array
     */
    public static void setFillValues(float[][] temps) {
        for (int i = 0; i < temps.length; i++) {
            for (int j = 0; j < temps[0].length; j++) {
                temps[i][j] = missing_value;
            }
        }
    }
}
