/*@***h* AMSU-A_FCDR_STAR/pub.attribute.AttributeItem.java
* NAME
*   pub.attribute.AttributeItem.java 
* 
* PURPOSE
*   To define attribute item (name, value) used by both global attributes and 
*   variable attributes.
* 
* DESCRIPTION
*   This class provides basic netCDF global and variable attribute object, it has 
*   two variables: attrName, attrValue
*   It also provides methods for creating netCDF global or variable attributes.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/01/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/04/2013 -- add methods for supporting variable attribute.
*
*@*****/

 

package pub.attribute;

import ucar.ma2.*;
import ucar.nc2.Attribute;
import ucar.nc2.NetcdfFileWriteable;

public class AttributeItem {

    String attrname; //attribute name
    Object value;    //attribute value

    /**
     * Constructor
     * @param name - attribute name
     * @param value - attribute value
     */
    public AttributeItem(String name, Object value) {
        attrname = name;
        this.value = value;
    }

    /**
     * Get name and value string of the attribute item
     * @return name and value string of the attribute item
     */
    @Override
    public String toString(){
        String str=attrname+"="+value.toString();
        return str;
    }
    
    /**
     * Set the value of a attribute time
     * @param value - value to be set
     */
    public void setValue(Object value) {
        this.value = value;
    }

    /**
     * Get the value of a attribute item
     * @return value object of this attribute item
     */
    public Object getValue(){
        return value;
    }
    
    /**
     * Create a global attribute in a netCDF file
     * @param ncfileWritable - netCDF file object
     * @return  0 if successful, otherwise -1.
     */
    public int createGlobalAttribute(NetcdfFileWriteable ncfileWritable) {
        if (value instanceof String) {
            String str = (String) (value);
            ncfileWritable.addGlobalAttribute(attrname, str);
        } else if (value instanceof Float) {
            ncfileWritable.addGlobalAttribute(attrname, (Float) (value));
        } else if (value instanceof Integer) {
            ncfileWritable.addGlobalAttribute(attrname, (Integer) (value));
        } else if (value instanceof Short) {
            ncfileWritable.addGlobalAttribute(attrname, (Short) (value));
        } else if (value instanceof Byte) {
            ncfileWritable.addGlobalAttribute(attrname, (Byte) (value));
        } else if (value instanceof Double) {
            ncfileWritable.addGlobalAttribute(attrname, (Double) (value));
        } else {
            return -1;
        }
        return 0;
    }

    /**
     * Update the value of global attribute in a netCDF file
     * @param ncfileWritable - netCDF file object
     * @return 0 if successful, otherwise -1.
     */
    public int updateGlobalAttribute(NetcdfFileWriteable ncfileWritable) {
        Attribute attr = null;
        if (attrname==VariableAttributesCF.VA_NAME_CF_FLAG_VALUES){
            attr=null;
        }
        if (value instanceof String) {
            attr = new Attribute(attrname, (String) value);
        } else if (value instanceof Float) {
            attr = new Attribute(attrname, (Float) (value));
        } else if (value instanceof Integer) {
            attr = new Attribute(attrname, (Integer) (value));
        } else if (value instanceof Short) {
            attr = new Attribute(attrname, (Short) (value));
        } else if (value instanceof Byte) {
            attr = new Attribute(attrname, (Byte) (value));
        } else if (value instanceof Double) {
            attr = new Attribute(attrname, (Double) (value));
        } else {
            return -1;
        }
        try {
            ncfileWritable.updateAttribute(null, attr);
        } catch (java.io.IOException e) {
            return -1;
        }
        return 0;
    }

    /**
     * Create a variable attribute in a netCDF file
     * @param ncfileWritable - netCDF file object
     * @param varName - variable name
     * @return 0 if successful, otherwise -1.
     */
    public int createVariableAttribute(NetcdfFileWriteable ncfileWritable, String varName) {
        if (value instanceof String) {
            String str = (String) (value);
            ncfileWritable.addVariableAttribute(varName, attrname, str);
        } else if (value instanceof Float) {
            ncfileWritable.addVariableAttribute(varName, attrname, (Float) (value));
        } else if (value instanceof Integer) {
            ncfileWritable.addVariableAttribute(varName, attrname, (Integer) (value));
        } else if (value instanceof Short) {
            ncfileWritable.addVariableAttribute(varName, attrname, (Short) (value));
        } else if (value instanceof Byte) {
            ncfileWritable.addVariableAttribute(varName, attrname, (Byte) (value));
        } else if (value instanceof Double) {
            ncfileWritable.addVariableAttribute(varName, attrname, (Double) (value));
        } else if (value instanceof byte[] ) {
            byte[] vs=(byte[])value;
            int[] dims={vs.length};
            ArrayByte arr= new ArrayByte(dims);
            for (int i=0;i<vs.length;i++){
                arr.setByte(i, vs[i]);
            }
             ncfileWritable.addVariableAttribute(varName, attrname, (Array) (arr));
        } else if (value instanceof short[]) {
            short[] vs=(short[])value;
            int[] dims={vs.length};
            ArrayShort arr= new ArrayShort(dims);
            for (int i=0;i<vs.length;i++){
                arr.setShort(i, vs[i]);
            }
             ncfileWritable.addVariableAttribute(varName, attrname, (Array) (arr));
        } else if (value instanceof float[]) {
            float[] vs=(float[])value;
            int[] dims={vs.length};
            ArrayFloat arr= new ArrayFloat(dims);
            for (int i=0;i<vs.length;i++){
                arr.setFloat(i, vs[i]);
            }
             ncfileWritable.addVariableAttribute(varName, attrname, (Array) (arr));
        } else {
            return -1;
        }
        return 0;
    }
}
