/*@***h* AMSU-A_FCDR_STAR/pub.attribute.AttributesPub.java
* NAME
*   pub.attribute.AttributesPub.java 
* 
* PURPOSE
*   To support a list of AttributeItem objects.
* 
* DESCRIPTION
*   This class supports multiple global or variable attribute items. 
*   It provides control flags that indicate whether an attribute item is required or optional.
*   It also provides methods for initializing, creating, updating attribute objects.
*   This class has a series of subclasses that support global and variable 
*   metadata, including GlobalAttributesCDR,
*   GlobalAttributesCF, GlobalAttributesGDS, GlobalAttributesUnidataACDD,
*   GlobalAttributesWWANG,GlobalAttributesWWANGL1C, VariableAttributesCF, 
*   VariableAttributesWWANG.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/01/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
* 
*
*@*****/
 
 
/*@***h* AMSU-A_FCDR_STAR/pub.attribute.AttributesPub.java
 * NAME
 * pub.attribute.AttributesPub.java
 * 
 * PURPOSE
 * List of AttributeItem
 * 
 * @DESCRIPTION
 * This class supports multiple global or variable attribute items. 
 * It provides control flags that indicate whether an attribute item is required or optional.
 * It also provides methods for initializing, creating, updating attribute objects.
 * This class has a series of subclasses that support global and variable 
 * metadata, including GlobalAttributesCDR,
 * GlobalAttributesCF, GlobalAttributesGDS, GlobalAttributesUnidataACDD,
 * GlobalAttributesWWANG,GlobalAttributesWWANGL1C, VariableAttributesCF, 
 * VariableAttributesWWANG.
 * 
 * AUTHOR
 * Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
 * 
 * CREATION DATE
 * 05/01/2013
 * 
 * COPYRIGHT
 * THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN
 * AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE FURNISHED "AS IS." 
 * THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES,OFFICERS, 
 * EMPLOYEES, AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS 
 * OF THE SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY 
 * (1) FOR THE USE OF THE SOFTWARE AND DOCUMENTATION; OR 
 * (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
 * 
 * HISTORY
 * 05/10/2013 - added modifyRequiredFlag method 
 * 05/04/2013 - added methods for supporting variable attribute
 *@*****/
package pub.attribute;

import java.util.ArrayList;
import ucar.nc2.NetcdfFileWriteable;

public class AttributesPub {
   static protected final String prefixSTAR="STAR_";

    ArrayList globalAttributes=new ArrayList(); //list of attribute items
    String[] names=null;                        //attribute names
    Object[] values=null;                       //attribute values
    boolean[] requireds=null;                   //whether the attributes are required or optional
    
    /**
     * Initialize a group of attribute objects 
     * @param names - names of attributes
     * @param values - values of attributes
     * @param requireds - flags that indicate whether each attribute in the group 
     *                    is required to be written to CDF file
     */
    public void initialize(String[] names,Object[] values,boolean[] requireds){
        this.names=names;
        this.values=values;
        this.requireds=requireds;
        
        AttributeItem attr;
        for (int i=0;i<names.length;i++){
            attr=new AttributeItem(names[i],values[i]);
            globalAttributes.add(attr);
        }
    }

    /**
     * Set the value of a global attribute
     * @param globalattrName - name of the global attribute
     * @param value - value of the global attribute
     * @return 0 if the value is set successfully, otherwise -1.
     */    
    public int setValue(String globalattrName,Object value){
        int i;
        AttributeItem attr;
        
        for (i=0;i<globalAttributes.size();i++){
            attr=(AttributeItem)globalAttributes.get(i);
            if (globalattrName.compareTo(attr.attrname)==0){
                attr.setValue(value);
                return 0;
            }
        }
        return -1;
    }

     /**
     * Set the value of a global attribute
     * @param globalattrName - name of the global attribute
     * @return attribute value if successful, otherwise null.
     */    
   public Object getValue(String globalattrName){
        int i;
        AttributeItem attr;
        
        for (i=0;i<globalAttributes.size();i++){
            attr=(AttributeItem)globalAttributes.get(i);
            if (globalattrName.compareTo(attr.attrname)==0){
                return attr.getValue();
            }
        }
        return null;
    }

    /**
     * Modify flag which indicates whether a global attribute will be written to the CDF file
     * @param globalattrName - name of the global attribute
     * @param flag - if true, the global attribute will be written to the CDF file, otherwise is not written.
     * @return 0 if the flag is set successfully, otherwise -1.
     */
    public int modifyRequiredFlag(String attrName,boolean value){
        int i;
        AttributeItem attr;
        
        for (i=0;i<globalAttributes.size();i++){
            attr=(AttributeItem)globalAttributes.get(i);
            if (attrName.compareTo(attr.attrname)==0){
                requireds[i]=value; 
                return 0;
            }
        }
        return -1;
    }

    /**
     * Create a group of global attributes in netCDF file
     * @param ncfileWritable - netCDF file object
     */
    public void createGlobalAttriabues(NetcdfFileWriteable ncfileWritable) {
        AttributeItem attr;
        for (int i = 0; i < names.length; i++) {
            if (requireds[i]) {
                attr = (AttributeItem) globalAttributes.get(i);
                attr.createGlobalAttribute(ncfileWritable);
            }
        }
    }
    
    /**
     * Create a group of variable attributes for a variable in netCDF file
     * @param ncfileWritable - netCDF file object
     * @param varName - variable name object
     */
    public void createVariableAttriabues(NetcdfFileWriteable ncfileWritable,String varName) {
        AttributeItem attr;
        for (int i = 0; i < names.length; i++) {
            if (requireds[i]) {
                attr = (AttributeItem) globalAttributes.get(i);
                attr.createVariableAttribute(ncfileWritable,varName);
            }
        }
    }
    
    /**
     * Update a global attribute in netCDF file
     * @param attrName - name of global attribute
     * @param ncfileWritable - netCDF file object
     * @return 0 if the update is successful, otherwise return -1.
     */
    public int update(String attrName, NetcdfFileWriteable ncfileWritable) {
        AttributeItem attr;
        for (int i = 0; i < names.length; i++) {
            if (attrName.compareTo(names[i]) == 0) {
                if (requireds[i]) {
                    attr = (AttributeItem) globalAttributes.get(i);
                    attr.updateGlobalAttribute(ncfileWritable);
                    return 0;
                }
            }
        }
        return -1;
    }
}
