/*@***h* AMSU-A_FCDR_STAR/pub.attribute.CDRGlobalAttributes.java
* NAME
*   pub.attribute.CDRGlobalAttributes.java 
* 
* PURPOSE
*   To support CDR global attributes metadata conventions defined by different 
*   organizations.
* 
* DESCRIPTION
*   This class currently support the CF Metadata Convention, Unidata ACDD, NCDC/
*   CDR, GDS, and self defined metadata for global attributes.
*   It also provides methods for setting and updating different global attributes. 
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/01/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/08/2003 - moved reading time coverage info from L1CFileNetCDF to this class.
*   05/04/2013 - added methods for setting and updating different global attributes.
*
*@*****/
 
 
package pub.attribute;

import java.util.ArrayList;
import ucar.nc2.NetcdfFile;
import ucar.nc2.NetcdfFileWriteable;
import util.Time;

public class CDRGlobalAttributes {
    static public String productVersionFormat1="Version 2.0";
    static public String productVersionFormat2="v01r00";
    
    public GlobalAttributesCF gaCF=new GlobalAttributesCF();
    public GlobalAttributesUnidataACDD gaUnidataACDD=new GlobalAttributesUnidataACDD();
    public GlobalAttributesCDR gaCDR=new GlobalAttributesCDR();
    public GlobalAttributesGDS gaGDS=new GlobalAttributesGDS();
    public GlobalAttributesWWANG gaWWANG=new GlobalAttributesWWANG();
    
    protected ArrayList gaAll=new ArrayList();
    
    public CDRGlobalAttributes(){
        gaAll.add(gaCF);
        gaAll.add(gaUnidataACDD);        
        gaAll.add(gaCDR);
        gaAll.add(gaGDS);
        gaAll.add(gaWWANG);
    }
    
        /**
     * Set time coverage attributes
     * @param timeStart - start time
     * @param timeEnd - end time
     */
    public void setTimeConverageAttributes(Time timeStart,Time timeEnd){
        gaWWANG.setTimeConverageAttributes(timeStart, timeEnd);
        gaUnidataACDD.setTimeConverageAttributes(timeStart, timeEnd);
    }
    
    /**
     * Update time coverage attributes
     * @param ncfileWritable - netCDF file to write
     * @param timeStart - start time
     * @param timeEnd - end time
     */
    public void updateTimeConverageAttributes(NetcdfFileWriteable ncfileWritable,Time timeStart,Time timeEnd){
        gaWWANG.updateTimeConverageAttributes(ncfileWritable, timeStart, timeEnd);
        gaUnidataACDD.updateTimeConverageAttributes(ncfileWritable, timeStart, timeEnd);
    }
    
    /**
     * Update geospatial attributes
     * @param ncfileWritable - netCDF file to write
     * @param lonMin - minimum longitude
     * @param lonMax - maximum longitude
     * @param latMin - minimum latitude
     * @param latMax - maximum latitude
     */
    public void updateGeospatialAttributes(NetcdfFileWriteable ncfileWritable,
            float lonMin,float lonMax,float latMin,float latMax){
        gaUnidataACDD.updateGeospatialAttributes(ncfileWritable, lonMin,lonMax,latMin,latMax);
    }
    
 
    /**
     * Create global attributes in a netCDF file.
     * @param ncfileWritable - target netCDF file
     */
    public void create(NetcdfFileWriteable ncfileWritable){
        AttributesPub ga;
        for (int i=0;i<gaAll.size();i++){
            ga=(AttributesPub)gaAll.get(i);
            ga.createGlobalAttriabues(ncfileWritable);
        }
    }

    /**
     * Update a global attribute in a netCDF file by attribute name
     * @param attrName - attribute name
     * @param ncfileWritable - netCDF file
     */
    public void update(String attrName,NetcdfFileWriteable ncfileWritable){
        AttributesPub ga;
        for (int i=0;i<gaAll.size();i++){
            ga=(AttributesPub)gaAll.get(i);
            
            if (ga.update(attrName,ncfileWritable)==0){
                break;
            }
        }
    }

    /**
     * Set a global attribute
     * @param name - attribute name
     * @param value - attribute value
     * @return 0 if successful, otherwise return 1.
     */
    public int setValue(String name,Object value){
        AttributesPub ga;
        int status=-1;
        for (int i=0;i<gaAll.size();i++){
            ga=(AttributesPub)gaAll.get(i);
            status=ga.setValue(name, value);
            if (status==0){
                return status;
            }
        }
        return status;
    }

    /**
     * Get the value of a attribute
     * @param name - attribute name
     * @return attribute value if successful, otherwise return null.
     */
    public Object getValue(String name){
        AttributesPub ga;
        Object value;
        for (int i=0;i<gaAll.size();i++){
            ga=(AttributesPub)gaAll.get(i);
            value=ga.getValue(name);
            if (value!=null){
                return value;
            }
        }
        return null;
        
    }
    /**
     * Set whether global attribute will be written in the CDR file
     * @param name - attribute name
     * @param flag - flag that indicates whether the global attribute will be written in the CDR file
     * @return 0 if successful, otherwise return 1 (global attribute does not exists).
     */
    public int modifyRequiredFlag(String name,boolean flag){
        AttributesPub ga;
        int status=-1;
        for (int i=0;i<gaAll.size();i++){
            ga=(AttributesPub)gaAll.get(i);
            status=ga.modifyRequiredFlag(name, flag);
            if (status==0){
                return status;
            }
        }
        return status;
    }

    /**
     * Read start time and end time from a netCDF file's global attributes
     * The method first tries to get time coverage from old STAR self-defined global attributes
       if not exist, then tries to read it from gaUnidataACDD global attributes
     * @param ncfile - netCDF file
     * @param timeStart - use for storing start time
     * @param timeEnd - use for storing end time
     * @return 0 if successful, otherwise return -1;
     */
    public int readStartTimeEndTime(NetcdfFile ncfile,Time timeStart,Time timeEnd){
        int status=gaWWANG.readStartTimeEndTime(ncfile, timeStart, timeEnd);

        if (status!=0){
            status=gaUnidataACDD.readStartTimeEndTime(ncfile, timeStart, timeEnd);
        }        
        
        return status;
    }
}
