/*@***h* AMSU-A_FCDR_STAR/pub.attribute.CDRVariableAttributes.java
* NAME
*   pub.attribute.CDRVariableAttributes.java 
* 
* PURPOSE
*   To support CDR variable attributes metadata conventions defined by different 
*   organizations.
* 
* DESCRIPTION
*   This class currently support the CF Metadata Conventionand self defined 
*   metadata for variable attributes.
*   It also provides methods for setting and updating different global attributes.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/04/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
* 
*
*@*****/
 
 
package pub.attribute;

import java.util.ArrayList;
import ucar.nc2.NetcdfFileWriteable;


public class CDRVariableAttributes {
    protected VariableAttributesCF vaCF=new VariableAttributesCF();
    protected VariableAttributesWWANG vaWWANG=new VariableAttributesWWANG();
    
    protected ArrayList vaAll=new ArrayList();
    
    public CDRVariableAttributes(){
        vaAll.add(vaCF);
        vaAll.add(vaWWANG);
    }
    
  
    /**
     * Create a variable attribute in a netCDF file
     * @param ncfileWritable - netCDF file
     * @param varName - attribute name
     */
    public void create(NetcdfFileWriteable ncfileWritable,String varName){
        AttributesPub ga;
        for (int i=0;i<vaAll.size();i++){
            ga=(AttributesPub)vaAll.get(i);
            ga.createVariableAttriabues(ncfileWritable,varName);
        }
    }

    /**
     * Set the value of a variable attribute
     * @param name - name of attribute
     * @param value - value of attribute
     * @return 0 if successful, otherwise return -1;
     */
     public int setValue(String name,Object value){
        AttributesPub ga;
        int status=-1;
        for (int i=0;i<vaAll.size();i++){
            ga=(AttributesPub)vaAll.get(i);
            status=ga.setValue(name, value);
            if (status==0)
                return status;
        }
        return -1;
    }
    
   /**
     * Set the value of a variable attribute
     * @param name - name of attribute
     * @param value - value of attribute
     * @return 0 if successful, otherwise return -1;
     */
     public int modifyRequiredFlag(String name,boolean flag){
        AttributesPub ga;
        int status=-1;
        for (int i=0;i<vaAll.size();i++){
            ga=(AttributesPub)vaAll.get(i);
            status=ga.modifyRequiredFlag(name, flag);
            if (status==0)
                return status;
        }
        return -1;
    }
    
     /**
      * Set long name attribute
      * @param longName - value of the long name attribute
      */
     public void setLongName(String longName){
         setValue(VariableAttributesCF.VA_NAME_CF_LONG_NAME,longName);
     }


     /**
      * Set standard name attribute
      * @param name - standard time
      */
     public void setStandardName(String name){
         setValue(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME,name);
     }
     
     /**
      * Set units attribute
      * @param units - unit
      */
     public void setUnits(String units){
         setValue(VariableAttributesCF.VA_NAME_CF_UNITS,units);         
     }

     public void setScaleFactor(float scaleFactor){
         setValue(VariableAttributesWWANG.VA_NAME_WWANG_SCALE_FACTOR,scaleFactor);
     }
     /**
      * Set fill value attribute
      * @param value - fill value
      */
     public void setFillValue(Object value){
         setValue(VariableAttributesCF.VA_NAME_CF_FILLVALUE,value);
     }
     
     /**
      * Set missing value attribute
      * @param value -missing value
      */
     public void setMissingValue(Object value){
         setValue(VariableAttributesCF.VA_NAME_CF_MISSINGVALUE,value);
     }
     
     /**
      * Set the valid maximum value attribute
      * @param value -maximum value
      */
     public void setValidMax(Object value){
         setValue(VariableAttributesCF.VA_NAME_CF_VALID_MAX,value);                  
     }

     
     /**
      * Set the valid minimum value attribute
      * @param value - minimum value
      */
     public void setValidMin(Object value){
         setValue(VariableAttributesCF.VA_NAME_CF_VALID_MIN,value);         
     }
     
     /**
      * Set the valid range attribute
      * @param value - valid range
      */
     public void setValidRange(Object value){
         setValue(VariableAttributesCF.VA_NAME_CF_VALID_RANGE,value);                  
     }
     /**
      * Set Note attribute
      * @param note - value of Note attribute
      */
     public void setNote(Object note){
         setValue(VariableAttributesWWANG.VA_NAME_WWANG_NOTE,note);
     }
     
     
}
