/*@***h* AMSU-A_FCDR_STAR/pub.attribute.GlobalAttributesUnidataACDD.java
* NAME
*  pub.attribute.GlobalAttributesUnidataACDD.java 
* 
* PURPOSE
*   To support the Unidata Attribute Convention for Dataset Recovery (ACDD).
* 
* DESCRIPTION
*   This class is of subclass of AttributesPub. It defined constants and variables
*   that related to the Unidata ACDD Metadata Convention.
*   It also provides methods for setting or updating the geospatial and time 
*   coverage attributes.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/01/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/04/2013 - add methods for setting or updating the geospatial and time 
*                coverage attributes.
*
*@*****/
 
 
package pub.attribute;

import ucar.nc2.Attribute;
import ucar.nc2.NetcdfFile;
import ucar.nc2.NetcdfFileWriteable;
import util.Time;

/**
 *
 * @author wwang
 */
public class GlobalAttributesUnidataACDD extends AttributesPub {

    public static final String GA_NAME_ACDD_METADATA_CONVENTIONS = "Metadata_Conventions";
    public static final String GA_NAME_ACDD_STANDARD_NAME_VOCABULARY = "standard_name_vocabulary";
    public static final String GA_NAME_ACDD_ID = "id";
    public static final String GA_NAME_ACDD_NAMING_AUTHORITY = "naming_authority";
    public static final String GA_NAME_ACDD_DATE_CREATED = "date_created";
    public static final String GA_NAME_ACDD_DATE_MODIFIED = "date_modified";
    public static final String GA_NAME_ACDD_LICENSE = "license";
    public static final String GA_NAME_ACDD_DATE_ISSUED = "date_issued";
    public static final String GA_NAME_ACDD_SUMMARY = "summary";
    public static final String GA_NAME_ACDD_KEYWORDS = "keywords";
    public static final String GA_NAME_ACDD_KEYWARDS_VOCABULARY = "keywords_vocabulary";
    public static final String GA_NAME_ACDD_CMD_DATA_TYPE = "cdm_data_type";
    public static final String GA_NAME_ACDD_PROJECT = "proejct";
    public static final String GA_NAME_ACDD_PROCESSING_LEVEL = "processing_level";
    public static final String GA_NAME_ACDD_CREATOR_NAME = "creator_name";
    public static final String GA_NAME_ACDD_CREATOR_URL = "creator_url";
    public static final String GA_NAME_ACDD_CREATOR_EMAIL = "creator_email";
    public static final String GA_NAME_ACDD_INSTITUTION = "institution";
    public static final String GA_NAME_ACDD_GEOSPATIAL_LAT_MIN = "geospatial_lat_min";
    public static final String GA_NAME_ACDD_GEOSPATIAL_LAT_MAX = "geospatial_lat_max";
    public static final String GA_NAME_ACDD_GEOSPATIAL_LON_MIN = "geospatial_lon_min";
    public static final String GA_NAME_ACDD_GEOSPATIAL_LON_MAX = "geospatial_lon_max";
    public static final String GA_NAME_ACDD_GEOSPATIAL_LAT_UNITS = "geospatial_lat_units";
    public static final String GA_NAME_ACDD_GEOSPATIAL_LAT_RESOLUTION = "geospatial_lat_resolution";
    public static final String GA_NAME_ACDD_GEOSPATIAL_LON_UNITS = "geospatial_lon_units";
    public static final String GA_NAME_ACDD_GEOSPATIAL_LON_RESOLUTION = "geospatial_lon_resolution";
    public static final String GA_NAME_ACDD_TIME_CONVERAGE_START = "time_coverage_start";
    public static final String GA_NAME_ACDD_TIME_CONVERAGE_END = "time_coverage_end";
    public static final String GA_NAME_ACDD_TIME_COVERAGE_DURATION = "time_coverage_duration";
    public static final String GA_NAME_ACDD_TIME_COVERAGE_RESOLUTION = "time_coverage_resolution";
    public static final String GA_NAME_ACDD_CONTRIBUTOR_NAME = "contributor_name";
    public static final String GA_NAME_ACDD_CONTRIBUTOR_ROLE = "contributor_role";
    public static final String GA_NAME_ACDD_ACKNOWLEDGMENT = "acknowledgment";
    String[] names = {GA_NAME_ACDD_METADATA_CONVENTIONS,
        GA_NAME_ACDD_STANDARD_NAME_VOCABULARY,
        GA_NAME_ACDD_ID,
        GA_NAME_ACDD_NAMING_AUTHORITY,
        GA_NAME_ACDD_DATE_CREATED,
        GA_NAME_ACDD_DATE_MODIFIED,
        GA_NAME_ACDD_LICENSE,
        GA_NAME_ACDD_DATE_ISSUED,
        GA_NAME_ACDD_SUMMARY,
        GA_NAME_ACDD_KEYWORDS,
        
        GA_NAME_ACDD_KEYWARDS_VOCABULARY,
        GA_NAME_ACDD_CMD_DATA_TYPE,
        GA_NAME_ACDD_PROJECT,
        GA_NAME_ACDD_PROCESSING_LEVEL,
        GA_NAME_ACDD_CREATOR_NAME,
        GA_NAME_ACDD_CREATOR_URL,
        GA_NAME_ACDD_CREATOR_EMAIL,
        GA_NAME_ACDD_INSTITUTION,
        GA_NAME_ACDD_GEOSPATIAL_LAT_MIN,
        GA_NAME_ACDD_GEOSPATIAL_LAT_MAX,
        
        GA_NAME_ACDD_GEOSPATIAL_LON_MIN,
        GA_NAME_ACDD_GEOSPATIAL_LON_MAX,
        GA_NAME_ACDD_GEOSPATIAL_LAT_UNITS,
        GA_NAME_ACDD_GEOSPATIAL_LAT_RESOLUTION,
        GA_NAME_ACDD_GEOSPATIAL_LON_UNITS,
        GA_NAME_ACDD_GEOSPATIAL_LON_RESOLUTION,
        GA_NAME_ACDD_TIME_CONVERAGE_START,
        GA_NAME_ACDD_TIME_CONVERAGE_END,
        GA_NAME_ACDD_TIME_COVERAGE_DURATION,        
        GA_NAME_ACDD_TIME_COVERAGE_RESOLUTION,
        
        GA_NAME_ACDD_CONTRIBUTOR_NAME,
        GA_NAME_ACDD_CONTRIBUTOR_ROLE,
        GA_NAME_ACDD_ACKNOWLEDGMENT
    };
    Object[] values = {"CF-1.6,Unidata Dataset Discovery v1.0, NOAA CDR v1.0, GDS v2.0",
        "CF Standard Name Table (v23, 23 March 2013)",
        "",
        "gov.noaa.ncdc",
        "", "",
        "No constraints on data access or use.",
        "",
        "NOAA/NESDIS/STAR Version 1.0 Advanced Microwave Sounding Unit A (AMSU-A) level 1c radiance Fundamental Climate Data Record (FCDR);"+
            "inter-calibrated channel brightness temperatures using the STAR Integrated Microwave Inter-Calibration Approach (IMICA, formerly known as simultaneous nadir overpass method, Zou and Wang 2011) and with quality flags, from 1998 - present; "+
            "NOAA operational calibrated brightness temperatures, as well as limb effect corrected IMICA and operational calibrated brightness temperatures using Goldberg (2001) method, are also provided.",
        "SPECTRAL/ENGINEERING>MICROWAVE>BRIGHTNESS TEMPERATURE", //keyword
//        "SPECTRAL/ENGINEERING>MICROWAVE>ANTENNA TEMPERATURE, SPECTRAL/ENGINEERING>MICROWAVE>BRIGHTNESS TEMPERATURE", //keyword
   
        "NASA Global Change Master Directory (GCMD) Earth Science Keywords, Version 8.0", //key word vocabulary
        "Swath", //"cdm_data_type"
        "NOAA/NESDIS/STAR MSU/AMSU/SSU climate data record", //prject
        "NOAA Level 2",
        "Cheng-Zhi Zou",
        "",
        "Cheng-Zhi.Zou@noaa.gov",
        "NOAA/NESDIS/STAR>Center for Satellite Application and Research, NESDIS, NOAA, U.S. Department of Commerce", //institution
        -9999.0f, -9999.0f, 
        
        -9999.0f, -9999.0f,
        "degrees_north",
        -9999.0f,
        "degrees_east",
        -9999.0f,
        "", //time start
        "", //time end
        "",
        "",
        
        "Cheng-Zhi Zou; Wenhui Wang",//contributor name
        "principalinvestigator; supportscientist", //contriubtor role
        "This project was supported by a grant from the NOAA Climate Data Record (CDR) Program for satellite" //achnowledgement
    };
    boolean[] requireds = {true, true, true, true, true,
        false, true, false, true, true,
        true, true, false, false, true,
        false, true, true, true, true,
        true, true, false, false, false,
        false, true, true, false, false,
        true, true, false};

    public GlobalAttributesUnidataACDD() {
        initialize(names, values, requireds);
    }

    /**
     * Update geospatial attribute of a netCDF file
     * @param ncfileWritable - netCDF file
     * @param lonMin - minimum longitude 
     * @param lonMax - maximum longitude 
     * @param latMin - minimum altitude
     * @param latMax - maximum altitude
     */
    public void updateGeospatialAttributes(NetcdfFileWriteable ncfileWritable,
            float lonMin, float lonMax, float latMin, float latMax) {
        int status=-1;
        status=setValue(GA_NAME_ACDD_GEOSPATIAL_LON_MIN, lonMin);
        status=update(GA_NAME_ACDD_GEOSPATIAL_LON_MIN, ncfileWritable);

        status=setValue(GA_NAME_ACDD_GEOSPATIAL_LON_MAX, lonMax);
        status=update(GA_NAME_ACDD_GEOSPATIAL_LON_MAX, ncfileWritable);

        status=setValue(GA_NAME_ACDD_GEOSPATIAL_LAT_MIN, latMin);
        status=update(GA_NAME_ACDD_GEOSPATIAL_LAT_MIN, ncfileWritable);

        status=setValue(GA_NAME_ACDD_GEOSPATIAL_LAT_MAX, latMax);
        status=update(GA_NAME_ACDD_GEOSPATIAL_LAT_MAX, ncfileWritable);
    }

    /**
     * Set time coverage attribute (not update the output file)
     * @param timeStart - start time
     * @param timeEnd - end time
     * @return 0 if successful, otherwise return 1
     */
    public int setTimeConverageAttributes(Time timeStart, Time timeEnd) {
        int status=-1;
        String strTimeStart = timeStart.toStringISO8601();
        String strTimeEnd = timeEnd.toStringISO8601();
        status=setValue(GA_NAME_ACDD_TIME_CONVERAGE_START, strTimeStart);
        if (status!=0)
            return -1;
        status=setValue(GA_NAME_ACDD_TIME_CONVERAGE_END, strTimeEnd);
        return status;
    }

    /**
     * Update time coverage attribute
     * @param ncfileWritable  - netCDF file writable
     * @param timeStart - start time
     * @param timeEnd - end time
     * @return 0 if successful, otherwise return 1
     */
    public int updateTimeConverageAttributes(NetcdfFileWriteable ncfileWritable, Time timeStart, Time timeEnd) {
        int status=-1;
        status=setTimeConverageAttributes(timeStart,timeEnd);
        if (status!=0)
            return status;
 
        status=update(GA_NAME_ACDD_TIME_CONVERAGE_START, ncfileWritable);
       if (status!=0)
            return status;
        status=update(GA_NAME_ACDD_TIME_CONVERAGE_END, ncfileWritable);
        return status;
    }
    
    public int readStartTimeEndTime(NetcdfFile ncfile,Time timeStart,Time timeEnd){
        if (readTimeAttributeKernel(ncfile,GA_NAME_ACDD_TIME_CONVERAGE_START,timeStart)==null){
            return -1;
        }
        if (readTimeAttributeKernel(ncfile,GA_NAME_ACDD_TIME_CONVERAGE_END,timeEnd)==null){
             return -1;
        }
        return 0;
    }
    
    public Time readTimeAttributeKernel(NetcdfFile ncfile, String attrName,Time time) {
        try {
            Attribute attr = ncfile.findGlobalAttributeIgnoreCase(attrName);
            String strTime=attr.getStringValue();
            if (time == null) {
                time = new Time();
            }
            time.setUsingISO8601String(strTime);
        } catch (java.lang.NullPointerException e) {
            return null;
        }
        return time;
    }
        
};
