/*@***h* AMSU-A_FCDR_STAR/pub.attribute.GlobalAttributesWWANG.java
* NAME
*  pub.attribute.GlobalAttributesWWANG.java 
* 
* PURPOSE
*   To support the self-defined global attribute of the STAR CDR products.
* 
* DESCRIPTION
*   This class is of subclass of AttributesPub. It defined constants and variables
*   that related to the STAR self defined global Metadata.
*   It also provides methods for setting or updating the time coverage attributes.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/01/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/14/2013 - add uncertainty global attributes
*   05/04/2013 - add methods for setting or updating the time coverage attributes. 
*
*@*****/
 
 
package pub.attribute;

import pub.QualityControl;
import pub.variable.VariableControl;
import ucar.nc2.Attribute;
import ucar.nc2.NetcdfFile;
import ucar.nc2.NetcdfFileWriteable;
import util.StringUtil;
import util.Time;

public class GlobalAttributesWWANG extends AttributesPub{
    public static final String GA_NAME_WWANG_SENSORNAME = prefixSTAR+"SensorName";
    public static final String GA_NAME_WWANG_SATELLITENAME = prefixSTAR+"SatelliteName";
    public static final String GA_NAME_WWANG_SATELLITEID = prefixSTAR+"SatelliteID";
    public static final String GA_NAME_WWANG_SATELLITEIDINTERNAL = prefixSTAR+"SatelliteIDInternal";
    public static  String GA_NAME_WWANG_STARTYEAR = prefixSTAR+"StartYear";
    public static  String GA_NAME_WWANG_STARTDAYOFYEAR = prefixSTAR+"StartDayOfYear";
    public static  String GA_NAME_WWANG_STARTMILLISECONDOFDAY = prefixSTAR+"StartMillisecondOfDay";
    public static  String GA_NAME_WWANG_ENDYEAR = prefixSTAR+"EndYear";
    public static  String GA_NAME_WWANG_ENDDAYOFYEAR = prefixSTAR+"EndDayOfYear";
    public static  String GA_NAME_WWANG_ENDMILLISECONDOFDAY = prefixSTAR+"EndMillisecondOfDay";
    public static  String GA_NAME_WWANG_Uncertainty_IMICA = "uncertainty_estimate_IMICA"; //prefixSTAR+"Uncertainty_IMICA";
    public static  String GA_NAME_WWANG_Uncertainty_OPR = prefixSTAR+"uncertainty_estimate_OPR";
    public static  String GA_NAME_WWANG_Uncertainty_LINEAR = prefixSTAR+"uncertainty_estimate_LINEAR";

    String[] names={
                    GA_NAME_WWANG_SENSORNAME,
                    GA_NAME_WWANG_SATELLITENAME,
                    GA_NAME_WWANG_SATELLITEID,
                    GA_NAME_WWANG_SATELLITEIDINTERNAL,
                    GA_NAME_WWANG_STARTYEAR,
                    GA_NAME_WWANG_STARTDAYOFYEAR,
                    GA_NAME_WWANG_STARTMILLISECONDOFDAY,
                    GA_NAME_WWANG_ENDYEAR,
                    GA_NAME_WWANG_ENDDAYOFYEAR,
                    GA_NAME_WWANG_ENDMILLISECONDOFDAY,
                    GA_NAME_WWANG_Uncertainty_IMICA,
                    GA_NAME_WWANG_Uncertainty_OPR,
                    GA_NAME_WWANG_Uncertainty_LINEAR,
    };
    
    Object[] values={"AMSUA","",(int)0,(int)0,
                     (short)-9999,(short)-9999,(int)-9999,
                     (short)-9999,(short)-9999,(int)-9999,
                     StringUtil.makeDummyString(' ',150),
                     StringUtil.makeDummyString(' ',200),
                     StringUtil.makeDummyString(' ',200),
    };
    
    boolean[] requireds={false,true,true,true,
                        true,true,true,
                        true,true,true,
                        true,false,false,
                        };
    
    /**
     * Constructor
     */
    public GlobalAttributesWWANG(){
        initialize(names,values,requireds);
        if (!VariableControl.isWrittenTimeNOAAL1BFORMAT()){
            for (int i=4;i<10;i++){
                requireds[i]=false;
            }
        }
    }

    /**
     * Set time coverage attribute (not update the output file) in STAR format
     * @param timeStart - start time
     * @param timeEnd - end time 
     */
    public void setTimeConverageAttributes(Time timeStart,Time timeEnd){
        Time time = timeStart;
        setValue(GlobalAttributesWWANG.GA_NAME_WWANG_STARTYEAR, time.getYear());
        setValue(GlobalAttributesWWANG.GA_NAME_WWANG_STARTDAYOFYEAR, time.getDayOfYear());
        setValue(GlobalAttributesWWANG.GA_NAME_WWANG_STARTMILLISECONDOFDAY, time.getMillisecondOfDay());
        time = timeEnd;
        setValue(GlobalAttributesWWANG.GA_NAME_WWANG_ENDYEAR, time.getYear());
        setValue(GlobalAttributesWWANG.GA_NAME_WWANG_ENDDAYOFYEAR, time.getDayOfYear());
        setValue(GlobalAttributesWWANG.GA_NAME_WWANG_ENDMILLISECONDOFDAY, time.getMillisecondOfDay());
        
    }
    
    /**
     * Update time coverage attribute in netCDF file
     * @param ncfileWritable  - netCDF file writable
     * @param timeStart - start time
     * @param timeEnd - end time
     */    
    public void updateTimeConverageAttributes(NetcdfFileWriteable ncfileWritable,Time timeStart,Time timeEnd){
        setTimeConverageAttributes(timeStart,timeEnd);
        update(GlobalAttributesWWANG.GA_NAME_WWANG_STARTYEAR, ncfileWritable);
        update(GlobalAttributesWWANG.GA_NAME_WWANG_STARTDAYOFYEAR, ncfileWritable);
        update(GlobalAttributesWWANG.GA_NAME_WWANG_STARTMILLISECONDOFDAY, ncfileWritable);
        
        update(GlobalAttributesWWANG.GA_NAME_WWANG_ENDYEAR, ncfileWritable);
        update(GlobalAttributesWWANG.GA_NAME_WWANG_ENDDAYOFYEAR, ncfileWritable);
        update(GlobalAttributesWWANG.GA_NAME_WWANG_ENDMILLISECONDOFDAY, ncfileWritable);
    }

    /**
     * Read starting and ending time from global attributes
     * @param ncfile - netCDF file object
     * @param timeStart - for storing starting time
     * @param timeEnd - for storing ending time
     * @return starting and ending time
     */
    public int readStartTimeEndTime(NetcdfFile ncfile,Time timeStart,Time timeEnd){
        if (readStartTime(ncfile,timeStart)==null){
            return -1;
        }
        if (readEndTime(ncfile,timeEnd)==null){
            return -1;
        }
        return 0;
    }
    
    /**
     * Read starting time from global attributes
     * @param ncfile - netCDF file object
     * @param time - for storing starting time
     * @return starting time
     */
    public Time readStartTime(NetcdfFile ncfile, Time time) {
        try {
            short year, dayOfYear;
            int millisecondOfDay;
            Attribute attr = ncfile.findGlobalAttributeIgnoreCase(GlobalAttributesWWANG.GA_NAME_WWANG_STARTYEAR);
            year = attr.getNumericValue().shortValue();

            attr = ncfile.findGlobalAttributeIgnoreCase(GlobalAttributesWWANG.GA_NAME_WWANG_STARTDAYOFYEAR);
            dayOfYear = attr.getNumericValue().shortValue();

            attr = ncfile.findGlobalAttributeIgnoreCase(GlobalAttributesWWANG.GA_NAME_WWANG_STARTMILLISECONDOFDAY);
            millisecondOfDay = attr.getNumericValue().intValue();
            if (time == null) {
                time = new Time();
            }
            time.set(year, dayOfYear, millisecondOfDay);
        } catch (java.lang.NullPointerException e) {
            return null;
        }
        return time;
    }
    
    /**
     * Read ending time from global attributes
     * @param ncfile - netCDF file object
     * @param time - for storing ending time
     * @return ending time
     */
    public Time readEndTime(NetcdfFile ncfile, Time time) {
        try {
            short year, dayOfYear;
            int millisecondOfDay;
            Attribute attr = ncfile.findGlobalAttributeIgnoreCase(GlobalAttributesWWANG.GA_NAME_WWANG_ENDYEAR);
            year = attr.getNumericValue().shortValue();

            attr = ncfile.findGlobalAttributeIgnoreCase(GlobalAttributesWWANG.GA_NAME_WWANG_ENDDAYOFYEAR);
            dayOfYear = attr.getNumericValue().shortValue();

            attr = ncfile.findGlobalAttributeIgnoreCase(GlobalAttributesWWANG.GA_NAME_WWANG_ENDMILLISECONDOFDAY);
            millisecondOfDay = attr.getNumericValue().intValue();
            if (time == null) {
                time = new Time();
            }
            time.set(year, dayOfYear, millisecondOfDay);
        } catch (java.lang.NullPointerException e) {
            return null;
        }
        return time;
    }
        
    /**
     * Read number of scan lines from global attribute
     * @param ncfile - netCDF file object
     * @return number of scan lines from global attribute
     */
    public int readNumberOfScanlines(NetcdfFile ncfile){
        int nLines=QualityControl.missing_value_short;
        Attribute attr = ncfile.findGlobalAttributeIgnoreCase(GlobalAttributesWWANGL1C.GA_NAME_WWANG_L1C_NSCANLINES);
        if (attr != null) {
            nLines = attr.getNumericValue().shortValue();
        }
        return nLines;
    }
}
