/*@***h* AMSU-A_FCDR_STAR/pub.physics.PlanckFunction.java
* NAME
*   pub.physics.PlanckFunction.java 
* 
* PURPOSE
*   To support temperature and radiance conversions.
* 
* DESCRIPTION
*   This class provies genetic methods for temperature/radiance conversions,
*   by specifying channel central wavelength.
*   
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   12/20/2008
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
* 
*
*@*****/
 
 

package pub.physics;


public class PlanckFunction {
    public static final float plcoef1 = 1.1910659E-05f;  //1.1910659~10-5
    public static final float plcoef2 = 1.438833f;       //1.438833
  
    //haifeng's or Likun
    //static final float plcoef1 = 1.1910427E-05f;
    //static final float plcoef2 = 1.4387752f;
    //    c1=1.1909E-8, c2=1.4388

    static public float temperature2Radiance(double wavenumber,float temp){ //mW · m-2 · steradian-1/cm-1.            
          double c1v3, c2v;
          double t1,rad;
          
          c1v3=plcoef1*wavenumber*wavenumber*wavenumber;
          c2v=plcoef2*wavenumber;

          t1=Math.exp(c2v/temp)-1;
          rad=c1v3/t1;
          

          return (float)rad;
          
    }
    
    static public float radiance2Temperature(double wavenumber,float rad){
          double c1v3, c2v;
          double t1,temp;
          
          c1v3=plcoef1*wavenumber*wavenumber*wavenumber;
          c2v=plcoef2*wavenumber;
          
          t1=Math.log(c1v3/rad + 1);
          temp=c2v/t1;
          return (float)temp;
       
    }

///////////////////////////////////////////////////////////////////////////////////////////////////////
//ONE ORBIT
    static public void radiance2Temperature(float[][][] radiances, float[][][] temperatures,
            float[]centralWVs){ 
        int nScans = radiances.length;

        for (int idxLine = 0; idxLine < nScans; idxLine++) {
            radiance2Temperature(radiances[idxLine], temperatures[idxLine], centralWVs);
        }        
    }
    
    //ONE scan line level
   static public void radiance2Temperature(float[][] radiances, float[][] temperatures,
            float[]centralWVs){ 
       int i;
       double wv;
       int nChannels = radiances.length;
       int nSpots = radiances[0].length;

       for (i = 0; i < nChannels; i++) {
           wv = centralWVs[i];
           for (int spot = 0; spot < nSpots; spot++) {
               temperatures[i][spot] = radiance2Temperature(wv, radiances[i][spot]);
           }
       }
   }
   
   static public void main(String[] args){
       float t=radiance2Temperature(669.998f,52.81782f); //ssu channel 1
       System.out.println("t="+t);
   }
}
