/*@***h* AMSU-A_FCDR_STAR/pub.variable.CDRVariablesL1C.java
* NAME
*   pub.variable.CDRVariablesL1C.java 
* 
* PURPOSE
*   Define names and data buffers of CDR variables.
* 
* DESCRIPTION
*   Names given in this class are default CDR variable names, they can be
*   customized in subclasses.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/05/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/14/2013 - added public constant NUM_CALIBRATION_METHODS.
*
*@*****/
 
 
package pub.variable;

import ucar.ma2.Array;
import pub.CalibrationInterface;

public class CDRVariablesL1C implements CalibrationInterface{

    static public String VAR_TIME = "scan_time"; //name of time variable in ISO8601 format
    static public String VAR_TIME_SINCE1978 = "scan_time_since78"; //name of time variable in seconds since 1978-01-01
    static public String VAR_YEAR = "Year";                        //name of year variable
    static public String VAR_DAYOFYEAR = "DayOfYear";              //name of day of year variable
    static public String VAR_MILLISECONDOFDAY = "MillisecondOfDay"; //name of millisecond of day variable

    static public String VAR_LATITUDE = "Latitude";                 //name of latitude variable
    static public String VAR_LONGITUDE = "Longitude";               //name of longitude variable

    static public String VAR_SOLARZENITH = "SolarZenith";           //name of solar zenith angle variables
    static public String VAR_SENSORZENITH = "SensorZenith";         //name of sensor zenith angle variables
    static public String VAR_LOCALAZIMUTH = "LocalAzimuth";         //name of local azimuth angle variables
    
    //quality flag variables for different calibration method
    static public String[] VAR_CFLAGs = {"CFlag_T_SNO", "CFlag_T_OPR", "CFlag_T_LINEAR"};  //scan level QC
    static public String[] VAR_CFLAGPIXELs = {"CFlagPixel_T_SNO", "CFlagPixel_T_OPR", "CFlagPixel_T_LINEAR"}; //Pixel level QC
    
    //calibration coefficients variable, //scanline,channel,0-2 orders
    //will replace VAR_CALCOEF in the future
    static public String[] VAR_CALCOEF_AMSUA={"CalCoefIMICA", 
                                          "CalCoefOPR", 
                                          "CalCoefLinear"};
    static public String[] VAR_CALCOEF = {"CalCoefOPR", //scanline,channel,0-2 orders
                                          "CalCoefLinear"};
    
    
    //temperature variables, can be modified in subclasses 
    static public final int VARINDEX_T_SNO = 0;
    static public final int VARINDEX_T_SNO_LIMB = 1;
    static public final int VARINDEX_T_OPR = 2;
    static public final int VARINDEX_T_OPR_LIMB = 3;
    static public final int VARINDEX_T_LINEAR = 4;
    static public final int VARINDEX_T_LINEAR_LIMB = 5;
    
    //names of different scene temperature variables
    static public String VAR_T_IMICA="T_SNO";
    static public String VAR_T_IMICA_LIMB="T_SNO_LIMB";
    static public String VAR_T_OPR="T_OPR";
    static public String VAR_T_OPR_LIMB="T_OPR_LIMB";
    static public String VAR_T_LINEAR="T_LINEAR";
    static public String VAR_T_LINEAR_LIMB="T_LINEAR_LIMB";    
    static public String[] VAR_Ts = {VAR_T_IMICA, VAR_T_IMICA_LIMB,
        VAR_T_OPR, VAR_T_OPR_LIMB, VAR_T_LINEAR, VAR_T_LINEAR_LIMB,};
    static public String[] longnamesVAR_Ts={    
            "IMICA recalibrated scene temperatures",
            "IMICA recalibrated scene temperatures with limb correction",
            "NOAA operational calibrated scene temperatures",
            "NOAA operational calibrated scene temperatures with limb correction",
            "linear calibrated scene Temperatures",
            "linear calibrated scene temperatures with limb correction",
        };

    //name of warm target temperature variables
    static public String VAR_WARMTARGETTEMPERATURE = "WarmTargetTemperature";

    //name of spacecraft altitude variable
    static public String VAR_ALTITUDE = "SpacecraftAltitude";  //counterpart: "Height" in MSU level 1c
 
    //internal level 1c variables, not used in the operational version
    static public final String[] VAR_CRTM_CORRECTIONS = {"", "T_limbcorr", "T_n15ch6freqcorr", "T_diurcorr", "T_msuequcorr"};
    static public String VAR_LECT = "LECT";  //local equator crossing time
    static public String VAR_MSUEQUIVALENTAMSUA_CORR = "MSUEQUIVALENTAMSU_CORR"; //T_Simulated_Nadir_MSU";
    
    
    //definition of different ascending/descending flags
    static public final byte ASD_UNDEFINED = -1;
    static public final byte ASD_ASCENDING = 0;
    static public final byte ASD_DESCENDING = 1;
    static public final byte ASD_ALL = 4;
    //name of ascending/descending flag variable    
    static public String VAR_ASDFLAG = "AscendingDesendingFlag";
    
    //name of satellite attitude variable
    static public final String VAR_ATTITUDE = "SatelliteAttitude";
    
    //names of raw counts variables
    static public final String VAR_SCENECOUNTS = "SceneCounts";
    static public String VAR_COLDSPACECOUNTS = "ColdSpaceCounts";
    static public String VAR_WARMTARGETCOUNTS = "WarmTargetCounts";
    
    

    
    //data buffer s for different variables 
    public Array[] dataCFlags, dataCFlagPixel; //quality control flags for linear,opr, sno calibrated temperatures
    public Array dataLatitude, dataLongitude;
    public Array dataSolarZenith, dataSensorZenith, dataLocalAzimuth;
    public Array dataWarmTargetTemperature;
    public Array dataSceneCounts, dataColdSpaceCounts, dataWarmTargetCounts;
    
    public Array dataASDFlag;
    public Array dataAltitude;
    public Array dataAttitude;
    public Array dataLECT;
    public Array[] dataTsNew, dataRads /*radiance*/;
    public Array dataT_MSUEQUIVALENTAMSUA_CORR;
    public Array[] dataCalCoef;
    public Array[] dataCalCoefAMSUA;

    public VariablesGroupTime varsTime=new VariablesGroupTime(); //time variable group object, support multiple time variables

    /**
     * Update the name of T(scene temperature) variables, should be called after customizing T variables
     */
    static public void updateVAR_Ts(){
        VAR_Ts[0]=VAR_T_IMICA;
        VAR_Ts[1]=VAR_T_IMICA_LIMB;
        VAR_Ts[2]=VAR_T_OPR;
        VAR_Ts[3]=VAR_T_OPR_LIMB;
        VAR_Ts[4]=VAR_T_LINEAR;
        VAR_Ts[5]=VAR_T_LINEAR_LIMB;
    }
    
}
