/*@***h* AMSU-A_FCDR_STAR/pub.variable.VariableItem.java
* NAME
*   pub.variable.VariableItem.java 
* 
* PURPOSE
*   To define a variable of climate data record.
* 
* DESCRIPTION
*   This class defines variable of climate data records.
*   Currently, it is only used for supporting the time variables. 
*   It will be used for all variables in the future.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/05/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/08/2013 - added bWrite2File to control whether the variable will be written
*                to the output file. 
*
*@*****/
 
 
package pub.variable;

import java.io.IOException;
import java.util.ArrayList;
import pub.attribute.*;
import ucar.ma2.*;
import ucar.nc2.*;

public class VariableItem {
    
    String varName;         //name of variable
    DataType dataType;      //data type of the variable
    ArrayList dims;         //dimensions of the variable
    CDRVariableAttributes varAttrs;//Attribute of the variable
    Array varData;          //data buffer for storing variable data in the memory
    boolean bWrite2File;    //to control whether the variable will be written
                           //to the output file
    /**
     * Constructor
     * @param name - name of variable
     */
    public VariableItem(String name){
        set(name,null,null,null,null);
        bWrite2File=true;
    }
    
    /**
     * Constructor
     * @param name - name of variable
     * @param dataType0 - data type of variable
     * @param dims0 - dimensions of variable
     * @param attr - attribute of variable
     * @param data - data buffer of variable
     */
    public VariableItem(String name, DataType dataType0,ArrayList dims0,CDRVariableAttributes attr, Array data ){
        set(name,dataType0,dims0,attr,data);
        bWrite2File=true;
    }

    /**
     * Set if this variable is written to output file
     * @param flag - flag that controls whether the variable will be written to output file
     */
    public void setWrite2FileFlag(boolean flag){
        bWrite2File=flag;
    }
    /**
     * Set variable 
     * @param name - name of variable
     * @param dataType0 - data type of variable
     * @param dims0 - dimensions of variable
     * @param attr - attribute of variable
     * @param data - data buffer of variable
     */
    public void set(String name, DataType dataType0,ArrayList dims0,CDRVariableAttributes attr, Array data ){
        varName=name;
        dataType=dataType0;
        dims=dims0;
        varAttrs=attr;
        varData=data;
    }
    
    /**
     * Set data buffer of a variable
     * @param obj - data buffer
     */
    public void setData(Array obj){
        varData=obj;
    }

    /**
     * Get data buffer of a variable
     * @return data buffer
     */
    public Array getData(){
        return varData;
    }
    
    /**
     * Create a netCDF variable and its attributes
     * @param ncfileWritable
     * @return 0 if successful, otherwise return -1
     */
    public int create(NetcdfFileWriteable ncfileWritable) {
        if (bWrite2File) {
            if (ncfileWritable.addVariable(varName, dataType, dims) == null) {
                return -1;
            }

            varAttrs.create(ncfileWritable, varName);
        }
        return 0;
    }
 
    /**
     * Read data of a variable from netCDF file
     * @param ncfile - netCDF file
     * @param org - start index
     * @param shape - shape of data to be read
     * @return 0 if successful, otherwise return -1
     */
    public int read(NetcdfFile ncfile,int[] org,int[] shape){
        int status = -1;
        try {
            Variable var = ncfile.findVariable(varName);
            if (var != null) {
                if (org==null || shape==null){
                    varData=var.read();
                }else{
                    varData = var.read(org, shape);
                }
                if (varData==null){
                    return -1;
                }
                status=0;
            } else {
                return -1;
            }
            
        } catch (java.io.IOException e) {
            return -1;
        } catch (InvalidRangeException e){
            return -1;
        }
        return status;    
    }
 
    /**
     * Write the data buffer of a variable to netCDF file
     * @param ncfileWritable - netCDF file
     * @return 0 if successful, otherwise return -1
   */
    public int writeDataToFile(NetcdfFileWriteable ncfileWritable) {
        
        try {
            if (bWrite2File) {
                ncfileWritable.write(varName, varData);
            }
        } catch (IOException e) {
            return -1;
        } catch (InvalidRangeException e) {
            return -1;
        }
        return 0;
    }

    /**
     * Get the name of a variable item
     * @return name of variable item
     */
    @Override
    public String toString(){
        return varName;
    }
}
