/*@***h* AMSU-A_FCDR_STAR/pub.variable.VariablesGroup.java
* NAME
*   pub.variable.VariablesGroup.java 
* 
* PURPOSE
*   To support multiple climate data record variables.
* 
* DESCRIPTION
*   This class supports a group of variables. It provides methods for creating 
*   multiple variables in netCDF file, as well as reading and writing multiple 
*   variables from and to netCDF file.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/05/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/08/2013 - added setWrite2FileFlag() to control whether variables in 
*                  this group will be written to the output file. 
*
*@*****/
 
 
package pub.variable;

import java.util.*;
import ucar.nc2.*;

public class VariablesGroup extends ArrayList{
    
    /**
     * Constructor
     */
    VariablesGroup(){
        
    }
    
    /**
     * Get one variable object by index
     * @param index - index of variable object
     * @return variable object
     */
    public VariableItem get(int index){
        VariableItem var=(VariableItem) super.get(index);
        return var;
    }
    
    /**
     * Create a group of variables in netCDF file
     * @param ncfileWritable - netCDF file
     * @return 0 if successful, otherwise return -1;
     */
    public int create(NetcdfFileWriteable ncfileWritable){
        int status=-1;
        VariableItem var;
        
        for (int i=0;i<size();i++){
            var=get(i);
            status=var.create(ncfileWritable);
            if (status!=0)
                return status;
        }
        
        return status;
    }
    
    /**
     * Read a group of variables from netCDF file
     * @param ncfile - netCDF FILE
     * @param org - int array specifying the starting index
     * @param shape - int array specifying the extents in each dimension
     * @return 0 if at least one variable is read successful, otherwise return -1.
     */
    public int read(NetcdfFile ncfile,int[] org,int[] shape){
        int status,ret=-1;
        VariableItem var;
        
        for (int i=0;i<size();i++){
            var=get(i);
            status=var.read(ncfile,org,shape);
            if (status==0)
                ret=0;
        }
        
        return ret;
        
    }
    
    /**
     * Discard variable data in the data buffer, set data buffer to null.
     */
    public void deallocate() {
        VariableItem var;
        for (int i = 0; i < size(); i++) {
            var = get(i);
            var.setData(null);
        }

    }

    /**
     * Write data buffers of all variables to file
     * @param ncfileWritable - netCDF file
     */
    public void writeDataToFile(NetcdfFileWriteable ncfileWritable){
        VariableItem var;
        for (int i = 0; i < size(); i++) {
            var = get(i);
            var.writeDataToFile(ncfileWritable);
        }
        
    }

    /**
     * Set theWrite2File flag to all variables in this group
     * @param flag - flag that controls whether variables in this group 
     *               will be written to output file
     */
    public void setWrite2FileFlag(boolean flag){
        VariableItem var;
        for (int i = 0; i < size(); i++) {
            var = get(i);
            var.setWrite2FileFlag(flag);
        }       
    }
}
