/*@***h* AMSU-A_FCDR_STAR/pub.variable.VariablesGroupTime.java
* NAME
*   pub.variable.VariablesGroupTime.java 
* 
* PURPOSE
*   To support time variables in different formats.
* 
* DESCRIPTION
*   This class supports different time variables. It provides
*   methods for creating different time variables in netCDF file, as well as
*   reading and writing time variables from and to netCDF file.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/05/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   05/08/2013 - added methods that control whether a time variable will be 
*                written to output file.
*
*@*****/
 
 

package pub.variable;

import java.util.ArrayList;
import pub.QualityControl;
import pub.attribute.CDRVariableAttributes;
import pub.attribute.VariableAttributesCF;
import ucar.ma2.Array;
import ucar.ma2.DataType;
import ucar.ma2.Index2D;
import ucar.nc2.Dimension;
import ucar.nc2.NetcdfFileWriteable;
import util.Time;

/**
 *
 * @author wwang
 */
public class VariablesGroupTime extends VariablesGroup {
    static final   Time time1=new Time((short)1998,(short)1,(int)1);
/***************
    static final   Time time2=new Time((short)2015,(short)1,(int)1);
    * A bug creating wrong scan_time/scan_time_since78 reported by NCDC,
    * Corrected to 2050 by hqian on Nov.2015 
    * Reprocess all 2015 L1B
*/    
    static final   Time time2=new Time((short)2050,(short)1,(int)1);
    static    double min=time1.getSecondsSince((short)1978);
    static    double max=time2.getSecondsSince((short)1978);

    static public String strISO8601TimeFormat = "[YYYY]-[MM]-[DD]T[HH]-[MM]-[SS]Z";
    VariableItem varYear=null, varDayOfYear=null, varMillisecondOfDay=null;
    VariableItem varTime=null, varTimeSince1978=null;
    int[] shapeTimeISO8601 = new int[2];

    public VariablesGroupTime() {
            varTime = new VariableItem(CDRVariablesL1C.VAR_TIME);
            add(varTime);
        
            varTimeSince1978 = new VariableItem(CDRVariablesL1C.VAR_TIME_SINCE1978);
            add(varTimeSince1978);
        
            varYear = new VariableItem(CDRVariablesL1C.VAR_YEAR);
            varDayOfYear = new VariableItem(CDRVariablesL1C.VAR_DAYOFYEAR);
            varMillisecondOfDay = new VariableItem(CDRVariablesL1C.VAR_MILLISECONDOFDAY);
            add(varYear);
            add(varDayOfYear);
            add(varMillisecondOfDay);
    }

    
    /**
     * Initialize ISO8601 format time variable
     *
     * @param scanDim - scan dimension
     * @param timeISO8601Dim - dimension that gives length of a ISO8601 format
     * time string
     */
    void initVariableTime(Dimension scanDim, Dimension timeISO8601Dim) {
        if (varTime != null) {
            String varName = CDRVariablesL1C.VAR_TIME;
            //initialize Year variable
            CDRVariableAttributes attr = new CDRVariableAttributes();
            attr.setValue(VariableAttributesCF.VA_NAME_CF_LONG_NAME,
                    "Scan start time (UTC) in ISO8601 date/time (" + strISO8601TimeFormat + ") format");
            attr.setValue(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, varName);
            attr.modifyRequiredFlag(VariableAttributesCF.VA_NAME_CF_UNITS, false);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_FILLVALUE, QualityControl.missing_value_string);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_MISSINGVALUE, QualityControl.missing_value_string);


            ArrayList dimsTime = new ArrayList();
            dimsTime.add(scanDim);
            dimsTime.add(timeISO8601Dim);
            shapeTimeISO8601[0] = scanDim.getLength();
            shapeTimeISO8601[1] = timeISO8601Dim.getLength();
            Array dataTime = Array.factory(DataType.CHAR, shapeTimeISO8601);
            varTime.set(varName, DataType.CHAR, dimsTime, attr, dataTime);
        }
    }

    /**
     * Initialize time variable in a referenced or elapsed time format
     *
     * @param dims - dimension of the time variable
     * @param shape - shape of the time variable
     */
    void initVariableTimeSince1978(ArrayList dims, int[] shape) {
        if (varTimeSince1978 != null) {

            String varName = CDRVariablesL1C.VAR_TIME_SINCE1978;
            //initialize Year variable
            CDRVariableAttributes attr = new CDRVariableAttributes();
            attr.setValue(VariableAttributesCF.VA_NAME_CF_LONG_NAME, "Scan start time (UTC) in a referenced or elapsed time format");
            attr.setValue(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, "time");
            attr.setValue(VariableAttributesCF.VA_NAME_CF_UNITS, "seconds since 1978-01-01 00:00:00");
            attr.setValue(VariableAttributesCF.VA_NAME_CF_FILLVALUE, QualityControl.missing_value_double);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_MISSINGVALUE, QualityControl.missing_value_double);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_VALID_MIN, 0);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_VALID_MAX, Double.MAX_VALUE);


            Array dataTime = Array.factory(DataType.DOUBLE, shape);
            varTimeSince1978.set(varName, DataType.DOUBLE, dims, attr, dataTime);
        }
    }

    /**
     * Initialize self-defined time_year variable
     *
     * @param dims - dimension of the variable
     * @param shape - shape of the variable
     */
    void initVariableYear(ArrayList dims, int[] shape) {
        if (varYear != null) {
            String varName = CDRVariablesL1C.VAR_YEAR;
            //initialize Year variable
            CDRVariableAttributes attr = new CDRVariableAttributes();
            attr.setValue(VariableAttributesCF.VA_NAME_CF_LONG_NAME, "Year of satellite observations");
            attr.setValue(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, varName);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_UNITS, "year");
            attr.setValue(VariableAttributesCF.VA_NAME_CF_FILLVALUE, (short) -9999);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_MISSINGVALUE, (short) -9999);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_VALID_MIN, (short) 1978);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_VALID_MAX, (short) 9999);


            Array dataYear = Array.factory(DataType.SHORT, shape);
            varYear.set(varName, DataType.SHORT, dims, attr, dataYear);
        }
    }

    /**
     * Initialize self-defined time_day_of_year variable
     *
     * @param dims - dimension of the variable
     * @param shape - shape of the variable
     */
    void initVariableDayOfYear(ArrayList dims, int[] shape) {
        if (varDayOfYear != null) {
            String varName = CDRVariablesL1C.VAR_DAYOFYEAR;
            //initialize varDayOfYear
            CDRVariableAttributes attr = new CDRVariableAttributes();
            attr.setValue(VariableAttributesCF.VA_NAME_CF_LONG_NAME, "Day of year of satellite observations");
            attr.setValue(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, varName);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_UNITS, "day");
            attr.setValue(VariableAttributesCF.VA_NAME_CF_FILLVALUE, (short) -9999);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_MISSINGVALUE, (short) -9999);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_VALID_MIN, (short) 1);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_VALID_MAX, (short) 366);

            Array dataDayOfYear = Array.factory(DataType.SHORT, shape);
            varDayOfYear.set(varName, DataType.SHORT, dims, attr, dataDayOfYear);
        }
    }

    /**
     * Initialize self-defined time_millisecond_of_day variable
     *
     * @param dims - dimension of the variable
     * @param shape - shape of the variable
     */
    void initVariableMillisecondOfDay(ArrayList dims, int[] shape) {
        if (varMillisecondOfDay != null) {
            String varName = CDRVariablesL1C.VAR_MILLISECONDOFDAY;
            CDRVariableAttributes attr = new CDRVariableAttributes();
            attr.setValue(VariableAttributesCF.VA_NAME_CF_LONG_NAME, "millisecond of day of satellite observations");
            attr.setValue(VariableAttributesCF.VA_NAME_CF_STANDARD_NAME, varName);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_UNITS, "millisecond");
            attr.setValue(VariableAttributesCF.VA_NAME_CF_FILLVALUE, (int) -9999);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_MISSINGVALUE, (int) -9999);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_VALID_MIN, (int) 1);
            attr.setValue(VariableAttributesCF.VA_NAME_CF_VALID_MAX, Time.MILLISECONDS_PER_DAY);
            Array dataMillisecondOfDay = Array.factory(DataType.INT, shape);
            varMillisecondOfDay.set(varName, DataType.INT, dims, attr, dataMillisecondOfDay);
        }
    }

    /**
     * Create all time variables in netCDF file
     *
     * @param ncfileWritable - netCDF file
     * @param scanDim - scan line dimension
     * @param timeISO8601Dim - dimension that gives length of a ISO8601 format
     * time string
     */
    public void create(NetcdfFileWriteable ncfileWritable, Dimension scanDim, Dimension timeISO8601Dim) {
        if (VariableControl.isWrittenTimeISO8601()) {
            initVariableTime(scanDim, timeISO8601Dim);
        }else{
            varTime.setWrite2FileFlag(false);
        }
        ArrayList dims = new ArrayList();
        dims.add(scanDim);
        int[] shape = {scanDim.getLength()};

        if (VariableControl.isWrittenTimeSince1978()) {
            initVariableTimeSince1978(dims, shape);
        }else{
            varTimeSince1978.setWrite2FileFlag(false);
        }

        if (VariableControl.isWrittenTimeNOAAL1BFORMAT()) {
            initVariableYear(dims, shape);
            initVariableDayOfYear(dims, shape);
            initVariableMillisecondOfDay(dims, shape);
        }else{
            varYear.setWrite2FileFlag(false);
            varDayOfYear.setWrite2FileFlag(false);
            varMillisecondOfDay.setWrite2FileFlag(false);
        }

        super.create(ncfileWritable);
    }

    /**
     * Get time_year data buffer
     *
     * @return data buffer of variable time_year
     */
    public Array getYearData() {
        if (varYear!=null)
            return varYear.getData();
        else
            return null;
    }

    /**
     * Get time_day_of_year data buffer
     *
     * @return data buffer of variable time_day_of_year
     */
    public Array getDayOfYearData() {
        if (varDayOfYear != null) {
            return varDayOfYear.getData();
        } else {
            return null;
        }
    }

    /**
     * Get time_millisecond_of_day data buffer
     *
     * @return data buffer of variable time_millisecond_of_day
     */
    public Array getMillisecondOfDayData() {
        if (varMillisecondOfDay != null) {
            return varMillisecondOfDay.getData();
        } else {
            return null;
        }
    }

///////////////////////////////////////////////////////////////////
    /**
     * Write time data of current scan line to l1c file data buffer
     *
     * @param t - time for current scan line
     */
    public void bufferTimeVariables(int indexLine, Time t) {
        short year = t.getYear(),
                dayOfYear = t.getDayOfYear();
        int millisecondOfDay = t.getMillisecondOfDay();
        try {
            if (VariableControl.isWrittenTimeNOAAL1BFORMAT()) {
                varYear.getData().setShort(indexLine, year);
                varDayOfYear.getData().setShort(indexLine, dayOfYear);
                varMillisecondOfDay.getData().setInt(indexLine, millisecondOfDay);
            }

            double seconds = t.getSecondsSince((short) 1978);  
            String strTimeISO8601 = t.toStringISO8601();
            if (VariableControl.isWrittenTimeSince1978()) {
                if (seconds<min || seconds>max){
                    seconds=QualityControl.missing_value_double;
                }
                varTimeSince1978.getData().setDouble(indexLine, seconds);
            }

            if (VariableControl.isWrittenTimeISO8601()) {
                if (seconds<min || seconds>max){
                    strTimeISO8601="-9999                    ";
                }
                Index2D index2 = new Index2D(shapeTimeISO8601);
                char ch;
                for (int i = 0; i < shapeTimeISO8601[1] && i < strTimeISO8601.length(); i++) {
                    ch = strTimeISO8601.charAt(i);
                    index2.set(indexLine, i);
                    varTime.getData().setChar(index2, ch);
                }
            }
        } catch (java.lang.NullPointerException e) {
        }
    }

    /**
     * Read time variable in seconds since a 1978-01-01 format for one scan line
     * @param indexLine - index of scan line
     * @param t - used for storing the time info. 
     * @return 0 if successful, otherwise return -1.
     */
    public int readTimeSince1978(int indexLine, Time t) {
        int status = -1;
        Array data = varTimeSince1978.getData();
        if (data != null) {
            double scendsSince1978 = data.getDouble(indexLine);
            t.setBySecondsSince(scendsSince1978, (short) 1978);
            status = 0;
        }
        return status;
    }

    /**
     * Read time variable in ISO8601 format for one scan line
     * @param indexLine - index of scan line
     * @param t - used for storing the time info. 
     * @return 0 if successful, otherwise return -1.
     */
    public int readTimeISO8601(int indexLine, Time t) {
        int status = -1;
        Array data = varTime.getData();
        if (data != null) {
            shapeTimeISO8601 = data.getShape();
            Index2D index2 = new Index2D(shapeTimeISO8601);
            char ch;
            String strTime = "";
            for (int i = 0; i < shapeTimeISO8601[1]; i++) {
                index2.set(indexLine, i);
                ch = data.getChar(index2);
                strTime += ch;
            }
            t.setUsingISO8601String(strTime);
            status = 0;
        }
        return status;
    }

    /**
     * Read time variable in NOAA L1B format for one scan line
     * @param indexLine - index of scan line
     * @param t - used for storing the time info. 
     * @return 0 if successful, otherwise return -1.
     */
    public int readTimeNOAAL1BFormat(int indexLine, Time t) {
        int status = -1;
        Array data = varYear.getData();
                    short year = data.getShort(indexLine);
                    short dayOfYear = varDayOfYear.getData().getShort(indexLine);
                    int millisecondOfDay = varMillisecondOfDay.getData().getInt(indexLine);
                    t.set(year, dayOfYear, millisecondOfDay);
                    status = 0;
       return status;
    }
    
    /**
     * Read observation time of a scan line
     * First try varTime, then try varYear/varDayOfYear/varMillisecondOfDay
     * 
     * @param indexLine - scan line index
     * @param time - time
     * @return 0 if successful, otherwise return -1;
     */
    public int readTime(int indexLine, Time t) {
        int status=-1;
        /*Time time1 = new Time(),
                time2 = new Time(),
                time3 = new Time();
                */
        status = readTimeSince1978(indexLine, t);
        //if (status != 0) {
            status = readTimeISO8601(indexLine, t);
            if (status != 0) {
                status = readTimeNOAAL1BFormat(indexLine, t);
            }
       //}
       return status;
    }

    /**
     * Set flag that controls whether to write varSTime to output file
     *
     * @param flag - true the variable will be written false the variable will
     * not be written
     */
    public void setVariableControl4Time(boolean flag) {
        varTime.setWrite2FileFlag(flag);
    }

    /**
     * Set flag that controls whether to write varSince1978 to output file
     *
     * @param flag - true the variable will be written false the variable will
     * not be written
     */
    public void setVariableControl4TimeSince1978(boolean flag) {
        varTimeSince1978.setWrite2FileFlag(flag);
    }

    /**
     * Set flag that controls whether to write NOAA L1B format time variables
     *
     * @param flag - true NOAA L1B format time variables will be written false
     * NOAA L1B format time variables will not be written
     */
    public void setVariableControl4TimeNOAAL1BFormat(boolean flag) {
        varYear.setWrite2FileFlag(flag);
        varDayOfYear.setWrite2FileFlag(flag);
        varMillisecondOfDay.setWrite2FileFlag(flag);
    }
}
