/*@***h* AMSU-A_FCDR_STAR/util.LandType.java
* NAME
*   util.LandType.java 
* 
* PURPOSE
*   To support 2.5 x 2.5 degree gridded land fraction dataset.
* 
* DESCRIPTION
*   This class defines constants, variables and methods for a 2.5 x 2.5 degree 
*   gridded land fraction dataset.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   12/10/2008
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
*   03/05/2010 Initial version developed for the 2010 Workshop on Climate Data Records
*              from Satellite Microwave Radiometry.
*
*@*****/
 
 
package util;

import grid.Grid;
import java.io.*;
import java.util.Scanner;

public class LandType {
    static public final byte LAND=1;
    static public final byte OCEAN=2;
    static public final byte LANDOCEAN=3;
    static public final byte UNDEFINED=-1;
    
    float resolution;      //spatial resolution of the gridded land fraction and land type data
    int nx,                //number of grid cells along the longitutinal direction
        ny;                //number of grid cells along the latitudinal direction
    float[][] landFraction;//land fraction,  <0.5 - ocean  >=0.5 land
    byte[][] landtypeByte; // land ocean mask derived from land fraction
    static public LandType landType=null;
  
    
    public LandType(int nx,int ny, float reso){
        this.nx=nx;
        this.ny=ny;
        landFraction=new float[ny][nx];
        landtypeByte=new byte[ny][nx];
        resolution=reso;
    }
    
    /**
     * Load land fraction dataset 
     * @param pathname - pathname of land fraction dataset
     * @return 0 if successful, otherwise return -1.
     */
    public int load(String pathname){
        float value;
        String line;
        try{
            File file=new File(pathname);
            FileReader fr=new FileReader(file);
            BufferedReader br = new BufferedReader(fr);
            
            
            for (int x=0;x<nx;x++){
                for (int y=0;y<ny;y++){
                    line=br.readLine();
                    value=Float.parseFloat(line);
                    landFraction[y][x]=value;
                    if (isOcean(x,y))
                        landtypeByte[y][x]=OCEAN;
                    else
                        landtypeByte[y][x]=LAND; 
                    
                }
            }
            br.close();
            
            
        }catch (IOException e){
            System.out.println("Load land type data failed: "+pathname);
            return -1;
        }
        return 0;
    }
    
    /**
     * To check whether is a grid is ocean.
     * @param x - x coordinate of the grid cell
     * @param y - y coordinate of the gird cell
     * @return true if its land fraction is less than 0.5; otherwise return false.
     */
    public boolean isOcean(int x,int y){
        int flag=0;
        if (!(x>=0 && x<=143 && y>=0 && y<=71))
            return false; 
        float type= landFraction[y][x];
        return (type<0.5f);   
    }

   /**
     * To check whether is a grid is land.
     * @param x - x coordinate of the grid cell
     * @param y - y coordinate of the gird cell
     * @return true if its land fraction is equal or larger than 0.5; 
     * otherwise return false.
     */
     public boolean isLand(int x,int y){
        if (!(x>=0 && x<=143 && y>=0 && y<=71)){
            return false; 
        }
        float type= landFraction[y][x];
        return (type>=0.5f);   
    }
    
   /**
     * To get land fraction of a grid cell.
     * @param x - x coordinate of the grid cell
     * @param y - y coordinate of the gird cell
     * @return land fraction of a grid cell
   */    
     public float getLandFraction(int x,int y){
        if (!(x>=0 && x<=143 && y>=0 && y<=71)){
            return UNDEFINED; 
        }
        float type= landFraction[y][x];
        return type;
    }
     
    /**
     * To get land type of a grid cell
     * @param x - x coordinate of the grid cell
     * @param y - y coordinate of the gird cell
     * @return land type of the grid cell
     */
    public byte getType1(int x,int y){
        if (!(x>=0 && x<=143 && y>=0 && y<=71))
            return UNDEFINED;
        return landtypeByte[y][x];
    }
    
    
    /**
     * To get land type of multiple grid cells
     * @param xy - coordinates of grid cells
     * @param landcovertype - for output 
     * @return land type of the grid cells
     */
   public byte[] getType1(int xy[][],byte[] landcovertype){
        for (int i=0;i<xy.length;i++){
            landcovertype[i]=getType1(xy[i][0],xy[i][1]);
        }
        return landcovertype;
    }

   /**
     * To get land type of multiple grid cells
     * @param xy - coordinates of grid cells in short data type
     * @param landcovertype - for output 
     * @return land type of the grid cells
     */
   public byte[] getType1(short xy[][],byte[] landcovertype){
        for (int i=0;i<xy.length;i++){
            landcovertype[i]=getType1(xy[i][0],xy[i][1]);
        }
        return landcovertype;
    }
    
   /**
    * Get land fraction data
    * @return land fraction data 
    */
    public float[][] getData(){
        return landFraction;
    }
    
    /**
     * Calculate the percentage of grid cells that are ocean
     * @return the percentage of grid cells that are ocean
     */
    public float calculateOceanPercentage(){
        int sumOcean=0,
            sumTotal=0;
        int x,y;
   
        for (y=0;y<ny;y++){
            for (x=0;x<nx;x++){
                if (isOcean(x,y))
                    sumOcean++;
                sumTotal++;
            }                            
        }
        
        float percentage=1.0f*sumOcean/sumTotal;
        return percentage;
        
    }
       
    /**
     * Old interface for initialize land type data
     * @param dirCoefficients - direction for the land type data
     * @return LandType
     */
    static public LandType createLandType(String dirCoefficients){
           String pathname=dirCoefficients+"/landfrac_2.5.txt";
           float res=2.5f;
           int nx,ny;
           
           nx=(int)(360/2.5);
           ny=(int)(180/2.5);
           
           landType=new LandType(nx,ny,2.5f);
           landType.load(pathname);   
           return landType;
    }

    /**
     * Initialize land type data.
     * Other class will access land type data through LandType.landType
     * 
     * @param dir - directory of the land fraction dataset
     * @return LandType
     */
    static public LandType init(String dir){
           String pathnameInput=dir+"landfrac_2.5.txt";
           float res=2.5f;
           int nx,ny;
           nx=Grid.GRID_X_MAX;
           ny=Grid.GRID_Y_MAX;
           
           landType=new LandType(nx,ny,res);
           landType.load(pathnameInput);   
           return landType;
    }

    
    
    static public byte[][] maskNorthAmerica;
    /**
     * Load North America mask
     * @param pathname - pathname of the North America mask
     * @param nx - total number of x grid cells
     * @param ny - total number of y grid cells
     * @return North America mask
     */
    public static int loadNorthAmericaContinentMask(String pathname, int nx, int ny) {
        maskNorthAmerica = new byte[ny][nx];

        //init
        for (int i = 0; i < ny; i++) {
            for (int j = 0; j < nx; j++) {
                maskNorthAmerica[i][j] = 0;
            }
        }

        String line;
        int n;
        float value;
        try {
            File file = new File(pathname);
            FileReader fr = new FileReader(file);
            BufferedReader br = new BufferedReader(fr);

            for (int y = 0; y < ny; y++) {
                line = br.readLine();
                n = line.length();
                //System.out.println("y="+y+" "+line);
                Scanner sc = new Scanner(line);
                sc.useDelimiter(",");
                for (int x = 0; x < nx; x++) {
                    maskNorthAmerica[y][x] = 0;
                    try {
                        value = sc.nextFloat();
                        if (value >= 0.5) {
                            maskNorthAmerica[y][x] = 1;
                        } else {
                            maskNorthAmerica[y][x] = 0;
                        }
                    } catch (java.util.InputMismatchException e) {
                        System.out.println("y=" + y + " x=" + x);
                    } catch (java.util.NoSuchElementException e) {
                        System.out.println("y=" + y + " x=" + x);
                    }
                }
                sc.close();
            }
            br.close();


        } catch (IOException e) {
            System.out.println("Load land type data failed: " + pathname);
            return -1;
        }
        return 0;
    }
    
    /**
     * For debug only
     * @param args 
     */
  public static void main(String[] args){    
      //  String nameOut="d:\\data\\amsu\\landtype.grid.csv";
      //  LandType landType=new LandType(144,72,2.5f);
      //  landType.load("d:\\data\\amsu\\coeffs\\wwang\\landfrac_2.5.txt");
      //  Grid2DFloat.dumpASCII(nameOut, landType.getData(), 0);
      //  Grid2DFloat.dumpShort(nameOut+".bin", landType.getData(), 0);
        
        loadNorthAmericaContinentMask("D:\\data\\Important.Data.Results\\LandType\\landtype.northamerica.csv",144,72);
    }}



