/*@***h* AMSU-A_FCDR_STAR/util.SurfaceElevation.java
* NAME
*   util.SurfaceElevation.java 
* 
* PURPOSE
*   To support gridded surface elevation dataset.
* 
* DESCRIPTION
*   This class provides variables and methods for gridded 
*   surface elevation dataset. It also provides methods for converting higher 
*   resolution of gridded elevation data to lower resolution.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*    05/20/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
* 
*
*@*****/
 

package util;
import grid.*;
import java.io.*;
import java.util.*;

public class SurfaceElevation {
    float[][] elev;
    int nx,ny;
    
    public SurfaceElevation(int nx,int ny){
        this.nx=nx;
        this.ny=ny;
        elev=new float[ny][nx];
    }
    
    public int loadASCII(String pathname){
        int status=0;
        float value;
        try {
            
            Scanner sc = new Scanner(new java.io.File(pathname));

            for (int y = 0; y < ny; y++) {
                String line=sc.nextLine();
                Scanner scLine=new Scanner(line);
                scLine.useDelimiter(",");
                for (int x = 0; x < nx; x++) {
                    value=scLine.nextFloat();
                    elev[y][x]=value;
                }
                scLine.close();
            }

            sc.close();
            System.out.println("Elevation binary file "+pathname+" loadded");
            //for testing Grid2DFloat.dumpASCII("d:\\1.csv", elev, 0);

        } catch (IOException e) {
            System.out.println("Elevation binary file "+pathname+" loading failed");
            status = 1;
        }
        return status;
    }
    
    public float getElevation(int x,int y){
        if (x<0)
            x=0;
        if (x>=nx)
            x=nx-1;
        if (y<0)
            y=0;
        if (y>=ny)
            y=ny-1;
        return elev[y][x];
    }
    static void loadHDegElevation(String pathname,float[][] data){
        float max=Float.MIN_VALUE,min=Float.MAX_VALUE;
        try {
            Scanner sc = new Scanner(new java.io.File(pathname));
            float value;
            
            for (int y = 359; y >=0; y--) {
                for (int x = 0; x < 720; x++) {
                    value = sc.nextFloat();
                    if (value>max)
                        max=value;
                    if (value<min)
                        min=value;
                    data[y][x] = value;
                }
                System.out.println("row "+(y+1));
            }
            sc.close();
        } catch (IOException e) {
            System.out.println(pathname + " loading failed!");
        }
        System.out.println(pathname + " loadded!");
    }
    
    static void convertHDegto25Deg(float[][] dataSrc,float[][] dataDst){
        float sum;
        int n;
        for (int y=0;y<72;y++){
            for (int x=0;x<144;x++){
                sum=0;
                n=0;
                for (int i=0;i<5;i++){
                    for (int j=0;j<5;j++){
                        sum+=dataSrc[y*5+i][x*5+j];
                        n++;
                    }
                }
                sum/=n;
                dataDst[y][x]=sum;
            }
        }
    }
    
    static void convertHDegto125Deg(float[][] dataSrc,float[][] dataDst){
        //1. 0.5 deg --> 0.25 deg
        float[][] dataQDeg=new float[720][1440];
        for (int y=0;y<720;y++){
            for (int x=0;x<1440;x++){
                dataQDeg[y][x]=dataSrc[y/2][x/2];
            }            
        }
         
        float sum;
        int n;
        for (int y=0;y<144;y++){
            for (int x=0;x<288;x++){
                sum=0;
                n=0;
                for (int i=0;i<5;i++){
                    for (int j=0;j<5;j++){
                        sum+=dataQDeg[y*5+i][x*5+j];
                        n++;
                    }
                }
                sum/=n;
                dataDst[y][x]=sum;
            }
        }
    }
    
    public static void main(String[] args){    
        //half degree data 
        String filenameIn="D:\\data\\Important.Data.Results\\elevation\\hydro1k_elevation_hdeg\\hydro1k_elev_mean_hd.asc";                       
        String dirOut="D:\\data\\Important.Data.Results\\elevation\\2.5deg\\";

        float[][] dataHDeg=new float[360][720];
        float[][] data25Deg=new float[72][144];
        float[][] data125Deg=new float[144][288];
        //load hdeg elevation data
        loadHDegElevation(filenameIn,dataHDeg);        
        //Grid2DFloat.dumpBinaryFloat(dirOut+"elev.hdg.grd", dataHDeg);
        
        //reduce resolution to 2.5 degree
        convertHDegto25Deg(dataHDeg,data25Deg);
        //Grid2DFloat.dumpBinaryFloat(dirOut+"elev.2.5deg.grd", data25Deg);
       // Grid2DFloat.dumpASCII(dirOut+"elev.2.5deg.csv", data25Deg,0);
        
        //reduce resolution to 1.25 degree
  //      convertHDegto125Deg(dataHDeg,data125Deg);
        //Grid2DFloat.dumpBinaryFloat(dirOut+"elev.1.25deg.grd", data125Deg);
//        Grid2DFloat.dumpASCII(dirOut+"elev.1.25deg.csv", data125Deg,0);
/*        LandType landType=new LandType(144,72,2.5f);
        landType.load("d:\\data\\amsu\\coeffs\\wwang\\landfrac_2.5.txt");
        Grid2DFloat.dumpASCII(nameOut, landType.getData(), 0);
        Grid2DFloat.dumpShort(nameOut+".bin", landType.getData(), 0);*/
    }
}
