/*@***h* AMSU-A_FCDR_STAR/vector.VectorFloat.java
* NAME
*   vector.VectorFloat.java 
* 
* PURPOSE
*   To support vector of floating number.
* 
* DESCRIPTION
*   This class defines a float vector, as well methods for manupulating a 
*   float vector, such as addition, subtraction, multiplication.
*   It also provides static methods for calculating mean and standard deviation
*   of a float array.
* 
* AUTHOR
*   Wenhui Wang,  I. M. Systems Group@NOAA/NESDIS/STAR
* 
* CREATION DATE
*   05/20/2013
* 
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
*   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
*   SUPPORT TO USERS.
*    
*   Fair Use of NOAA's CDR Data Sets, Algorithms and Documentation: ALTHOUGH 
*   UNRESTRICTED, USERS OF THE SOFTWARE AND DATASETS SHOULD ACKNOWLEDGE 
*   AND CITE THE NOAA MSU/AMSU CDR TEAM IN THEIR PUBLICATIONS, PRESENTATIONS, 
*   AND/OR ANY DERIVED DATA PRODUCTS THAT WERE PRODUCED USING THE ORIGINAL 
*   DATA SET.
*    
*   Acknowledgement Request:  The AMSU-A FCDR used in this study was acquired 
*   from NOAA's National Climatic Data Center (http://www.ncdc.noaa.gov). This 
*   FCDR was originally developed by Cheng-Zhi Zou and Wenhui Wang through 
*   support from NOAA's CDR Program.
*    
*   Citation Request: 
*       Zou, Cheng-Zhi and Wenhui Wang (2011), Inter-satellite calibration of 
*       AMSU-A observations for weather and climate applications, J. Geophys. Res.,
*       Vol. 116, D23113, doi:10.1029/2011JD016205
*       Zou, C.-Z., M. Goldberg, Z. Cheng, N. Grody, J. Sullivan, C. Cao, and 
*       D. Tarpley (2006) Recalibration of microwave sounding unit for climate 
*       studies using simultaneous nadir overpasses. J. Geophys. Res., 111, D19114, 
*       doi:10.1029/2005JD006798
*       Goldberg, M. D., D. S. Crosby, and L. Zhou, (2001) The limb adjustment of 
*       AMSU-A observations: Methodology and validation. J. Appl. Meteor., 40, 70.83
* 
* REVISION HISTORY
*   05/20/2013 Initial Version prepared for NCDC
* 
*
*@*****/
 
 
package vector;

import java.io.*;
import pub.QualityControl;


public class VectorFloat {
    protected float[] data;  //data buffer for the vector
    
    /**
     * Constructor
     * @param n - maximum length of float vector
     */
    public VectorFloat(int n){
        data=new float[n];
    }
    
    /**
     * Initialize the vector using a given float value
     * @param v - value to be assigned 
     */
    public void init(float v){
        for (int i=0;i<data.length;i++)
            data[i]=v;
    }
    
    /**
     * Get data buffer of the vector
     * @return data buffer
     */    
    public float[] getData(){
        return data;
    }
    
    /**
     * Set a value to a specified slot
     * @param index - index of slot
     * @param v - value
     */
    public void set(int index, float v){
        data[index]=v;
    }
    
    /**
     * Set values of a float vector using another float vector
     * @param vec - float vector whose length is equal or longer than the 
     *              current float vector
     */
    public void set(VectorFloat vec){
         for (int i=0;i<data.length;i++){
            data[i]=vec.data[i];
         }
    }
    
    /**
     * Get value at specified index
     * @param index - index
     * @return value at specified index
     */
    public float get(int index){
        return data[index];        
    }

    /**
     * Float vector addition.
     * @param vec - the second float vector.
     */
    public void plus(VectorFloat vec){
         float v;
         for (int i=0;i<data.length;i++){
             v=vec.data[i];
             if (!QualityControl.isMissingValue(v))
                data[i]+=vec.data[i];
         }
       
    }
    
    /**
     * Float vector subtraction.
     * @param vec - the float vector to be subtracted.
     */    
    public void subtract(VectorFloat vec){
         float v;
         for (int i=0;i<data.length;i++){
             v=vec.data[i];
             if (!QualityControl.isMissingValue(v))
                data[i]-=v;
         }
       
    }

 
    /**
     * Multiply a value to each elements in the vector
     * @param m - multiplier
     */
    public void multiply(float m){
         for (int i=0;i<data.length;i++){
            data[i]*=m;
         }
        
    }
    
    /**
     * Write a group of float vectors to a text file in comma delimited (.csv) format.
     * @param pathnameOut - pathname of the output file
     * @param data - float vectors
     * @return  0 if successful, otherwise return -1;
     */
    static public int dumpCSV(String pathnameOut,VectorFloat[] data){
        
        int x,y;
        String str;
        String strHeader;
        int ny=data[0].getData().length;
        int nx=data.length;
        
        try {
            DataOutputStream fp = new DataOutputStream(
                          new FileOutputStream(pathnameOut)); 
            strHeader="Row,";
            for (x=0;x<nx;x++){
                strHeader+="Col"+(x+1);
                if (x!=nx-1)
                    strHeader+=",";
                else
                    strHeader+="\r\n";
            }
            fp.writeBytes(strHeader);
            
            for (y=0;y<ny;y++){
                str=""+(y+1)+",";
                for (x=0;x<nx;x++){
                    
                  //  if (landType.getType(x, y)<0.5)
                        str+=data[x].getData()[y];
                  //  else
                  //      str+=-9999;
                    if (x!=nx-1)
                        str+=",";                                                                            
                }
                str+="\r\n";
                fp.writeBytes(str);   
            }
    
            fp.close();
        }catch (IOException e){
          String errMsg="Pentad File "+pathnameOut+" write failed";          
          System.out.println(errMsg);
      }
        return 0;
    }
   
    
    /**
     * Calculate mean of a float array using specified start and end index, 
     * with quality control.
     * @param data - float array
     * @param indexStart - starting index
     * @param indexEnd - ending index
     * @param min  -- minimum valid value
     * @param max  -- maximum valid value
     * @return mean value. if no good element exist, return missing_value.
     */    
    public static float mean(float[] data,int indexStart,int indexEnd,float min,float max){
         int n=0;
         float v,sum=0;
         for (int i=indexStart;i<=indexEnd;i++){
             v=data[i];
             if ((Float.isNaN(min) || v>=min) && (Float.isNaN(max) || v<=max)){
                 sum+=v;
                 n++;
             }else{
                 //System.out.println("VectorFloat::stddev value="+v+" throw out"+" min="+min+" max="+max);
             }
         }
         float mean=QualityControl.missing_value;
         if (n>0)
             mean=sum/n;
          return mean;
     }

    
/**
     * Calculate standard deviation of a float array using specified start and end index, 
     * with quality control.
     * @param data - float array
     * @param indexStart - starting index
     * @param indexEnd - ending index
     * @param min  - minimum valid value
     * @param max  - maximum valid value
     * @param mean - mean of the specified float array 
     * @return standard deviation value. if no good element exist, return missing_value.
     */    
     public static float stddev(float[] data,int indexStart,int indexEnd,float min,float max,float mean){
         
         int n=0;
         float v,sum=0;
         sum=0;
         n=0;

         if (Float.isNaN(mean))
             mean=mean(data,indexStart,indexEnd,min,max);
         
         for (int i=indexStart;i<=indexEnd;i++){
             v=data[i];
             if ((Float.isNaN(min) || v>=min) && (Float.isNaN(max) || v<=max)){
                    sum+=(v-mean)*(v-mean);
                 n++;
             }else{
                 //System.out.println("VectorFloat::stddev value="+v+" throw out"+" min="+min+" max="+max);
             }
         }
         
         float sigma;
         if (n>1){
             sigma=(float)Math.sqrt(sum/(n-1));             
         }else{
             sigma=Float.NaN;
         }
         return sigma;
     }
    

     /**
      * Calculate mean of a float array by excluding elements whose distance from
      * the mean is larger than 3-sigma
      * @param data - float array
      * @param indexStart - starting index
      * @param indexEnd - ending index
      * @param min  - minimum valid value
      * @param max  - maximum valid value
      * @param sigmaRet - used for return the standard deviation of the float array.
      * @return mean of the float array
      */
     static public float calMeanWith3Sigma(float[] data,int indexStart,int indexEnd,float min,float max,float[] sigmaRet){
         int n=indexEnd-indexStart+1;
         float mean;
         
         mean=mean(data,indexStart,indexEnd,min,max);
         
         if (n > 1) {
             float sigma = stddev(data, indexStart, indexEnd, min, max, mean);
             if (sigmaRet != null) {
                 sigmaRet[0] = sigma;
             }
             //recalculate mean by excluding data >3*sigma
             mean = mean(data, indexStart, indexEnd, mean - 3 * sigma, mean + 3 * sigma);
         }else{
             if (sigmaRet!=null)
                sigmaRet[0]=Float.NaN;
         }
         return mean;
     }
}
